@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: fitcircle.c,v 1.35 2008/03/24 08:58:30 guru Exp $
+ *    $Id: fitcircle.c,v 1.41 2009/07/11 03:16:29 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -64,7 +64,7 @@ struct FITCIRCLE_CTRL {	/* All control options for this program (except common a
 	/* active is TRUE if the option has been activated */
 	struct L {	/* -L[<n>] */
 		BOOLEAN active;
-		int norm;	/* 1, 2, or 3 (both) */
+		GMT_LONG norm;	/* 1, 2, or 3 (both) */
 	} L;
 	struct S {	/* -S */
 		BOOLEAN active;
@@ -79,15 +79,15 @@ int main (int argc, char **argv)
 {
 	BOOLEAN	error = FALSE, greenwich = FALSE;
 
-	char	format[BUFSIZ];
+	char	format[BUFSIZ], *not_used = NULL;
 
-	int	imin, imax, nrots, n_fields, n_expected_fields;
+	GMT_LONG	imin, imax, nrots, n_fields, n_expected_fields;
 	GMT_LONG	i, j, k, n_alloc, n_data, n, np;
 
 	double	lonsum, latsum, *in;
 	double	meanv[3], cross[3], cross_sum[3], gcpole[3], scpole[3];		/* Extra vectors  */
 	double	*a, *lambda, *v, *b, *z;	/* Matrix stuff */
-	double	get_small_circle (struct FITCIRCLE_DATA *data, GMT_LONG ndata, double *center, double *gcpole, double *scpole, int norm, double *work);
+	double	get_small_circle (struct FITCIRCLE_DATA *data, GMT_LONG ndata, double *center, double *gcpole, double *scpole, GMT_LONG norm, double *work);
 	double	rad, *work = VNULL;
 
 	FILE	*fp = NULL;
@@ -179,7 +179,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 
@@ -195,7 +195,7 @@ int main (int argc, char **argv)
 
 	data = (struct FITCIRCLE_DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct FITCIRCLE_DATA), GMT_program);
 
-	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (format, BUFSIZ, fp);
+	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (format, BUFSIZ, fp);
 
 	sprintf (format, "%s\t%s", gmtdefs.d_format, gmtdefs.d_format);
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 2;
@@ -203,13 +203,13 @@ int main (int argc, char **argv)
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_data);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_data);
 			exit (EXIT_FAILURE);
 		}
 
-		lonsum += in[0];
-		latsum += in[1];
-		GMT_geo_to_cart (&in[1], &in[0], data[n_data].x, TRUE);
+		lonsum += in[GMT_X];
+		latsum += in[GMT_Y];
+		GMT_geo_to_cart (in[GMT_Y], in[GMT_X], data[n_data].x, TRUE);
 		n_data++;
 
 		if (n_data == n_alloc) {
@@ -269,7 +269,7 @@ int main (int argc, char **argv)
 		fprintf (GMT_stdout, format, lonsum, latsum);
 		fprintf (GMT_stdout, "\tL1 N Hemisphere Great Circle Pole (Cross-Averaged)\n");
 		latsum = -latsum;
-		lonsum = d_atan2(-cross_sum[1], -cross_sum[0]) * R2D;
+		lonsum = d_atan2d(-cross_sum[1], -cross_sum[0]);
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
 		fprintf (GMT_stdout, format, lonsum, latsum);
 		fprintf (GMT_stdout, "\tL1 S Hemisphere Great Circle Pole (Cross-Averaged)\n");
@@ -306,7 +306,7 @@ int main (int argc, char **argv)
 			fprintf(stderr,"%s: Eigenvalue routine failed to converge in 50 sweeps.\n", GMT_program);
 			fprintf(stderr,"%s: The reported L2 positions might be garbage.\n", GMT_program);
 		}
-		if (gmtdefs.verbose) fprintf(stderr,"%s: Eigenvalue routine converged in %d rotations.\n", GMT_program, nrots);
+		if (gmtdefs.verbose) fprintf(stderr,"%s: Eigenvalue routine converged in %ld rotations.\n", GMT_program, nrots);
 		imax = 0;
 		imin = 2;
 		if (d_acos (GMT_dot3v (v, meanv)) > M_PI_2)
@@ -328,7 +328,7 @@ int main (int argc, char **argv)
 		fprintf (GMT_stdout, format, lonsum, latsum);
 		fprintf (GMT_stdout, "\tL2 N Hemisphere Great Circle Pole (Eigenval Method)\n");
 		latsum = -latsum;
-		lonsum = d_atan2(-gcpole[1], -gcpole[0]) * R2D;
+		lonsum = d_atan2d(-gcpole[1], -gcpole[0]);
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
 		fprintf (GMT_stdout, format, lonsum, latsum);
 		fprintf (GMT_stdout, "\tL2 S Hemisphere Great Circle Pole (Eigenval Method)\n");
@@ -362,7 +362,7 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-double	get_small_circle (struct FITCIRCLE_DATA *data, GMT_LONG ndata, double *center, double *gcpole, double *scpole, int norm, double *work)
+double	get_small_circle (struct FITCIRCLE_DATA *data, GMT_LONG ndata, double *center, double *gcpole, double *scpole, GMT_LONG norm, double *work)
 {
 
 	/* Find scpole, the pole to the best-fit small circle, 
@@ -373,7 +373,7 @@ double	get_small_circle (struct FITCIRCLE_DATA *data, GMT_LONG ndata, double *ce
 	GMT_LONG	i, j;
 	double	temppole[3], a[3], b[3], oldpole[3];
 	double	trypos, tryneg, afit, bfit, afactor, bfactor, fit, oldfit;
-	double	length_ab, length_aold, length_bold, circle_misfit(struct FITCIRCLE_DATA *data, GMT_LONG ndata, double *pole, int norm, double *work, double *circle_distance), circle_distance;
+	double	length_ab, length_aold, length_bold, circle_misfit(struct FITCIRCLE_DATA *data, GMT_LONG ndata, double *pole, GMT_LONG norm, double *work, double *circle_distance), circle_distance;
 
 	/* First find out if solution is between center and gcpole,
 		or center and -gcpole:  */
@@ -403,8 +403,8 @@ double	get_small_circle (struct FITCIRCLE_DATA *data, GMT_LONG ndata, double *ce
 	bfit = circle_misfit(data, ndata, b, norm, work, &circle_distance);
 	j = 1;
 	do {
-		afactor = sin(j * D2R);
-		bfactor = cos(j * D2R);
+		afactor = sind(j);
+		bfactor = cosd(j);
 		for (i = 0; i < 3; i++) temppole[i] = (afactor * a[i] + bfactor * b[i]);
 		GMT_normalize3v (temppole);
 		fit = circle_misfit(data, ndata, temppole, norm, work, &circle_distance);
@@ -472,7 +472,7 @@ double	get_small_circle (struct FITCIRCLE_DATA *data, GMT_LONG ndata, double *ce
 	return (R2D * circle_distance);
 }
 
-double	circle_misfit(struct FITCIRCLE_DATA *data, GMT_LONG ndata, double *pole, int norm, double *work, double *circle_distance)
+double	circle_misfit(struct FITCIRCLE_DATA *data, GMT_LONG ndata, double *pole, GMT_LONG norm, double *work, double *circle_distance)
 {
 	/* Find the L(norm) misfit between a small circle through
 		center with pole pole.  Return misfit in radians.  */
