@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt2rgb.c,v 1.48 2007/09/30 04:16:48 guru Exp $
+ *	$Id: gmt2rgb.c,v 1.56 2009/05/13 21:06:41 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -51,14 +51,15 @@ struct GMT2RGB_CTRL {
 	} L;
 	struct W {	/* -W<width/height>[/<n_bytes>] */
 		BOOLEAN active;
-		int nx, ny;	/* Dimension of image */
-		int size;	/* Number of bytes per pixels */
+		GMT_LONG nx, ny;	/* Dimension of image */
+		GMT_LONG size;	/* Number of bytes per pixels */
 	} W;
 };
 
 int main (int argc, char **argv)
 {
-	int i, k, k3, nm, error = 0, index, irgb[3], entry, one_or_zero, pos;
+	GMT_LONG i, error = 0, index, entry, one_or_zero, pos, nm, k, k3;
+	int irgb[3];
 	BOOLEAN guess = FALSE;
 	float *z;
 	double w, e, s, n;
@@ -69,8 +70,8 @@ int main (int argc, char **argv)
 	unsigned char *picture;
 	struct GMT2RGB_CTRL *Ctrl;
 	
-	void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny);
-	unsigned char *loadraw (char *file, struct imageinfo *header, int byte_per_pixel, int nx, int ny);
+	void guess_width (char *file, GMT_LONG byte_per_pixel, GMT_LONG *raw_nx, GMT_LONG *raw_ny);
+	unsigned char *loadraw (char *file, struct imageinfo *header, GMT_LONG byte_per_pixel, GMT_LONG nx, GMT_LONG ny);
 	void *New_Gmt2rgb_Ctrl (), Free_Gmt2rgb_Ctrl (struct GMT2RGB_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -160,7 +161,7 @@ int main (int argc, char **argv)
 		fprintf (stderr,"\t      the default values of dx = dy = 1 and region 1/ncols/1/nrows.\n");
 		fprintf (stderr,"\t  (2) A regular z grid file.  Use -C to provide a cpt file with which\n");
 		fprintf (stderr,"\t      to convert z to r/g/b triplets. -R, -I, and -F are ignored.\n");
-		fprintf (stderr,"\t  (3) A RGB or RGBA raw rasterfile. Since raw rastefiles have no header, you have to\n");
+		fprintf (stderr,"\t  (3) A RGB or RGBA raw rasterfile. Since raw rasterfiles have no header, you have to\n");
 		fprintf (stderr,"\t      give the image dimensions via -W\n");
 		fprintf (stderr,"\t      However, you may take the chance of letting the program try to\n");
 		fprintf (stderr,"\t      guess the image dimensions (slow, via FFT spectrum).\n");
@@ -185,6 +186,8 @@ int main (int argc, char **argv)
 
 	/* Check that the options selected are mutually consistent */
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (!Ctrl->C.active) {
 		if (!file) {
 			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify input raster file\n", GMT_program);
@@ -206,11 +209,11 @@ int main (int argc, char **argv)
 			guess_width (file, Ctrl->W.size, &Ctrl->W.nx, &Ctrl->W.ny);
 
 		if (Ctrl->W.active && Ctrl->W.nx <= 0) {
-			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Witdth of raw raster file must be a positive integer. Not %d\n", GMT_program, Ctrl->W.nx);
+			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Witdth of raw raster file must be a positive integer. Not %ld\n", GMT_program, Ctrl->W.nx);
 			error++;
 		}
 		if (Ctrl->W.active && Ctrl->W.ny <= 0) {
-			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Height of raw raster file must be a positive integer. Not %d\n", GMT_program, Ctrl->W.ny);
+			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Height of raw raster file must be a positive integer. Not %ld\n", GMT_program, Ctrl->W.ny);
 			error++;
 		}
 	}
@@ -239,7 +242,8 @@ int main (int argc, char **argv)
 	if (Ctrl->C.active) {
 		GMT_read_cpt (Ctrl->C.file);
 		GMT_err_fail (GMT_read_grd_info (file, &grd), file);
-		nm = grd.nx * grd.ny;
+		nm = GMT_get_nm (grd.nx, grd.ny);
+		
 		z = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 		for (i = 0; i < 3; i++) {	/* Do the r, g, and b channels */
 			if (Ctrl->L.active && Ctrl->L.layer != rgb[i]) continue;
@@ -312,7 +316,7 @@ int main (int argc, char **argv)
 		grd.x_min = w;	grd.x_max = e;
 		grd.y_min = s;	grd.y_max = n;
 		grd.x_inc = Ctrl->I.xinc;	grd.y_inc = Ctrl->I.yinc;
-		nm = grd.nx * grd.ny;
+		nm = GMT_get_nm (grd.nx, grd.ny);
 
 		GMT_err_fail (GMT_grd_RI_verify (&grd, 1), Ctrl->G.name);
 
@@ -347,11 +351,10 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-
-unsigned char *loadraw (char *file, struct imageinfo *header, int byte_per_pixel, int nx, int ny) {
+unsigned char *loadraw (char *file, struct imageinfo *header, GMT_LONG byte_per_pixel, GMT_LONG nx, GMT_LONG ny) {
 	/* loadraw reads a raw binary grb or rgba rasterfile of depth 24, or 32 into memory */
 
-	int j, i, nm;
+	GMT_LONG j, i, nm;
 	unsigned char *buffer;
 
 	FILE *fp;
@@ -366,7 +369,7 @@ unsigned char *loadraw (char *file, struct imageinfo *header, int byte_per_pixel
 	header->depth = 24;
 	header->width = nx;
 	header->height = ny;
-	nm = nx * ny * byte_per_pixel;
+	nm = ((size_t)nx) * ((size_t)ny) * ((size_t)byte_per_pixel);
 	header->length = nm;
 
 	buffer = (unsigned char *) GMT_memory (VNULL, (size_t)nm, sizeof (unsigned char), GMT_program);
@@ -390,10 +393,11 @@ unsigned char *loadraw (char *file, struct imageinfo *header, int byte_per_pixel
 	return (buffer);
 }
 
-void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
+void guess_width (char *file, GMT_LONG byte_per_pixel, GMT_LONG *raw_nx, GMT_LONG *raw_ny) {
 	unsigned char *buffer;
 	float	*work, *datac, *img_pow, pow_max = -FLT_MAX, pm;
-	int	narray, img_size, k = 0, j, rgb[3], inc, n_pix, i, l, even;
+	GMT_LONG k = 0, j, inc, i, l, even, narray, img_size, n_pix;
+	int rgb[3];
 	FILE *fp;
 
 	if ((fp = GMT_fopen (file, "rb")) == NULL) {
@@ -462,12 +466,12 @@ void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
 		fprintf (stderr, "%s WARNING: first test based on FFT failed to guess image dimensions.\n\tI'll do now a second try\t", GMT_program);
 		k = 1;		pm = 1;		l = 1;
 		while (k < 41) {
-			i = *raw_ny + (int)copysign((float)l,pm);
+			i = *raw_ny + (int)irint (copysign((double)l, (double)pm));
 			pm *= -1.;
-			j = (*raw_nx) + (int)copysign((float)l,pm);
+			j = (*raw_nx) + (int)irint (copysign((double)l, (double)pm));
 			if (i*j == n_pix) {	/* Got a good candidate */
 				*raw_ny = i;	*raw_nx = j;
-				fprintf (stderr, "... SUCESS (W = %d, H = %d)\n", *raw_nx, *raw_ny);
+				fprintf (stderr, "... SUCESS (W = %ld, H = %ld)\n", *raw_nx, *raw_ny);
 				break;
 			}
 			even = (k%2 == 0) ? 1: 0;
@@ -476,11 +480,11 @@ void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
 		}
 	}
 	else
-		if (gmtdefs.verbose) fprintf (stderr, "File %s has %d Lines and %d Cols\n", file, *raw_ny, *raw_nx);
+		if (gmtdefs.verbose) fprintf (stderr, "File %s has %ld Lines and %ld Cols\n", file, *raw_ny, *raw_nx);
 
 	/* If both attempts failed */
 	if ((*raw_nx) * (*raw_ny) != n_pix) {
-		fprintf (stderr, "FAILURE while guessing image dimensions (W = %d, H = %d)\n", *raw_nx, *raw_ny);
+		fprintf (stderr, "FAILURE while guessing image dimensions (W = %ld, H = %ld)\n", *raw_nx, *raw_ny);
 		exit (EXIT_FAILURE);
 	}
 
@@ -494,7 +498,7 @@ void guess_width (char *file, int byte_per_pixel, int *raw_nx, int *raw_ny) {
 void *New_Gmt2rgb_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GMT2RGB_CTRL *C;
 	
-	C = (struct GMT2RGB_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GMT2RGB_CTRL), "New_Gmt2rgb_Ctrl");
+	C = (struct GMT2RGB_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMT2RGB_CTRL), "New_Gmt2rgb_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
