@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_grd.h,v 1.34 2009/01/09 04:02:32 guru Exp $
+ *	$Id: gmt_grd.h,v 1.37 2009/05/13 21:06:41 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -47,13 +47,14 @@ struct GRD_HEADER {
 	int ny;				/* Number of rows */
 	int node_offset;		/* 0 for node grids, 1 for pixel grids */
 /* This section is flexible. It is not copied to any grid header */
-	int type;			/* Grid format */
+	GMT_LONG type;			/* Grid format */
 	char name[GMT_LONG_TEXT];	/* Actual name of the file after any ?<varname> and =<stuff> has been removed */
 	char varname[GRD_VARNAME_LEN];	/* NetCDF: variable name */
 	int y_order;			/* NetCDF: 1 if S->N, -1 if N->S */
 	int z_id;			/* NetCDF: id of z field */
 	int ncid;			/* NetCDF: file ID */
 	int t_index[3];			/* NetCDF: index of higher coordinates */
+	int xy_dim[2];			/* NetCDF: dimension order of x and y; normally {1, 0} */
 	double nan_value;		/* Missing value as stored in grid file */
 	double xy_off;			/* 0.0 (node_offset == 0) or 0.5 ( == 1) */
 /* The following elements should not be changed. They are copied verbatim to the native grid header */
