@@ -1,4 +1,4 @@
-/*	$Id: gmt_mgg_header2.c,v 1.28 2008/04/03 20:01:18 guru Exp $
+/*	$Id: gmt_mgg_header2.c,v 1.38 2009/07/11 09:11:05 guru Exp $
  *
  *	Code donated by David Divens, NOAA/NGDC
  *	Distributed under the GNU Public License (see COPYING for details)
@@ -18,6 +18,8 @@
 #define SEC_PER_DEG		(SEC_PER_MIN * MIN_PER_DEG)
 #define BYTE_SIZE
 
+int swap_header (MGG_GRID_HEADER_2 *header);
+
 static double dms2degrees(int deg, int min, int sec)
 {
     double  decDeg = (double)deg;
@@ -95,7 +97,7 @@ static void MGG2_2GMT(MGG_GRID_HEADER_2 *mgg, struct GRD_HEADER *gmt)
 {
 	int one_or_zero;
 	
-	memset(gmt, 0, sizeof(struct GRD_HEADER));
+	/* Do not memset the gmt header since it has the file name set */
 	
 	gmt->type = GMT_grd_format_decoder ("rf");
 	gmt->node_offset    = mgg->cellRegistration;
@@ -118,17 +120,14 @@ static void MGG2_2GMT(MGG_GRID_HEADER_2 *mgg, struct GRD_HEADER *gmt)
 
 static void swap_word(void* ptr)
 {
-#ifdef GMTSWAP
    unsigned char *tmp = ptr;
    unsigned char a = tmp[0];
    tmp[0] = tmp[1];
    tmp[1] = a;
-#endif
 }
 
-static void swap_long(void *ptr)
+static void swap_long (void *ptr)
 {
-#ifdef GMTSWAP
    unsigned char *tmp = ptr;
    unsigned char a = tmp[0];
    tmp[0] = tmp[3];
@@ -137,41 +136,48 @@ static void swap_long(void *ptr)
    a = tmp[1];
    tmp[1] = tmp[2];
    tmp[2] = a;
-#endif
 }
 
-static void swap_header(MGG_GRID_HEADER_2 *header)
+int swap_header (MGG_GRID_HEADER_2 *header)
 {
-   int i;
-   swap_long(&header->version);
-   swap_long(&header->length);
-   swap_long(&header->dataType);
-   swap_long(&header->latDeg);
-   swap_long(&header->latMin);
-   swap_long(&header->latSec);
-   swap_long(&header->latSpacing);
-   swap_long(&header->latNumCells);
-   swap_long(&header->lonDeg);
-   swap_long(&header->lonMin);
-   swap_long(&header->lonSec);
-   swap_long(&header->lonSpacing);
-   swap_long(&header->lonNumCells);
-   swap_long(&header->minValue);
-   swap_long(&header->maxValue);
-   swap_long(&header->gridRadius);
-   swap_long(&header->precision);
-   swap_long(&header->nanValue);
-   swap_long(&header->numType);
-   swap_long(&header->waterDatum);
-   swap_long(&header->dataLimit);
-   swap_long(&header->cellRegistration);
-   for (i = 0; i < GRD98_N_UNUSED; i++) swap_long(&header->unused[i]);
+	int i, version;
+	/* Determine if swapping is needed */
+	if (header->version == (MGG_MAGIC_NUM + MGG_VERSION)) return (0);	/* Version matches, No need to swap */
+	version = header->version;
+	swap_long (&version);
+	if (version != (MGG_MAGIC_NUM + MGG_VERSION)) return (-1);		/* Cannot make sense of header */
+	/* Here we come when we do need to swap */
+	swap_long (&header->version);
+	swap_long (&header->length);
+	swap_long (&header->dataType);
+	swap_long (&header->latDeg);
+	swap_long (&header->latMin);
+	swap_long (&header->latSec);
+	swap_long (&header->latSpacing);
+	swap_long (&header->latNumCells);
+	swap_long (&header->lonDeg);
+	swap_long (&header->lonMin);
+	swap_long (&header->lonSec);
+	swap_long (&header->lonSpacing);
+	swap_long (&header->lonNumCells);
+	swap_long (&header->minValue);
+	swap_long (&header->maxValue);
+	swap_long (&header->gridRadius);
+	swap_long (&header->precision);
+	swap_long (&header->nanValue);
+	swap_long (&header->numType);
+	swap_long (&header->waterDatum);
+	swap_long (&header->dataLimit);
+	swap_long (&header->cellRegistration);
+	for (i = 0; i < GRD98_N_UNUSED; i++) swap_long (&header->unused[i]);
+	return (1);	/* Signal we need to swap the data also */
 }
 
-int GMT_is_mgg2_grid (struct GRD_HEADER *header)
+GMT_LONG GMT_is_mgg2_grid (struct GRD_HEADER *header)
 {	/* Determine if file is a GRD98 file */
 	FILE *fp = NULL;
 	MGG_GRID_HEADER_2 mggHeader;
+	int ok;
 
 	if (!strcmp(header->name, "=")) return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
 	if ((fp = GMT_fopen(header->name, GMT_io.r_mode)) == NULL) return (GMT_GRDIO_OPEN_FAILED);
@@ -179,19 +185,20 @@ int GMT_is_mgg2_grid (struct GRD_HEADER *header)
 	memset(&mggHeader, '\0', sizeof(MGG_GRID_HEADER_2));
 	if (GMT_fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_READ_FAILED);
 
-	/* Swap header bytes if necessary */
-	swap_header(&mggHeader);
+	/* Swap header bytes if necessary; ok is 0|1 if successful and -1 if bad file */
+	ok = swap_header (&mggHeader);
 
 	/* Check the magic number and size of header */
-	if (mggHeader.version < MGG_MAGIC_NUM + MGG_VERSION) return (-1);	/* Not this kind of file */
+	if (ok == -1) return (-1);	/* Not this kind of file */
 	header->type = GMT_grd_format_decoder ("rf");
 	return (header->type);
 }
 
-int mgg2_read_grd_info (struct GRD_HEADER *header)
+GMT_LONG mgg2_read_grd_info (struct GRD_HEADER *header)
 {
 	FILE			*fp = NULL;
 	MGG_GRID_HEADER_2	mggHeader;
+	int ok;
 
 	if (!strcmp(header->name, "=")) {
 		fp = GMT_stdin;
@@ -201,11 +208,11 @@ int mgg2_read_grd_info (struct GRD_HEADER *header)
 	memset(&mggHeader, '\0', sizeof(MGG_GRID_HEADER_2));
 	if (GMT_fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_READ_FAILED);
 
-	/* Swap header bytes if necessary */
-	swap_header(&mggHeader);
+	/* Swap header bytes if necessary; ok is 0|1 if successful and -1 if bad file */
+	ok = swap_header(&mggHeader);
 
 	/* Check the magic number and size of header */
-	if (mggHeader.version < MGG_MAGIC_NUM + MGG_VERSION) {
+	if (ok == -1) {
 		fprintf(stderr, "GMT Fatal Error: Unrecognized header, expected 0x%04X saw 0x%04X\n", MGG_MAGIC_NUM + MGG_VERSION, mggHeader.version);
 		return (GMT_GRDIO_GRD98_BADMAGIC);
 	}
@@ -222,11 +229,11 @@ int mgg2_read_grd_info (struct GRD_HEADER *header)
 	return (GMT_NOERROR);
 }
 
-int mgg2_write_grd_info (struct GRD_HEADER *header)
+GMT_LONG mgg2_write_grd_info (struct GRD_HEADER *header)
 {
 	FILE			*fp;
 	MGG_GRID_HEADER_2	mggHeader;
-	int err;
+	GMT_LONG err;
 	
 	if (!strcmp(header->name, "=")) {
 		fp = GMT_stdout;
@@ -235,9 +242,6 @@ int mgg2_write_grd_info (struct GRD_HEADER *header)
 	
 	if ((err = GMT2MGG2(header, &mggHeader))) return (err);
 
-	/* Swap header bytes if necessary */
-	swap_header(&mggHeader);
-
 	if (GMT_fwrite (&mggHeader, sizeof(MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
 
 	if (fp != GMT_stdout) GMT_fclose (fp);
@@ -245,17 +249,18 @@ int mgg2_write_grd_info (struct GRD_HEADER *header)
 	return (GMT_NOERROR);
 }
 
-int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int pad[], BOOLEAN complex)
+GMT_LONG mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, GMT_LONG pad[], BOOLEAN complex)
 {
 	MGG_GRID_HEADER_2	mggHeader;
 	FILE  *fp     = NULL;
 	int  *tLong  = NULL;
 	short *tShort = NULL;
 	char  *tChar  = NULL;
+	float *tFloat = NULL;
 	GMT_LONG first_col, last_col, first_row, last_row, one_or_zero;
 	GMT_LONG j, j2, width_in, height_in, i_0_out, inc = 1;
 	GMT_LONG i, kk, ij, nm, width_out;
-	BOOLEAN piping = FALSE, geo = FALSE;
+	BOOLEAN piping = FALSE, geo = FALSE, swap_all = FALSE, is_float = FALSE;
 	double half_or_zero, x, small;
 	long long_offset;	/* For fseek only */
 	
@@ -266,8 +271,9 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		piping = TRUE;
 	}
 	else if ((fp = GMT_fopen (header->name, GMT_io.r_mode)) != NULL) {
-		if (GMT_fread(&mggHeader, (size_t)1, sizeof(MGG_GRID_HEADER_2), fp) != 1) return (GMT_GRDIO_READ_FAILED);
-		swap_header(&mggHeader);
+		if (GMT_fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_READ_FAILED);
+		swap_all = swap_header(&mggHeader);
+		if (swap_all == -1) return (GMT_GRDIO_GRD98_BADMAGIC);
 		if (mggHeader.numType == 0) mggHeader.numType = sizeof(int);
 	}
 
@@ -275,24 +281,33 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		return (GMT_GRDIO_OPEN_FAILED);
 	}
 	
+	is_float = (mggHeader.numType < 0 && abs(mggHeader.numType) == (int)sizeof(float));	/* Float file */
+	
 	if (w == 0.0 && e == 0.0) {	/* Get entire file and return */
 		nm = ((GMT_LONG)header->nx) * ((GMT_LONG)header->ny);
 		tLong  = (int *) GMT_memory (CNULL, (size_t)nm, sizeof (int), "mgg_read_grd");
 		tShort = (short *) tLong;
 		tChar  = (char *) tLong;
+		tFloat  = (float *) tLong;
 
-		if (GMT_fread ((void *)tLong, (size_t)mggHeader.numType, (size_t)nm, fp) != (size_t)nm) return (GMT_GRDIO_READ_FAILED);
+		if (GMT_fread ((void *)tLong, (size_t)abs(mggHeader.numType), (size_t)nm, fp) != (size_t)nm) return (GMT_GRDIO_READ_FAILED);
 		for (i = 0; i < nm; i++) {
 			/* 4-byte values */
 			if (mggHeader.numType == sizeof(int)) {
-                swap_long(&tLong[i]);
+				if (swap_all) swap_long (&tLong[i]);
 				if (tLong[i] == mggHeader.nanValue) grid[i] = GMT_f_NaN;
 				else grid[i] = (float) tLong[i] / (float) mggHeader.precision;
 			}
 
+			else if (is_float) {
+				if (swap_all) swap_long (&tLong[i]);
+				if (tLong[i] == mggHeader.nanValue) grid[i] = GMT_f_NaN;
+				else grid[i] = tFloat[i];
+			}
+
 			/* 2-byte values */
 			else if (mggHeader.numType == sizeof(short)) {
-                swap_word(&tShort[i]);
+                		if (swap_all) swap_word(&tShort[i]);
 				if (tShort[i] == mggHeader.nanValue) grid[i] = GMT_f_NaN;
 				else grid[i] = (float) tShort[i] / (float) mggHeader.precision;;
 			}
@@ -349,6 +364,7 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		tLong  = (int *) GMT_memory (CNULL, (size_t)header->nx, sizeof (int), "mgg_read_grd");
 		tShort = (short *)tLong;
 		tChar  = (char *)tLong;
+		tFloat  = (float *)tLong;
 
 		k = (int *) GMT_memory (CNULL, (size_t)width_in, sizeof (int), "mgg_read_grd");
 		
@@ -364,27 +380,32 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		}
 		if (piping)	{ /* Skip data by reading it */
 			for (j = 0; j < first_row; j++) {
-				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+				if (GMT_fread ((void *) tLong, (size_t)abs(mggHeader.numType), (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		} else { /* Simply seek by it */
-			long_offset = (long)(first_row * header->nx * mggHeader.numType);
+			long_offset = (long)(first_row * header->nx * abs(mggHeader.numType));
 			if (GMT_fseek (fp, long_offset, 1)) return (GMT_GRDIO_SEEK_FAILED);
 		}
 		
 		for (j = first_row, j2 = 0; j <= last_row; j++, j2++) {
-			if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+			if (GMT_fread ((void *) tLong, (size_t)abs(mggHeader.numType), (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			
 			ij = (j2 + pad[3]) * width_out + i_0_out;
 			for (i = 0; i < width_in; i++) {
 				kk = ij+i*inc;
 				if (mggHeader.numType == sizeof(int)) {
-					swap_long(&tLong[k[i]]);
+					if (swap_all) swap_long (&tLong[k[i]]);
 					if (tLong[k[i]] == mggHeader.nanValue) grid[kk] = GMT_f_NaN;
 					else grid[kk] = (float) tLong[k[i]] / (float) mggHeader.precision;
 				}
+				else if (is_float) {
+					if (swap_all) swap_long (&tLong[k[i]]);
+					if (tLong[k[i]] == mggHeader.nanValue) grid[kk] = GMT_f_NaN;
+					else grid[kk] = tFloat[k[i]];
+				}
 				
 				else if (mggHeader.numType == sizeof(short)) {
-					swap_word(&tShort[k[i]]);
+					if (swap_all) swap_word(&tShort[k[i]]);
 					if (tShort[k[i]] == mggHeader.nanValue) grid[kk] = GMT_f_NaN;
 					else grid[kk] = (float) tShort[k[i]] / (float) mggHeader.precision;;
 				}
@@ -401,7 +422,7 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		}
 		if (piping)	{ /* Skip data by reading it */
 			for (j = last_row + 1; j < header->ny; j++) {
-				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+				if (GMT_fread ((void *) tLong, (size_t)abs(mggHeader.numType), (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		}
 			
@@ -411,32 +432,38 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		tLong = (int *) GMT_memory (CNULL, (size_t)header->nx, sizeof (int), "mgg_read_grd");
 		tShort = (short *) tLong;
 		tChar  = (char *) tLong;
+		tFloat  = (float *)tLong;
 
 		if (piping)	{ /* Skip data by reading it */
 			for (j = 0; j < first_row; j++) {
-				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+				if (GMT_fread ((void *) tLong, (size_t)abs(mggHeader.numType), (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		} else {/* Simply seek by it */
-			long_offset = (long) (first_row * header->nx * mggHeader.numType);
+			long_offset = (long) (first_row * header->nx * abs(mggHeader.numType));
 			if (GMT_fseek (fp, long_offset, 1)) return (GMT_GRDIO_SEEK_FAILED);
 		}
 		
 		for (j = first_row, j2 = 0; j <= last_row; j++, j2++) {
 			ij = (j2 + pad[3]) * width_out + i_0_out;
-			if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+			if (GMT_fread ((void *) tLong, (size_t)abs(mggHeader.numType), (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			
 			for (i = 0; i < width_in; i++) {
 				kk = ij+i;
 				/* 4-byte values */
 				if (mggHeader.numType == sizeof(int)) {
-					swap_long(&tLong[first_col + i]);
+					if (swap_all) swap_long (&tLong[first_col + i]);
 					if (tLong[first_col + i] == mggHeader.nanValue) grid[kk] = GMT_f_NaN;
 					else grid[kk] = (float)tLong[first_col + i] / (float) mggHeader.precision;
 				}
+				else if (is_float) {
+					if (swap_all) swap_long (&tLong[first_col + i]);
+					if (tLong[first_col + i] == mggHeader.nanValue) grid[kk] = GMT_f_NaN;
+					else grid[kk] = tFloat[first_col + i];
+				}
 				
 				/* 2-byte values */
 				else if (mggHeader.numType == sizeof(short)) {
-					swap_word(&tShort[first_col + i]);
+					if (swap_all) swap_word(&tShort[first_col + i]);
 					if (tShort[first_col + i] == mggHeader.nanValue) grid[kk] = GMT_f_NaN;
 					else grid[kk] = (float) tShort[first_col + i] / (float) mggHeader.precision;;
 				} 
@@ -455,7 +482,7 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 
 		if (piping)	{/* Skip data by reading it */
 			for (j = last_row + 1; j < header->ny; j++) {
-				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) < (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+				if (GMT_fread ((void *) tLong, (size_t)abs(mggHeader.numType), (size_t)header->nx, fp) < (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		}
 	}
@@ -483,7 +510,7 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 }
 
 	
-int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
+GMT_LONG mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, GMT_LONG *pad, BOOLEAN complex)
 {
 	MGG_GRID_HEADER_2	mggHeader;
 	int *k;
@@ -492,12 +519,13 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 	GMT_LONG first_col, last_col, first_row, last_row;
 	GMT_LONG i, nm, ij, width_in;
 	
-	BOOLEAN geo = FALSE;
+	BOOLEAN geo = FALSE, is_float = FALSE;
 	double half_or_zero, small, x;
 	
 	int  *tLong;
 	short *tShort;
 	char  *tChar;
+	float *tFloat;
 	FILE *fp;
 	
 	if (complex) return (GMT_GRDIO_GRD98_COMPLEX);
@@ -519,22 +547,23 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 			header->z_max = MAX (header->z_max, grid[ij]);
 		}
 		if ((err = GMT2MGG2(header, &mggHeader))) return (err);
-		swap_header(&mggHeader);
 		if (GMT_fwrite (&mggHeader, sizeof (MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
-		swap_header(&mggHeader);
+		is_float = (mggHeader.numType < 0 && abs(mggHeader.numType) == (int)sizeof(float));	/* Float file */
 
 		tLong  = (int *) GMT_memory (CNULL, (size_t)nm, sizeof (int), "mgg_write_grd");
 		tShort = (short*)tLong;
 		tChar  = (char*)tLong;
+		tFloat  = (float*)tLong;
 		
 		for (i = 0; i < nm; i++) {
 			if (GMT_is_fnan (grid[i])) {
 				if (mggHeader.numType == sizeof(int)) {
-			       tLong[i]  = mggHeader.nanValue;
-			       swap_long(&tLong[i]);
+					tLong[i]  = mggHeader.nanValue;
+				}
+				else if (is_float) {
+					tFloat[i]  = (float)mggHeader.nanValue;
 				} else if (mggHeader.numType == sizeof(short)) {
 					tShort[i] = (short)mggHeader.nanValue;
-					swap_word(&tShort[i]);
 				} else if (mggHeader.numType == sizeof(char)) {
 					tChar[i]  = (char)mggHeader.nanValue;
 				} else {
@@ -545,12 +574,13 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 
 				if (mggHeader.numType == sizeof(int)) {
 					tLong[i] = (int)rint((double)grid[i] * mggHeader.precision);
-					swap_long(&tLong[i]);
+				}
+				else if (is_float) {
+					tFloat[i] = grid[i];
 				}
 				
 				else if (mggHeader.numType == sizeof(short)) {
 					tShort[i] = (short) rint((double)grid[i] * mggHeader.precision);
-					swap_word(&tShort[i]);
 				}
 				
 				else if (mggHeader.numType == sizeof(char)) {
@@ -562,7 +592,7 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 				}
 			}
 		}
-		if (GMT_fwrite (tLong, (size_t)mggHeader.numType, (size_t)nm, fp) != (size_t)nm) return (GMT_GRDIO_WRITE_FAILED);
+		if (GMT_fwrite (tLong, (size_t)abs(mggHeader.numType), (size_t)nm, fp) != (size_t)nm) return (GMT_GRDIO_WRITE_FAILED);
 		GMT_free ((void *)tLong);
 		return (GMT_NOERROR);
 	}
@@ -613,13 +643,13 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 	
 	/* store header information and array */
 	if ((err = GMT2MGG2(header, &mggHeader))) return (err);;
-	swap_header(&mggHeader);
 	if (GMT_fwrite (&mggHeader, sizeof (MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
-	swap_header(&mggHeader);
+	is_float = (mggHeader.numType < 0 && abs(mggHeader.numType) == (int)sizeof(float));	/* Float file */
 
 	tLong  = (int *) GMT_memory (CNULL, (size_t)width_in, sizeof (int), "mgg_write_grd");
 	tShort = (short *) tLong;
-	tChar  = (char *)  tLong;
+	tChar  = (char *) tLong;
+	tFloat  = (float *) tLong;
 	
 	if (geo) {
 		k = (int *) GMT_memory (CNULL, (size_t)width_out, sizeof (int), "mgg_write_grd");
@@ -641,6 +671,7 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 				kk = ij+k[i];
 				if (GMT_is_fnan (grid[kk])) {
 					if (mggHeader.numType == sizeof(int))       tLong[i]  = mggHeader.nanValue;
+					else if (is_float) tFloat[i]  = (float)mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(short)) tShort[i] = (short)mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(char))  tChar[i] = (char)mggHeader.nanValue;
 					else {
@@ -652,6 +683,9 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 					if (mggHeader.numType == sizeof(int)) {
 						tLong[i] = (int)rint ((double)grid[kk] * mggHeader.precision);
 					}
+					else if (is_float) {
+						tFloat[i] = grid[kk];
+					}
 					
 					else if (mggHeader.numType == sizeof(short)) {
 						tShort[i] = (short) rint((double)grid[kk] * mggHeader.precision);
@@ -665,10 +699,8 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 						return (GMT_GRDIO_UNKNOWN_TYPE);
 					}
 				}
-				if (mggHeader.numType == sizeof(int)) swap_long(&tLong[i]);
-				else if (mggHeader.numType == sizeof(short)) swap_word(&tShort[i]);
 			}
-			GMT_fwrite ((void *)tLong, (size_t)mggHeader.numType, (size_t)width_out, fp);
+			if (GMT_fwrite ((void *)tLong, (size_t)abs(mggHeader.numType), (size_t)width_out, fp) != (size_t)width_out) return (GMT_GRDIO_WRITE_FAILED);
 		}
 		GMT_free ((void *)k);
 	}
@@ -680,6 +712,7 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 				kk = ij+i;
 				if (GMT_is_fnan (grid[kk])) {
 					if (mggHeader.numType == sizeof(int))       tLong[i]  = mggHeader.nanValue;
+					else if (is_float) tFloat[i]  = (float)mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(short)) tShort[i] = (short)mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(char))  tChar[i] = (char)mggHeader.nanValue;
 					else {
@@ -690,6 +723,9 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 					if (mggHeader.numType == sizeof(int)) {
 						tLong[i] = (int) rint ((double)grid[kk] * mggHeader.precision);
 					}
+					else if (is_float) {
+						tFloat[i] = grid[kk];
+					}
 					
 					else if (mggHeader.numType == sizeof(short)) {
 						tShort[i] = (short) rint ((double)grid[kk] * mggHeader.precision);
@@ -703,10 +739,8 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 						return (GMT_GRDIO_UNKNOWN_TYPE);
 					}
 				}
-				if (mggHeader.numType == sizeof(int)) swap_long(&tLong[i]);
-				else if (mggHeader.numType == sizeof(short)) swap_word(&tShort[i]);
 			}
-			GMT_fwrite ((void *)tLong, (size_t)mggHeader.numType, (size_t)width_out, fp);
+			if (GMT_fwrite ((void *)tLong, (size_t)abs(mggHeader.numType), (size_t)width_out, fp) != (size_t)width_out) return (GMT_GRDIO_WRITE_FAILED);
 		}
 	}
 	if (fp != GMT_stdout) GMT_fclose (fp);
