@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_shore.c,v 1.31 2007/09/29 23:44:14 remko Exp $
+ *	$Id: gmt_shore.c,v 1.48 2009/07/08 21:41:40 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -42,27 +42,43 @@
  *
  */
 
+#define RIVERLAKE	5	/* Fill array id for riverlakes */
 
-int GMT_copy_to_shore_path (double *lon, double *lat, struct GMT_SHORE *s, int id);
-int GMT_shore_get_first_entry (struct GMT_SHORE *c, int dir, int *side);
-int GMT_shore_get_position (int side, short int x, short int y);
-int GMT_shore_get_next_entry (struct GMT_SHORE *c, int dir, int side, int id);
-int GMT_copy_to_br_path (double *lon, double *lat, struct GMT_BR *s, int id);
+GMT_LONG GMT_copy_to_shore_path (double *lon, double *lat, struct GMT_SHORE *s, GMT_LONG id);
+GMT_LONG GMT_shore_get_first_entry (struct GMT_SHORE *c, GMT_LONG dir, GMT_LONG *side);
+GMT_LONG GMT_shore_get_position (GMT_LONG side, short int x, short int y);
+GMT_LONG GMT_shore_get_next_entry (struct GMT_SHORE *c, GMT_LONG dir, GMT_LONG side, GMT_LONG id);
+GMT_LONG GMT_copy_to_br_path (double *lon, double *lat, struct GMT_BR *s, GMT_LONG id);
 void GMT_shore_to_degree (struct GMT_SHORE *c, short int dx, short int dy, double *lon, double *lat);
 void GMT_shore_pau_sides (struct GMT_SHORE *c);
-void GMT_shore_path_shift (double *lon, double *lat, int n, double edge);
-void GMT_shore_path_shift2 (double *lon, double *lat, int n, double west, double east, BOOLEAN leftmost);
+void GMT_shore_path_shift (double *lon, double *lat, GMT_LONG n, double edge);
+void GMT_shore_path_shift2 (double *lon, double *lat, GMT_LONG n, double west, double east, BOOLEAN leftmost);
 void GMT_br_to_degree (struct GMT_BR *c, short int dx, short int dy, double *lon, double *lat);
-void shore_prepare_sides(struct GMT_SHORE *c, int dir);
+void shore_prepare_sides(struct GMT_SHORE *c, GMT_LONG dir);
 int GMT_shore_asc_sort (const void *a, const void *b);
 int GMT_shore_desc_sort(const void *a, const void *b);
 char *GMT_shore_getpathname (char *name, char *path);
+void GMT_shore_check (BOOLEAN ok[5]);
+GMT_LONG GMT_res_to_int (char res);
 
-int GMT_set_resolution (char *res, char opt)
+void GMT_set_levels (char *info, struct GMT_SHORE_SELECT *I)
+{	/* Decode GMT's -A option for coastline levels */
+	int n;
+	char *p = NULL;
+	if (strstr (info, "+l"))  I->flag = GMT_NO_RIVERLAKES;
+	if (strstr (info, "+r"))  I->flag = GMT_NO_LAKES;
+	if ((p = strstr (info, "+p"))) {	/* Requested percentage limit on small features */
+		I->fraction = irint (1e6 * 0.01 * atoi (&p[2]));	/* Convert to integer microfraction */
+	}
+	n = sscanf (info, "%lf/%d/%d", &I->area, &I->low, &I->high);
+	if (n == 1) I->low = 0, I->high = GMT_MAX_GSHHS_LEVEL;
+}
+
+GMT_LONG GMT_set_resolution (char *res, char opt)
 {
 	/* Decodes the -D<res> option and returns the base integer value */
 
-	int base;
+	GMT_LONG base;
 
 	switch (*res) {
 		case 'f':	/* Full */
@@ -90,11 +106,46 @@ int GMT_set_resolution (char *res, char opt)
 	return (base);
 }
 
-int GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e, double s, double n)
-/* res: Resolution (f, h, i, l, c */
-                
+void GMT_shore_check (BOOLEAN ok[5])
+/* Sets ok to TRUE for those resolutions available in share for
+ * resolution (f, h, i, l, c) */
 {
-	int i, nb, idiv, iw, ie, is, in, this_south, this_west, err;
+	GMT_LONG i, j, n_found;
+	char stem[GMT_TEXT_LEN], path[BUFSIZ], *res = "clihf", *kind[3] = {"GSHHS", "river", "border"};
+	
+	for (i = 0; i < 5; i++) {	/* For each resolution... */
+		ok[i] = FALSE;
+		for (j = n_found = 0; j < 3; j++) {	/* For each data type... */
+			sprintf (stem, "binned_%s_%c", kind[j], res[i]);
+	        	if (!GMT_shore_getpathname (stem, path)) continue;	/* Failed to find file */
+			n_found++;	/* Increment how many found so far for this resolution */
+		}
+		ok[i] = (n_found == 3);	/* Need all three sets to say this resolution is covered */
+	}
+}
+
+GMT_LONG GMT_res_to_int (char res)
+{	/* Turns a resolution letter into a 0-4 integer */
+	GMT_LONG i, j;
+	char *type = "clihf";
+	
+	for (i = -1, j = 0; i == -1 && j < 5; j++) if (res == type[j]) i = j;
+	return (i);
+}
+
+char GMT_shore_adjust_res (char res) {	/* Returns the highest available resolution <= to specified resolution */
+	GMT_LONG k, orig;
+	BOOLEAN ok[5];	
+	char *type = "clihf";
+	(void)GMT_shore_check (ok);		/* See which resolutions we have */
+	k = orig = GMT_res_to_int (res);	/* Get integer value of requested resolution */
+	while (!ok[k] && k >= 0) k--;		/* Drop down one level to see if we have a lower resolution available */
+	if (k >= 0 && k != orig) fprintf (stderr, "%s: Warning: Resolution %c not available, substituting resolution %c\n", GMT_program, res, type[k]);
+	return ((k == -1) ? res : type[k]);	/* Return the chosen resolution */
+}
+
+GMT_LONG GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e, double s, double n, struct GMT_SHORE_SELECT *info) {	/* res: Resolution (f, h, i, l, c */
+	GMT_LONG i, nb, idiv, iw, ie, is, in, this_south, this_west, err;
 	short *stmp;
 	int *itmp;
 	size_t start[1], count[1];
@@ -102,8 +153,9 @@ int GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e, double s,
 	
 	sprintf (stem, "binned_GSHHS_%c", res);
 	
-        if (!GMT_shore_getpathname (stem, path)) return (GMT_GRDIO_FILE_NOT_FOUND);	/* Failed to find file */
-        
+	if (!GMT_shore_getpathname (stem, path)) return (GMT_GRDIO_FILE_NOT_FOUND);	/* Failed to find file */
+	memset ((void *)c, 0, sizeof (struct GMT_SHORE));
+		
 	GMT_err_trap (nc_open (path, NC_NOWRITE,&c->cdfid));
                 
 	/* Get all id tags */
@@ -121,11 +173,15 @@ int GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e, double s,
         GMT_err_trap (nc_inq_varid (c->cdfid, "Id_of_first_point_in_a_segment", &c->seg_start_id));
         GMT_err_trap (nc_inq_varid (c->cdfid, "Relative_longitude_from_SW_corner_of_bin", &c->pt_dx_id));
         GMT_err_trap (nc_inq_varid (c->cdfid, "Relative_latitude_from_SW_corner_of_bin", &c->pt_dy_id));
+	c->seg_frac_id = -1;	/* In case we use old GSHHS which does not have this ID */
+        nc_inq_varid (c->cdfid, "Micro_fraction_of_full_resolution_area", &c->seg_frac_id);
 
 	/* Get attributes */
 	GMT_err_trap (nc_get_att_text (c->cdfid, c->pt_dx_id, "units", c->units));
         GMT_err_trap (nc_get_att_text (c->cdfid, NC_GLOBAL, "title", c->title));
         GMT_err_trap (nc_get_att_text (c->cdfid, NC_GLOBAL, "source", c->source));
+	c->version[0] = '\0';
+	(void) nc_get_att_text (c->cdfid, NC_GLOBAL, "version", c->version);
 
 	/* Get global variables */
 
@@ -138,30 +194,35 @@ int GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e, double s,
         GMT_err_trap (nc_get_var1_int (c->cdfid, c->n_seg_id, start, &c->n_seg));
         GMT_err_trap (nc_get_var1_int (c->cdfid, c->n_pt_id, start, &c->n_pt));
 
+	c->min_area = info->area;	/* Limit the features */
+	c->min_level = info->low;	
+	c->max_level = (info->low == info->high && info->high == 0) ? GMT_MAX_GSHHS_LEVEL : info->high;	/* Default to all if not set */
+	c->flag = info->flag;
+	c->fraction = (c->seg_frac_id != -1) ? info->fraction : 0;	/* Cannot request fraction on old GSHHS files */
 
 	c->scale = (c->bin_size / 60.0) / 65535.0;
 	c->bsize = c->bin_size / 60.0;
 
-	c->bins = (int *) GMT_memory (VNULL, (size_t)c->n_bin, sizeof (int), "GMT_init_shore");
+	c->bins = (GMT_LONG *) GMT_memory (VNULL, (size_t)c->n_bin, sizeof (GMT_LONG), "GMT_init_shore");
 	
 	/* Round off area to nearest multiple of block-dimension */
 		
-	iw = (int)(floor (w / c->bsize) * c->bsize);
-	ie =  (int)(ceil (e / c->bsize) * c->bsize);
-	is = 90 - (int)(ceil ((90.0 - s) / c->bsize) * c->bsize);
-	in = 90 - (int)(floor ((90.0 - n) / c->bsize) * c->bsize);
+	iw = (GMT_LONG)(floor (w / c->bsize) * c->bsize);
+	ie =  (GMT_LONG)(ceil (e / c->bsize) * c->bsize);
+	is = 90 - (GMT_LONG)(ceil ((90.0 - s) / c->bsize) * c->bsize);
+	in = 90 - (GMT_LONG)(floor ((90.0 - n) / c->bsize) * c->bsize);
 	idiv = irint (360.0 / c->bsize);	/* Number of blocks per latitude band */
 	
 	for (i = nb = 0; i < c->n_bin; i++) {	/* Find which bins are needed */
-		this_south = 90 - (int)(c->bsize * ((i / idiv) + 1));
+		this_south = 90 - (GMT_LONG)(c->bsize * ((i / idiv) + 1));
 		if (this_south < is || this_south >= in) continue;
-		this_west = (int)(c->bsize * (i % idiv)) - 360;
+		this_west = (GMT_LONG)(c->bsize * (i % idiv)) - 360;
 		while (this_west < iw) this_west += 360;
 		if (this_west >= ie) continue;
 		c->bins[nb] = i;
 		nb++;
 	}
-	c->bins = (int *) GMT_memory ((void *)c->bins, (size_t)nb, sizeof (int), "GMT_init_shore");
+	c->bins = (GMT_LONG *) GMT_memory ((void *)c->bins, (size_t)nb, sizeof (GMT_LONG), "GMT_init_shore");
 	c->nb = nb;
 	
 	/* Get bin variables, then extract only those corresponding to the bins to use */
@@ -191,21 +252,21 @@ int GMT_init_shore (char res, struct GMT_SHORE *c, double w, double e, double s,
 	return (GMT_NOERROR);
 }
 
-int GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_level, int max_level)
+GMT_LONG GMT_get_shore_bin (GMT_LONG b, struct GMT_SHORE *c)
 /* b: index number into c->bins */
 /* min_area: Polygons with area less than this are ignored */
 /* min_level: Polygons with lower levels are ignored */
 /* max_level: Polygons with higher levels are ignored */
 {
 	size_t start[1], count[1];
-	int cut_area, *seg_area, *seg_info, *seg_start;
-	int s, i, err;
+	int *seg_area, *seg_frac = NULL, *seg_info, *seg_start;
+	GMT_LONG s, i, err, cut_area, level;
 	double w, e, dx;
-	
-	c->node_level[0] = (unsigned char)MIN (((unsigned short)c->bin_info[b] >> 9) & 7, max_level);
-	c->node_level[1] = (unsigned char)MIN (((unsigned short)c->bin_info[b] >> 6) & 7, max_level);
-	c->node_level[2] = (unsigned char)MIN (((unsigned short)c->bin_info[b] >> 3) & 7, max_level);
-	c->node_level[3] = (unsigned char)MIN ((unsigned short)c->bin_info[b] & 7, max_level);
+		
+	c->node_level[0] = (unsigned char)MIN (((unsigned short)c->bin_info[b] >> 9) & 7, c->max_level);
+	c->node_level[1] = (unsigned char)MIN (((unsigned short)c->bin_info[b] >> 6) & 7, c->max_level);
+	c->node_level[2] = (unsigned char)MIN (((unsigned short)c->bin_info[b] >> 3) & 7, c->max_level);
+	c->node_level[3] = (unsigned char)MIN ((unsigned short)c->bin_info[b] & 7, c->max_level);
 	dx = c->bin_size / 60.0;
 	c->lon_sw = (c->bins[b] % c->bin_nx) * dx;
 	c->lat_sw = 90.0 - ((c->bins[b] / c->bin_nx) + 1) * dx;
@@ -220,24 +281,30 @@ int GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_leve
 
 	if (c->bin_nseg[b] == 0) return (GMT_NOERROR);
 	
-	cut_area = irint (10.0 * min_area);
+	cut_area = irint (10.0 * c->min_area);
 	start[0] = c->bin_firstseg[b];
 	count[0] = c->bin_nseg[b];
 	
 	seg_area = (int *) GMT_memory (VNULL, (size_t)c->bin_nseg[b], sizeof (int), "GMT_get_shore_bin");
 	seg_info = (int *) GMT_memory (VNULL, (size_t)c->bin_nseg[b], sizeof (int), "GMT_get_shore_bin");
 	seg_start = (int *) GMT_memory (VNULL, (size_t)c->bin_nseg[b], sizeof (int), "GMT_get_shore_bin");
+	if (c->fraction) seg_frac = (int *) GMT_memory (VNULL, (size_t)c->bin_nseg[b], sizeof (int), "GMT_get_shore_bin");
 
 	GMT_err_trap (nc_get_vara_int (c->cdfid, c->seg_area_id, start, count, seg_area));
         GMT_err_trap (nc_get_vara_int (c->cdfid, c->seg_info_id, start, count, seg_info));
         GMT_err_trap (nc_get_vara_int (c->cdfid, c->seg_start_id, start, count, seg_start));
+	if (c->fraction) GMT_err_trap (nc_get_vara_int (c->cdfid, c->seg_frac_id, start, count, seg_frac));
 
 	/* First tally how many useful segments */
 
 	for (s = i = 0; i < c->bin_nseg[b]; i++) {
-		if (cut_area > 0 && seg_area[i] < cut_area) continue;
-		if (((seg_info[i] >> 6) & 7) < min_level) continue;
-		if (((seg_info[i] >> 6) & 7) > max_level) continue;
+		if (c->fraction && seg_frac[i] < c->fraction) continue;	/* Area of this feature is too small relative to its original size */
+		if (cut_area > 0 && abs(seg_area[i]) < cut_area) continue;	/* Use abs() since double-lined-river lakes have negative area */
+		level = ((seg_info[i] >> 6) & 7);
+		if (level < c->min_level) continue;
+		if (level > c->max_level) continue;
+		if (level == 2 && seg_area[i] < 0 && c->flag == GMT_NO_RIVERLAKES) continue;
+		if (level == 2 && seg_area[i] > 0 && c->flag == GMT_NO_LAKES) continue;
 		seg_area[s] = seg_area[i];
 		seg_info[s] = seg_info[i];
 		seg_start[s] = seg_start[i];
@@ -249,6 +316,7 @@ int GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_leve
 		GMT_free ((void *) seg_info);	
 		GMT_free ((void *) seg_area);	
 		GMT_free ((void *) seg_start);
+		if (c->fraction) GMT_free ((void *) seg_frac);
 		return (GMT_NOERROR);
 	}
 
@@ -259,6 +327,7 @@ int GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_leve
 		c->seg[s].n = (seg_info[s] >> 9);
 		c->seg[s].entry = (seg_info[s] >> 3) & 7;
 		c->seg[s].exit = seg_info[s] & 7;
+		c->seg[s].fid = (seg_area[s] < 0) ? RIVERLAKE : c->seg[s].level;
 		c->seg[s].dx = (short *) GMT_memory (VNULL, (size_t)c->seg[s].n, sizeof (short), "GMT_get_shore_bin");
 		c->seg[s].dy = (short *) GMT_memory (VNULL, (size_t)c->seg[s].n, sizeof (short), "GMT_get_shore_bin");
 		start[0] = seg_start[s];
@@ -270,15 +339,16 @@ int GMT_get_shore_bin (int b, struct GMT_SHORE *c, double min_area, int min_leve
 	GMT_free ((void *) seg_info);	
 	GMT_free ((void *) seg_area);	
 	GMT_free ((void *) seg_start);	
+	if (c->fraction) GMT_free ((void *) seg_frac);
 
 	return (GMT_NOERROR);
 }
 
-int GMT_init_br (char which, char res, struct GMT_BR *c, double w, double e, double s, double n)
+GMT_LONG GMT_init_br (char which, char res, struct GMT_BR *c, double w, double e, double s, double n)
 /* which: r(iver) or b(order) */
 /* res: Resolution (f, h, i, l, c */
 {
-	int i, nb, idiv, iw, ie, is, in, this_south, this_west, err;
+	GMT_LONG i, nb, idiv, iw, ie, is, in, this_south, this_west, err;
 	short *stmp;
 	int *itmp;
 	size_t start[1], count[1];
@@ -315,6 +385,8 @@ int GMT_init_br (char which, char res, struct GMT_BR *c, double w, double e, dou
 	GMT_err_trap (nc_get_att_text (c->cdfid, c->pt_dx_id, "units", c->units));
         GMT_err_trap (nc_get_att_text (c->cdfid, NC_GLOBAL, "title", c->title));
         GMT_err_trap (nc_get_att_text (c->cdfid, NC_GLOBAL, "source", c->source));
+	c->version[0] = '\0';
+	(void) nc_get_att_text (c->cdfid, NC_GLOBAL, "version", c->version);
 
 	/* Get global variables */
 
@@ -331,26 +403,26 @@ int GMT_init_br (char which, char res, struct GMT_BR *c, double w, double e, dou
 	c->scale = (c->bin_size / 60.0) / 65535.0;
 	c->bsize = c->bin_size / 60.0;
 
-	c->bins = (int *) GMT_memory (VNULL, (size_t)c->n_bin, sizeof (int), "GMT_init_br");
+	c->bins = (GMT_LONG *) GMT_memory (VNULL, (size_t)c->n_bin, sizeof (GMT_LONG), "GMT_init_br");
 	
 	/* Round off area to nearest multiple of block-dimension */
 		
-	iw = (int)(floor (w / c->bsize) * c->bsize);
-	ie =  (int)(ceil (e / c->bsize) * c->bsize);
-	is = 90 - (int)(ceil ((90.0 - s) / c->bsize) * c->bsize);
-	in = 90 - (int)(floor ((90.0 - n) / c->bsize) * c->bsize);
+	iw = (GMT_LONG)(floor (w / c->bsize) * c->bsize);
+	ie =  (GMT_LONG)(ceil (e / c->bsize) * c->bsize);
+	is = 90 - (GMT_LONG)(ceil ((90.0 - s) / c->bsize) * c->bsize);
+	in = 90 - (GMT_LONG)(floor ((90.0 - n) / c->bsize) * c->bsize);
 	idiv = irint (360.0 / c->bsize);	/* Number of blocks per latitude band */
 	
 	for (i = nb = 0; i < c->n_bin; i++) {	/* Find which bins are needed */
-		this_south = 90 - (int)(c->bsize * ((i / idiv) + 1));
+		this_south = 90 - (GMT_LONG)(c->bsize * ((i / idiv) + 1));
 		if (this_south < is || this_south >= in) continue;
-		this_west = (int)(c->bsize * (i % idiv)) - 360;
+		this_west = (GMT_LONG)(c->bsize * (i % idiv)) - 360;
 		while (this_west < iw) this_west += 360;
 		if (this_west >= ie) continue;
 		c->bins[nb] = i;
 		nb++;
 	}
-	c->bins = (int *) GMT_memory ((void *)c->bins, (size_t)nb, sizeof (int), "GMT_init_br");
+	c->bins = (GMT_LONG *) GMT_memory ((void *)c->bins, (size_t)nb, sizeof (GMT_LONG), "GMT_init_br");
 	c->nb = nb;
 	
 	/* Get bin variables, then extract only those corresponding to the bins to use */
@@ -376,7 +448,7 @@ int GMT_init_br (char which, char res, struct GMT_BR *c, double w, double e, dou
 	return (0);
 }
 
-int GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
+GMT_LONG GMT_get_br_bin (GMT_LONG b, struct GMT_BR *c, GMT_LONG *level, GMT_LONG n_levels)
 /* b: index number into c->bins */
 /* level: Levels of features to extract */
 /* n_levels: # of such levels. 0 means use all levels */
@@ -384,7 +456,7 @@ int GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
 	size_t start[1], count[1];
 	int *seg_start;
 	short *seg_n, *seg_level;
-	int s, i, k, skip, err;
+	GMT_LONG s, i, k, skip, err;
 	
 	c->lon_sw = (c->bins[b] % c->bin_nx) * c->bin_size / 60.0;
 	c->lat_sw = 90.0 - ((c->bins[b] / c->bin_nx) + 1) * c->bin_size / 60.0;
@@ -403,9 +475,7 @@ int GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
         GMT_err_trap (nc_get_vara_short (c->cdfid, c->seg_level_id, start, count, seg_level));
         GMT_err_trap (nc_get_vara_int (c->cdfid, c->seg_start_id, start, count, seg_start));
 
-
-	c->seg = (struct GMT_BR_SEGMENT *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct GMT_BR_SEGMENT), "GMT_get_br_bin");
-	
+	c->seg = NULL;
 	for (s = i = 0; i < c->ns; i++) {
 		if (n_levels == 0)
 			skip = FALSE;
@@ -414,7 +484,7 @@ int GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
 				if (seg_level[i] == level[k]) skip = FALSE;
 		}
 		if (skip) continue;
-		
+		if (!c->seg) c->seg = (struct GMT_BR_SEGMENT *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct GMT_BR_SEGMENT), "GMT_get_br_bin");
 		c->seg[s].n = seg_n[i];
 		c->seg[s].level = seg_level[i];
 		c->seg[s].dx = (short *) GMT_memory (VNULL, (size_t)c->seg[s].n, sizeof (short), "GMT_get_br_bin");
@@ -436,7 +506,7 @@ int GMT_get_br_bin (int b, struct GMT_BR *c, int *level, int n_levels)
 	return (GMT_NOERROR);
 }
 
-int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN assemble, BOOLEAN shift, double west, double east, struct GMT_GSHHS_POL **pol)
+GMT_LONG GMT_assemble_shore (struct GMT_SHORE *c, GMT_LONG dir, BOOLEAN assemble, BOOLEAN shift, double west, double east, struct GMT_GSHHS_POL **pol)
                 
                      
 /* assemble: TRUE if polygons is needed */
@@ -445,8 +515,8 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 
 {
 	struct GMT_GSHHS_POL *p;
-	int start_side, next_side, id, P = 0, more, p_alloc, wet_or_dry, use_this_level, high_seg_level = GMT_MAX_GSHHS_LEVEL;
-	int n_alloc, cid, nid, add, first_pos, entry_pos, n, low_level, high_level, nseg_at_level[GMT_MAX_GSHHS_LEVEL+1];
+	GMT_LONG start_side, next_side, id, P = 0, more, p_alloc, wet_or_dry, use_this_level, high_seg_level = GMT_MAX_GSHHS_LEVEL;
+	GMT_LONG n_alloc, cid, nid, add, first_pos, entry_pos, n, low_level, high_level, fid, nseg_at_level[GMT_MAX_GSHHS_LEVEL+1];
 	BOOLEAN completely_inside;
 	double *xtmp, *ytmp, plon, plat;
 	
@@ -459,6 +529,7 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 			p[id].lat = (double *) GMT_memory (VNULL, (size_t)c->seg[id].n, sizeof (double), "GMT_assemble_shore");
 			p[id].n = GMT_copy_to_shore_path (p[id].lon, p[id].lat, c, id);
 			p[id].level = c->seg[id].level;
+			p[id].fid = c->seg[id].fid;
 			p[id].interior = FALSE;
 			GMT_shore_path_shift2 (p[id].lon, p[id].lat, p[id].n, west, east, c->leftmost_bin);
 		}
@@ -469,18 +540,18 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 	
 	/* Check the consistency of node levels in case some features are dropped */
 	
-	memset ((void *)nseg_at_level, 0, (size_t)((GMT_MAX_GSHHS_LEVEL + 1) * sizeof (int)));
+	memset ((void *)nseg_at_level, 0, (size_t)((GMT_MAX_GSHHS_LEVEL + 1) * sizeof (GMT_LONG)));
 	for (id = 0; id < c->ns; id++) if (c->seg[id].entry != 4) nseg_at_level[c->seg[id].level]++;	/* Only count segments that crosses the bin */
 	for (n = 0; n <= GMT_MAX_GSHHS_LEVEL; n++) if (nseg_at_level[n]) high_seg_level = n;
 	
 	if (c->ns == 0) for (n = 0; n < 4; n++) high_seg_level = MIN (c->node_level[n], high_seg_level);	/* Initialize to lowest when there are no segments */
 	for (n = high_level = 0; n < 4; n++) {
-		c->node_level[n] = MIN (c->node_level[n], high_seg_level);
+		c->node_level[n] = (unsigned char)MIN (c->node_level[n], high_seg_level);
 		high_level = MAX (c->node_level[n], high_level);
 	}
 	
 	wet_or_dry = (dir == 1) ? 1 : 0;
-	use_this_level = (high_level%2 == wet_or_dry && high_level >= first_level);
+	use_this_level = (high_level%2 == wet_or_dry && high_level >= c->min_level);
 
 	if (c->ns == 0 && !use_this_level) return (0);	/* No polygons for this bin */
 	
@@ -498,6 +569,7 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 	if (completely_inside && use_this_level) {	/* Must include path of this bin outline as first polygon */
 		p[0].n = GMT_graticule_path (&p[0].lon, &p[0].lat, dir, c->lon_corner[3], c->lon_corner[1], c->lat_corner[0], c->lat_corner[2]);
 		p[0].level = c->node_level[0];	/* Any corner will do */
+		p[0].fid = p[0].level;	/* Assumes no riverlake is that big to contain an entire bin */
 		p[0].interior = FALSE;
 		P = 1;
 	}		
@@ -510,10 +582,11 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 		next_side = c->seg[id].exit;
 		
 		n_alloc = c->seg[id].n;
+		fid = c->seg[id].fid;
 		p[P].lon = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "GMT_assemble_shore");
 		p[P].lat = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "GMT_assemble_shore");
 		n = GMT_copy_to_shore_path (p[P].lon, p[P].lat, c, id);
-		if ((int)c->seg[id].level < low_level) low_level = (int)c->seg[id].level;
+		if ((GMT_LONG)c->seg[id].level < low_level) low_level = (GMT_LONG)c->seg[id].level;
 		
 		more = TRUE;
 		first_pos = GMT_shore_get_position (start_side, c->seg[id].dx[0], c->seg[id].dy[0]);
@@ -533,7 +606,7 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 					n += add;
 				}
 				next_side = ((id + 4) + dir + 4) % 4;
-				if ((int)c->node_level[nid] < low_level) low_level = (int)c->node_level[nid];
+				if ((GMT_LONG)c->node_level[nid] < low_level) low_level = (GMT_LONG)c->node_level[nid];
 			}
 			else {
 				GMT_shore_to_degree (c, c->seg[id].dx[0], c->seg[id].dy[0], &plon, &plat);
@@ -554,7 +627,7 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 					p[P].lat = (double *) GMT_memory ((void *)p[P].lat, (size_t)n_alloc, sizeof (double), "GMT_assemble_shore");
 					n += GMT_copy_to_shore_path (&p[P].lon[n], &p[P].lat[n], c, id);
 					next_side = c->seg[id].exit;
-					if ((int)c->seg[id].level < low_level) low_level = (int)c->seg[id].level;
+					if ((GMT_LONG)c->seg[id].level < low_level) low_level = (GMT_LONG)c->seg[id].level;
 				}
 			}
 			if (add) {
@@ -565,9 +638,10 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 		p[P].n = n;
 		p[P].interior = FALSE;
 		p[P].level = (dir == 1) ? 2 * ((low_level - 1) / 2) + 1: 2 * (low_level/2);
+		p[P].fid = (p[P].level == 2 && fid == RIVERLAKE) ? RIVERLAKE : p[P].level;	/* Not sure about this yet */
 		P++;
 		if (P == p_alloc) {
-			p_alloc += GMT_SMALL_CHUNK;
+			p_alloc <<= 1;
 			p = (struct GMT_GSHHS_POL *) GMT_memory ((void *)p, (size_t)p_alloc, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_shore");
 		}
 		
@@ -583,9 +657,10 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 		p[P].n = GMT_copy_to_shore_path (p[P].lon, p[P].lat, c, id);
 		p[P].interior = TRUE;
 		p[P].level = c->seg[id].level;
+		p[P].fid = c->seg[id].fid;
 		P++;
 		if (P == p_alloc) {
-			p_alloc += GMT_SMALL_CHUNK;
+			p_alloc <<= 1;
 			p = (struct GMT_GSHHS_POL *) GMT_memory ((void *)p, (size_t)p_alloc, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_shore");
 		}
 	}
@@ -600,12 +675,12 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 	return (P);
 }
 		
-int GMT_assemble_br (struct GMT_BR *c, BOOLEAN shift, double edge, struct GMT_GSHHS_POL **pol)
+GMT_LONG GMT_assemble_br (struct GMT_BR *c, BOOLEAN shift, double edge, struct GMT_GSHHS_POL **pol)
 /* shift: TRUE if longitudes may have to be shifted */
 /* edge: Edge test for shifting */       
 {
 	struct GMT_GSHHS_POL *p;
-	int id;
+	GMT_LONG id;
 	
 	p = (struct GMT_GSHHS_POL *) GMT_memory (VNULL, (size_t)c->ns, sizeof (struct GMT_GSHHS_POL), "GMT_assemble_br");
 	
@@ -623,7 +698,7 @@ int GMT_assemble_br (struct GMT_BR *c, BOOLEAN shift, double edge, struct GMT_GS
 		
 void GMT_free_shore (struct GMT_SHORE *c)
 {	/* Removes allocated variables for this block only */
-	int i;
+	GMT_LONG i;
 	
 	for (i = 0; i < c->ns; i++) {
 		GMT_free ((void *)c->seg[i].dx);
@@ -634,13 +709,14 @@ void GMT_free_shore (struct GMT_SHORE *c)
 		
 void GMT_free_br (struct GMT_BR *c)
 {	/* Removes allocated variables for this block only */
-	int i;
+	GMT_LONG i;
 	
 	for (i = 0; i < c->ns; i++) {
 		GMT_free ((void *)c->seg[i].dx);
 		GMT_free ((void *)c->seg[i].dy);
 	}
 	if (c->ns) GMT_free ((void *)c->seg);
+	
 }
 		
 void GMT_shore_cleanup (struct GMT_SHORE *c)
@@ -661,7 +737,7 @@ void GMT_br_cleanup (struct GMT_BR *c)
 	
 }
 
-int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN sample, double step, int anti_bin)
+GMT_LONG GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, GMT_LONG np, BOOLEAN sample, double step, GMT_LONG anti_bin)
 {
 	/* This function will go through each of the polygons and determine
 	 * if the polygon is clipped by the map boundary, and if so if it
@@ -677,7 +753,7 @@ int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN sample, dou
 	 * We also explicitly close all polygons if they are not so already.
 	 */
 
-	int k, np_new, n_use, n, start, n_alloc;
+	GMT_LONG k, np_new, n_use, n, start, n_alloc;
 	BOOLEAN close;
 	double *xtmp, *ytmp;
 	struct GMT_GSHHS_POL *p;
@@ -688,12 +764,12 @@ int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN sample, dou
 		
 	for (k = 0; k < np; k++) {
 			
-		if (sample) p[k].n = GMT_fix_up_path (&p[k].lon, &p[k].lat, p[k].n, step, 0);
+		if (sample) p[k].n = GMT_fix_up_path (&p[k].lon, &p[k].lat, (GMT_LONG)p[k].n, step, 0);
 		
 		/* Clip polygon against map boundary if necessary and return plot x,y in inches */
 				
-		if ((n = GMT_clip_to_map (p[k].lon, p[k].lat, p[k].n, &xtmp, &ytmp)) == 0) {	/* Completely outside */
-			p[k].n = 0;
+		if ((n = GMT_clip_to_map (p[k].lon, p[k].lat, (GMT_LONG)p[k].n, &xtmp, &ytmp)) == 0) {	/* Completely outside */
+			p[k].n = 0;	/* Note the memory in lon, lat not freed yet */
 			continue;
 		}
 			
@@ -737,13 +813,14 @@ int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN sample, dou
 			p[np_new].n = n_alloc;
 			p[np_new].interior = p[k].interior;
 			p[np_new].level = p[k].level;
+			p[np_new].fid = p[k].fid;
 			np_new++;
 		}
 		else {
 			n_use = GMT_compact_line (xtmp, ytmp, n, FALSE, 0);
 			if (project_info.three_D) GMT_2D_to_3D (xtmp, ytmp, project_info.z_level, n_use);
 			if (anti_bin > 0 && step == 0.0) {	/* Must warn for donut effect */
-				if (gmtdefs.verbose) fprintf (stderr, "%s: GMT Warning: Antipodal bin # %d not filled!\n", GMT_program, anti_bin);
+				if (gmtdefs.verbose) fprintf (stderr, "%s: GMT Warning: Antipodal bin # %ld not filled!\n", GMT_program, anti_bin);
 				GMT_free ((void *)xtmp);
 				GMT_free ((void *)ytmp);
 				continue;
@@ -775,18 +852,18 @@ int GMT_prep_polygons (struct GMT_GSHHS_POL **p_old, int np, BOOLEAN sample, dou
 
 /* ---------- LOWER LEVEL FUNCTIONS CALLED BY THE ABOVE ------------ */
 
-int GMT_copy_to_shore_path (double *lon, double *lat, struct GMT_SHORE *s, int id)
+GMT_LONG GMT_copy_to_shore_path (double *lon, double *lat, struct GMT_SHORE *s, GMT_LONG id)
 {
-	int i;
-	for (i = 0; i < (int)s->seg[id].n; i++)
+	GMT_LONG i;
+	for (i = 0; i < (GMT_LONG)s->seg[id].n; i++)
 		GMT_shore_to_degree (s, s->seg[id].dx[i], s->seg[id].dy[i], &lon[i], &lat[i]);
 	return (s->seg[id].n);
 }
 
-int GMT_copy_to_br_path (double *lon, double *lat, struct GMT_BR *s, int id)
+GMT_LONG GMT_copy_to_br_path (double *lon, double *lat, struct GMT_BR *s, GMT_LONG id)
 {
-	int i;
-	for (i = 0; i < (int)s->seg[id].n; i++)
+	GMT_LONG i;
+	for (i = 0; i < (GMT_LONG)s->seg[id].n; i++)
 		GMT_br_to_degree (s, s->seg[id].dx[i], s->seg[id].dy[i], &lon[i], &lat[i]);
 	return (s->seg[id].n);
 }
@@ -803,13 +880,13 @@ void GMT_br_to_degree (struct GMT_BR *c, short int dx, short int dy, double *lon
 	*lat = c->lat_sw + ((unsigned short)dy) * c->scale;
 }
 
-int GMT_shore_get_next_entry (struct GMT_SHORE *c, int dir, int side, int id)
+GMT_LONG GMT_shore_get_next_entry (struct GMT_SHORE *c, GMT_LONG dir, GMT_LONG side, GMT_LONG id)
 {
 	/* Finds the next entry point on the given side that is further away
 	 * in the <dir> direction than previous point.  It removes the info
 	 * regarding the new entry from the GSHHS_SIDE structure */
 	 
-	int k, pos, n;
+	GMT_LONG k, pos, n;
 	
 	if (id < 0)
 		pos = (dir == 1) ? 0 : GSHHS_MAX_DELTA;
@@ -819,13 +896,13 @@ int GMT_shore_get_next_entry (struct GMT_SHORE *c, int dir, int side, int id)
 	}
 
 	if (dir == 1) {
-		for (k = 0; k < (int)c->nside[side] && (int)c->side[side][k].pos < pos; k++);
+		for (k = 0; k < (GMT_LONG)c->nside[side] && (GMT_LONG)c->side[side][k].pos < pos; k++);
 		id = c->side[side][k].id;
 		for (k++; k < c->nside[side]; k++) c->side[side][k-1] = c->side[side][k];
 		c->nside[side]--;
 	}
 	else {
-		for (k = 0; k < (int)c->nside[side] && (int)c->side[side][k].pos > pos; k++);
+		for (k = 0; k < (GMT_LONG)c->nside[side] && (GMT_LONG)c->side[side][k].pos > pos; k++);
 		id = c->side[side][k].id;
 		for (k++; k < c->nside[side]; k++) c->side[side][k-1] = c->side[side][k];
 		c->nside[side]--;
@@ -834,9 +911,9 @@ int GMT_shore_get_next_entry (struct GMT_SHORE *c, int dir, int side, int id)
 	return (id);
 }
 
-int GMT_shore_get_first_entry (struct GMT_SHORE *c, int dir, int *side)
+GMT_LONG GMT_shore_get_first_entry (struct GMT_SHORE *c, GMT_LONG dir, GMT_LONG *side)
 {
-	int try = 0;
+	GMT_LONG try = 0;
 	while (try < 4 && (c->nside[*side] == 0 || (c->nside[*side] == 1 && c->side[*side][0].id < 0))) {	/* No entries or only a corner left on this side */
 		try++;
 		*side = (*side + dir + 4) % 4;
@@ -861,37 +938,37 @@ int GMT_shore_desc_sort (const void *a, const void *b)
 
 void GMT_shore_pau_sides (struct GMT_SHORE *c)
 {
-	int i;
+	GMT_LONG i;
 	for (i = 0; i < 4; i++) GMT_free ((void *)c->side[i]);
 }
 
-void GMT_free_polygons (struct GMT_GSHHS_POL *p, int n)
+void GMT_free_polygons (struct GMT_GSHHS_POL *p, GMT_LONG n)
 {
-	int k;
+	GMT_LONG k;
 	for (k = 0; k < n; k++) {
 		GMT_free ((void *)p[k].lon);
 		GMT_free ((void *)p[k].lat);
 	}
 }
 
-void GMT_shore_path_shift (double *lon, double *lat, int n, double edge)
+void GMT_shore_path_shift (double *lon, double *lat, GMT_LONG n, double edge)
 {
-	int i;
+	GMT_LONG i;
 	
 	for (i = 0; i < n; i++) if (lon[i] >= edge) lon[i] -= 360.0;
 }
 
-void GMT_shore_path_shift2old (double *lon, double *lat, int n, double west, double east)
+void GMT_shore_path_shift2old (double *lon, double *lat, GMT_LONG n, double west, double east)
 {
-	int i;
+	GMT_LONG i;
 	
 	/* for (i = 0; i < n; i++) if (lon[i] >= east && (lon[i]-360.0) > west) lon[i] -= 360.0; */
 	for (i = 0; i < n; i++) if (lon[i] > east && (lon[i]-360.0) >= west) lon[i] -= 360.0;
 }
 
-void GMT_shore_path_shift2 (double *lon, double *lat, int n, double west, double east, BOOLEAN leftmost)
+void GMT_shore_path_shift2 (double *lon, double *lat, GMT_LONG n, double west, double east, BOOLEAN leftmost)
 {
-	int i;
+	GMT_LONG i;
 	
 	if (leftmost) {	/* Must check this bin differently  */
 		for (i = 0; i < n; i++) if (lon[i] >= east && (lon[i]-360.0) >= west) lon[i] -= 360.0;
@@ -901,16 +978,16 @@ void GMT_shore_path_shift2 (double *lon, double *lat, int n, double west, double
 	}
 }
 
-int GMT_shore_get_position (int side, short int x, short int y)
+GMT_LONG GMT_shore_get_position (GMT_LONG side, short int x, short int y)
 {
 	/* Returns the position along the given side */
 	
 	return ((side%2) ? ((side == 1) ? (unsigned short)y : GSHHS_MAX_DELTA - (unsigned short)y) : ((side == 0) ? (unsigned short)x : GSHHS_MAX_DELTA - (unsigned short)x));
 }
 
-void shore_prepare_sides (struct GMT_SHORE *c, int dir)
+void shore_prepare_sides (struct GMT_SHORE *c, GMT_LONG dir)
 {
-	int s, i, n[4];
+	GMT_LONG s, i, n[4];
 	
 	c->lon_corner[0] = c->lon_sw + ((dir == 1) ? c->bsize : 0.0);
 	c->lon_corner[1] = c->lon_sw + c->bsize;
@@ -935,8 +1012,8 @@ void shore_prepare_sides (struct GMT_SHORE *c, int dir)
 	for (s = 0; s < c->ns; s++) {	/* Add entry points */
 		/* if (c->seg[s].level > 2 || (i = c->seg[s].entry) == 4) continue; */
 		if ((i = c->seg[s].entry) == 4) continue;
-		c->side[i][n[i]].pos = GMT_shore_get_position (i, c->seg[s].dx[0], c->seg[s].dy[0]);
-		c->side[i][n[i]].id = s;
+		c->side[i][n[i]].pos = (unsigned short)GMT_shore_get_position (i, c->seg[s].dx[0], c->seg[s].dy[0]);
+		c->side[i][n[i]].id = (short)s;
 		n[i]++;
 	}
 	
