@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_stat.c,v 1.58 2007/09/28 19:49:43 remko Exp $
+ *	$Id: gmt_stat.c,v 1.68 2009/05/13 21:06:42 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -45,7 +45,7 @@
  *					platforms without native library support.
  *		09-MAY-2000 P. Wessel: Added GMT_chi2 and GMT_cumpoisson
  *		18-AUG-2000 P. Wessel: Moved GMT_mode and GMT_median from gmt_support to here.
- *					Added float versions of these two functions for grd data.
+ *					Added float versions of GMT_mode for grd data.
  *		27-JUL-2005 P. Wessel: Added Chebyshev polynomials Tn(x)
  *		07-SEP-2005 P. Wessel: Added GMT_corrcoeff (x,y)
  *		24-MAR-2006 P. Wessel: Added GMT_zdist (x)
@@ -89,24 +89,24 @@
 #define GMT_WITH_NO_PS
 #include "gmt.h"
 
-int GMT_inc_beta (double a, double b, double x, double *ibeta);
-int GMT_ln_gamma_r (double x, double *lngam);
-int GMT_f_test_new (double chisq1, int nu1, double chisq2, int nu2, double *prob, int iside);
-int GMT_student_t_a (double t, int n, double *prob);
+GMT_LONG GMT_inc_beta (double a, double b, double x, double *ibeta);
+GMT_LONG GMT_ln_gamma_r (double x, double *lngam);
+GMT_LONG GMT_f_test_new (double chisq1, GMT_LONG nu1, double chisq2, GMT_LONG nu2, double *prob, GMT_LONG iside);
+GMT_LONG GMT_student_t_a (double t, GMT_LONG n, double *prob);
 double GMT_ln_gamma (double xx);
 double GMT_cf_beta (double a, double b, double x);
 double GMT_bei (double x);
 double GMT_kei (double x);
 double GMT_ber (double x);
 double GMT_ker (double x);
-double GMT_plm (int l, int m, double x);
-double GMT_factorial (int n);
+double GMT_plm (GMT_LONG l, GMT_LONG m, double x);
+double GMT_factorial (GMT_LONG n);
 double GMT_i0 (double x);
 double GMT_i1 (double x);
-double GMT_in (int n, double x);
+double GMT_in (GMT_LONG n, double x);
 double GMT_k0 (double x);
 double GMT_k1 (double x);
-double GMT_kn (int n, double x);
+double GMT_kn (GMT_LONG n, double x);
 double GMT_dilog (double x);
 double GMT_ln_gamma (double xx);		/*	Computes natural log of the gamma function	*/
 double GMT_erfinv (double y);
@@ -138,7 +138,7 @@ double GMT_yn (int n, double x);
 void sincos (double a, double *s, double *c);
 #endif
 
-int	GMT_f_test_new (double chisq1, int nu1, double chisq2, int nu2, double *prob, int iside)
+GMT_LONG	GMT_f_test_new (double chisq1, GMT_LONG nu1, double chisq2, GMT_LONG nu2, double *prob, GMT_LONG iside)
 {
 	/* Given chisq1 and chisq2, random variables distributed as chi-square
 		with nu1 and nu2 degrees of freedom, respectively, except that
@@ -189,7 +189,7 @@ int	GMT_f_test_new (double chisq1, int nu1, double chisq2, int nu2, double *prob
 }
 
 
-int	GMT_f_test (double chisq1, int nu1, double chisq2, int nu2, double *prob)
+GMT_LONG	GMT_f_test (double chisq1, GMT_LONG nu1, double chisq2, GMT_LONG nu2, double *prob)
 {
 	/* Routine to compute the probability that
 		two variances are the same.
@@ -241,7 +241,7 @@ int	GMT_f_test (double chisq1, int nu1, double chisq2, int nu2, double *prob)
 	return(0);
 }
 
-int	GMT_sig_f (double chi1, int n1, double chi2, int n2, double level, double *prob)
+GMT_LONG	GMT_sig_f (double chi1, GMT_LONG n1, double chi2, GMT_LONG n2, double level, double *prob)
 {
 	/* Returns TRUE if chi1/n1 significantly less than chi2/n2
 		at the level level.  Returns FALSE if:
@@ -250,7 +250,7 @@ int	GMT_sig_f (double chi1, int n1, double chi2, int n2, double level, double *p
 
 			Changed 12 August 1999 to use GMT_f_test_new()  */
 
-	int	trouble;
+	GMT_LONG	trouble;
 
 	trouble = GMT_f_test_new (chi1, n1, chi2, n2, prob, -1);
 	if (trouble) return(0);
@@ -259,7 +259,7 @@ int	GMT_sig_f (double chi1, int n1, double chi2, int n2, double level, double *p
 
 /* --------- LOWER LEVEL FUNCTIONS ------- */
 
-int	GMT_inc_beta (double a, double b, double x, double *ibeta)
+GMT_LONG	GMT_inc_beta (double a, double b, double x, double *ibeta)
 {
 	double	bt, gama, gamb, gamab;
 
@@ -314,7 +314,7 @@ int	GMT_inc_beta (double a, double b, double x, double *ibeta)
 	return(-1);
 }
 
-int	GMT_ln_gamma_r(double x, double *lngam)
+GMT_LONG	GMT_ln_gamma_r(double x, double *lngam)
 {
 	/* Get natural logrithm of Gamma(x), x > 0.
 		To maintain full accuracy, this
@@ -362,7 +362,7 @@ double	GMT_ln_gamma (double xx)
 	static double	stp = 2.50662827465, half = 0.5, one = 1.0, fpf = 5.5;
 	double	x, tmp, ser;
 
-	int	i;
+	GMT_LONG	i;
 
 	x = xx - one;
 	tmp = x + fpf;
@@ -379,14 +379,14 @@ double	GMT_cf_beta (double a, double b, double x)
 {
 	/* Continued fraction method called by GMT_inc_beta.  */
 
-	static int	itmax = 100;
+	static GMT_LONG	itmax = 100;
 	static double	eps = 3.0e-7;
 
 	double	am = 1.0, bm = 1.0, az = 1.0;
 	double	qab, qap, qam, bz, em, tem, d;
 	double	ap, bp, app, bpp, aold;
 
-	int	m = 0;
+	GMT_LONG	m = 0;
 
 	qab = a + b;
 	qap = a + 1.0;
@@ -422,7 +422,7 @@ void	GMT_gamma_ser (double *gamser, double a, double x, double *gln) {
 	/* Returns the incomplete gamma function P(a,x) by series rep.
 	 * Press et al, gser() */
 
-	int n;
+	GMT_LONG n;
 	double sum, del, ap;
 
 	GMT_ln_gamma_r (a, gln);
@@ -453,7 +453,7 @@ void	GMT_gamma_ser (double *gamser, double a, double x, double *gln) {
 void	GMT_gamma_cf (double *gammcf, double a, double x, double *gln) {
 	/* Returns the incomplete gamma function P(a,x) by continued fraction.
 	 * Press et al, gcf() */
-	int n;
+	GMT_LONG n;
 	double gold = 0.0, g, fac = 1.0, b1 = 1.0;
 	double b0 = 0.0, anf, ana, an, a1, a0 = 1.0;
 
@@ -727,11 +727,11 @@ double GMT_i1 (double x)
 	return (res);
 }
 
-double GMT_in (int n, double x)
+double GMT_in (GMT_LONG n, double x)
 {
 	/* Modified Bessel function In(x) */
 
-	int j, m, IACC = 40;
+	GMT_LONG j, m, IACC = 40;
 	double res, tox, bip, bi, bim;
 	double BIGNO = 1.0e10, BIGNI = 1.0e-10;
 
@@ -742,7 +742,7 @@ double GMT_in (int n, double x)
 	tox = 2.0 / fabs (x);
 	bip = res = 0.0;
 	bi = 1.0;
-	m = 2 * (n + irint (sqrt (IACC * n)));
+	m = 2 * (n + irint (sqrt ((double)(IACC * n))));
 	for (j = m; j >= 1; j--) {
 		bim = bip + ((double)j) * tox * bi;
 		bip = bi;
@@ -797,11 +797,11 @@ double GMT_k1 (double x)
 	return (res);
 }
 
-double GMT_kn (int n, double x)
+double GMT_kn (GMT_LONG n, double x)
 {
 	/* Modified Bessel function Kn(x) */
 
-	int j;
+	GMT_LONG j;
 	double bkm, bk, bkp, tox;
 
 	if (n == 0) return (GMT_k0 (x));
@@ -819,12 +819,12 @@ double GMT_kn (int n, double x)
 	return (bk);
 }
 
-double GMT_plm (int l, int m, double x)
+double GMT_plm (GMT_LONG l, GMT_LONG m, double x)
 {
 	/* Unnormalized associated Legendre polynomial of degree l and order m, including
 	 * Condon-Shortley phase (-1)^m */
 	double fact, pll = 0, pmm, pmmp1, somx2;
-	int i, ll;
+	GMT_LONG i, ll;
 
 	/* x is cosine of colatitude and must be -1 <= x <= +1 */
 	if (fabs(x) > 1.0) {
@@ -860,7 +860,7 @@ double GMT_plm (int l, int m, double x)
 	return (pll);
 }
 
-double GMT_plm_bar (int l, int m, double x, BOOLEAN ortho)
+double GMT_plm_bar (GMT_LONG l, GMT_LONG m, double x, BOOLEAN ortho)
 {
 	/* This function computes the normalized associated Legendre function of x for degree
 	 * l and order m. u must be in the range [-1;1] and 0 <= |m| <= l.
@@ -897,7 +897,7 @@ double GMT_plm_bar (int l, int m, double x, BOOLEAN ortho)
 	 * recursive computation of very high degree and order normalised associated Legendre functions.
 	 * Journal of Geodesy, 76, 279-299, 2002. doi:10.1007/s00190-002-0216-2.
 	 */
-	int i;
+	GMT_LONG i;
 	BOOLEAN csphase = FALSE;
 	double scalef=1.0e280, u, r, pmm, pmm0, pmm1, pmm2;
 
@@ -931,7 +931,7 @@ double GMT_plm_bar (int l, int m, double x, BOOLEAN ortho)
 	for (i = 1; i <= m; i++) pmm = d_sqrt (1.0 + 0.5/i) * u * pmm;
 
 	/* If orthonormalization is requested: multiply by sqrt(1/4pi)
-	   In case of geophysical convertion : multiply by sqrt(2-delta_0m) */
+	   In case of geophysical conversion : multiply by sqrt(2-delta_0m) */
 
 	if (ortho) {
 		pmm *= 0.5 / d_sqrt(M_PI);
@@ -951,7 +951,7 @@ double GMT_plm_bar (int l, int m, double x, BOOLEAN ortho)
 	   First compute P_m+1,m / P_m,m */
 
 	pmm0 = 1.0/scalef;
-	pmm1 = pmm0 * x * d_sqrt (2*m + 3);
+	pmm1 = pmm0 * x * d_sqrt ((double)(2*m + 3));
 
 	/* Use second modified column forward recurrence relation to compute P_m+2,m / P_m,m */
 
@@ -980,9 +980,9 @@ double GMT_sinc (double x)
 
 /* GMT_factorial (n) calculates the factorial n! */
 
-double GMT_factorial (int n)
+double GMT_factorial (GMT_LONG n)
 {
-	int i;
+	GMT_LONG i;
 	double val = 1.0;
 
 	if (n < 0) {
@@ -1003,7 +1003,7 @@ double GMT_dilog (double x)
 	   is needed for x in the range 0 <= x <= 1 when solving the
 	   spherical spline interpolation in section 2.07 of Parker.
 
-	   I tested this for x in range 0 to 1 by reproducting Figure
+	   I tested this for x in range 0 to 1 by reproducing Figure
 	   2.07c of Parker's book.  I also tested that the result was
 	   smooth for x out to 25.  I also checked d[dilog(x)]/dx
 	   obtained numerically from this routine against an analytic
@@ -1130,7 +1130,7 @@ static double q2[5] = {1.872952849923460, 5.279051029514284e-01,
 #if HAVE_ERF == 0
 double GMT_erf (double y)
 {
-	int i, sign = 1;
+	GMT_LONG i, sign = 1;
 	double x, res, xsq, xnum, xden, xi;
 
 	x = y;
@@ -1184,7 +1184,7 @@ double GMT_erf (double y)
 #if HAVE_ERFC == 0
 double GMT_erfc (double y)
 {
-	int i, sign = 1;
+	GMT_LONG i, sign = 1;
 	double x, res, xsq, xnum, xden, xi;
 
 	x = y;
@@ -1241,7 +1241,7 @@ double GMT_erfc (double y)
 
 #if HAVE_STRDUP == 0
 char	*GMT_strdup (const char *s) {
-	int n;
+	GMT_LONG n;
 	char *p;
 
 	n = strlen (s) + 1;
@@ -1281,7 +1281,7 @@ double	GMT_strtod (const char *s, char **ends) {
 
 	char	*t, savechar;
 	double	x = 0.0;
-	int	i, nsign[2], nradix[2], nexp, ndigits, error;
+	GMT_LONG	i, nsign[2], nradix[2], nexp, ndigits, error;
 	BOOLEAN inside = FALSE;
 
 	t = (char *)s;
@@ -1346,7 +1346,7 @@ double	GMT_strtod (const char *s, char **ends) {
 }
 #endif
 
-int     GMT_f_q (double chisq1, int nu1, double chisq2, int nu2, double *prob)
+GMT_LONG     GMT_f_q (double chisq1, GMT_LONG nu1, double chisq2, GMT_LONG nu2, double *prob)
 {
 	/* Routine to compute Q(F, nu1, nu2) = 1 - P(F, nu1, nu2), where nu1
 		and nu2 are positive integers, chisq1 and chisq2 are random
@@ -1403,7 +1403,7 @@ int     GMT_f_q (double chisq1, int nu1, double chisq2, int nu2, double *prob)
 	return (0);
 }
 
-int	GMT_student_t_a(double t, int n, double *prob)
+GMT_LONG	GMT_student_t_a(double t, GMT_LONG n, double *prob)
 {
 	/* Probability integral called A(t,n) by Abramowitz &
 	Stegun for the student's t distribution with n degrees
@@ -1437,7 +1437,7 @@ int	GMT_student_t_a(double t, int n, double *prob)
 */
 
 	double	theta, s, c, csq, term, sum;
-	int	k, kstop, kt, kb;
+	GMT_LONG	k, kstop, kt, kb;
 
 	if (t < 0.0 || n <= 0) {
 		fprintf (stderr, "GMT_student_t_a:  Bad argument(s).\n");
@@ -1586,7 +1586,7 @@ double GMT_tcrit (double alpha, double nu)
 {
 	/* Critical values for Student t-distribution */
 
-	int NU;
+	GMT_LONG NU;
 	BOOLEAN done;
 	double t_low, t_high, t_mid, p_high, p_mid, p, sign;
 
@@ -1600,7 +1600,7 @@ double GMT_tcrit (double alpha, double nu)
 	}
 	t_low = GMT_zcrit (alpha);
 	t_high = 5.0;
-	NU = irint(nu);
+	NU = (GMT_LONG)irint(nu);
 	GMT_student_t_a (t_high, NU, &p_high);
 	while (p_high < p) {	/* Must pick higher starting point */
 		t_high *= 2.0;
@@ -1665,7 +1665,7 @@ double GMT_Fcrit (double alpha, double nu1, double nu2)
 {
 	/* Critical values for F-distribution */
 
-	int NU1, NU2;
+	GMT_LONG NU1, NU2;
 	BOOLEAN done;
 	double F_low, F_high, F_mid, p_high, p_mid, p, chisq1, chisq2;
 	void F_to_ch1_ch2 (double F, double nu1, double nu2, double *chisq1, double *chisq2);
@@ -1674,8 +1674,8 @@ double GMT_Fcrit (double alpha, double nu1, double nu2)
 	p = 1.0 - alpha;
 	F_low = 0.0;
 	F_to_ch1_ch2 (F_high, nu1, nu2, &chisq1, &chisq2);
-	NU1 = irint (nu1);
-	NU2 = irint (nu2);
+	NU1 = (GMT_LONG)irint (nu1);
+	NU2 = (GMT_LONG)irint (nu2);
 	GMT_f_q (chisq1, NU1, chisq2, NU2, &p_high);
 	while (p_high > p) {	/* Must pick higher starting point */
 		F_high *= 2.0;
@@ -2128,23 +2128,23 @@ double	GMT_atanh (double x) {
 
 #endif
 
-int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial, double *med)
+GMT_LONG GMT_median (double *x, GMT_LONG n, double xmin, double xmax, double m_initial, double *med)
 {
 	double	lower_bound, upper_bound, m_guess, t_0, t_1, t_middle;
 	double	lub, glb, xx, temp;
-	size_t	i, n_above, n_below, n_equal, n_lub, n_glb;
-	int	finished = FALSE;
-	int	iteration = 0;
+	GMT_LONG	i, n_above, n_below, n_equal, n_lub, n_glb, one;	/* These must be signed integers */
+	GMT_LONG	iteration = 0;
+	BOOLEAN	finished = FALSE;
 
-	if (n == 0) {
+	if (n == (GMT_LONG)0) {
 		*med = m_initial;
 		return (1);
 	}
-	if (n == 1) {
+	if (n == (GMT_LONG)1) {
 		*med = x[0];
 		return (1);
 	}
-	if (n == 2) {
+	if (n == (GMT_LONG)2) {
 		*med = 0.5 * (x[0] + x[1]);
 		return (1);
 	}
@@ -2152,13 +2152,14 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 	m_guess = m_initial;
 	lower_bound = xmin;
 	upper_bound = xmax;
-	t_0 = 0;
-	t_1 = n - 1;
-	t_middle = 0.5 * (n - 1);
+	one = (GMT_LONG)1;
+	t_0 = 0.0;
+	t_1 = (double)(n - one);
+	t_middle = 0.5 * t_1;
 
 	do {
 
-		n_above = n_below = n_equal = n_lub = n_glb = 0;
+		n_above = n_below = n_equal = n_lub = n_glb = (GMT_LONG)0;
 		lub = xmax;
 		glb = xmin;
 
@@ -2172,7 +2173,7 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 				n_above++;
 				if (xx < lub) {
 					lub = xx;
-					n_lub = 1;
+					n_lub = one;
 				}
 				else if (xx == lub) {
 					n_lub++;
@@ -2182,7 +2183,7 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 				n_below++;
 				if (xx > glb) {
 					glb = xx;
-					n_glb = 1;
+					n_glb = one;
 				}
 				else if (xx == glb) {
 					n_glb++;
@@ -2196,12 +2197,7 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 
 		if ((labs((long)(n_above - n_below))) <= (long)n_equal) {
 
-			if (n_equal) {
-				*med = m_guess;
-			}
-			else {
-				*med = 0.5 * (lub + glb);
-			}
+			*med = (n_equal) ? m_guess : 0.5 * (lub + glb);
 			finished = TRUE;
 		}
 		else if ((labs ((long)((n_above - n_lub) - (n_below + n_equal)))) < (long)n_lub) {
@@ -2219,14 +2215,14 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 		else if ( n_above > (n_below + n_equal) ) {  /* Guess is too low  */
 
 			lower_bound = m_guess;
-			t_0 = n_below + n_equal - 1;
+			t_0 = (double)(n_below + n_equal - one);
 			temp = lower_bound + (upper_bound - lower_bound) * (t_middle - t_0) / (t_1 - t_0);
 			m_guess = (temp > lub) ? temp : lub;	/* Move guess at least to lub  */
 		}
 		else if ( n_below > (n_above + n_equal) ) {  /* Guess is too high  */
 
 			upper_bound = m_guess;
-			t_1 = n_below + n_equal - 1;
+			t_1 = (double)(n_below + n_equal - one);
 			temp = lower_bound + (upper_bound - lower_bound) * (t_middle - t_0) / (t_1 - t_0);
 			m_guess = (temp < glb) ? temp : glb;	/* Move guess at least to glb  */
 		}
@@ -2241,10 +2237,10 @@ int GMT_median (double *x, size_t n, double xmin, double xmax, double m_initial,
 	return (iteration);
 }
 
-int GMT_mode (double *x, size_t n, size_t j, int sort, int mode_selection, int *n_multiples, double *mode_est)
+GMT_LONG GMT_mode (double *x, GMT_LONG n, GMT_LONG j, GMT_LONG sort, GMT_LONG mode_selection, GMT_LONG *n_multiples, double *mode_est)
 {
-	size_t	i, istop;
-	int multiplicity;
+	GMT_LONG	i, istop;
+	GMT_LONG multiplicity;
 	double	mid_point_sum = 0.0, length, short_length = DBL_MAX, this_mode;
 
 	if (n == 0) return (0);
@@ -2296,10 +2292,10 @@ int GMT_mode (double *x, size_t n, size_t j, int sort, int mode_selection, int *
 	return (0);
 }
 
-int GMT_mode_f (float *x, size_t n, size_t j, int sort, int mode_selection, int *n_multiples, double *mode_est)
+GMT_LONG GMT_mode_f (float *x, GMT_LONG n, GMT_LONG j, GMT_LONG sort, GMT_LONG mode_selection, GMT_LONG *n_multiples, double *mode_est)
 {
-	size_t	i, istop;
-	int multiplicity;
+	GMT_LONG	i, istop;
+	GMT_LONG multiplicity;
 	double	mid_point_sum = 0.0, length, short_length = FLT_MAX, this_mode;
 
 	if (n == 0) return (0);
@@ -2352,12 +2348,12 @@ int GMT_mode_f (float *x, size_t n, size_t j, int sort, int mode_selection, int
 
 /* Replacement slower functions until we figure out the problem with the algorithm */
 
-void GMT_getmad (double *x, size_t n, double location, double *scale)
+void GMT_getmad (double *x, GMT_LONG n, double location, double *scale)
 {
-	size_t i;
+	GMT_LONG i;
 	double *dev, med;
 
-	dev = (double *) GMT_memory (VNULL, n, sizeof(double), GMT_program);
+	dev = (double *) GMT_memory (VNULL, (size_t)n, sizeof(double), GMT_program);
 	for (i = 0; i < n; i++) dev[i] = fabs (x[i] - location);
 	qsort ((void *)dev, (size_t)n, sizeof (double), GMT_comp_double_asc);
 	for (i = n; GMT_is_dnan (dev[i-1]) && i > 1; i--);
@@ -2369,13 +2365,13 @@ void GMT_getmad (double *x, size_t n, double location, double *scale)
 	*scale = 1.4826 * med;
 }
 
-void GMT_getmad_f (float *x, size_t n, double location, double *scale)
+void GMT_getmad_f (float *x, GMT_LONG n, double location, double *scale)
 {
-	size_t i;
+	GMT_LONG i;
 	float *dev;
 	double med;
 
-	dev = (float *) GMT_memory (VNULL, n, sizeof(float), GMT_program);
+	dev = (float *) GMT_memory (VNULL, (size_t)n, sizeof(float), GMT_program);
 	for (i = 0; i < n; i++) dev[i] = (float) fabs ((double)(x[i] - location));
 	qsort ((void *)dev, (size_t)n, sizeof (float), GMT_comp_float_asc);
 	for (i = n; GMT_is_fnan (dev[i-1]) && i > 1; i--);
@@ -2387,12 +2383,12 @@ void GMT_getmad_f (float *x, size_t n, double location, double *scale)
 	*scale = 1.4826 * med;
 }
 
-void GMT_getmad_BROKEN (double *x, int n, double location, double *scale)
+void GMT_getmad_BROKEN (double *x, GMT_LONG n, double location, double *scale)
 {
 	/* Compute MAD (Median Absolute Deviation) for a double data set */
 
 	double	e_low, e_high, error, last_error;
-	int	i_low, i_high, n_dev, n_dev_stop;
+	GMT_LONG	i_low, i_high, n_dev, n_dev_stop;
 
 	i_low = 0;
 	while (i_low < n && x[i_low] <= location) i_low++;
@@ -2450,12 +2446,12 @@ void GMT_getmad_BROKEN (double *x, int n, double location, double *scale)
 	*scale = (n%2) ? (1.4826 * error) : (0.7413 * (error + last_error));
 }
 
-void GMT_getmad_f_BROKEN (float *x, int n, double location, double *scale)
+void GMT_getmad_f_BROKEN (float *x, GMT_LONG n, double location, double *scale)
 {
 	/* Compute MAD (Median Absolute Deviation) for a float data set */
 
 	double	e_low, e_high, error, last_error;
-	int	i_low, i_high, n_dev, n_dev_stop;
+	GMT_LONG	i_low, i_high, n_dev, n_dev_stop;
 
 	i_low = 0;
 	while (i_low < n && x[i_low] <= location) i_low++;
@@ -2514,7 +2510,7 @@ void GMT_getmad_f_BROKEN (float *x, int n, double location, double *scale)
 	*scale = (n%2) ? (1.4826 * error) : (0.7413 * (error + last_error));
 }
 
-double GMT_extreme (double x[], size_t n, double x_default, int kind, int way)
+double GMT_extreme (double x[], GMT_LONG n, double x_default, GMT_LONG kind, GMT_LONG way)
 {
 	/* Returns the extreme value in the x array according to:
 	*  kind: -1 means only consider negative values.
@@ -2526,7 +2522,7 @@ double GMT_extreme (double x[], size_t n, double x_default, int kind, int way)
 	* If kind == 0 we expect x_default to be set so that x[0] will reset x_select.
 	*/
 
-	size_t i, k;
+	GMT_LONG i, k;
 	double x_select = GMT_d_NaN;
 
 	for (i = k = 0; i < n; i++) {
@@ -2541,7 +2537,7 @@ double GMT_extreme (double x[], size_t n, double x_default, int kind, int way)
 	return ((k) ? x_select : x_default);
 }
 
-int GMT_chebyshev (double x, int n, double *t)
+GMT_LONG GMT_chebyshev (double x, GMT_LONG n, double *t)
 {
 	/* Calculates the n'th Chebyshev polynomial at x */
 
@@ -2577,13 +2573,13 @@ int GMT_chebyshev (double x, int n, double *t)
 	return (GMT_NOERROR);
 }
 
-double GMT_corrcoeff (double *x, double *y, size_t n, int mode)
+double GMT_corrcoeff (double *x, double *y, GMT_LONG n, GMT_LONG mode)
 {
 	/* Returns plain correlation coefficient, r.
 	 * If mode = 1 we assume mean(x) = mean(y) = 0.
 	 */
 
-	size_t i, n_use;
+	GMT_LONG i, n_use;
 	double xmean = 0.0, ymean = 0.0, dx, dy, vx, vy, vxy, r;
 
 	if (mode == 0) {
@@ -2612,13 +2608,13 @@ double GMT_corrcoeff (double *x, double *y, size_t n, int mode)
 	return (r);
 }
 
-double GMT_corrcoeff_f (float *x, float *y, size_t n, int mode)
+double GMT_corrcoeff_f (float *x, float *y, GMT_LONG n, GMT_LONG mode)
 {
 	/* Returns plain correlation coefficient, r.
 	 * If mode = 1 we assume mean(x) = mean(y) = 0.
 	 */
 
-	size_t i, n_use;
+	GMT_LONG i, n_use;
 	double xmean = 0.0, ymean = 0.0, dx, dy, vx, vy, vxy, r;
 
 	if (mode == 0) {
@@ -2648,12 +2644,12 @@ double GMT_corrcoeff_f (float *x, float *y, size_t n, int mode)
 }
 
 
-double GMT_quantile (double *x, double q, size_t n)
+double GMT_quantile (double *x, double q, GMT_LONG n)
 {
 	/* Returns the q'th (q in percent) quantile of x (assumed sorted).
 	 * q is expected to be 0 < q < 100 */
 
-	int i_f;
+	GMT_LONG i_f;
 	double p, f, df;
 
 	while (n > 1 && GMT_is_dnan (x[n-1])) n--;	/* Skip any NaNs at the end of x */
@@ -2661,7 +2657,7 @@ double GMT_quantile (double *x, double q, size_t n)
 	if (q == 0.0) return (x[0]);			/* 0% quantile == min(x) */
 	if (q == 100.0) return (x[n-1]);		/* 100% quantile == max(x) */
 	f = (n - 1) * q / 100.0;
-	i_f = (int)floor (f);
+	i_f = (GMT_LONG)floor (f);
 	if ((df = (f - (double)i_f)) > 0.0)		/* Must interpolate between the two neighbors */
 		p = x[i_f+1] * df + x[i_f] * (1.0 - df);
 	else						/* Exactly on a node */
@@ -2670,12 +2666,12 @@ double GMT_quantile (double *x, double q, size_t n)
 	return (p);
 }
 
-double GMT_quantile_f (float *x, double q, size_t n)
+double GMT_quantile_f (float *x, double q, GMT_LONG n)
 {
 	/* Returns the q'th (q in percent) quantile of x (assumed sorted).
 	 * q is expected to be 0 < q < 100 */
 
-	int i_f;
+	GMT_LONG i_f;
 	double p, f, df;
 
 	while (n > 1 && GMT_is_fnan (x[n-1])) n--;	/* Skip any NaNs at the end of x */
@@ -2683,7 +2679,7 @@ double GMT_quantile_f (float *x, double q, size_t n)
 	if (q == 0.0) return ((double)x[0]);		/* 0% quantile == min(x) */
 	if (q == 100.0) return ((double)x[n-1]);	/* 100% quantile == max(x) */
 	f = (n - 1) * q / 100.0;
-	i_f = (int)floor (f);
+	i_f = (GMT_LONG)floor (f);
 	if ((df = (f - (double)i_f)) > 0.0)		/* Must interpolate between the two neighbors */
 		p = (double)(x[i_f+1] * df + x[i_f] * (1.0 - df));
 	else						/* Exactly on a node */
@@ -2692,18 +2688,26 @@ double GMT_quantile_f (float *x, double q, size_t n)
 	return (p);
 }
 
+#define RE 0
+#define IM 1
+
 static void Cmul (double A[], double B[], double C[])
 {	/* Complex multiplication */
-	C[0] = A[0]*B[0] - A[1]*B[1];
-	C[1] = A[0]*B[1] + A[1]*B[0];
+	C[RE] = A[RE]*B[RE] - A[IM]*B[IM];
+	C[IM] = A[RE]*B[IM] + A[IM]*B[RE];
 }
 
 static void Cdiv (double A[], double B[], double C[])
 {	/* Complex division */
 	double denom;
-	denom = B[0]*B[0] + B[1]*B[1];
-	C[0] = (A[0]*B[0] + A[1]*B[1])/denom;
-	C[1] = (A[1]*B[0] - A[0]*B[1])/denom;
+	denom = B[RE]*B[RE] + B[IM]*B[IM];
+	C[RE] = (A[RE]*B[RE] + A[IM]*B[IM])/denom;
+	C[IM] = (A[IM]*B[RE] - A[RE]*B[IM])/denom;
+}
+
+static double Cabs (double A[])
+{
+	return (hypot (A[RE], A[IM]));
 }
 
 double GMT_psi (double zz[], double p[])
@@ -2714,7 +2718,7 @@ double GMT_psi (double zz[], double p[])
 *        Psi(z) = --log(Gamma(z))
 *                 dz
 *
-* zz[0] is real and zz[1] is imaginary component; same for p on output (only if p != NULL).
+* zz[RE] is real and zz[IM] is imaginary component; same for p on output (only if p != NULL).
 * We also return the real component as function result.
 */
 	static double c[15] = { 0.99999999999999709182, 57.156235665862923517, -59.597960355475491248,
@@ -2723,162 +2727,214 @@ double GMT_psi (double zz[], double p[])
 			-0.21026444172410488319e-3, 0.21743961811521264320e-3, -0.16431810653676389022e-3,
 			0.84418223983852743293e-4, -0.26190838401581408670e-4, 0.36899182659531622704e-5};
 	double z[2], g[2], dx[2], dd[2], d[2], n[2], gg[2], f[2], x0, A[2], B[2], C[2], sx, cx, e;
-	int k;
+	GMT_LONG k;
 
-	if (zz[1] == 0.0 && rint(zz[0]) == zz[0] && zz[0] <= 0.0) {
-		if (p) { p[0] = GMT_d_NaN; p[1] = 0.0;}
+	if (zz[IM] == 0.0 && rint(zz[RE]) == zz[RE] && zz[RE] <= 0.0) {
+		if (p) { p[RE] = GMT_d_NaN; p[IM] = 0.0;}
 		return (GMT_d_NaN);	/* Singular points */
 	}
 
-	z[0] = zz[0];	z[1] = zz[1];
-	if ((x0 = z[0]) < 0.5) {	/* reflection point */
-		z[0] = 1.0 - z[0];
-		z[1] = -z[1];
+	z[RE] = zz[RE];	z[IM] = zz[IM];
+	if ((x0 = z[RE]) < 0.5) {	/* reflection point */
+		z[RE] = 1.0 - z[RE];
+		z[IM] = -z[IM];
 	}
 
 	/* Lanczos approximation */
 
-	g[0] = 607.0/128.0;	g[1] = 0.0; /* best results when 4<=g<=5 */
-	n[0] = d[0] = n[1] = d[1] = 0.0;
+	g[RE] = 607.0/128.0;	g[IM] = 0.0; /* best results when 4<=g<=5 */
+	n[RE] = d[RE] = n[IM] = d[IM] = 0.0;
 	for (k = 14; k > 0; k--) {
-		A[0] = 1.0;	A[1] = 0.0;
-		B[0] = z[0] + k - 1.0;	B[1] = z[1];
+		A[RE] = 1.0;	A[IM] = 0.0;
+		B[RE] = z[RE] + k - 1.0;	B[IM] = z[IM];
 		Cdiv (A, B, dx);
-		dd[0] = c[k] * dx[0];	dd[1] = c[k] * dx[1];
-		d[0] += dd[0];	d[1] += dd[1];
+		dd[RE] = c[k] * dx[RE];	dd[IM] = c[k] * dx[IM];
+		d[RE] += dd[RE];	d[IM] += dd[IM];
 		Cmul (dd, dx, B);
-		n[0] -= B[0];	n[1] -= B[1];
+		n[RE] -= B[RE];	n[IM] -= B[IM];
 	}
-	d[0] += c[0];
-	gg[0] = z[0] + g[0] - 0.5;	gg[1] = z[1];
+	d[RE] += c[RE];
+	gg[RE] = z[RE] + g[RE] - 0.5;	gg[IM] = z[IM];
 	Cdiv (n, d, A);
 	Cdiv (g, gg, B);
-	f[0] = log (hypot(gg[0], gg[1])) + A[0] - B[0];
-	f[1] = atan2 (gg[1], gg[0])  + A[1] - B[1];
+	f[RE] = log (hypot(gg[RE], gg[IM])) + A[RE] - B[RE];
+	f[IM] = atan2 (gg[IM], gg[RE])  + A[IM] - B[IM];
 	if (x0 < 0.5) {
-		C[0] = M_PI * zz[0];	C[1] = M_PI * zz[1];
-		e = exp (-2*C[1]);	sx = sin (2*C[0]);	cx = cos (2*C[0]);
-		A[0] = -e * sx;	A[1] = e * cx + 1.0;
-		B[0] = e * cx - 1.0;	B[1] = e * sx;
+		C[RE] = M_PI * zz[RE];	C[IM] = M_PI * zz[IM];
+		e = exp (-2*C[IM]);	sx = sin (2*C[RE]);	cx = cos (2*C[RE]);
+		A[RE] = -e * sx;	A[IM] = e * cx + 1.0;
+		B[RE] = e * cx - 1.0;	B[IM] = e * sx;
 		Cdiv (A, B, C);
-		f[0] -= M_PI * C[0];	f[1] -= M_PI * C[1];
+		f[RE] -= M_PI * C[RE];	f[IM] -= M_PI * C[IM];
 	}
 	if (p) {
-		p[0] = f[0];
-		p[1] = f[1];
+		p[RE] = f[RE];
+		p[IM] = f[IM];
 	}
-	return (f[0]);
+	return (f[RE]);
 }
 
 #ifndef M_SQRT_PI
 #define M_SQRT_PI 1.772453850905516
 #endif
 
-void GMT_PvQv (double x, double v_ri[], double pq[], int *iter)
+#define PV_RE 0
+#define PV_IM 1
+#define QV_RE 2
+#define QV_IM 3
+void GMT_PvQv (double x, double v_ri[], double pq[], GMT_LONG *iter)
 {
 	/* Here, -1 <= x <= +1, v_ri is an imaginary number [r,i], and we return
-	 * both Pv(x) and Qv(x) in the pq array since no savings in doing it separately.
-	 * Based on recipe in Spanier/Oldham: An Atlas of Functions.
-	 * pq[0-1] is real/imag Pv
-	 * pq[2-3] is real/imag Qv
-	 */
-
-	double v[2], a[2], R[2], r[2], z[2], tmp[2], X[2], ep, em, s[2], c[2];
-	double M, L, K, vp1[2], G[2], Xn, x2, g, u, t, f, k, k1, sx, cx, w, sL, cL;
+	 * the real amd imaginary parts of Pv(x) and Qv(x) in the pq array.
+	 * Based on recipe in An Atlas of Functions */
 
+	BOOLEAN p_set, q_set;
+	double M, L, K, Xn, x2, k, k1, ep, em, sx, cx, fact;
+	double a[2], v[2], vp1[2], G[2], g[2], u[2], t[2], f[2];
+	double R[2], r[2], z[2], s[2], c[2], w[2], tmp[2], X[2], A[2], B[2];
+	
 	*iter = 0;
-	if (x == -1) {	/* Both blow up */
-    		pq[0] = pq[2] = GMT_d_NaN;
-    		pq[1] = pq[3] = 0.0;
-		return;
+    	pq[PV_RE] = pq[QV_RE] = pq[PV_IM] = pq[QV_IM] = 0.0;	/* Initialize all answers to zero first */
+	p_set = q_set = FALSE;
+	if (x == -1 && v_ri[IM] == 0.0) {	/* Check special values for real nu when x = -1 */
+			/* Special Pv(-1) values */
+		if ((v_ri[RE] > 0.0 && fmod (v_ri[RE], 2.0) == 1.0) || (v_ri[RE] < 0.0 && fmod (v_ri[RE], 2.0) == 0.0)) {	/* v = 1,3,5,.. or -2, -4, -6 */
+ 			pq[PV_RE] = -1.0; p_set = TRUE;
+		}
+		else if ((v_ri[RE] >= 0.0 && fmod (v_ri[RE], 2.0) == 0.0) || (v_ri[RE] < 0.0 && fmod (v_ri[RE]+1.0, 2.0) == 0.0)) {	/* v = 0,2,4,6 or -1,-3,-5,-7 */
+ 			pq[PV_RE] = 1.0; p_set = TRUE;
+		}
+		else if (v_ri[RE] > 0.0 && ((fact = v_ri[RE]-2.0*floor(v_ri[RE]/2.0)) < 1.0 && fact > 0.0)) { /* 0 < v < 1, 2 < v < 3, 4 < v < 5, ... */
+   			pq[PV_RE] = GMT_d_NaN; p_set = TRUE;	/* -inf */
+		}
+		else if (v_ri[RE] < 1.0 && ((fact = v_ri[RE]-2.0*floor(v_ri[RE]/2.0)) < 1.0 && fact > 0.0)) {	/* -2 < v < -1, -4 < v < -3, -6 < v < -5 */
+   			pq[PV_RE] = GMT_d_NaN; p_set = TRUE;	/* -inf */
+		}
+		else if (v_ri[RE] > 1.0 && (v_ri[RE]-2.0*floor(v_ri[RE]/2.0)) > 1.0) {	/* 1 < v < 2, 3 < v < 4, 5 < v < 6, .. */
+   			pq[PV_RE] = GMT_d_NaN; p_set = TRUE;	/* +inf */
+		}
+		else if (v_ri[RE] < 2.0 && ( v_ri[RE]-2.0*floor(v_ri[RE]/2.0)) > 1.0) {	/* -3 < v < -2, -5 < v < -4, -7 < v < -6, .. */
+   			pq[PV_RE] = GMT_d_NaN; p_set = TRUE;	/* +inf */
+		}
+		/* Special Qv(-1) values */
+		if (v_ri[RE] > 0.0 && fmod (2.0 * v_ri[RE] - 1.0, 4.0) == 0.0) {	/* v = 1/2, 5/2, 9/2, ... */
+   			pq[QV_RE] = -M_PI_2; q_set = TRUE;
+		}
+		else if (v_ri[RE] > -1.0 && fmod (2.0 * v_ri[RE] + 1.0, 4.0) == 0.0) {	/* v = -1/2, 3/2, 7/2, ... */
+   			pq[QV_RE] = M_PI_2; q_set = TRUE;
+		}
+		else if (v_ri[RE] < -1.0 && fmod (2.0 * v_ri[RE] - 1.0, 4.0) == 0.0) {	/* v = -3/2, -7/2, -11/2, ... */
+   			pq[QV_RE] = -M_PI_2; q_set = TRUE;
+		}
+		else if (v_ri[RE] < -2.0 && fmod (2.0 * v_ri[RE] + 1.0, 4.0) == 0.0) {	/* v = -5/2, -9/2, -13/2, ... */
+   			pq[QV_RE] = M_PI_2; q_set = TRUE;
+		}
+		else {	/* Either -inf or +inf */
+   			pq[QV_RE] = GMT_d_NaN; q_set = TRUE;
+		}
 	}
-	else if (x == +1) {	/* Pv = 1, Qv = inf */
-		pq[0] = 1;
-		pq[2] = GMT_d_NaN;
-    		pq[1] = pq[3] = 0.0;
-		return;
+	else if (x == +1 && v_ri[IM] == 0.0) {	/* Check special values for real nu when x = +1 */
+		pq[PV_RE] = 1.0; p_set = TRUE;
+		pq[QV_RE] = GMT_d_NaN; q_set = TRUE;
 	}
+	if (p_set && q_set) return;
+	
 	/* General case of |x| < 1 */
-	a[0] = a[1] = R[0] = 1.0;	R[1] = 0.0;
-	v[0] = v_ri[0];	v[1] = v_ri[1];
+
+	a[0] = a[1] = R[RE] = 1.0;	R[IM] = 0.0;
+	v[RE] = v_ri[RE];	v[IM] = v_ri[IM];
 	Cmul (v, v, z);
-	z[0] = v[0] - z[0];	z[1] = v[1] - z[1];
-	K = 4.0 * sqrt (hypot(z[0], z[1]));
-	vp1[0] = v[0] + 1.0;	vp1[1] = v[1];
-	if ((hypot(vp1[0], vp1[1]) + floor(vp1[1])) == 0.0) {
+	z[RE] = v[RE] - z[RE];	z[IM] = v[IM] - z[IM];
+	K = 4.0 * sqrt (Cabs(z));
+	vp1[RE] = v[RE] + 1.0;	vp1[IM] = v[IM];
+	if ((Cabs(vp1) + floor(vp1[RE])) == 0.0) {
 		a[0] = GMT_d_NaN;
 		a[1] = 0.0;
-		v[0] = -1.0 - v[0];
-		v[1] = -v[1];
+		v[RE] = -1 - v[RE];
+		v[IM] = -v[IM];
 	}
-	z[0] = 0.5 * M_PI * v[0];	z[1] = 0.5 * M_PI * v[1];
-	ep = exp (z[1]);	em = exp (-z[1]);
-	sincos (z[0], &sx, &cx);
-	s[0] = +0.5 * sx * (em + ep);
-	s[1] = -0.5 * cx * (em - ep);
-	c[0] = +0.5 * cx * (em + ep);
-	c[1] = +0.5 * sx * (em - ep);
-	w = (0.5 + v[0])*(0.5 + v[0]) - v[1] * v[1];
-	z[1] = v[1];
-	while (v[0] <= 6.0) {
-		v[0] += 2.0;
-		z[0] = v[0] - 1.0;
+	z[RE] = 0.5 * M_PI * v[RE];	z[IM] = 0.5 * M_PI * v[IM];
+	ep = exp (z[IM]);	em = exp (-z[IM]);
+	sincos (z[RE], &sx, &cx);
+	s[RE] = 0.5 * sx * (em + ep); 
+	s[IM] = -0.5 * cx * (em - ep); 
+	c[RE] = 0.5 * cx * (em + ep); 
+	c[IM] = 0.5 * sx * (em - ep);
+	tmp[RE] = 0.5 + v[RE];	tmp[IM] = v[IM];
+	Cmul (tmp, tmp, w);
+	z[IM] = v[IM];
+	while (v[RE] <= 6.0) {
+		v[RE] = v[RE] + 2.0;
+		z[RE] = v[RE] - 1.0;
 		Cdiv (z, v, tmp);
 		Cmul (R,tmp,r);
-		R[0] = r[0];	R[1] = r[1];
+		R[RE] = r[RE];	R[IM] = r[IM];
 	}
-	z[0] = v[0] + 1.0;
-	tmp[0] = 0.25;	tmp[1] = 0.0;
+	z[RE] = v[RE] + 1.0;
+	tmp[RE] = 0.25;	tmp[IM] = 0.0;
 	Cdiv (tmp, z, X);
-	tmp[0] = 0.35 + 6.1 * X[0];	tmp[1] = 6.1*X[1];
+	tmp[RE] = 0.35 + 6.1 * X[RE];	tmp[IM] = 6.1*X[IM];
 	Cmul (X, tmp, z);
-	z[0] = 1.0 - 3.0*z[0];	z[1] = -3.0*z[1];
+	z[RE] = 1.0 - 3.0*z[RE];	z[IM] = -3.0*z[IM];
 	Cmul (X, z, tmp);
-	G[0] = 1.0 + 5.0 * tmp[0];	G[1] = 5.0 * tmp[1];
-	z[0] = 8.0 * X[0];	z[1] = 8.0 * X[1];
-	M = sqrt(hypot(z[0], z[1]));
-	L = 0.5 * atan2 (z[1], z[0]);
-	sincos (L, &sL, &cL);
-	tmp[0] = M * cL;	tmp[1] = M * sL;
+	G[RE] = 1.0 + 5.0 * tmp[RE];	G[IM] = 5.0 * tmp[IM];
+	z[RE] = 8.0 * X[RE];	z[IM] = 8.0 * X[IM];
+	M = sqrt(hypot(z[RE], z[IM]));
+	L = 0.5 * atan2 (z[IM], z[RE]);
+	tmp[RE] = M * cos(L);	tmp[IM] = M * sin(L);
 	Cmul (G, X, z);
-	z[0] = 1.0 - 0.5*z[0];	z[1] = -0.5*z[1];
+	z[RE] = 1.0 - 0.5*z[RE];	z[IM] = -0.5*z[IM];
 	Cmul (X, z, r);
-	r[0] = 1.0 - r[0];	r[1] = -r[1];
+	r[RE] = 1.0 - r[RE];	r[IM] = -r[IM];
 	Cmul (R, r, z);
 	Cdiv (z, tmp, R);
-	u = g = 2.0 * x;
-	f = t = 1.0;
+	u[RE] = g[RE] = 2.0 * x;	u[IM] = g[IM] = f[IM] = t[IM] = 0.0;
+	f[RE] = t[RE] = 1.0;
 	k = 0.5;
 	x2 = x * x;
-	Xn = 1.0 + (1e8/(1.0 - x2));
+	Xn = 1.0 + (1e8/(1 - x2));
 	k1 = k + 1.0;
-	t = t * x2 * (k*k - w) / (k1*k1 - 0.25);
+	fact = x2 / (k1*k1 - 0.25);
+	t[RE] = (k*k - w[RE]) * fact;	t[IM] = -w[IM] * fact;
 	k += 1.0;
-	f += t;
+	f[RE] += t[RE];	f[IM] += t[IM];
 	k1 = k + 1.0;
-	u = u * x2 * (k*k - w) / (k1*k1 - 0.25);
-	k += 1.0;
-	g += u;
-	while (k < K || fabs (Xn*t) > fabs(f)) {
+	fact = u[RE] * x2 / (k1*k1 - 0.25);
+	u[RE] = (k*k - w[RE]) * fact;	u[IM] = -w[IM] * fact;
+	k += 1;
+	g[RE] += u[RE];	g[IM] += u[IM];
+	tmp[RE] = Xn * t[RE];	tmp[IM] = Xn * t[IM];
+	while (k < K || Cabs (tmp) > Cabs(f)) {
 		(*iter)++;
 		k1 = k + 1.0;
-		t = t * x2 * (k*k - w) / (k1*k1 - 0.25);
+		tmp[RE] = k*k - w[RE];	tmp[IM] = -w[IM];	fact = x2 / (k1*k1 - 0.25);
+		Cmul (t, tmp, A);
+		t[RE] = A[RE] * fact;	t[IM] = A[IM] * fact;
 		k += 1.0;
 		k1 = k + 1.0;
-		f += t;
-		u = u * x2 * (k*k - w) / (k1*k1 - 0.25);
+		f[RE] += t[RE];	f[IM] += t[IM];
+		tmp[RE] = k*k - w[RE];	tmp[IM] = -w[IM];	fact = x2 / (k1*k1 - 0.25);
+		Cmul (u, tmp, B);
+		u[RE] = B[RE] * fact;	u[IM] = B[IM] * fact;
 		k += 1.0;
-		g += u;
+		g[RE] += u[RE];	g[IM] += u[IM];
+		tmp[RE] = Xn * t[RE];	tmp[IM] = Xn * t[IM];
+	}
+	fact = x2 / (1.0 - x2);
+	f[RE] += t[RE] * fact;	f[IM] += t[IM] * fact;
+	g[RE] += u[RE] * fact;	g[IM] += u[IM] * fact;
+	if (!p_set) {
+		Cmul(s,R,z);
+		Cdiv(c,R,tmp);
+		Cmul (g, z, A);	Cmul (f, tmp, B);
+		pq[PV_RE] = (A[RE] + B[RE])/M_SQRT_PI;
+		pq[PV_IM] = (A[IM] + B[IM])/M_SQRT_PI;
+	}
+	if (!q_set) {
+		Cmul(c,R,z);
+		Cdiv(s,R,tmp);
+		Cmul (g, z, A);	Cmul (f, tmp, B);
+		pq[QV_RE] = a[0]*M_SQRT_PI*(A[RE] - B[RE])/2.0;
+		pq[QV_IM] = a[1]*M_SQRT_PI*(A[IM] - B[IM])/2.0;
 	}
-	L = x2 / (1.0 - x2);
-	f += (t * L);
-	g += (u * L);
-	Cmul(s,R,z);
-	Cdiv(c,R,tmp);
-	pq[0] = (g*z[0] + f*tmp[0])/M_SQRT_PI;
-	pq[1] = (g*z[1] + f*tmp[1])/M_SQRT_PI;
-	Cmul(c,R,z);
-	Cdiv(s,R,tmp);
-	pq[2] = a[0]*M_SQRT_PI*(g*z[0] - f*tmp[0])/2.0;
-	pq[3] = a[1]*M_SQRT_PI*(g*z[1] - f*tmp[1])/2.0;
 }
