@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_vector.c,v 1.15 2007/09/30 13:12:09 remko Exp $
+ *	$Id: gmt_vector.c,v 1.30 2009/05/15 09:32:23 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -27,9 +27,7 @@
 
 #define MAX_SWEEPS 50
 
-double GMT_mag3v (double *a);
-
-int	GMT_jacobi (double *a, int *n, int *m, double *d, double *v, double *b, double *z, int *nrots) {
+GMT_LONG	GMT_jacobi (double *a, GMT_LONG *n, GMT_LONG *m, double *d, double *v, double *b, double *z, GMT_LONG *nrots) {
 /*
  *
  * Find eigenvalues & eigenvectors of a square symmetric matrix by Jacobi's
@@ -120,7 +118,7 @@ int	GMT_jacobi (double *a, int *n, int *m, double *d, double *v, double *b, doub
  * Revised:	PW: 12-MAR-1998 for GMT 3.1
  * Revision by WHF Smith, March 03, 2000, to speed up loop indexes.
  */
-	int	p, q, pp, pq, mp1, pm, qm, nsweeps, j, jm, i, k;
+	GMT_LONG	p, q, pp, pq, mp1, pm, qm, nsweeps, j, jm, i, k;
 	double	sum, threshold, g, h, t, theta, c, s, tau;
 
 
@@ -275,13 +273,14 @@ int	GMT_jacobi (double *a, int *n, int *m, double *d, double *v, double *b, doub
 	/* Return 0 if converged; else print warning and return -1:  */
 
 	if (nsweeps == MAX_SWEEPS) {
-		fprintf (stderr, "GMT_jacobi:  Failed to converge in %d sweeps\n", nsweeps);
+		fprintf (stderr, "GMT_jacobi:  Failed to converge in %ld sweeps\n", nsweeps);
 		return(-1);
 	}
 	return(0);
 }
 
-int	GMT_jacobi_old (double *a, int *n, int *m, double *d, double *v, double *b, double *z, int *nrots)
+#ifdef OBSOLETE
+GMT_LONG	GMT_jacobi_old (double *a, GMT_LONG *n, GMT_LONG *m, double *d, double *v, double *b, double *z, GMT_LONG *nrots)
 /*
  *
  * Find eigenvalues & eigenvectors of a square symmetric matrix by Jacobi's
@@ -310,7 +309,7 @@ int	GMT_jacobi_old (double *a, int *n, int *m, double *d, double *v, double *b,
    	       	/* Returned.  number of Givens rotations performed.  */
 
 {
-	int	ip, iq, nsweeps, i, j, k;
+	GMT_LONG	ip, iq, nsweeps, i, j, k;
 	double	sum, threshold, g, h, t, theta, c, s, tau, p;
 
 
@@ -469,6 +468,148 @@ int	GMT_jacobi_old (double *a, int *n, int *m, double *d, double *v, double *b,
 	}
 	return(0);
 }
+#endif
+
+void GMT_gauss (double *a, double *vec, GMT_LONG n_in, GMT_LONG nstore_in, double test, GMT_LONG *ierror, GMT_LONG itriag)
+{
+ 
+/* subroutine gauss, by william menke */
+/* july 1978 (modified feb 1983, nov 85) */
+ 
+/* a subroutine to solve a system of n linear equations in n unknowns*/
+/* gaussian reduction with partial pivoting is used */
+/*      a               (sent, destroyed)       n by n matrix           */
+/*      vec             (sent, overwritten)     n vector, replaced w/ solution*/
+/*      nstore          (sent)                  dimension of a  */
+/*      test            (sent)                  div by zero check number*/
+/*      ierror          (returned)              zero on no error*/
+/*      itriag          (sent)                  matrix triangularized only*/
+/*                                               on TRUE useful when solving*/
+/*                                               multiple systems with same a */
+        static GMT_LONG l1;
+        GMT_LONG *line, *isub, i = 0, j, k, l, j2, n, nstore;
+	GMT_LONG iet, ieb;
+        double big, testa, b, sum;
+
+        iet=0;  /* initial error flags, one for triagularization*/
+        ieb=0;  /* one for backsolving */
+	n = n_in;
+	nstore = nstore_in;
+	(void)GMT_alloc_memory2 ((void **)&line, (void **)&isub, n, 0, sizeof (GMT_LONG), "GMT_gauss");
+
+/* triangularize the matrix a*/
+/* replacing the zero elements of the triangularized matrix */
+/* with the coefficients needed to transform the vector vec */
+
+        if (itriag) {   /* triangularize matrix */
+ 
+                for( j=0; j<n; j++ ) {      /*line is an array of flags*/
+                        line[j]=0; 
+                        /* elements of a are not moved during pivoting*/
+                        /* line=0 flags unused lines */
+                        }    /*end for j*/
+                        
+                for( j=0; j<n-1; j++ ) {
+                        /*  triangularize matrix by partial pivoting */
+                       big = 0.0; /* find biggest element in j-th column*/
+                                  /* of unused portion of matrix*/
+                       for( l1=0; l1<n; l1++ ) {
+                               if( line[l1]==0 ) {
+                                       testa=(double) fabs((double) (*(a+l1*nstore+j)));
+                                       if (testa>big) {
+                                                i=l1;
+                                                big=testa;
+                                                } /*end if*/
+                                        } /*end if*/
+                                } /*end for l1*/
+                       if( big<=test) {   /* test for div by 0 */
+                               iet=1;
+                               } /*end if*/
+ 
+                       line[i]=1;  /* selected unused line becomes used line */
+                       isub[j]=i;  /* isub points to j-th row of tri. matrix */
+ 
+                       sum=1.0/(*(a+i*nstore+j)); 
+                                /*reduce matrix towards triangle */
+                       for( k=0; k<n; k++ ) {
+                                if( line[k]==0 ) {
+                                        b=(*(a+k*nstore+j))*sum;
+                                        for( l=j+1; l<n; l++ ) {
+                                               *(a+k*nstore+l)= (*(a+k*nstore+l)) -b*(*(a+i*nstore+l));
+                                               } /*end for l*/
+                                       *(a+k*nstore+j)=b;
+                                        } /*end if*/
+                                } /*end for k*/
+                        } /*end for j*/
+ 
+               for( j=0; j<n; j++ ) {
+                        /*find last unused row and set its pointer*/
+                        /*  this row contains the apex of the triangle*/
+                        if( line[j]==0) {
+                                l1=j;   /*apex of triangle*/
+                                isub[n-1]=j;
+                                break;
+                                } /*end if*/
+                        } /*end for j*/
+ 
+                } /*end if itriag true*/
+                
+        /*start backsolving*/
+        
+        for( i=0; i<n; i++ ) {  /* invert pointers. line(i) now gives*/
+                                /* row no in triang matrix of i-th row*/
+                                /* of actual matrix */
+                line[isub[i]] = i;
+                } /*end for i*/
+ 
+        for( j=0; j<n-1; j++) { /*transform the vector to match triang. matrix*/
+               b=vec[isub[j]];
+               for( k=0; k<n; k++ ) {
+                      if (line[k]>j) {  /* skip elements outside of triangle*/
+                                vec[k]=vec[k]-(*(a+k*nstore+j))*b;
+                                } /*end if*/
+                        } /*end for k*/
+                } /*end for j*/
+ 
+      b = *(a+l1*nstore+(n-1));   /*apex of triangle*/
+      if( ((double)fabs( (double) b))<=test) {
+                /*check for div by zero in backsolving*/
+                ieb=2;
+                } /*end if*/
+      vec[isub[n-1]]=vec[isub[n-1]]/b;
+ 
+      for( j=n-2; j>=0; j-- ) { /* backsolve rest of triangle*/
+                sum=vec[isub[j]];
+                for( j2=j+1; j2<n; j2++ ) {
+                        sum = sum - vec[isub[j2]] * (*(a+isub[j]*nstore+j2));
+                        } /*end for j2*/
+                        b = *(a+isub[j]*nstore+j);
+               if( ((double)fabs((double)b))<=test) {
+                        /* test for div by 0 in backsolving */
+                        ieb=2;
+                        } /*end if*/
+                vec[isub[j]]=sum/b;   /*solution returned in vec*/
+                } /*end for j*/
+
+/*put the solution vector into the proper order*/
+
+      for( i=0; i<n; i++ ) {    /* reorder solution */
+                for( k=i; k<n; k++ ) {  /* search for i-th solution element */
+                        if( line[k]==i ) {
+                                j=k;
+                                break;
+                                } /*end if*/
+                        } /*end for k*/
+               b = vec[j];       /* swap solution and pointer elements*/
+               vec[j] = vec[i];
+               vec[i] = b;
+               line[j] = line[i];
+                } /*end for i*/
+ 
+		GMT_free ((void *)isub);
+		GMT_free ((void *)line);
+      *ierror = iet + ieb;   /* set final error flag*/
+}
 
 /*  cartesian_stuff.c  --  bits and pieces for doing spherical trig
  *  in terms of dot and cross products.
@@ -490,7 +631,7 @@ double GMT_mag3v (double *a)
 void GMT_normalize3v (double *a)
 {
 	double r_length;
-	r_length = (GMT_mag3v(a));
+	r_length = GMT_mag3v(a);
 	if (r_length != 0.0) {
 		r_length = 1.0 / r_length;
 		a[0] *= r_length;
@@ -506,41 +647,41 @@ void GMT_cross3v (double *a, double *b, double *c)
 	c[2] = a[0] * b[1] - a[1] * b[0];
 }
 
-void GMT_geo_to_cart (double *alat, double *alon, double *a, int rads)
+void GMT_geo_to_cart (double lat, double lon, double *a, BOOLEAN degrees)
 {
-	/* Convert geographic latitude and longitude (alat, alon)
-	   to a 3-vector of unit length (a).  rads = TRUE if we
-	   need to convert alat, alon from degrees to radians  */
+	/* Convert geographic latitude and longitude (lat, lon)
+	   to a 3-vector of unit length (a). If degrees = TRUE,
+	   input coordinates are in degrees, otherwise in radian */
 
 	double clat, clon, slon;
 
-	if (rads) {
-		*alat *= D2R;
-		*alon *= D2R;
+	if (degrees) {
+		lat *= D2R;
+		lon *= D2R;
 	}
-	sincos (*alat, &a[2], &clat);
-	sincos (*alon, &slon, &clon);
+	sincos (lat, &a[2], &clat);
+	sincos (lon, &slon, &clon);
 	a[0] = clat * clon;
 	a[1] = clat * slon;
 }
 
-void GMT_cart_to_geo (double *alat, double *alon, double *a, int rads)
+void GMT_cart_to_geo (double *lat, double *lon, double *a, BOOLEAN degrees)
 {
 	/* Convert a 3-vector (a) of unit length into geographic
-	   coordinates (alat, alon).  rads = TRUE if we want the
-	   lat and lon converted from radians into degrees.  */
+	   coordinates (lat, lon). If degrees = TRUE, the output coordinates
+	   are in degrees, otherwise in radian. */
 
-	if(rads) {
-		*alat = R2D * d_asin(a[2]);
-		*alon = R2D * d_atan2(a[1], a[0]);
+	if (degrees) {
+		*lat = d_asind (a[2]);
+		*lon = d_atan2d (a[1], a[0]);
 	}
 	else {
-		*alat = d_asin(a[2]);
-		*alon = d_atan2(a[1],a[0]);
+		*lat = d_asin (a[2]);
+		*lon = d_atan2 (a[1], a[0]);
 	}
 }
 
-int GMT_fix_up_path (double **a_lon, double **a_lat, int n, double step, int mode)
+GMT_LONG GMT_fix_up_path (double **a_lon, double **a_lat, GMT_LONG n, double step, GMT_LONG mode)
 {
 	/* Takes pointers to a list of <n> lon/lat pairs (in degrees) and adds
 	 * auxiliary points if the great circle distance between two given points exceeds
@@ -556,7 +697,8 @@ int GMT_fix_up_path (double **a_lon, double **a_lat, int n, double step, int mod
 	 * inches of map projection.
 	 */
       
-	int i, j, n_tmp, n_step = 0, n_alloc;
+	GMT_LONG i, j, n_tmp, n_step = 0, n_alloc = 0;
+	BOOLEAN meridian;
 	double *lon_tmp, *lat_tmp, *old;
 	double a[3], b[3], x[3], *lon, *lat;
 	double c, d, fraction, theta, minlon, maxlon;
@@ -564,86 +706,68 @@ int GMT_fix_up_path (double **a_lon, double **a_lat, int n, double step, int mod
 	lon = *a_lon;
 	lat = *a_lat;
 	
-	n_alloc = n;
-	lon_tmp = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-	lat_tmp = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-	
-	GMT_geo_to_cart (&lat[0], &lon[0], a, TRUE);
+	GMT_geo_to_cart (lat[0], lon[0], a, TRUE);
+	n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, 1, 0, sizeof (double), "GMT_fix_up_path");
 	lon_tmp[0] = lon[0];	lat_tmp[0] = lat[0];
 	n_tmp = 1;
 	if (step <= 0.0) step = 1.0;
 	
 	for (i = 1; i < n; i++) {
 		
-		GMT_geo_to_cart (&lat[i],&lon[i], b, TRUE);
+		GMT_geo_to_cart (lat[i], lon[i], b, TRUE);
 
 		if (mode == 1) {	/* First follow meridian, then parallel */
-			theta = fabs(lon[i]-lon[i-1]) * cos(lat[i-1]);
-			n_step = irint (theta * R2D / step);
+			theta = fabs(lon[i]-lon[i-1]) * cosd(lat[i-1]);
+			n_step = irint (theta / step);
 			for (j = 1; j < n_step; j++) {
 				c = j / (double)n_step;
+				if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
 				lon_tmp[n_tmp] = lon[i-1] * (1 - c) + lon[i] * c;
 				lat_tmp[n_tmp] = lat[i-1];
 				n_tmp++;
-				if (n_tmp == n_alloc) {
-					n_alloc += GMT_CHUNK;
-					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-				}
 			}
 			theta = fabs(lat[i]-lat[i-1]);
-			n_step = irint (theta * R2D / step);
+			n_step = irint (theta / step);
 			for (j = 0; j < n_step; j++) {	/* Start at 0 to make sure corner point is saved */
 				c = j / (double)n_step;
+				if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
 				lon_tmp[n_tmp] = lon[i];
 				lat_tmp[n_tmp] = lat[i-1] * (1 - c) + lat[i] * c;
 				n_tmp++;
-				if (n_tmp == n_alloc) {
-					n_alloc += GMT_CHUNK;
-					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-				}
 			}
 		}
 
 		else if (mode == 2) {	/* First follow parallel, then meridian */
 			theta = fabs(lat[i]-lat[i-1]);
-			n_step = irint (theta * R2D / step);
+			n_step = irint (theta / step);
 			for (j = 1; j < n_step; j++) {
 				c = j / (double)n_step;
+				if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
 				lon_tmp[n_tmp] = lon[i-1];
 				lat_tmp[n_tmp] = lat[i-1] * (1 - c) + lat[i] * c;
 				n_tmp++;
-				if (n_tmp == n_alloc) {
-					n_alloc += GMT_CHUNK;
-					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-				}
 			}
-			theta = fabs(lon[i]-lon[i-1]) * cos(lat[i]);
-			n_step = irint (theta * R2D / step);
+			theta = fabs(lon[i]-lon[i-1]) * cosd(lat[i]);
+			n_step = irint (theta / step);
 			for (j = 0; j < n_step; j++) {	/* Start at 0 to make sure corner point is saved */
 				c = j / (double)n_step;
+				if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
 				lon_tmp[n_tmp] = lon[i-1] * (1 - c) + lon[i] * c;
 				lat_tmp[n_tmp] = lat[i];
 				n_tmp++;
-				if (n_tmp == n_alloc) {
-					n_alloc += GMT_CHUNK;
-					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-				}
 			}
 		}
 
 		/* Follow great circle */
-		else if ((theta = d_acos (GMT_dot3v (a, b))) == M_PI) {	/* trouble, no unique great circle */
+		else if ((theta = d_acosd (GMT_dot3v (a, b))) == 180.0) {	/* trouble, no unique great circle */
 			if (gmtdefs.verbose) fprintf (stderr, "%s: GMT Warning: Two points in input list are antipodal - no resampling taken place!\n", GMT_program);
 		}
 
-		else if ((n_step = irint (theta * R2D / step)) > 1) {	/* Must insert (n_step - 1) points, i.e. create n_step intervals */
+		else if ((n_step = irint (theta / step)) > 1) {	/* Must insert (n_step - 1) points, i.e. create n_step intervals */
 			fraction = 1.0 / (float) n_step;
 			minlon = MIN(lon[i-1],lon[i]);
 			maxlon = MAX(lon[i-1],lon[i]);
+			meridian = GMT_IS_ZERO (maxlon - minlon);	/* A meridian; make a gap so tests below will give right range */
 			for (j = 1; j < n_step; j++) {
 				c = j * fraction;
 				d = 1 - c;
@@ -651,30 +775,23 @@ int GMT_fix_up_path (double **a_lon, double **a_lat, int n, double step, int mod
 				x[1] = a[1] * d + b[1] * c;
 				x[2] = a[2] * d + b[2] * c;
 				GMT_normalize3v (x);
-				GMT_cart_to_geo (&lat_tmp[n_tmp], &lon_tmp[n_tmp], x, FALSE);
-				if (lon_tmp[n_tmp] < minlon)
-					lon_tmp[n_tmp] += TWO_PI;
+				if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
+				GMT_cart_to_geo (&lat_tmp[n_tmp], &lon_tmp[n_tmp], x, TRUE);
+				if (meridian)
+					lon_tmp[n_tmp] = minlon;
+				else if (lon_tmp[n_tmp] < minlon)
+					lon_tmp[n_tmp] += 360.0;
 				else if (lon_tmp[n_tmp] > maxlon)
-					lon_tmp[n_tmp] -= TWO_PI;
+					lon_tmp[n_tmp] -= 360.0;
 				n_tmp++;
-				if (n_tmp == n_alloc) {
-					n_alloc += GMT_CHUNK;
-					lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-					lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-				}
 			}
 		}
+		if (n_tmp == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, n_tmp, n_alloc, sizeof (double), "GMT_fix_up_path");
 		lon_tmp[n_tmp] = lon[i];	lat_tmp[n_tmp] = lat[i];
 		n_tmp++;
-		if (n_tmp == n_alloc) {
-			n_alloc += GMT_CHUNK;
-			lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-			lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_alloc, sizeof (double), "GMT_fix_up_path");
-		}
 		a[0] = b[0];	a[1] = b[1];	a[2] = b[2];
 	}
-	lon_tmp = (double *) GMT_memory ((void *) lon_tmp, (size_t)n_tmp, sizeof (double), "GMT_fix_up_path");
-	lat_tmp = (double *) GMT_memory ((void *) lat_tmp, (size_t)n_tmp, sizeof (double), "GMT_fix_up_path");
+	n_alloc = GMT_alloc_memory2 ((void **)&lon_tmp, (void **)&lat_tmp, 0, n_tmp, sizeof (double), "GMT_fix_up_path");
 	
 	old = lon;
 	lon = lon_tmp;
@@ -682,16 +799,12 @@ int GMT_fix_up_path (double **a_lon, double **a_lat, int n, double step, int mod
 	old = lat;
 	lat = lat_tmp;
 	GMT_free ((void *) old);
-	for (i = 0; i < n_tmp; i++) {
-		lon[i] *= R2D;
-		lat[i] *= R2D;
-	}
 	*a_lon = lon;
 	*a_lat = lat;
 	return (n_tmp);
 }		
 
-int GMT_chol_dcmp (double *a, double *d, double *cond, int nr, int n) {
+GMT_LONG GMT_chol_dcmp (double *a, double *d, double *cond, GMT_LONG nr, GMT_LONG n) {
 
 	/* Given a, a symmetric positive definite matrix
 	of size n, and row dimension nr, compute a lower
@@ -720,7 +833,7 @@ int GMT_chol_dcmp (double *a, double *d, double *cond, int nr, int n) {
 	
 	W H F Smith, 18 Feb 2000.
 */
-	int	i, j, k, ik, ij, kj, kk, nrp1;
+	GMT_LONG	i, j, k, ik, ij, kj, kk, nrp1;
 	double	eigmax, eigmin;
 
 	nrp1 = nr + 1;
@@ -747,7 +860,7 @@ int GMT_chol_dcmp (double *a, double *d, double *cond, int nr, int n) {
 	return (0);
 }
 
-void GMT_chol_recover (double *a, double *d, int nr, int n, int nerr, int donly) {
+void GMT_chol_recover (double *a, double *d, GMT_LONG nr, GMT_LONG n, GMT_LONG nerr, GMT_LONG donly) {
 
 	/* Given a, a symmetric positive definite matrix of row dimension nr,
 	and size n >= abs(nerr), one uses GMT_chol_dcmp() to attempt to find
@@ -782,12 +895,12 @@ void GMT_chol_recover (double *a, double *d, int nr, int n, int nerr, int donly)
 	W H F Smith, 18 Feb 2000
 */
 	
-	int	kbad, i, j, ii, ij, ji, nrp1;
+	GMT_LONG	kbad, i, j, ii, ij, ji, nrp1;
 	
-	kbad = abs(nerr) - 1;
+	kbad = labs((long)nerr) - 1;
 	nrp1 = nr + 1;
 	
-	for (i = 0, ii = 0; i <=kbad; i++, ii += nrp1) {
+	for (i = 0, ii = 0; i <= kbad; i++, ii += nrp1) {
 		a[ii] = d[i];
 	}
 	
@@ -801,7 +914,7 @@ void GMT_chol_recover (double *a, double *d, int nr, int n, int nerr, int donly)
 	return;
 }
 
-void GMT_chol_solv (double *a, double *x, double *y, int nr, int n) {
+void GMT_chol_solv (double *a, double *x, double *y, GMT_LONG nr, GMT_LONG n) {
 	/* Given an n by n linear system ax = y, with a a symmetric, 
 	positive-definite matrix, y a known vector, and x an unknown
 	vector, this routine finds x, if a holds the lower-triangular
@@ -819,7 +932,7 @@ void GMT_chol_solv (double *a, double *x, double *y, int nr, int n) {
 	
 	W H F Smith, 18 Feb 2000
 */
-	int	i, j, ij, ji, ii, nrp1;
+	GMT_LONG	i, j, ij, ji, ii, nrp1;
 	
 	nrp1 = nr + 1;
 
