@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtselect.c,v 1.91 2008/03/24 08:58:31 guru Exp $
+ *	$Id: gmtselect.c,v 1.105 2009/07/11 03:16:30 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -65,9 +65,7 @@ struct GMTSELECT_CTRL {	/* All control options for this program (except common a
 	/* active is TRUE if the option has been activated */
 	struct A {	/* -A<min_area>[/<min_level>/<max_level>] */
 		BOOLEAN active;
-		int low;	/* Lowest hierarchical level to use [0] */
-		int high;	/* Highest hierarchical level to use [4] */
-		double area;	/* Area of smallest geographical feature to include [0] */
+		struct GMT_SHORE_SELECT info;
 	} A;
 	struct C {	/* [-C[f]<dist>/<ptfile>] */
 		BOOLEAN active;
@@ -77,11 +75,12 @@ struct GMTSELECT_CTRL {	/* All control options for this program (except common a
 	} C;
 	struct D {	/* -D<resolution> */
 		BOOLEAN active;
+		BOOLEAN force;	/* if TRUE, select next highest level if current set is not avaialble */
 		char set;	/* One of f, h, i, l, c */
 	} D;
 	struct L {	/* -L[p][<dist>/<lfile>] */
 		BOOLEAN active;
-		int mode;	/* Controls what happens beyond segment endpoints */
+		GMT_LONG mode;	/* Controls what happens beyond segment endpoints */
 		double dist;	/* Distance of influence for each line */
 		char *file;	/* Name of file with lines */
 	} L;
@@ -96,7 +95,7 @@ struct GMTSELECT_CTRL {	/* All control options for this program (except common a
 	struct N {	/* -N<maskvalues>[o] */
 		BOOLEAN active;
 		BOOLEAN edge;	/* TRUE if edges are considere outside */
-		int mask[GMTSELECT_N_CLASSES];	/* Mask for each level */
+		GMT_LONG mask[GMTSELECT_N_CLASSES];	/* Mask for each level */
 	} N;
 	struct Z {	/* -Z<min>/<max> */
 		BOOLEAN active;
@@ -107,10 +106,9 @@ struct GMTSELECT_CTRL {	/* All control options for this program (except common a
 
 int main (int argc, char **argv)
 {
-	int i, j, k, fno, n_files = 0, n_args, err;
-	int n_fields, n_expected_fields, ind, bin, last_bin = -1, n_output = -1;
-	int np[2], base = 3, wd[2], id, this_node, side, is_inside, row, col, pos;
-	
+	GMT_LONG i, j, k, fno, n_files = 0, n_args, err, n_minimum = 2;
+	GMT_LONG n_fields, n_expected_fields, ind, bin, last_bin = -1, n_output = -1;
+	GMT_LONG np[2], base = 3, wd[2], id, this_node, side, is_inside, row, col, pos;
 	GMT_LONG n_read = 0, n_pass = 0;
 
 	BOOLEAN error = FALSE, dry_wet_only = FALSE, long_verbose = FALSE, need_header, shuffle;
@@ -121,7 +119,7 @@ int main (int argc, char **argv)
 	double *in, west_border = 0.0, east_border = 0.0;
 	double xmin, xmax, ymin, ymax, lon, step = 0.0;
 
-	char buffer[BUFSIZ], ptr[BUFSIZ], za[GMT_TEXT_LEN], zb[GMT_TEXT_LEN];
+	char buffer[BUFSIZ], ptr[BUFSIZ], za[GMT_TEXT_LEN], zb[GMT_TEXT_LEN], *not_used = NULL;
 	char *shore_resolution[5] = {"full", "high", "intermediate", "low", "crude"};
 
 	FILE *fp = NULL;
@@ -131,7 +129,7 @@ int main (int argc, char **argv)
 	struct GMT_SHORE c;
 	struct GMTSELECT_CTRL *Ctrl;
 
-	PFI near_a_line, near_a_point;
+	PFL near_a_line, near_a_point;
 
 	int compare_x (const void *point_1, const void *point_2);
 	void *New_Gmtselect_Ctrl (), Free_Gmtselect_Ctrl (struct GMTSELECT_CTRL *C);
@@ -149,11 +147,13 @@ int main (int argc, char **argv)
 				case 'V':
 					if (argv[i][2] == 'l') long_verbose = TRUE;
 				case 'H':
+				case 'M':
 				case 'J':
 				case 'R':
 				case ':':
 				case 'b':
 				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
@@ -162,8 +162,7 @@ int main (int argc, char **argv)
 
 				case 'A':
 					Ctrl->A.active = TRUE;
-					j = sscanf (&argv[i][2], "%lf/%d/%d", &Ctrl->A.area, &Ctrl->A.low, &Ctrl->A.high);
-					if (j == 1) Ctrl->A.low = 0, Ctrl->A.high = GMT_MAX_GSHHS_LEVEL;
+					GMT_set_levels (&argv[i][2], &Ctrl->A.info);
 					break;
 				case 'C':
 					Ctrl->C.active = TRUE;
@@ -182,6 +181,7 @@ int main (int argc, char **argv)
 				case 'D':
 					Ctrl->D.active = TRUE;
 					Ctrl->D.set = argv[i][2];
+					if (argv[i][3] == '+') Ctrl->D.force = TRUE;
 					break;
 				case 'L':
 					if (argv[i][2]) {	/* Set line options */
@@ -239,9 +239,6 @@ int main (int argc, char **argv)
 						}
 					}
 					break;
-				case 'M':               /* Multiple line segments input */
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
 					strcpy (buffer, &argv[i][2]);
@@ -277,8 +274,9 @@ int main (int argc, char **argv)
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -Z option:  Specify z_min and z_max\n", GMT_program);
 						exit (EXIT_FAILURE);
 					}
-					if (!(za[0] == '-' && za[1] == '\0')) Ctrl->Z.min = atof (za);
-					if (!(zb[0] == '-' && zb[1] == '\0')) Ctrl->Z.max = atof (zb);
+					if (!(za[0] == '-' && za[1] == '\0')) error += GMT_verify_expectations (GMT_io.in_col_type[GMT_Z], GMT_scanf_arg (za, GMT_io.in_col_type[GMT_Z], &Ctrl->Z.min), za);
+					if (!(zb[0] == '-' && zb[1] == '\0')) error += GMT_verify_expectations (GMT_io.in_col_type[GMT_Z], GMT_scanf_arg (zb, GMT_io.in_col_type[GMT_Z], &Ctrl->Z.max), zb);
+					
 					break;
 				case '+':	/* Undocumented option to increase path-fix resolution */
 					step = atof (&argv[i][2]);
@@ -295,22 +293,21 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "gmtselect %s - Select data subsets based on multiple spatial criteria\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: gmtselect <infiles> [-A<min_area>[/<min_level>/<max_level>]] [-C[f]<dist>/<ptfile>]\n");
-		fprintf (stderr, "\t[-D<resolution>] [-F<polygon>] [%s] [-L[p]<dist>/<lfile>] [%s] [-I[cflrsz]\n", GMT_J_OPT, GMT_H_OPT);
-		fprintf (stderr, "\t[%s] [-N<maskvalues>[o]] [%s] [-V[l]]\n\t[%s] [-Z<min>/<max>] [-%s] [%s]\n\n", GMT_M_OPT, GMT_Rgeo_OPT, GMT_t_OPT, GMT_b_OPT, GMT_f_OPT);
+		fprintf (stderr, "usage: gmtselect <infiles> [%s] [-C[f]<dist>/<ptfile>]\n", GMT_A_OPT);
+		fprintf (stderr, "\t[-D<resolution>][+] [-F<polygon>] [%s] [-L[p]<dist>/<lfile>] [%s] [-I[cflrsz]\n", GMT_J_OPT, GMT_H_OPT);
+		fprintf (stderr, "\t[-N<maskvalues>[o]] [%s] [-V[l]]\n\t[%s] [-Z<min>/<max>] [-%s] [%s] [%s]\n\n", GMT_Rgeo_OPT, GMT_t_OPT, GMT_b_OPT, GMT_f_OPT, GMT_m_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\tinfiles (in ASCII or binary) have 2 or more columns with (x,y) or (y,x) in first columns.\n");
+		fprintf (stderr, "\tinfiles (in ASCII, binary, netCDF) have 2 or more columns with (x,y) or (y,x) in first columns.\n");
 		fprintf (stderr, "\t  If no file(s) is given, standard input is read.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-A coastline features smaller than <min_area> (in km^2) or of levels (0-4) outside the min-max levels\n");
-		fprintf (stderr, "\t   will be skipped [0/4] (see pscoast for details)].  Ignored unless -N is set\n");
+		GMT_GSHHS_syntax ('A', "Place limits on coastline features from the GSHHS data base (ignored  unless -N is set).");
 		fprintf (stderr, "\t-C pass locations that are within <dist> of any point in the ASCII <ptfile>\n");
 		fprintf (stderr, "\t   Give distance as 0 if 3rd column of <ptfile> has individual distances.\n");
 		fprintf (stderr, "\t   Distances are Cartesian and in user units [or spherical in km if -fg is used].\n");
 		fprintf (stderr, "\t   Use -Cf for approximate (flat earth) rather than exact geodesic distances.\n");
-		fprintf (stderr, "\t   If current ELLIPSOID is Sphere then great circle distances replace geodesics.\n");
+		fprintf (stderr, "\t   If current ELLIPSOID is spherical then great circle distances replace geodesics.\n");
 		fprintf (stderr, "\t   Use -R -J to compute mapped Cartesian distances in cm, inch, m, or points [%s]\n", GMT_unit_names[gmtdefs.measure_unit]);
 		fprintf (stderr, "\t-D Choose one of the following resolutions: (Ignored unless -N is set)\n");
 		fprintf (stderr, "\t   f - full resolution (may be very slow for large regions)\n");
@@ -318,10 +315,11 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   i - intermediate resolution\n");
 		fprintf (stderr, "\t   l - low resolution [Default]\n");
 		fprintf (stderr, "\t   c - crude resolution, for tasks that need crude continent outlines only\n");
+		fprintf (stderr, "\t   Append + to use a lower resolution should the chosen one not be available [abort].\n");
 		fprintf (stderr, "\t-L Pass locations that are within <dist> of any line in ASCII <linefile>\n");
 		fprintf (stderr, "\t   Give distance as 0 if 2nd column of segment headers have individual distances.\n");
 		fprintf (stderr, "\t   Distances are Cartesian and in user units [or spherical in km if -fg is used].\n");
-		fprintf (stderr, "\t   If current ELLIPSOID is Sphere then great circle distances replace geodesics.\n");
+		fprintf (stderr, "\t   If current ELLIPSOID is spherical then great circle distances replace geodesics.\n");
 		fprintf (stderr, "\t   Use -R -J to compute mapped Cartesian distances in cm, inch, m, or points [%s]\n", GMT_unit_names[gmtdefs.measure_unit]);
 		fprintf (stderr, "\t   Optionally, use -Lp to exclude points projecting beyond a line's endpoints.\n");
 		fprintf (stderr, "\t-F pass locations that are inside the polygons in the ASCII <polygon> file\n");
@@ -335,7 +333,6 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   s will pass locations that otherwise would be skipped in -N\n");
 		fprintf (stderr, "\t   z will pass locations outside the range given in -Z\n");
 		GMT_explain_option ('J');
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N set if a point outside or inside a geographic feature should be s(kipped) or k(ept).\n");
 		fprintf (stderr, "\t   Append o to let feature boundary be considered outside [Default is inside].\n");
 		fprintf (stderr, "\t   Specify this information with s or k using 1 of 2 formats:\n");
@@ -354,6 +351,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -366,8 +364,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  -Z must have zmax > zmin!\n", GMT_program);
 		error++;
 	}
-	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
+	if ((GMT_io.binary[GMT_IN] || GMT_io.netcdf[GMT_IN]) && GMT_io.io_header[GMT_IN]) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary and netCDF input data cannot have header -H\n", GMT_program);
 		error++;
 	}
         if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = 2 + Ctrl->Z.active;
@@ -381,7 +379,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -389,6 +387,7 @@ int main (int argc, char **argv)
 #endif
 
 	shuffle = (gmtdefs.xy_toggle[GMT_IN] != gmtdefs.xy_toggle[GMT_OUT]);	/* Must rewrite output record */
+	n_minimum = (Ctrl->Z.active) ? 3 : 2;	/* Minimum number of columns in ASCII input */
 	
 	if (!project_info.region_supplied && Ctrl->N.active) {	/* If we use coastline data or used -fg but didnt give -R we imply -Rg */
 		project_info.region_supplied = TRUE;
@@ -397,7 +396,7 @@ int main (int argc, char **argv)
 		GMT_io.in_col_type[1] = GMT_IS_LAT;
 	}
 	if (project_info.region_supplied) {	/* -R was set directly or indirectly; hence must set -J if not supplied */
-		if (project_info.projection == GMT_NO_PROJ) {	/* -J not specifed, set one implicitly */
+		if (project_info.projection == GMT_NO_PROJ) {	/* -J not specified, set one implicitly */
 			if (GMT_io.in_col_type[0] == GMT_IS_LON) {	/* We know we have geographic data (either via -f or -N) */
 				project_info.degree[0] = project_info.degree[1] = TRUE;
 			}
@@ -421,22 +420,24 @@ int main (int argc, char **argv)
 	if (do_project && gmtdefs.verbose) fprintf (stderr, "%s: Warning: -J means all data will be projected before tests are applied\n", GMT_program);
 	 
 	if (Ctrl->N.active) {
+		if (Ctrl->D.force) Ctrl->D.set = GMT_shore_adjust_res (Ctrl->D.set);
 		if (Ctrl->D.active) base = GMT_set_resolution (&Ctrl->D.set, 'D');
 		if (dry_wet_only) {	/* Post-process -N choice */
 			Ctrl->N.mask[3] = Ctrl->N.mask[1];
 			Ctrl->N.mask[2] = Ctrl->N.mask[4] = Ctrl->N.mask[0];
 		}
-		if (GMT_init_shore (Ctrl->D.set, &c, west, east, south, north)) {
+		if (GMT_init_shore (Ctrl->D.set, &c, west, east, south, north, &Ctrl->A.info)) {
 			fprintf (stderr, "%s: %s resolution shoreline data base not installed\n", GMT_program, shore_resolution[base]);
 			exit (EXIT_FAILURE);
 		}
+		if (gmtdefs.verbose == 2) fprintf (stderr, "GSHHS version %s\n%s\n%s\n", c.version, c.title, c.source);
 		west_border = floor (project_info.w / c.bsize) * c.bsize;
 		east_border = ceil (project_info.e / c.bsize) * c.bsize;
 		wd[0] = 1;	wd[1] = -1;
 		np[0] = np[1] = 0;
 	}
 
-	just_copy_record = !(GMT_io.binary[GMT_IN] || GMT_io.binary[GMT_OUT] || shuffle);
+	just_copy_record = !(GMT_io.binary[GMT_IN] || GMT_io.netcdf[GMT_IN] || GMT_io.binary[GMT_OUT] || GMT_io.netcdf[GMT_OUT] || shuffle);
 
 	/* Initiate pointer to distance calculation function */
 	if (GMT_io.in_col_type[0] & GMT_IS_GEO && !do_project) {	/* Geographic data and no -R -J conversion */
@@ -444,13 +445,13 @@ int main (int argc, char **argv)
 			GMT_distance_func = GMT_flatearth_dist_km;
 		else
 			GMT_distance_func = (PFD) ((GMT_IS_SPHERICAL) ? GMT_geodesic_dist_km : GMT_great_circle_dist_km);
-		near_a_line  = (PFI) GMT_near_a_line_spherical;
-		near_a_point = (PFI) GMT_near_a_point_spherical;
+		near_a_line  = (PFL) GMT_near_a_line_spherical;
+		near_a_point = (PFL) GMT_near_a_point_spherical;
 	}
 	else {	/* Cartesian data (or lon/lat projected via -R -J) */
 		GMT_distance_func = (PFD) GMT_cartesian_dist;
-		near_a_line  = (PFI) GMT_near_a_line_cartesian;
-		near_a_point = (PFI) GMT_near_a_point_cartesian;
+		near_a_line  = (PFL) GMT_near_a_line_cartesian;
+		near_a_point = (PFL) GMT_near_a_point_cartesian;
 	}
 
 	if (Ctrl->C.active) { 	/* Initialize point structure used in test for proximity to points */
@@ -566,7 +567,7 @@ int main (int argc, char **argv)
 
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
-				GMT_fgets (buffer, BUFSIZ, fp);
+				not_used = GMT_fgets (buffer, BUFSIZ, fp);
 				if (first && GMT_io.io_header[GMT_OUT]) fprintf (GMT_stdout, "%s", buffer);
 			}
 			first = FALSE;
@@ -581,16 +582,19 @@ int main (int argc, char **argv)
 			}
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
-			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_read);
+			if (!just_copy_record && (GMT_io.status & GMT_IO_MISMATCH)) {
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 
 			n_read++;
 			if (long_verbose && n_read%1000 == 0) fprintf (stderr, "%s: Read %ld records, passed %ld records\r", GMT_program, n_read, n_pass);
 
-			if (Ctrl->Z.active && n_expected_fields < 3) {
-				fprintf (stderr, "%s: -Z requires a data file with at least 3 columns; this file only has %d. Exiting.\n", GMT_program, n_fields);
+			if (n_fields < n_minimum) {
+				if (Ctrl->Z.active)
+					fprintf (stderr, "%s: -Z requires a data file with at least 3 columns; this file only has %ld near line %ld. Exiting.\n", GMT_program, n_fields, n_read);
+				else
+					fprintf (stderr, "%s: Data file must have at least 2 columns; this file only has %ld near line %ld. Exiting.\n", GMT_program, n_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 
@@ -600,17 +604,17 @@ int main (int argc, char **argv)
 				if (inside != Ctrl->I.pass[5]) { output_header = need_header; continue;}
 			}
 
-			lon = in[0];
+			lon = in[GMT_X];
 			if (project_info.region_supplied) {
-				inside = !GMT_map_outside (lon, in[1]);
+				inside = !GMT_map_outside (lon, in[GMT_Y]);
 				if (inside != Ctrl->I.pass[0]) { output_header = need_header; continue;}
 			}
 
 			if (do_project)
-				GMT_geo_to_xy (lon, in[1], &xx, &yy);
+				GMT_geo_to_xy (lon, in[GMT_Y], &xx, &yy);
 			else {
 				xx = lon;
-				yy = in[1];
+				yy = in[GMT_Y];
 			}
 			
 			if (Ctrl->C.active) {	/* Check for distance to points */
@@ -653,7 +657,7 @@ int main (int argc, char **argv)
 			if (Ctrl->N.active) {
 				xx = lon;
 				while (xx < 0.0) xx += 360.0;
-				row = ((int)floor ((90.0 - in[1]) / c.bsize));
+				row = ((int)floor ((90.0 - in[GMT_Y]) / c.bsize));
 				if (row >= c.bin_ny) row = c.bin_ny - 1;	/* Presumably only kicks in for south pole */
 				col = (int)floor (xx / c.bsize);
 				bin = row * c.bin_nx + col;
@@ -663,7 +667,7 @@ int main (int argc, char **argv)
 					if (ind == c.nb) continue;			/* Bin not among the chosen ones */
 					last_bin = bin;
 					GMT_free_shore (&c);	/* Free previously allocated arrays */
-					if ((err = GMT_get_shore_bin (ind, &c, Ctrl->A.area, Ctrl->A.low, Ctrl->A.high))) {
+					if ((err = GMT_get_shore_bin (ind, &c))) {
 						fprintf (stderr, "%s: %s [%s resolution shoreline]\n", GMT_program, GMT_strerror(err), shore_resolution[base]);
 						exit (EXIT_FAILURE);
 					}
@@ -672,7 +676,7 @@ int main (int argc, char **argv)
 					for (id = 0; id < 2; id++) {
 						GMT_free_polygons (p[id], np[id]);
 						if (np[id]) GMT_free ((void *)p[id]);
-						np[id] = GMT_assemble_shore (&c, wd[id],  Ctrl->A.low, TRUE, greenwich, west_border, east_border, &p[id]);
+						np[id] = GMT_assemble_shore (&c, wd[id], TRUE, greenwich, west_border, east_border, &p[id]);
 						np[id] = GMT_prep_polygons (&p[id], np[id], TRUE, step, -1);
 					}
 				}
@@ -682,7 +686,7 @@ int main (int argc, char **argv)
 				}
 				else {
 					this_node = 0;
-					GMT_geo_to_xy (lon, in[1], &xx, &yy);
+					GMT_geo_to_xy (lon, in[GMT_Y], &xx, &yy);
 					for (id = 0; id < 2; id++) {
 
 						for (k = 0; k < np[id]; k++) {
@@ -773,17 +777,17 @@ int compare_x (const void *point_1, const void *point_2)
 }
 
 void *New_Gmtselect_Ctrl () {	/* Allocate and initialize a new control structure */
-	int i;
+	GMT_LONG i;
 	struct GMTSELECT_CTRL *C;
 	
 	C = (struct GMTSELECT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMTSELECT_CTRL), "New_Gmtselect_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
-	C->A.high = GMT_MAX_GSHHS_LEVEL;				/* Include all GSHHS levels */
+	C->A.info.high = GMT_MAX_GSHHS_LEVEL;				/* Include all GSHHS levels */
 	C->D.set = 'l';							/* Low-resolution coastline data */
 	for (i = 0; i < GMTSELECT_N_TESTS; i++) C->I.pass[i] = TRUE;	/* Default is to pass if we are inside */
-	memset ((void *)C->N.mask, 0, (size_t)(GMTSELECT_N_CLASSES * sizeof (int)));	/* Default for "wet" areas = 0 (outside) */
+	memset ((void *)C->N.mask, 0, (size_t)(GMTSELECT_N_CLASSES * sizeof (GMT_LONG)));	/* Default for "wet" areas = 0 (outside) */
 	C->N.mask[1] = C->N.mask[3] = 1;				/* Default for "dry" areas = 1 (inside) */
 	C->Z.min = -DBL_MAX;	C->Z.max = DBL_MAX;			/* No limits on z-range */
 	
