@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grd2cpt.c,v 1.46 2007/09/30 04:16:48 guru Exp $
+ *	$Id: grd2cpt.c,v 1.55 2009/05/13 21:06:42 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -67,7 +67,7 @@ struct GRD2CPT_CTRL {
 	} N;
 	struct Q {	/* -Q[i|o */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 	} Q;
 	struct S {	/* -S<z_start>/<z_stop>/<z_inc> */
 		BOOLEAN active;
@@ -77,7 +77,7 @@ struct GRD2CPT_CTRL {
 	} S;
 	struct T {	/* -T<kind> */
 		BOOLEAN active;
-		int kind; /* -1 symmetric +-zmin, +1 +-zmax, -2 = +-Minx(|zmin|,|zmax|), +2 = +-Max(|zmin|,|zmax|), 0 = min to max [Default] */
+		GMT_LONG kind; /* -1 symmetric +-zmin, +1 +-zmax, -2 = +-Minx(|zmin|,|zmax|), +2 = +-Max(|zmin|,|zmax|), 0 = min to max [Default] */
 	} T;
 	struct Z {	/* -Z */
 		BOOLEAN active;
@@ -88,15 +88,17 @@ int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, global = FALSE;
 
-	char *grdfile = CNULL, CPT_lis[BUFSIZ], CPT_file[BUFSIZ], format[BUFSIZ], kind;
+#define mgrd 1000
 
-	int i, j, nxy, nx, ny, nfound, ngood;
+	char *grdfile[mgrd], CPT_lis[BUFSIZ], CPT_file[BUFSIZ], format[BUFSIZ], kind;
+
+	GMT_LONG j, ngrd = 0, i, nxy, nxyg, nfound, ngood;
 
 	float	*zdata;
 
 	double *z, mean, sd, w, e, s, n;
 
-	struct GRD_HEADER grd;
+	struct GRD_HEADER grd, new;
 	struct CDF_CPT {
 		double	z;	/* Data value  */
 		double	f;	/* Cumulative distribution function f(z)  */
@@ -224,19 +226,35 @@ int main (int argc, char **argv)
 			}
 		}
 		else {
-			grdfile = argv[i];
-			if (GMT_err_pass (GMT_read_grd_info (grdfile, &grd), grdfile)) error++;
+			if (ngrd > mgrd) {
+				fprintf (stderr, "%s: GMT USAGE ERROR:  Cannot handle more than %i grids\n", GMT_program, mgrd);
+				exit(EXIT_FAILURE);
+			}
+
+			grdfile[ngrd] = argv[i];
+
+			if (ngrd == 0) {
+				if (GMT_err_pass (GMT_read_grd_info (grdfile[0], &grd), grdfile[0])) error++;
+			}
+			else {
+				if (GMT_err_pass (GMT_read_grd_info (grdfile[ngrd], &new), grdfile[ngrd])) error++;
+				if (new.x_min != grd.x_min || new.x_max != grd.x_max || new.y_min != grd.y_min || new.y_max != grd.y_max || new.nx != grd.nx || new.ny != grd.ny) {
+					fprintf (stderr, "%s: GMT USAGE ERROR:  Grid %s does not have the same dimensions and size as grid %s\n", GMT_program, grdfile[ngrd], grdfile[0]);
+					exit (EXIT_FAILURE);
+				}
+			}
+			ngrd++;
 		}
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "grd2cpt %s - Make a a linear or histogram-equalized color palette table from a grdfile\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: grd2cpt <grdfile> [-C<table>] [-D] [-E<nlevels> [-I] [-L<min_limit>/<max_limit>]\n");
+		fprintf (stderr, "grd2cpt %s - Make a linear or histogram-equalized color palette table from a grdfile\n\n", GMT_VERSION);
+		fprintf (stderr, "usage: grd2cpt <grdfiles> [-C<table>] [-D] [-E<nlevels> [-I] [-L<min_limit>/<max_limit>]\n");
 		fprintf (stderr, "\t[-M] [-N] [-Q[i|o]] [%s] [-S<z_start>/<z_stop>/<z_inc>] [-T<-|+|=|_>] [-V] [-Z]\n", GMT_Rgeo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
-		fprintf (stderr, "\t<grdfile> is the name of the 2-D binary data set\n");
+		fprintf (stderr, "\t<grdfiles> names of one or more 2-D binary data sets\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-C  Specify a colortable [Default is rainbow]:\n");
 		fprintf (stderr, "\t   [Original z-range is given in brackets]\n");
@@ -277,6 +295,10 @@ int main (int argc, char **argv)
 
 	error += GMT_set_cpt_path (CPT_file, Ctrl->C.file);
 
+	if (ngrd < 1) {
+		fprintf(stderr,"%s: GMT USAGE ERROR:  No grid name(s) specified.\n", GMT_program);
+		error++;
+	}
 	if (Ctrl->L.active && Ctrl->L.min >= Ctrl->L.max) {
 		fprintf(stderr,"%s: GMT SYNTAX ERROR -L option:  min_limit must be less than max_limit.\n", GMT_program);
 		error++;
@@ -306,19 +328,18 @@ int main (int argc, char **argv)
 			fprintf (stderr, "%s: GMT ERROR: Subset exceeds data domain!\n", GMT_program);
 			exit (EXIT_FAILURE);
 		}
-		nx = GMT_get_n (w, e, grd.x_inc, grd.node_offset);
-		ny = GMT_get_n (s, n, grd.y_inc, grd.node_offset);
-		nxy = nx * ny;
-
-		zdata = (float *) GMT_memory (VNULL, (size_t) nxy, sizeof (float), GMT_program);
-
-		GMT_err_fail (GMT_read_grd (grdfile, &grd, zdata, w, e, s, n, GMT_pad, FALSE), grdfile);
+		nxy = GMT_get_nm (GMT_get_n (w, e, grd.x_inc, grd.node_offset), GMT_get_n (s, n, grd.y_inc, grd.node_offset));
 	}
-	else {
-		nxy = grd.nx * grd.ny;
-		zdata = (float *) GMT_memory (VNULL, (size_t) nxy, sizeof (float), GMT_program);
+	else
+		nxy = GMT_get_nm (grd.nx, grd.ny);
 
-		GMT_err_fail (GMT_read_grd (grdfile, &grd, zdata, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), grdfile);
+	nxyg = GMT_get_nm (nxy, ngrd);
+
+	zdata = (float *) GMT_memory (VNULL, (size_t) nxyg, sizeof (float), GMT_program);
+
+	for (i = 0; i < ngrd; i++) {
+		GMT_err_fail (GMT_read_grd_info (grdfile[i], &grd), grdfile[i]);
+		GMT_err_fail (GMT_read_grd (grdfile[i], &grd, &(zdata[i*nxy]), w, e, s, n, GMT_pad, FALSE), grdfile[i]);
 	}
 
 	/* Loop over the file and find NaNs.  If set limits, may create more NaNs  */
@@ -329,7 +350,7 @@ int main (int argc, char **argv)
 
 		grd.z_min = Ctrl->L.min;
 		grd.z_max = Ctrl->L.max;
-		for (i = 0; i < nxy; i++) {
+		for (i = 0; i < nxyg; i++) {
 			if (GMT_is_fnan (zdata[i]))
 				nfound++;
 			else {
@@ -347,7 +368,7 @@ int main (int argc, char **argv)
 	else {
 		Ctrl->L.min = grd.z_max;	/* This is just to double check grd.z_min, grd.z_max  */
 		Ctrl->L.max = grd.z_min;
-		for (i = 0; i < nxy; i++) {
+		for (i = 0; i < nxyg; i++) {
 			if (GMT_is_fnan (zdata[i]))
 				nfound++;
 			else {
@@ -360,7 +381,7 @@ int main (int argc, char **argv)
 		grd.z_min = Ctrl->L.min;
 		grd.z_max = Ctrl->L.max;
 	}
-	ngood = nxy - nfound;	/* This is the number of non-NaN points for the cdf function  */
+	ngood = nxyg - nfound;	/* This is the number of non-NaN points for the cdf function  */
 	mean /= ngood;
 	sd /= ngood;
 	sd = sqrt(sd - mean*mean);
@@ -373,7 +394,7 @@ int main (int argc, char **argv)
 	if (Ctrl->S.active) {
 		/* Use predefined levels and interval */
 		Ctrl->E.levels = (grd.z_min < Ctrl->S.low) ? 1 : 0;
-		Ctrl->E.levels += (int)floor((Ctrl->S.high - Ctrl->S.low)/Ctrl->S.inc) + 1;
+		Ctrl->E.levels += (GMT_LONG)floor((Ctrl->S.high - Ctrl->S.low)/Ctrl->S.inc) + 1;
 		if (grd.z_max > Ctrl->S.high) Ctrl->E.levels++;
 		cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)Ctrl->E.levels, sizeof(struct CDF_CPT), GMT_program);
 		if (grd.z_min < Ctrl->S.low) {
@@ -457,7 +478,7 @@ int main (int argc, char **argv)
 			cdf_cpt[j].f = 1.0;
 		else {
 			nfound = 0;
-			for (i = 0; i < nxy; i++) {
+			for (i = 0; i < nxyg; i++) {
 				if (!GMT_is_fnan (zdata[i]) && zdata[i] <= cdf_cpt[j].z) nfound++;
 			}
 			cdf_cpt[j].f = (double)(nfound-1)/(double)(ngood-1);
@@ -494,7 +515,7 @@ int main (int argc, char **argv)
 void *New_Grd2cpt_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRD2CPT_CTRL *C;
 
-	C = (struct GRD2CPT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRD2CPT_CTRL), "New_Grd2cpt_Ctrl");
+	C = (struct GRD2CPT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRD2CPT_CTRL), "New_Grd2cpt_Ctrl");
 
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	return ((void *)C);
