@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: grdblend.c,v 1.63 2008/05/01 03:00:38 guru Exp $
+ *    $Id: grdblend.c,v 1.76 2009/07/08 21:41:40 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -67,8 +67,9 @@ struct GRDBLEND_CTRL {
 
 struct GRDBLEND_INFO {	/* Structure with info about each input grid file */
 	struct GMT_GRDFILE G;				/* I/O structure for grid files, including grd header */
-	int in_i0, in_i1, out_i0, out_i1;		/* Indices of outer and inner x-coordinates (in output grid coordinates) */
-	int in_j0, in_j1, out_j0, out_j1;		/* Indices of outer and inner y-coordinates (in output grid coordinates) */
+	GMT_LONG in_i0, in_i1, out_i0, out_i1;		/* Indices of outer and inner x-coordinates (in output grid coordinates) */
+	GMT_LONG in_j0, in_j1, out_j0, out_j1;		/* Indices of outer and inner y-coordinates (in output grid coordinates) */
+	GMT_LONG offset;				/* grid offset when the grid extends beyond north */
 	long skip;					/* Byte offset to skip in native binary files */
 	BOOLEAN outside;				/* TRUE if the current output row is outside the range of this grid */
 	BOOLEAN invert;					/* TRUE if weight was given as negative and we want to taper to zero INSIDE the grid region */
@@ -81,18 +82,19 @@ struct GRDBLEND_INFO {	/* Structure with info about each input grid file */
 
 int main (int argc, char **argv)
 {
-	int i, col, row, k, kk, m, n_blend, err, error = 0;
-	size_t n_fill, n_tot;
+	GMT_LONG i, col, pcol, row, nx_360 = 0, k, kk, m, n_blend, err, error = 0;
+	GMT_LONG n_fill, n_tot, type;
 	double wt_x, w, wt;
 	float *z, no_data_f;
-	char mode[2] = {'w', 'W'}, type;
+	char mode[2] = {'w', 'W'};
+	BOOLEAN wrap_x;
 	FILE *fp = NULL;
 	struct GRDBLEND_INFO *blend = NULL;
 	struct GMT_GRDFILE S;
 	struct GRDBLEND_CTRL *Ctrl;
 
-	void sync_input_rows (int row, struct GRDBLEND_INFO *blend, int n_blend, double half);
-	int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend);
+	void sync_input_rows (GMT_LONG row, struct GRDBLEND_INFO *blend, GMT_LONG n_blend, double half);
+	GMT_LONG init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend);
 	void *New_Grdblend_Ctrl (), Free_Grdblend_Ctrl (struct GRDBLEND_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -173,8 +175,9 @@ int main (int argc, char **argv)
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\t<blendfile> is an ASCII file (or stdin) with blending parameters for each input grid\n");
-		fprintf (stderr, "\t   Each record has:  grid filename -Rw/e/s/n weight\n");
-		fprintf (stderr, "\t   Relative weights are <weight> inside the given -R and cosine taper to 0 at actual -R.\n");
+		fprintf (stderr, "\t   Each record has three items:  filename -Rw/e/s/n weight\n");
+		fprintf (stderr, "\t   Relative weights are <weight> inside the given -R and cosine taper to 0 at actual grid -R.\n");
+		fprintf (stderr, "\t   Give filename - weight if inner region should equal the actual region\n");
 		fprintf (stderr, "\t   Give a negative weight to invert the sense of the taper (i.e., |<weight>| outside given R.\n");
 		fprintf (stderr, "\t-G <grdfile> is the name of the final 2-D binary data set\n");
 		GMT_inc_syntax ('I', 0);
@@ -191,6 +194,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, NULL, &Ctrl->I.active);
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -R:  Must specify region\n", GMT_program);
 		error++;
@@ -243,7 +248,7 @@ int main (int argc, char **argv)
 
 	GMT_err_fail (GMT_grd_RI_verify (&S.header, 1), Ctrl->G.file);
 
-	n_tot = ((size_t)S.header.nx) * ((size_t)S.header.ny);
+	n_tot = GMT_get_nm (S.header.nx, S.header.ny);
 
 	z = (float *) GMT_memory (VNULL, (size_t)S.header.nx, sizeof (float), GMT_program);	/* Memory for one output row */
 
@@ -257,6 +262,8 @@ int main (int argc, char **argv)
 
 	S.header.z_min = DBL_MAX;	/* These will be updated in the loop below */
 	S.header.z_max = -DBL_MAX;
+	wrap_x = (GMT_io.out_col_type[GMT_X] == GMT_IS_LON);	/* Periodic geographic grid */
+	if (wrap_x) nx_360 = irint (360.0 / S.header.x_inc);
 
 	for (row = 0; row < S.header.ny; row++) {	/* For every output row */
 
@@ -269,13 +276,21 @@ int main (int argc, char **argv)
 			w = 0.0;
 			for (k = m = 0; k < n_blend; k++) {	/* Loop over every input grid */
 				if (blend[k].outside) continue;					/* This grid is currently outside the s/n range */
-				if (col < blend[k].out_i0 || col > blend[k].out_i1) continue;	/* This grid is currently outside the w/e range */
-				kk = col - blend[k].out_i0;					/* kk is the local column variable for this grid */
+				if (wrap_x) {	/* Special testing for periodic x coordinates */
+					pcol = col + nx_360;
+					while (pcol > blend[k].out_i1) pcol -= nx_360;
+					if (pcol < blend[k].out_i0) continue;	/* This grid is currently outside the w/e range */
+				}
+				else {	/* Not periodic */
+					if (col < blend[k].out_i0 || col > blend[k].out_i1) continue;	/* This grid is currently outside the xmin/xmax range */
+					pcol = col;
+				}
+				kk = pcol - blend[k].out_i0;					/* kk is the local column variable for this grid */
 				if (GMT_is_fnan (blend[k].z[kk])) continue;			/* NaNs do not contribute */
-				if (col <= blend[k].in_i0)					/* Left cosine-taper weight */
-					wt_x = 0.5 * (1.0 - cos ((col - blend[k].out_i0 + S.header.xy_off) * blend[k].wxl));
-				else if (col >= blend[k].in_i1)					/* Right cosine-taper weight */
-					wt_x = 0.5 * (1.0 - cos ((blend[k].out_i1 - col + S.header.xy_off) * blend[k].wxr));
+				if (pcol <= blend[k].in_i0)					/* Left cosine-taper weight */
+					wt_x = 0.5 * (1.0 - cos ((pcol - blend[k].out_i0 + S.header.xy_off) * blend[k].wxl));
+				else if (pcol >= blend[k].in_i1)					/* Right cosine-taper weight */
+					wt_x = 0.5 * (1.0 - cos ((blend[k].out_i1 - pcol + S.header.xy_off) * blend[k].wxr));
 				else								/* Inside inner region, weight = 1 */
 					wt_x = 1.0;
 				wt = wt_x * blend[k].wt_y;					/* Actual weight is 2-D cosine taper */
@@ -301,10 +316,10 @@ int main (int argc, char **argv)
 		}
 		GMT_err_fail (GMT_write_grd_row (&S, 0, z), Ctrl->G.file);
 
-		if (gmtdefs.verbose && row%10 == 0)  fprintf (stderr, "%s: Processed row %7d of %d\r", GMT_program, row, S.header.ny);
+		if (gmtdefs.verbose && row%10 == 0)  fprintf (stderr, "%s: Processed row %7ld of %d\r", GMT_program, row, S.header.ny);
 
 	}
-	if (gmtdefs.verbose)  fprintf (stderr, "%s: Processed row %7d\n", GMT_program, row);
+	if (gmtdefs.verbose)  fprintf (stderr, "%s: Processed row %7ld\n", GMT_program, row);
 
 	GMT_close_grd (&S);	/* Close the output gridfile */
 	if (!Ctrl->Q.active) GMT_err_fail (GMT_update_grd_info (Ctrl->G.file, &S.header), Ctrl->G.file);
@@ -338,25 +353,30 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend) {
-	int n = 0, nr, one_or_zero = 0;
-	size_t n_alloc = GMT_SMALL_CHUNK;
+GMT_LONG init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend) {
+	GMT_LONG n = 0, nr, one_or_zero = 0, n_alloc = 0, type;
 	struct GRDBLEND_INFO *B;
-	char line[BUFSIZ], r_in[GMT_LONG_TEXT], type, *sense[2] = {"normal", "inverse"};
+	char line[BUFSIZ], r_in[GMT_LONG_TEXT], *sense[2] = {"normal", "inverse"};
 	void decode_R (char *string, double *w, double *e, double *s, double *n);
 
-	B = (struct GRDBLEND_INFO *) GMT_memory (VNULL, n_alloc, sizeof (struct GRDBLEND_INFO), GMT_program);
-
+	GMT_set_meminc (GMT_SMALL_CHUNK);
 	while (GMT_fgets (line, BUFSIZ, fp)) {	/* Read each input file for grid information */
 		if (line[0] == '#' || line[0] == '\n' || line[0] == '\r') continue;	/* Skip comment lines or blank lines */
 
+		if (n == n_alloc) n_alloc = GMT_alloc_memory ((void **)&B, n, n_alloc, sizeof (struct GRDBLEND_INFO), GMT_program);
+		memset ((void *)&B[n], 0, sizeof (struct GRDBLEND_INFO));	/* Initialize memory */
 		nr = sscanf (line, "%s %s %lf", B[n].file, r_in, &B[n].weight);
 		if (nr != 3) {
-			fprintf (stderr, "%s: Read error for blending parameters near row %d\n", GMT_program, n);
+			fprintf (stderr, "%s: Read error for blending parameters near row %ld\n", GMT_program, n);
 			exit (EXIT_FAILURE);
 		}
-		decode_R (&r_in[2], &B[n].w_in, &B[n].e_in, &B[n].s_in, &B[n].n_in);	/* Decode inner region */
 		GMT_err_fail (GMT_read_grd_info (B[n].file, &B[n].G.header), B[n].file);	/* Read header structure */
+		if (!strcmp (r_in, "-")) {	/* Set inner = outer region */
+			B[n].w_in = B[n].G.header.x_min;	B[n].e_in = B[n].G.header.x_max;
+			B[n].s_in = B[n].G.header.y_min;	B[n].n_in = B[n].G.header.y_max;
+		}
+		else	/* Must decode the -R string */
+			decode_R (&r_in[2], &B[n].w_in, &B[n].e_in, &B[n].s_in, &B[n].n_in);	/* Decode inner region */
 
 		/* Skip the file if its outer region does not lie within the final grid region */
 		if (h->x_min > B[n].e_in || h->x_max < B[n].w_in || h->y_min > B[n].n_in || h->y_max < B[n].s_in) {
@@ -413,9 +433,9 @@ int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend
 		if (B[n].out_j0 < 0) {	/* Must skip to first row inside the present -R */
 			type = GMT_grdformats[B[n].G.header.type][0];
 			if (type == 'c')	/* Old-style, 1-D netcdf grid */
-				B[n].G.start[0] += B[n].G.header.nx * abs (B[n].out_j0);
+				B[n].offset = B[n].G.header.nx * abs (B[n].out_j0);
 			else if (type == 'n')	/* New, 2-D netcdf grid */
-				B[n].G.start[0] -= abs (B[n].out_j0);
+				B[n].offset = B[n].out_j0;
 			else
 				B[n].skip = (long)(B[n].G.n_byte * abs (B[n].out_j0));	/* do the fseek when we are ready to read first row */
 		}
@@ -423,26 +443,23 @@ int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend
 		/* Allocate space for one entire row */
 
 		B[n].z = (float *) GMT_memory (VNULL, (size_t)(B[n].G.header.nx), sizeof (float), GMT_program);
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Blend file %s in %g/%g/%g/%g with %s weight %g [%d-%d]\n",
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Blend file %s in %g/%g/%g/%g with %s weight %g [%ld-%ld]\n",
 			GMT_program, B[n].G.header.name, B[n].w_in, B[n].e_in, B[n].s_in, B[n].n_in, sense[B[n].invert], B[n].weight, B[n].out_j0, B[n].out_j1);
 
 		GMT_close_grd (&B[n].G);
 
 		n++;
-		if (n == (GMT_LONG)n_alloc) {
-			n_alloc <<= 1;
-			B = (struct GRDBLEND_INFO *) GMT_memory ((void *)B, n_alloc, sizeof (struct GRDBLEND_INFO), GMT_program);
-		}
 	}
 
-	B = (struct GRDBLEND_INFO *) GMT_memory ((void *)B, (size_t)n, sizeof (struct GRDBLEND_INFO), GMT_program);
+	n_alloc = GMT_alloc_memory ((void **)&B, 0, n, sizeof (struct GRDBLEND_INFO), GMT_program);
 	*blend = B;
+	GMT_reset_meminc ();
 
 	return (n);
 }
 
-void sync_input_rows (int row, struct GRDBLEND_INFO *B, int n_blend, double half) {
-	int k;
+void sync_input_rows (GMT_LONG row, struct GRDBLEND_INFO *B, GMT_LONG n_blend, double half) {
+	GMT_LONG k;
 
 	for (k = 0; k < n_blend; k++) {	/* Get every input grid ready for the new row */
 		if (row < B[k].out_j0 || row > B[k].out_j1) {	/* Either done with grid or haven't gotten to this range yet */
@@ -466,6 +483,7 @@ void sync_input_rows (int row, struct GRDBLEND_INFO *B, int n_blend, double half
 		if (!B[k].open) {
 			GMT_err_fail (GMT_open_grd (B[k].file, &B[k].G, 'r'), B[k].file);	/* Open the grid for incremental row reading */
 			if (B[k].skip) GMT_fseek (B[k].G.fp, B[k].skip, SEEK_CUR);	/* Position for native binary files */
+			B[k].G.start[0] += B[k].offset;					/* Start position for netCDF files */
 			B[k].open = TRUE;
 		}
 
@@ -475,7 +493,7 @@ void sync_input_rows (int row, struct GRDBLEND_INFO *B, int n_blend, double half
 
 void decode_R (char *string, double *w, double *e, double *s, double *n) {
 	double *p[4];
-	int i, pos, error = 0;
+	GMT_LONG i, pos, error = 0;
 	char text[BUFSIZ];
 
 	/* Needed to decode the inner region -Rw/e/s/n string */
