@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdcontour.c,v 1.120 2007/10/05 22:17:16 guru Exp $
+ *	$Id: grdcontour.c,v 1.140 2009/06/12 02:06:45 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -66,7 +66,7 @@ struct GRDCONTOUR_CTRL {
 	} E;
 	struct F {	/* -F<way> */
 		BOOLEAN active;
-		int value;
+		GMT_LONG value;
 	} F;
 	struct G {	/* -G[d|f|n|l|L|x|X]<params> */
 		BOOLEAN active;
@@ -77,11 +77,11 @@ struct GRDCONTOUR_CTRL {
 	} L;
 	struct Q {	/* -Q<cut> */
 		BOOLEAN active;
-		int min;
+		GMT_LONG min;
 	} Q;
 	struct S {	/* -S<smooth> */
 		BOOLEAN active;
-		int value;
+		GMT_LONG value;
 	} S;
 	struct T {	/* -T[+|-][<gap>[c|i|m|p]/<length>[c|i|m|p]][:LH] */
 		BOOLEAN active;
@@ -105,17 +105,19 @@ struct GRDCONTOUR_CTRL {
 struct SAVE {
 	double *x, *y;
 	double cval;
-	int n;
+	GMT_LONG n;
 	struct GMT_PEN pen;
 	BOOLEAN do_it, high;
 };
 
 int main (int argc, char **argv)
 {
-	int i, j, k, n, nn, nm, c, n_edges, n_alloc, section, n_contours, side, id;
-	int nx, ny, n_save = 0, got, rgb[3], *edge;
+	GMT_LONG j, k, c, n_edges, n_alloc = 0, section, n_contours, side, id;
+	GMT_LONG nx, ny, n_save = 0, got, nm, i, n, nn, *edge;
+	
+	int rgb[3];
 
-	BOOLEAN error = FALSE, first = TRUE, interior, begin, subset = FALSE;
+	BOOLEAN error = FALSE, interior, begin, subset = FALSE;
 
 	char *grdfile = 0, line[BUFSIZ], *cont_type, *cont_do_tick;
 	char txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
@@ -132,7 +134,7 @@ int main (int argc, char **argv)
 	struct SAVE *save = NULL;
 	struct GRDCONTOUR_CTRL *Ctrl;
 
-	void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float *grd, double tick_gap, double tick_length, BOOLEAN tick_low, BOOLEAN tick_high, BOOLEAN tick_label, char Llabel, char Hlabel);
+	void sort_and_plot_ticks (struct SAVE *save, GMT_LONG n, struct GRD_HEADER *h, float *grd, double tick_gap, double tick_length, BOOLEAN tick_low, BOOLEAN tick_high, BOOLEAN tick_label, char Llabel, char Hlabel);
 	void GMT_grd_minmax (float *a, struct GRD_HEADER *h, double xyz[2][3]);
 	void adjust_hill_label (struct GMT_CONTOUR *G, struct GRD_HEADER *h, float *z);
 	void *New_Grdcontour_Ctrl (), Free_Grdcontour_Ctrl (struct GRDCONTOUR_CTRL *C);
@@ -150,6 +152,7 @@ int main (int argc, char **argv)
 				case 'B':
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -162,6 +165,7 @@ int main (int argc, char **argv)
 				case 'b':
 				case 'c':
 				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
@@ -191,7 +195,7 @@ int main (int argc, char **argv)
 					Ctrl->C.active = TRUE;
 					if (!GMT_access (&argv[i][2], R_OK)) {	/* Gave a readable file */
 						Ctrl->C.interval = 1.0;
-						Ctrl->C.cpt = (!strncmp (&argv[i][strlen(argv[i])-4], ".cpt", 4)) ? TRUE : FALSE;
+						Ctrl->C.cpt = (!strncmp (&argv[i][strlen(argv[i])-4], ".cpt", (size_t)4)) ? TRUE : FALSE;
 						Ctrl->C.file = strdup (&argv[i][2]);
 					}
 					else
@@ -204,7 +208,7 @@ int main (int argc, char **argv)
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
-					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
+					error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'F':
 					Ctrl->F.active = TRUE;
@@ -231,10 +235,6 @@ int main (int argc, char **argv)
 					Ctrl->L.active = TRUE;
 					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->L.low, &Ctrl->L.high);
 					break;
-				case 'M':	/* with -D, create one multiple line segments */
-					GMT_multisegment (&argv[i][2]);
-					GMT_io.multi_segments[GMT_OUT] = 2;
-					break;
 				case 'N':	/* Backwards compatibility - now done in -A instead */
 					if (argv[i][2])
 						strcpy (Ctrl->contour.unit, &argv[i][2]);
@@ -303,14 +303,14 @@ int main (int argc, char **argv)
 						/* Get the argument following a or c and up to first comma, slash (or to the end) */
 						n = k+1;
 						while (!(argv[i][n] == ',' || argv[i][n] == '/' || argv[i][n] == '\0')) n++;
-						strncpy (txt_a, &argv[i][k], n-k);	txt_a[n-k] = '\0';
+						strncpy (txt_a, &argv[i][k], (size_t)(n-k));	txt_a[n-k] = '\0';
 						if (GMT_colorname2index (txt_a) >= 0) j = k;	/* Found a colorname; wind j back by 1 */
 						id = (argv[i][k] == 'a') ? 1 : 0;
 						if (GMT_getpen (&argv[i][j], &Ctrl->W.pen[id])) {
 							GMT_pen_syntax ('W', " ");
 							error++;
 						}
-						if (j == k) Ctrl->W.pen[1] = Ctrl->W.pen[0];	/* Must copy since it wasnt -Wc or -Wa after all */
+						if (j == k) Ctrl->W.pen[1] = Ctrl->W.pen[0];	/* Must copy since it was not -Wc nor -Wa after all */
 					}
 					break;
 				case 'Z':
@@ -331,10 +331,10 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"grdcontour %s - Contouring of 2-D gridded data sets\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdcontour <grdfile> -C<cont_int> %s\n", GMT_J_OPT);
-		fprintf (stderr, "\t[-A[-|<annot_int>][<labelinfo>] [%s] [-D<dumpfile>] [-Eazimuth/elevation] [-F[l|r]] [%s]\n", GMT_B_OPT, GMT_CONTG);
-		fprintf (stderr, "\t[-K] [-L<Low/high>] [%s] [-O] [-P] [-Q<cut>] [%s] [-S<smooth>]\n", GMT_Mo_OPT, GMT_Rgeo_OPT);
+		fprintf (stderr, "\t[-A[-|<annot_int>][<labelinfo>] [%s] [-D<dumpfile>] [%s] [-F[l|r]] [%s]\n", GMT_B_OPT, GMT_E_OPT, GMT_CONTG);
+		fprintf (stderr, "\t[-K] [-L<Low/high>] [-O] [-P] [-Q<cut>] [%s] [-S<smooth>]\n", GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[-T[+|-][<gap>[c|i|m|p]/<length>[c|i|m|p]][:LH]] [%s] [-V] [-W[+]<type><pen>]\n", GMT_U_OPT);
-		fprintf (stderr, "\t[%s] [%s] [-Z[<fact>[/shift>]][p]] [%s] [%s]\n\n", GMT_X_OPT, GMT_Y_OPT, GMT_bo_OPT, GMT_c_OPT);
+		fprintf (stderr, "\t[%s] [%s] [-Z[<fact>[/shift>]][p]] [%s] [%s] [%s]\n\n", GMT_X_OPT, GMT_Y_OPT, GMT_bo_OPT, GMT_c_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -353,20 +353,18 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   <labelinfo> controls the specifics of the labels.  Append what you need:\n");
 		GMT_label_syntax (5, 0);
 		GMT_explain_option ('b');
-		fprintf (stderr, "\t-D to Dump contour lines to individual files (but see -M)\n");
+		fprintf (stderr, "\t-D to Dump contour lines to individual files (but see -m)\n");
 		fprintf (stderr, "\t   Append file prefix [contour].  Files will be called <dumpfile>_<cont>_#[_i].xyz|b\n");
 		fprintf (stderr, "\t   where <cont> is the contour value and # is a segment counter.\n");
 		fprintf (stderr, "\t   _i is inserted for interior (closed) contours, with xyz (ascii) or b (binary) as extension.\n");
 		fprintf (stderr, "\t   However, if -D- is given then files are C#_e or C#_i plus extension, where # is a running number.\n");
-		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		GMT_explain_option ('E');
 		fprintf (stderr, "\t-F force dumped contours to be oriented so that the higher z-values are to the left (-Fl [Default])\n");
 		fprintf (stderr, "\t   or right (-Fr) as we move along the contour [Default is not oriented]\n");
 		fprintf (stderr, "\t-G Controls placement of labels along contours.  Choose among five algorithms:\n");
 		GMT_cont_syntax (3, 0);
 		GMT_explain_option ('K');
 		fprintf (stderr, "\t-L only contour inside this range\n");
-		fprintf (stderr, "\t-M Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
-		fprintf (stderr, "\t   whose first character is <flag> ['>'].  This header also has the contour level value\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
 		fprintf (stderr, "\t-Q Do not draw contours with less than <cut> points [Draw all contours]\n");
@@ -376,7 +374,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   at approximately gridsize/<smooth> intervals\n");
 		fprintf (stderr, "\t-T will embellish innermost, closed contours with ticks pointing in the downward direction\n");
 		fprintf (stderr, "\t   User may specify to tick only highs (-T+) or lows (-T-) [-T means both]\n");
-		fprintf (stderr, "\t   Append spacing/ticklength (apppend units) to change defaults [%g/%g %s]\n",
+		fprintf (stderr, "\t   Append spacing/ticklength (append units) to change defaults [%g/%g %s]\n",
 			Ctrl->T.spacing*GMT_u2u[GMT_INCH][gmtdefs.measure_unit], Ctrl->T.length*GMT_u2u[GMT_INCH][gmtdefs.measure_unit], GMT_unit_names[gmtdefs.measure_unit]);
 		fprintf (stderr, "\t   Append :LH to plot the characters L and H in the center of closed contours\n");
 		fprintf (stderr, "\t   for local Lows and Highs (e.g, give :-+ to plot - and + signs)\n");
@@ -394,6 +392,8 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		GMT_explain_option ('c');
 		GMT_explain_option ('f');
+		fprintf (stderr, "\t-m Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
+		fprintf (stderr, "\t   whose first character is <flag> ['>'].  This header also has the contour level value\n");
 		exit (EXIT_FAILURE);
 	}
 
@@ -446,6 +446,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: contours will be written to files contour_*.xyz\n", GMT_program);
 		strcpy (Ctrl->D.file,"contour");
 	}
+	if (GMT_io.multi_segments[GMT_OUT]) GMT_io.multi_segments[GMT_OUT] = 2;
+
 	GMT_z_periodic = Ctrl->Z.periodic;	/* Phase data */
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
@@ -473,42 +475,26 @@ int main (int argc, char **argv)
 
 	/* Determine the wesn to be used to read the grid file */
 
-	if (GMT_grd_setregion (&header, &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
+	if (GMT_grd_setregion (&header, &data_west, &data_east, &data_south, &data_north, BCR_BILINEAR)) {
+		/* No grid to plot; just do empty map and exit */
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: No data within specified region\n", GMT_program);
 		GMT_plotinit (argc, argv);
 		GMT_map_basemap ();
 		GMT_plotend ();
+		Free_Grdcontour_Ctrl (Ctrl);	/* Deallocate control structure */
 		GMT_end (argc, argv);
-	}
-
-	/* Read data */
-
-	if (subset && header.node_offset) {	/* Possibly extend the subset by 1 row/col since pixel tiles need 4 corners to work */
-		BOOLEAN global;
-		global = (GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (header.x_max, header.x_min));
-		if (global && data_west > project_info.w)
-			data_west -= header.x_inc;
-		else if (GMT_x_to_i (data_west, header.x_min, header.x_inc, header.xy_off, header.nx) > 0)
-			data_west -= header.x_inc;
-		if (global && data_east < project_info.e)
-			data_east += header.x_inc;
-		else if (GMT_x_to_i (data_east, header.x_min, header.x_inc, header.xy_off, header.nx) < (header.nx)) {
-			data_east += header.x_inc;
-			if (data_east > header.x_max) data_east -= header.x_inc;
-		}
-		if (GMT_y_to_j (data_south, header.y_min, header.y_inc, header.xy_off, header.ny) < (header.ny)) data_south -= header.y_inc;
-		if (GMT_y_to_j (data_north, header.y_min, header.y_inc, header.xy_off, header.ny) > 0) data_north += header.y_inc;
-		if (data_south < header.y_min) data_south += header.y_inc;
+		exit (EXIT_SUCCESS);
 	}
 
 	nx = GMT_get_n (data_west, data_east, header.x_inc, header.node_offset);
 	ny = GMT_get_n (data_south, data_north, header.y_inc, header.node_offset);
-	nm = nx * ny;
+	nm = GMT_get_nm (nx, ny);
+
+	/* Read data */
+	
 	grd_original = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 	GMT_err_fail (GMT_read_grd (grdfile, &header, grd_original, data_west, data_east, data_south, data_north, GMT_pad, FALSE), grdfile);
 
-	n_edges = header.ny * (int )ceil (header.nx / 16.0);
-	edge = (int *) GMT_memory (VNULL, (size_t)n_edges, sizeof (int), GMT_program);
-
 	if (!(Ctrl->Z.scale == 1.0 && Ctrl->Z.offset == 0.0)) {
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Subtracting %g and multiplying grid by %g\n", GMT_program, Ctrl->Z.offset, Ctrl->Z.scale);
 		for (i = 0; i < nm; i++) grd_original[i] = (float)((grd_original[i] - Ctrl->Z.offset) * Ctrl->Z.scale);
@@ -518,17 +504,19 @@ int main (int argc, char **argv)
 	}
 	if (Ctrl->L.low > header.z_min) header.z_min = Ctrl->L.low;
 	if (Ctrl->L.high < header.z_max) header.z_max = Ctrl->L.high;
-
+	if (Ctrl->L.active && header.z_max < header.z_min) {	/* Specified contour range outside range of data - quit */
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: No contours within specified -L range\n", GMT_program);
+		GMT_free ((void *)grd_original);
+		Free_Grdcontour_Ctrl (Ctrl);	/* Deallocate control structure */
+		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
+	}
+	
 	small = Ctrl->C.interval * 1.0e-6;
 	if (Ctrl->A.interval == 0.0) Ctrl->A.interval = Ctrl->C.interval;
 
 	small_x = 0.01 * header.x_inc;	small_y = 0.01 * header.y_inc;
 
-	GMT_plotinit (argc, argv);
-	if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
-
-	GMT_map_clip_on (GMT_no_rgb, 3);
-
 	if (Ctrl->contour.annot) {	/* Wants annotated contours */
 		aval = floor (header.z_min / Ctrl->A.interval) * Ctrl->A.interval;
 		if (aval < header.z_min) aval += Ctrl->A.interval;
@@ -536,12 +524,6 @@ int main (int argc, char **argv)
 	else
 		aval = header.z_max + 1.0;
 
-	n_alloc = GMT_CHUNK;
-	contour = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-	cont_type = (char *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (char), GMT_program);
-	cont_do_tick = (char *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (char), GMT_program);
-	cont_angle = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-
 	if (Ctrl->C.cpt) {	/* Presumably got a cpt-file */
 		GMT_read_cpt (Ctrl->C.file);
 #ifdef GMT_CPT2	
@@ -551,12 +533,8 @@ int main (int argc, char **argv)
 		}
 #endif
 		n_contours = GMT_n_colors + 1;
-		if (n_contours > n_alloc) {
-			contour = (double *) GMT_memory ((void *)contour, (size_t)n_contours, sizeof (double), GMT_program);
-			cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_contours, sizeof (char), GMT_program);
-			cont_do_tick = (char *) GMT_memory ((void *)cont_do_tick, (size_t)n_contours, sizeof (char), GMT_program);
-			cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_contours, sizeof (double), GMT_program);
-		}
+		(void)GMT_alloc_memory2 ((void **)&contour, (void **)&cont_angle, n_contours, 0, sizeof (double), GMT_program);
+		n_alloc = GMT_alloc_memory2 ((void **)&cont_type, (void **)&cont_do_tick, n_contours, 0, sizeof (char), GMT_program);
 		for (i = c = 0; i < GMT_n_colors; i++) {
 			if (GMT_lut[i].skip) continue;
 			contour[c] = GMT_lut[i].z_low;
@@ -591,19 +569,16 @@ int main (int argc, char **argv)
 		}
 		while (GMT_fgets (line, BUFSIZ, fpc)) {
 			if (line[0] == '#') continue;
+			if (n_contours == n_alloc) {
+				(void)GMT_alloc_memory2 ((void **)&contour, (void **)&cont_angle, n_contours, n_alloc, sizeof (double), GMT_program);
+				n_alloc = GMT_alloc_memory2 ((void **)&cont_type, (void **)&cont_do_tick, n_contours, n_alloc, sizeof (char), GMT_program);
+			}
 			got = sscanf (line, "%lf %c %lf", &contour[n_contours], &cont_type[n_contours], &tmp);
 			if (cont_type[n_contours] == 0) cont_type[n_contours] = 'C';
 			cont_do_tick[n_contours] = (Ctrl->T.active && ((cont_type[n_contours] == 'C') || (cont_type[n_contours] == 'A'))) ? 1 : 0;
 			cont_angle[n_contours] = (got == 3) ? tmp : GMT_d_NaN;
 			if (got == 3) Ctrl->contour.angle_type = 2;	/* Must set this directly if angles are provided */
 			n_contours++;
-			if (n_contours == n_alloc) {
-				n_alloc += GMT_CHUNK;
-				contour = (double *) GMT_memory ((void *)contour, (size_t)n_alloc, sizeof (double), GMT_program);
-				cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_alloc, sizeof (char), GMT_program);
-				cont_do_tick = (char *) GMT_memory ((void *)cont_do_tick, (size_t)n_alloc, sizeof (char), GMT_program);
-				cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_alloc, sizeof (double), GMT_program);
-			}
 		}
 		GMT_fclose (fpc);
 	}
@@ -612,10 +587,8 @@ int main (int argc, char **argv)
 		max = ceil (header.z_max / Ctrl->C.interval) * Ctrl->C.interval; if (max > header.z_max) max -= Ctrl->C.interval;
 		for (c = irint (min/Ctrl->C.interval), n_contours = 0; c <= irint (max/Ctrl->C.interval); c++, n_contours++) {
 			if (n_contours == n_alloc) {
-				n_alloc += GMT_CHUNK;
-				contour = (double *) GMT_memory ((void *)contour, (size_t)n_alloc, sizeof (double), GMT_program);
-				cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_alloc, sizeof (char), GMT_program);
-				cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_alloc, sizeof (double), GMT_program);
+				(void)GMT_alloc_memory2 ((void **)&contour, (void **)&cont_angle, n_contours, n_alloc, sizeof (double), GMT_program);
+				n_alloc = GMT_alloc_memory2 ((void **)&cont_type, (void **)&cont_do_tick, n_contours, n_alloc, sizeof (char), GMT_program);
 			}
 			contour[n_contours] = c * Ctrl->C.interval;
 			if (Ctrl->contour.annot && (contour[n_contours] - aval) > GMT_SMALL) aval += Ctrl->A.interval;
@@ -624,17 +597,30 @@ int main (int argc, char **argv)
 			cont_do_tick[n_contours] = Ctrl->T.active;
 		}
 	}
-
-	if (GMT_z_periodic && fabs (contour[n_contours-1] - contour[0] - 360.0) < GMT_SMALL) {	/* Get rid of redundant contour */
+	if (GMT_z_periodic && n_contours > 1 && fabs (contour[n_contours-1] - contour[0] - 360.0) < GMT_SMALL) {	/* Get rid of redundant contour */
 		n_contours--;
 	}
 
-	contour = (double *) GMT_memory ((void *)contour, (size_t)n_contours, sizeof (double), GMT_program);
-	cont_type = (char *) GMT_memory ((void *)cont_type, (size_t)n_contours, sizeof (char), GMT_program);
-	cont_do_tick = (char *) GMT_memory ((void *)cont_do_tick, (size_t)n_contours, sizeof (char), GMT_program);
-	cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_contours, sizeof (double), GMT_program);
+	if (n_contours == 0) {	/* No contours within range of data */
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: No contours found\n", GMT_program);
+		GMT_plotinit (argc, argv);
+		GMT_map_basemap ();
+		GMT_plotend ();
+		GMT_free ((void *)contour);
+		GMT_free ((void *)cont_type);
+		GMT_free ((void *)cont_angle);
+		GMT_free ((void *)cont_do_tick);
+		GMT_free ((void *)grd_original);
+		Free_Grdcontour_Ctrl (Ctrl);	/* Deallocate control structure */
+		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
+	}
+	
+	/* OK, now we know we have contouring to do */
+	
+	(void)GMT_alloc_memory2 ((void **)&contour, (void **)&cont_angle, 0, n_contours, sizeof (double), GMT_program);
+	n_alloc = GMT_alloc_memory2 ((void **)&cont_type, (void **)&cont_do_tick, 0, n_contours, sizeof (char), GMT_program);
 
-	if (Ctrl->T.active) save = (struct SAVE *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct SAVE), GMT_program);
 	GMT_grd_minmax (grd_original, &header, xyz);
 	if (GMT_contlabel_prep (&Ctrl->contour, xyz)) exit (EXIT_FAILURE);	/* Prep for crossing lines, if any */
 
@@ -644,6 +630,14 @@ int main (int argc, char **argv)
 	 * original grid values and subtract the current contour value. */
 	 
 	grd = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
+	n_edges = header.ny * (int )ceil (header.nx / 16.0);
+	edge = (GMT_LONG *) GMT_memory (VNULL, (size_t)n_edges, sizeof (GMT_LONG), GMT_program);
+
+
+	GMT_plotinit (argc, argv);
+	if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
+
+	GMT_map_clip_on (GMT_no_rgb, 3);
 
 	for (c = 0; c < n_contours; c++) {	/* For each contour value cval */
 
@@ -660,7 +654,6 @@ int main (int argc, char **argv)
 		}
 
 		section = 0;
-		first = FALSE;
 		id = (cont_type[c] == 'A' || cont_type[c] == 'a') ? 1 : 0;
 
 		Ctrl->contour.line_pen = Ctrl->W.pen[id];	/* Load current pen into contour structure */
@@ -672,6 +665,7 @@ int main (int argc, char **argv)
 
 		side = 0;
 		begin = TRUE;
+		n_alloc = 0;
 		while (side <= GMT_LAST_CONTOUR_SIDE) {
 			while ((n = GMT_contours (grd, &header, Ctrl->S.value, gmtdefs.interpolant, Ctrl->F.value, &side, edge, begin, &x, &y)) > 0) {
 
@@ -693,8 +687,8 @@ int main (int argc, char **argv)
 						else
 							cont_label[0] = (char)0;
 						if (cont_do_tick[c] && interior) {
-							save[n_save].x = (double *) GMT_memory (VNULL, (size_t)nn, sizeof (double), GMT_program);
-							save[n_save].y = (double *) GMT_memory (VNULL, (size_t)nn, sizeof (double), GMT_program);
+							if (n_save == n_alloc) n_alloc = GMT_alloc_memory ((void **)&save, n_save, n_alloc, sizeof (struct SAVE), GMT_program);
+							(void)GMT_alloc_memory2 ((void **)&save[n_save].x, (void **)&save[n_save].y, nn, 0, sizeof (double), GMT_program);
 							memcpy ((void *)save[n_save].x, (void *)xp, (size_t)(nn * sizeof (double)));
 							memcpy ((void *)save[n_save].y, (void *)yp, (size_t)(nn * sizeof (double)));
 							save[n_save].n = nn;
@@ -702,10 +696,6 @@ int main (int argc, char **argv)
 							save[n_save].do_it = TRUE;
 							save[n_save].cval = cval;
 							n_save++;
-							if (n_save == n_alloc) {
-								n_alloc += GMT_CHUNK;
-								save = (struct SAVE *) GMT_memory ((void *)save, (size_t)n_alloc, sizeof (struct SAVE), GMT_program);
-							}
 						}
 						GMT_hold_contour (&xp, &yp, nn, cval, cont_label, cont_type[c], cont_angle[c], interior, &Ctrl->contour);
 						GMT_free ((void *)xp);
@@ -724,7 +714,8 @@ int main (int argc, char **argv)
 	if (Ctrl->W.pen[0].texture || Ctrl->W.pen[1].texture) ps_setdash (CNULL, 0);
 
 	if (Ctrl->T.active && n_save) {
-		save = (struct SAVE *) GMT_memory ((void *)save, (size_t)n_save, sizeof (struct SAVE), GMT_program);
+		(void)GMT_alloc_memory ((void **)&save, 0, n_save, sizeof (struct SAVE), GMT_program);
+
 		sort_and_plot_ticks (save, n_save, &header, grd_original, Ctrl->T.spacing, Ctrl->T.length, Ctrl->T.low, Ctrl->T.high, Ctrl->T.label, Ctrl->T.L_label, Ctrl->T.H_label);
 		for (i = 0; i < n_save; i++) {
 			GMT_free ((void *)save[i].x);
@@ -737,6 +728,7 @@ int main (int argc, char **argv)
 	if (Ctrl->contour.hill_label) adjust_hill_label (&Ctrl->contour, &header, grd_original);	/* Must possibly adjust label angles so that label is readable when following contours */
 	
 	GMT_contlabel_plot (&Ctrl->contour);
+	GMT_contlabel_free (&Ctrl->contour);
 
 	GMT_map_clip_off ();
 
@@ -765,9 +757,9 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float *grd, double tick_gap, double tick_length, BOOLEAN tick_low, BOOLEAN tick_high, BOOLEAN tick_label, char Llabel, char Hlabel)
+void sort_and_plot_ticks (struct SAVE *save, GMT_LONG n, struct GRD_HEADER *h, float *grd, double tick_gap, double tick_length, BOOLEAN tick_low, BOOLEAN tick_high, BOOLEAN tick_label, char Llabel, char Hlabel)
 {
-	int np, i, j, inside, ix, iy, iy_s, done, n_ticks;
+	GMT_LONG np, i, j, inside, ix, iy, iy_s, done, n_ticks;
 	double x0, y0, add, sign, dx, dy, x_back, y_back, x_front, y_front;
 	double xmin, xmax, ymin, ymax, x_mean = 0.0, y_mean = 0.0, inc, dist, a, small, this_lon, this_lat, sa, ca, *s;
 	char txt[2][2];
@@ -783,7 +775,7 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 	for (i = 0; i < n; i++) {	/* Mark polygons that have other polygons inside them */
 		np = save[i].n;
 		for (j = 0; save[i].do_it && j < n; j++) {
-			inside = GMT_non_zero_winding (save[j].x[0], save[j].y[0], save[i].x, save[i].y, np);
+			inside = GMT_non_zero_winding (save[j].x[0], save[j].y[0], save[i].x, save[i].y, (GMT_LONG)np);
 			if (inside == 2) save[i].do_it = FALSE;
 		}
 	}
@@ -793,7 +785,6 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 	for (i = 0; i < n; i++) {
 		if (!save[i].do_it) continue;
 		np = save[i].n;
-		s = (double *) GMT_memory (VNULL, (size_t)np, sizeof (double), GMT_program);
 
 		/* Get rectangular surrounding box for this polygon */
 
@@ -811,16 +802,18 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 		 */
 
 		GMT_xy_to_geo (&this_lon, &this_lat, 0.5 * (xmin + xmax), ymax);
+		if (GMT_is_dnan (this_lon) || GMT_is_dnan (this_lat)) continue;
 		ix = GMT_x_to_i (this_lon, h->x_min, h->x_inc, h->xy_off, h->nx);
 		iy = GMT_y_to_j (this_lat, h->y_min, h->y_inc, h->xy_off, h->ny);
 		this_lon = GMT_i_to_x (ix, h->x_min, h->x_max, h->x_inc, h->xy_off, h->nx);	/* This is that mid longitude */
 		GMT_xy_to_geo (&a, &this_lat, 0.5 * (xmin + xmax), ymin);			/* Get bottom latitude */
+		if (GMT_is_dnan (a)) continue;
 		iy_s = MIN (h->ny, GMT_y_to_j (this_lat, h->y_min, h->y_inc, h->xy_off, h->ny) + 1);	/* And its j-row */
 		done = FALSE;
 		while (!done && iy < iy_s) {
 			this_lat = GMT_j_to_y (iy, h->y_min, h->y_max, h->y_inc, h->xy_off, h->ny);
 			GMT_geo_to_xy (this_lon, this_lat, &x0, &y0);
-			inside = GMT_non_zero_winding (x0, y0, save[i].x, save[i].y, np);
+			inside = GMT_non_zero_winding (x0, y0, save[i].x, save[i].y, (GMT_LONG)np);
 			if (inside == 2)	/* OK, this point is inside */
 				done = TRUE;
 			else
@@ -829,13 +822,14 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 		if (iy == h->ny) iy--;	/* Check to make sure we don't exceed array bounds in next line. However, iy may thus be wrong */ 
 		save[i].high = (grd[iy*h->nx+ix] > save[i].cval);
 
-		if (save[i].high && !tick_high) continue;	/* Dont tick highs */
-		if (!save[i].high && !tick_low) continue;	/* Dont tick lows */
+		if (save[i].high && !tick_high) continue;	/* Do not tick highs */
+		if (!save[i].high && !tick_low) continue;	/* Do not tick lows */
 
 		if (tick_label) {
 			x_mean = save[i].x[0];
 			y_mean = save[i].y[0];
 		}
+		s = (double *) GMT_memory (VNULL, (size_t)np, sizeof (double), GMT_program);
 		for (j = 1, s[0] = 0.0; j < np; j++) {
 			s[j] = s[j-1] + hypot (save[i].x[j]-save[i].x[j-1], save[i].y[j]-save[i].y[j-1]);
 			if (tick_label) {
@@ -853,7 +847,7 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 		sincos (a, &sa, &ca);
 		x0 = 0.5 * (save[i].x[0] + save[i].x[1]) + small * ca;
 		y0 = 0.5 * (save[i].y[0] + save[i].y[1]) + small * sa;
-		inside = GMT_non_zero_winding (x0, y0, save[i].x, save[i].y, np);
+		inside = GMT_non_zero_winding (x0, y0, save[i].x, save[i].y, (GMT_LONG)np);
 		sign = (inside == 2) ? 1.0 : -1.0;
 
 		n_ticks = (int)(s[np-1] / tick_gap);
@@ -904,7 +898,7 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 
 void GMT_grd_minmax (float *a, struct GRD_HEADER *h, double xyz[2][3])
 {
-	int i, i_min, i_max;
+	GMT_LONG i, i_min, i_max;
 	float z_min, z_max;
 	double half;
 
@@ -931,7 +925,7 @@ void GMT_grd_minmax (float *a, struct GRD_HEADER *h, double xyz[2][3])
 
 void adjust_hill_label (struct GMT_CONTOUR *G, struct GRD_HEADER *h, float *z)
 {	
-	int i, k, i0, j0;
+	GMT_LONG i, k, i0, j0;
 	double nx, ny, x_on, y_on, x_node, y_node, x_node_p, y_node_p, dx, dy, dz, dot, angle;
 	struct GMT_CONTOUR_LINE *C;
 	
@@ -972,7 +966,7 @@ void adjust_hill_label (struct GMT_CONTOUR *G, struct GRD_HEADER *h, float *z)
 void *New_Grdcontour_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDCONTOUR_CTRL *C;
 	
-	C = (struct GRDCONTOUR_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDCONTOUR_CTRL), "New_Grdcontour_Ctrl");
+	C = (struct GRDCONTOUR_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDCONTOUR_CTRL), "New_Grdcontour_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
