@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdfilter.c,v 1.57 2007/09/30 04:16:48 guru Exp $
+ *	$Id: grdfilter.c,v 1.72 2009/06/30 17:50:37 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -47,14 +47,14 @@
 struct GRDFILTER_CTRL {
 	struct D {	/* -D<distflag> */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 	} D;
 	struct F {	/* <type>[-]<filter_width>[<mode>] */
 		BOOLEAN active;
 		BOOLEAN highpass;
 		char filter;	/* Character codes for the filter */
 		double width;
-		int mode;
+		GMT_LONG mode;
 	} F;
 	struct G {	/* -G<file> */
 		BOOLEAN active;
@@ -64,12 +64,16 @@ struct GRDFILTER_CTRL {
 		BOOLEAN active;
 		double xinc, yinc;
 	} I;
+	struct N {	/* -Np|i|r */
+		BOOLEAN active;
+		GMT_LONG mode;	/* 0 is default (i), 1 is replace (r), 2 is preserve (p) */
+	} N;
 	struct T {	/* -T */
 		BOOLEAN active;
 	} T;
 };
 
-#define IMG2LAT(img) (2.0*atan(exp((img)*D2R))*R2D - 90.0)
+#define IMG2LAT(img) (2.0*atand(exp((img)*D2R))-90.0)
 #define LAT2IMG(lat) (R2D*log(tand(0.5*((lat)+90.0))))
 
 #define GRDFILTER_WIDTH		0
@@ -80,12 +84,16 @@ struct GRDFILTER_CTRL {
 
 #define GRDFILTER_N_FILTERS	9
 
+#define NAN_IGNORE	0
+#define NAN_REPLACE	1
+#define NAN_PRESERVE	2
+
 struct FILTER_INFO {
-	int nx;			/* The max number of filter weights in x-direction */
-	int ny;			/* The max number of filter weights in y-direction */
-	int x_half_width;	/* Number of filter nodes to either side needed at this latitude */
-	int y_half_width;	/* Number of filter nodes above/below this point (ny_f/2) */
-	int d_flag;
+	GMT_LONG nx;			/* The max number of filter weights in x-direction */
+	GMT_LONG ny;			/* The max number of filter weights in y-direction */
+	GMT_LONG x_half_width;	/* Number of filter nodes to either side needed at this latitude */
+	GMT_LONG y_half_width;	/* Number of filter nodes above/below this point (ny_f/2) */
+	GMT_LONG d_flag;
 	double dx, dy;		/* Grid spacing in original units */
 	double *x, *y;		/* Distances in original units along x and y to distance nodes */
 	double par[5];		/* [0] is filter width, [1] is 0.5*filter_width, [2] is xscale, [3] is yscale, [4] is 1/r_half for filter */
@@ -95,13 +103,15 @@ struct FILTER_INFO {
 
 int main (int argc, char **argv)
 {
-	int	nx_out, ny_out, n_in_median, n_nan = 0, i_west, i_east, j_s_pole, j_n_pole;
-	int	j_origin, i_out, j_out, half_nx, i_orig = 0;
-	int	i_in, j_in, ii, jj, i, j, ij_in, ij_out, ij_wt, effort_level;
-	int	filter_type, one_or_zero = 1, GMT_n_multiples = 0, *i_origin;
+	GMT_LONG	nx_out, ny_out, n_in_median, n_nan = 0, i_west, i_east, j_s_pole, j_n_pole;
+	GMT_LONG	j_origin, i_out, j_out, half_nx, i_orig = 0, n_bad = 0, nm;
+	GMT_LONG	i_in, j_in, ii, jj, i, j, ij_in, ij_out, ij_wt, effort_level;
+	GMT_LONG	filter_type, one_or_zero = 1, GMT_n_multiples = 0;
+	
+	GMT_LONG *i_origin;
 
 
-	BOOLEAN	error, new_range, fast_way, slow = FALSE;
+	BOOLEAN	error, new_range, fast_way, slow = FALSE, same_grid = FALSE;
 	BOOLEAN wrap_case_x = FALSE, wrap_case_y = FALSE, pole_check = FALSE;
 	BOOLEAN full_360, full_180, duplicate_check, i_west_used, i_east_used, row_ok;
 
@@ -184,8 +194,27 @@ int main (int argc, char **argv)
 						error = TRUE;
 					}
 					break;
-				case 'N':	/* Pixel registration OBSOLETE but BACKWARD COMPATIBLE */
-					one_or_zero = 0;
+				case 'N':
+					if (!argv[i][2]) {	/* Pixel registration OBSOLETE but BACKWARD COMPATIBLE for now */
+						one_or_zero = 0;
+					}
+					else {	/* Treatment of NaNs */
+						Ctrl->N.active = TRUE;
+						switch (argv[i][2]) {
+							case 'i':
+								Ctrl->N.mode = NAN_IGNORE;	/* Default */
+								break;
+							case 'r':
+								Ctrl->N.mode = NAN_REPLACE;	/* Replace */
+								break;
+							case 'p':
+								Ctrl->N.mode = NAN_PRESERVE;	/* Preserve */
+								break;
+							default:
+								fprintf (stderr, "%s: GMT SYNTAX ERROR -N option.  Correct syntax: -Ni|p|r\n", GMT_program);
+								break;
+						}
+					}
 					break;
 				case 'T':	/* Toggle registration */
 					Ctrl->T.active = TRUE;
@@ -203,7 +232,7 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "grdfilter %s - Filter a 2-D grid file in the space (or time) domain\n\n", GMT_VERSION);
 		fprintf(stderr,"usage: grdfilter input_file -D<distance_flag> -F<type>[-]<filter_width>[<mode>]\n");
-		fprintf(stderr,"\t-G<output_file> [%s] [%s] [-T] [-V] [%s]\n", GMT_I_OPT, GMT_Rgeo_OPT, GMT_f_OPT);
+		fprintf(stderr,"\t-G<output_file> [%s] [-Ni|p|r] [%s] [-T] [-V] [%s]\n", GMT_I_OPT, GMT_Rgeo_OPT, GMT_f_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -237,6 +266,11 @@ int main (int argc, char **argv)
 		fprintf(stderr, "\n\tOPTIONS:\n");
 		GMT_inc_syntax ('I', 0);
 		fprintf(stderr,"\t   The new xinc and yinc should be divisible by the old ones (new lattice is subset of old).\n");
+		fprintf(stderr, "\t-N specifies how NaNs in the input grid should be treated.  There are three options:\n");
+		fprintf(stderr, "\t   -Ni skips all NaN values and returns a filtered value unless all are NaN [Default]\n");
+		fprintf(stderr, "\t   -Np sets filtered output to NaN is any NaNs are found inside filter circle.\n");
+		fprintf(stderr, "\t   -Nr sets filtered output to NaN if the corresponding input node was NaN.\n");
+		fprintf(stderr, "\t      (only possible if the input and output grids are coregistered).\n");
 		fprintf(stderr, "\t-T Toggles between grid and pixel registration for output grid [Default is same as input registration]\n");
 		fprintf(stderr, "\t-R for new Range of output grid; enter <WESN> (xmin, xmax, ymin, ymax) separated by slashes.\n");
 		GMT_explain_option ('V');
@@ -245,6 +279,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, NULL, &Ctrl->I.active);
+
 	if (!Ctrl->G.file) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option:  Must specify output file\n", GMT_program);
 		error++;
@@ -280,7 +316,7 @@ int main (int argc, char **argv)
 	
 	if (error) exit (EXIT_FAILURE);
 
-	/* Assing filter_type number */
+	/* Assign filter_type number */
 	
 	for (filter_type = 0; filter_type < GRDFILTER_N_FILTERS && filter_code[filter_type] != Ctrl->F.filter; filter_type++);
 	
@@ -297,9 +333,13 @@ int main (int argc, char **argv)
 
 	/* Read the input grid file and close it  */
 
-	input = (float *) GMT_memory (VNULL, (size_t)(h.nx * h.ny), sizeof(float), GMT_program);
+	nm = GMT_get_nm (h.nx, h.ny);
+	input = (float *) GMT_memory (VNULL, (size_t)nm, sizeof(float), GMT_program);
 
 	GMT_err_fail (GMT_read_grd (fin, &h, input, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), fin);
+	
+	full_360 = (Ctrl->D.mode && GMT_360_RANGE (h.x_max, h.x_min));	/* Periodic geographic grid */
+	full_180 = (Ctrl->D.mode && GMT_180_RANGE (h.y_min, h.y_max));	/* Full latitude range for geographic grid */
 
 	last_median = 0.5 * (h.z_min + h.z_max);
 
@@ -316,8 +356,10 @@ int main (int argc, char **argv)
 		Ctrl->I.yinc = h.y_inc;
 	}
 
-	if (west_new < h.x_min) error = TRUE;
-	if (east_new > h.x_max) error = TRUE;
+	if (!full_360) {
+		if (west_new < h.x_min) error = TRUE;
+		if (east_new > h.x_max) error = TRUE;
+	}
 	if (south_new < h.y_min) error = TRUE;
 	if (north_new > h.y_max) error = TRUE;
 	if (Ctrl->I.xinc <= 0.0) error = TRUE;
@@ -337,13 +379,12 @@ int main (int argc, char **argv)
 	/* Copy back in case test_h were changed */
 	west_new  = test_h.x_min;	east_new  = test_h.x_max;	Ctrl->I.xinc = test_h.x_inc;
 	south_new = test_h.y_min;	north_new = test_h.y_max;	Ctrl->I.yinc = test_h.y_inc;
-	
 
 	/* We can save time by computing a weight matrix once [or once pr scanline] only
 	   if new grid spacing is a multiple of old spacing */
 
 	fast_way = (fabs (fmod (Ctrl->I.xinc / h.x_inc, 1.0)) < GMT_SMALL && fabs (fmod (Ctrl->I.yinc / h.y_inc, 1.0)) < GMT_SMALL);
-
+	same_grid = !(new_range  || Ctrl->I.active || h.node_offset == one_or_zero);
 	if (!fast_way && gmtdefs.verbose) {
 		fprintf (stderr, "%s: Warning - Your output grid spacing is such that filter-weights must\n", GMT_program);
 		fprintf (stderr, "be recomputed for every output node, so expect this run to be slow.  Calculations\n");
@@ -351,12 +392,16 @@ int main (int argc, char **argv)
 		fprintf (stderr, "of the input grid spacing.  If the odd output grid is necessary, consider using\n");
 		fprintf (stderr, "a \'fast\' grid for filtering and then resample onto your desired grid with grdsample.\n");
 	}
-
+	if (Ctrl->N.mode == NAN_REPLACE && !same_grid) {
+		fprintf (stderr, "%s: Warning: -Nr requires co-registered input/output grids, option is ignored\n", GMT_program);
+		Ctrl->N.mode = NAN_IGNORE;
+	}
 	nx_out = one_or_zero + irint ( (east_new - west_new) / Ctrl->I.xinc);
 	ny_out = one_or_zero + irint ( (north_new - south_new) / Ctrl->I.yinc);
 
-	output = (float *) GMT_memory (VNULL, (size_t)(nx_out*ny_out), sizeof(float), GMT_program);
-	i_origin = (int *) GMT_memory (VNULL, (size_t)nx_out, sizeof(int), GMT_program);
+	nm = GMT_get_nm (nx_out, ny_out);
+	output = (float *) GMT_memory (VNULL, (size_t)nm, sizeof(float), GMT_program);
+	i_origin = (GMT_LONG *) GMT_memory (VNULL, (size_t)nx_out, sizeof(GMT_LONG), GMT_program);
 	if (!fast_way) x_shift = (double *) GMT_memory (VNULL, (size_t)nx_out, sizeof(double), GMT_program);
 
 	xincnew2 = (one_or_zero) ? 0.0 : 0.5 * Ctrl->I.xinc;
@@ -368,8 +413,6 @@ int main (int argc, char **argv)
 		x_fix = 0.5 * h.x_inc;
 		y_fix = 0.5 * h.y_inc;
 	}
-	full_360 = (Ctrl->D.mode && GMT_360_RANGE (h.x_max, h.x_min));	/* Periodic geographic grid */
-	full_180 = (Ctrl->D.mode && GMT_180_RANGE (h.y_min, h.y_max));	/* Full latitude range for geographic grid */
 
 	/* Set up the distance scalings for lon and lat, and assign pointer to distance function  */
 
@@ -400,11 +443,10 @@ int main (int argc, char **argv)
 			if (full_180) wrap_case_y = wrap_case_x;	/* For spherical filtering */
 			break;
 		case 5:	/* Great circle distances with Mercator coordinates */
-			x_scale = 0.0;
 			/* Get the max |lat| extent of the grid */
 			max_lat = IMG2LAT (MAX (fabs (h.y_min), fabs (h.y_max)));
 			merc_range = LAT2IMG (max_lat + (0.5 * Ctrl->F.width / project_info.KM_PR_DEG)) - LAT2IMG (max_lat);
-			y_scale = 0.5 * Ctrl->F.width / merc_range;
+			x_scale = y_scale = 0.5 * Ctrl->F.width / merc_range;
 			F.radius_func = SphericalRadius;
 			if (full_360) wrap_case_x = TRUE;		/* For spherical filtering */
 			break;
@@ -435,8 +477,8 @@ int main (int argc, char **argv)
 	F.dy = h.y_inc;
 	x_width = Ctrl->F.width / (h.x_inc * x_scale);
 	y_width = Ctrl->F.width / (h.y_inc * y_scale);
-	F.y_half_width = (int) (ceil(y_width) / 2.0);
-	F.x_half_width = (int) (ceil(x_width) / 2.0);
+	F.y_half_width = (GMT_LONG) (ceil(y_width) / 2.0);
+	F.x_half_width = (GMT_LONG) (ceil(x_width) / 2.0);
 	F.nx = 2 * F.x_half_width + 1;
 	F.ny = 2 * F.y_half_width + 1;
 	if (x_scale == 0.0 || F.nx < 0 || F.nx > h.nx) {	/* Safety valve when x_scale -> 0.0 */
@@ -460,7 +502,7 @@ int main (int argc, char **argv)
 	}
 
 	if (gmtdefs.verbose) {
-		fprintf(stderr,"%s: Input nx,ny = (%d %d), output nx,ny = (%d %d), filter nx,ny = (%d %d)\n", GMT_program, h.nx, h.ny, nx_out, ny_out, F.nx, F.ny);
+		fprintf(stderr,"%s: Input nx,ny = (%d %d), output nx,ny = (%ld %ld), filter nx,ny = (%ld %ld)\n", GMT_program, h.nx, h.ny, nx_out, ny_out, F.nx, F.ny);
 		fprintf(stderr,"%s: Filter type is %s.\n", GMT_program, filter_name[filter_type]);
 	}
 
@@ -489,7 +531,7 @@ int main (int argc, char **argv)
 	half_nx = (h.node_offset) ? h.nx / 2 : (h.nx - 1) / 2;
 	pole_weight = 0.25 * M_PI * (h.y_inc / h.x_inc);	/* This is the weight of the pole point for gridregistered grids */
 	pole_check = (Ctrl->D.mode && !h.node_offset);		/* Must check to make sure we only use the N and S pole value once */
-	duplicate_check = (full_360 && !h.node_offset);		/* Must avoid using the duplicated Greenwhich node twice */
+	duplicate_check = (full_360 && !h.node_offset);		/* Must avoid using the duplicated Greenwich node twice */
 	j_s_pole = (h.y_min == -90.0) ? h.ny - 1 : INT_MIN;	/* row number of S pole (if in the grid)*/
 	j_n_pole = (h.y_max == 90.0) ? 0 : INT_MIN;		/* row number of N pole (if in the grid)*/
 	i_west = 0;		/* column number of western edge */
@@ -497,11 +539,11 @@ int main (int argc, char **argv)
 	
 	for (j_out = 0; j_out < ny_out; j_out++) {
 
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Processing output line %d\r", GMT_program, j_out);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Processing output line %ld\r", GMT_program, j_out);
 		y_out = north_new - j_out * Ctrl->I.yinc - yincnew2;
 		lat_out = (Ctrl->D.mode == 5) ? IMG2LAT (y_out) : y_out;
 		j_origin = GMT_y_to_j (y_out, h.y_min, h.y_inc, h.xy_off, h.ny);
-		if (Ctrl->D.mode == 3) par[GRDFILTER_X_SCALE] = project_info.KM_PR_DEG * cos (D2R * lat_out);	/* Update flat-earth longitude scale */
+		if (Ctrl->D.mode == 3) par[GRDFILTER_X_SCALE] = project_info.KM_PR_DEG * cosd (lat_out);	/* Update flat-earth longitude scale */
 
 		if (Ctrl->D.mode > 2) {	/* Update max filterweight nodes to deal with at this latitude */
 			y = fabs (lat_out);
@@ -516,8 +558,9 @@ int main (int argc, char **argv)
 
 			if (effort_level == 3) set_weight_matrix (&F, weight, y_out, par, x_shift[i_out], y_shift);
 			wt_sum = value = 0.0;
-			n_in_median = 0;
+			n_in_median = n_bad = 0;
 			ij_out = j_out * nx_out + i_out;
+			if (Ctrl->N.mode == NAN_REPLACE && GMT_is_fnan (input[ij_out])) continue;	/* Since output will be NaN we bypass the filter loop */
 
 			/* Now loop over the filter domain and collect those points that should be considered by the filter operation */
 			
@@ -541,7 +584,10 @@ int main (int argc, char **argv)
 					if (pole_check && (j_in == j_n_pole || j_in == j_s_pole)) {	/* N or S pole on gridline grid */
 						row_ok = FALSE;	/* Only use one point from this row since we are at a pole */
 						ij_in = j_in*h.nx;	/* All points on pole rows are the same; pick the first one */
-						if (GMT_is_fnan (input[ij_in])) continue;	/* Whaddaya know; the pole is NaN */
+						if (GMT_is_fnan (input[ij_in])) {	/* Whaddaya know; the pole is NaN */
+							n_bad++;
+							continue;
+						}
 						/* Get here when pole value is usable  */
 						if (slow) {
 							work_array[n_in_median] = input[ij_in];
@@ -567,7 +613,10 @@ int main (int argc, char **argv)
 					if (weight[ij_wt] <= 0.0) continue;
 
 					ij_in = j_in*h.nx + i_in;
-					if (GMT_is_fnan (input[ij_in])) continue;
+					if (GMT_is_fnan (input[ij_in])) {
+						n_bad++;
+						continue;
+					}
 
 					/* Get here when point is usable  */
 					if (slow) {
@@ -583,7 +632,15 @@ int main (int argc, char **argv)
 
 			/* Now we have done the convolution and we can get the value  */
 
-			if (slow) {
+			if (Ctrl->N.mode == NAN_REPLACE && GMT_is_fnan (input[ij_out])) {
+				output[ij_out] = GMT_f_NaN;
+				n_nan++;
+			}
+			else if (Ctrl->N.mode == NAN_PRESERVE && n_bad) {	/* -Np in effect and there were NaNs inside circle */
+				output[ij_out] = GMT_f_NaN;
+				n_nan++;
+			}
+			else if (slow) {
 				if (n_in_median) {
 					switch (filter_type) {
 						case 3:	/* Median */
@@ -615,8 +672,8 @@ int main (int argc, char **argv)
 			}
 			else {
 				if (wt_sum == 0.0) {	/* Assign value = GMT_f_NaN */
-					n_nan++;
 					output[ij_out] = GMT_f_NaN;
+					n_nan++;
 				}
 				else
 					output[ij_out] = (float)(value / wt_sum);
@@ -632,8 +689,8 @@ int main (int argc, char **argv)
 	
 	/* At last, that's it!  Output: */
 
-	if (n_nan && gmtdefs.verbose) fprintf (stderr, "%s: Unable to estimate value at %d nodes, set to NaN\n", GMT_program, n_nan);
-	if (GMT_n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %d multiple modes found\n", GMT_program, GMT_n_multiples);
+	if (n_nan && gmtdefs.verbose) fprintf (stderr, "%s: Unable to estimate value at %ld nodes, set to NaN\n", GMT_program, n_nan);
+	if (GMT_n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %ld multiple modes found\n", GMT_program, GMT_n_multiples);
 
 	h.nx = nx_out;
 	h.ny = ny_out;
@@ -671,7 +728,7 @@ void set_weight_matrix (struct FILTER_INFO *F, double *weight, double output_lat
 	 * par[4] is the normalization distance needed for the Cosine-bell or Gaussian weight function.
 	 */
 
-	int	i, j, ij;
+	GMT_LONG	i, j, ij;
 	double	x, y, yc, y0, x_jump = 0.0, normalize = 1.0, r, dy = 1.0, dy_half = 0.5, dy_shrink = 1.0;
 
 	yc = y0 = output_lat - y_off;
@@ -757,7 +814,7 @@ double GaussianWeight (double r, double par[])
 void *New_Grdfilter_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDFILTER_CTRL *C;
 	
-	C = (struct GRDFILTER_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDFILTER_CTRL), "New_Grdfilter_Ctrl");
+	C = (struct GRDFILTER_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDFILTER_CTRL), "New_Grdfilter_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->D.mode = -1;	
