@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdgradient.c,v 1.35 2007/09/30 04:16:48 guru Exp $
+ *	$Id: grdgradient.c,v 1.46 2009/05/13 21:06:42 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -55,13 +55,13 @@ struct GRDGRADIENT_CTRL {
 	} A;
 	struct D {	/* -D[a][o][n] */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 	} D;
 	struct E {	/* -E[s|p]<azim>/<elev[ambient/diffuse/specular/shine]> */
 		BOOLEAN active;
 		double azimuth, elevation;
 		double ambient, diffuse, specular, shine;
-		int mode;
+		GMT_LONG mode;
 	} E;
 	struct G {	/* -G<file> */
 		BOOLEAN active;
@@ -76,7 +76,7 @@ struct GRDGRADIENT_CTRL {
 	} M;
 	struct N {	/* -N[t_or_e][<amp>[/<sigma>[/<offset>]]] */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 		double norm, sigma, offset;
 	} N;
 	struct S {	/* -S<slopefile> */
@@ -91,7 +91,9 @@ int main (int argc, char **argv)
 	
 	BOOLEAN	error = FALSE, sigma_set = FALSE, offset_set = FALSE, bad;
 	
-	int	i, j, ij, k, n, nm, nm2, mx, my, p[4], n_used = 0, pos, entry;
+	GMT_LONG pos, p[4], entry, mx, my;
+	
+	GMT_LONG	i, j, ij, k, n, nm, nm2, n_used = 0;
 	
 	float *data, *slp = VNULL;
 
@@ -164,7 +166,7 @@ int main (int argc, char **argv)
 						case 'p':	/* Peucker */
 							Ctrl->E.mode = 1;
 							break;
-						case 's':	/* "simple" Lmbertian case */
+						case 's':	/* "simple" Lambertian case */
 							Ctrl->E.mode = 2;
 							if (sscanf(&argv[i][3], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation) != 2) {
 								fprintf(stderr,"%s: GMT SYNTAX ERROR -Es option: Must append azimuth/elevation\n", GMT_program);
@@ -209,7 +211,7 @@ int main (int argc, char **argv)
 					break;
 				case 'L':
 					Ctrl->L.active = TRUE;
-					strncpy (Ctrl->L.mode, &argv[i][2], 4);
+					strncpy (Ctrl->L.mode, &argv[i][2], (size_t)4);
 					break;
 				case 'M':
 					Ctrl->M.active = TRUE;
@@ -267,7 +269,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t    Specify '=' to get the default value (e.g. -E60/30/=/0.5)\n");
 		fprintf (stderr, "\t   Append s to use a simpler Lambertian algorithm (note that with this form\n");
 		fprintf (stderr, "\t   you only have to provide the azimuth and elevation parameters)\n");
-		fprintf (stderr, "\t   Append p to use the Peucker picewise linear aproximation (simpler but faster algorithm)\n");
+		fprintf (stderr, "\t   Append p to use the Peucker piecewise linear approximation (simpler but faster algorithm)\n");
 		fprintf (stderr, "\t   Note that in this case the azimuth and elevation are hardwired to 315 and 45 degrees.\n");
 		fprintf (stderr, "\t   This means that even if you provide other values they will be ignored.\n");
 		fprintf (stderr, "\t-G output file for results from -A or -D\n");
@@ -344,24 +346,24 @@ int main (int argc, char **argv)
 		while (Ctrl->E.azimuth > 360.0) Ctrl->E.azimuth -= 360.0;
 	}
 	if (Ctrl->E.mode == 2) {
-		p0 = cos((90.0 - Ctrl->E.azimuth)*D2R) * tan((90.0 - Ctrl->E.elevation)*D2R);
-		q0 = sin((90.0 - Ctrl->E.azimuth)*D2R) * tan((90.0 - Ctrl->E.elevation)*D2R);
+		p0 = cosd(90.0 - Ctrl->E.azimuth) * tand(90.0 - Ctrl->E.elevation);
+		q0 = sind(90.0 - Ctrl->E.azimuth) * tand(90.0 - Ctrl->E.elevation);
 		p0q0_cte = sqrt(1 + p0*p0 + q0*q0);
 	}
 	if (Ctrl->E.mode == 3) {
 		Ctrl->E.elevation = 90 - Ctrl->E.elevation;
-		s[0] = sin(Ctrl->E.azimuth*D2R) * cos(Ctrl->E.elevation*D2R);
-		s[1] = cos(Ctrl->E.azimuth*D2R) * cos(Ctrl->E.elevation*D2R);
-		s[2] =  sin(Ctrl->E.elevation*D2R);
+		s[0] = sind(Ctrl->E.azimuth) * cosd(Ctrl->E.elevation);
+		s[1] = cosd(Ctrl->E.azimuth) * cosd(Ctrl->E.elevation);
+		s[2] = sind(Ctrl->E.elevation);
 		k_ads = Ctrl->E.ambient + Ctrl->E.diffuse + Ctrl->E.specular;
 	}
 	GMT_boundcond_param_prep (&header, &edgeinfo);
 
 	GMT_grd_init (&header, argc, argv, TRUE);
-	nm = header.nx * header.ny;
+	nm = GMT_get_nm (header.nx, header.ny);
 	mx = header.nx + 4;
 	my = header.ny + 4;
-	nm2 = mx * my;
+	nm2 = GMT_get_nm (mx, my);
 	data = (float *) GMT_memory (VNULL, (size_t)nm2, sizeof (float), GMT_program);
 	GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
 	if (Ctrl->S.active) slp = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
@@ -409,7 +411,7 @@ int main (int argc, char **argv)
 	for (j = k = 0; j < header.ny; j++) {
 		if (Ctrl->M.active) {
 			lat = GMT_j_to_y (j, header.y_min, header.y_max, header.y_inc, 0.5 * header.node_offset, header.ny);
-			dx_grid = project_info.M_PR_DEG * header.x_inc * cos (D2R * lat);
+			dx_grid = project_info.M_PR_DEG * header.x_inc * cosd (lat);
 			if (dx_grid > 0.0) x_factor = -1.0 / (2.0 * dx_grid);	/* Use previous value at the poles */
 			if (Ctrl->A.active) {
 				if (Ctrl->A.two) {
@@ -423,6 +425,7 @@ int main (int argc, char **argv)
 			for (n = 0, bad = FALSE; !bad && n < 4; n++) if (GMT_is_fnan (data[ij+p[n]])) bad = TRUE;
 			if (bad) {	/* One of corners = NaN, skip */
 				data[k] = GMT_f_NaN;
+				if (Ctrl->S.active) slp[k] = GMT_f_NaN;
 				continue;
 			}
 
@@ -449,7 +452,7 @@ int main (int argc, char **argv)
 				max_gradient = MAX (max_gradient, data[k]);
 			}
 			else if (Ctrl->D.active) {
-				azim = (Ctrl->D.mode & 1) ? atan2 (-dzdy, -dzdx) * R2D : 90.0 - atan2 (-dzdy, -dzdx) * R2D;
+				azim = (Ctrl->D.mode & 1) ? atan2d (-dzdy, -dzdx) : 90.0 - atan2d (-dzdy, -dzdx);
 				if (Ctrl->D.mode & 4) azim += 90.0;
 				if (azim < 0.0) azim += 360.0;
 				if (azim >= 360.0) azim -= 360.0;
@@ -478,6 +481,21 @@ int main (int argc, char **argv)
 		}
 	}
 
+	if (Ctrl->M.active || GMT_io.in_col_type[GMT_Y] == GMT_IS_LAT) {	/* Data is geographic */
+		double sum;
+		/* If the N or S poles are included then we only want a single estimate at these repeating points */
+		if (header.y_min == -90.0 && header.node_offset == 0) {	/* Average all the multiple N pole estimates */
+			for (k = 0, sum = 0.0; k < header.nx; k++) sum += data[k];
+			sum /= header.nx;	/* Average gradient */
+			for (k = 0; k < header.nx; k++) data[k] = (float)sum;
+		}
+		if (header.y_min == -90.0 && header.node_offset == 0) {	/* Average all the multiple S pole estimates */
+			for (i = 0, k = header.nx * (header.ny - 1) - 1, sum = 0.0; i < header.nx; i++, k++) sum += data[k];
+			sum /= header.nx;	/* Average gradient */
+			for (i = 0, k = header.nx * (header.ny - 1) - 1; i < header.nx; i++, k++) data[k] = (float)sum;
+		}
+	}
+	
 	if (Ctrl->E.active) {	/* data must be scaled to the [-1,1] interval, but we'll do it into [-.95, .95] to not get too bright */
 		scale = 1.0 / (r_max - r_min);
 		for (k = 0; k < nm; k++) {
@@ -592,9 +610,9 @@ double specular (double nx, double ny, double nz, double *s) {
 	   to the light source and to the viewer, respectively. 
 	   For the time beeing I'm using V = [azim elev] = [0 90] so the following
 
-	   V[0] =  sin(V[0]*D2R)*cos(V[1]*D2R);
-	   V[1] = -cos(V[0]*D2R)*cos(V[1]*D2R);
-	   V[2] =  sin(V[1]*D2R);
+	   V[0] =  sind(V[0])*cosd(V[1]);
+	   V[1] = -cosd(V[0])*cosd(V[1]);
+	   V[2] =  sind(V[1]);
 
 	   Reduces to V[0] = 0;		V[1] = 0;	V[2] = 1 */
 
@@ -606,7 +624,7 @@ double specular (double nx, double ny, double nz, double *s) {
 void *New_Grdgradient_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDGRADIENT_CTRL *C;
 	
-	C = (struct GRDGRADIENT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDGRADIENT_CTRL), "New_Grdgradient_Ctrl");
+	C = (struct GRDGRADIENT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDGRADIENT_CTRL), "New_Grdgradient_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.ambient = 0.55;
