@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdimage.c,v 1.72 2007/09/30 04:16:48 guru Exp $
+ *	$Id: grdimage.c,v 1.88 2009/07/08 21:41:40 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -39,7 +39,7 @@ struct GRDIMAGE_CTRL {
 	struct E {	/* -Ei|<dpi> */
 		BOOLEAN active;
 		BOOLEAN device_dpi;
-		int dpi;
+		GMT_LONG dpi;
 	} E;
 	struct G {	/* -G[f|b]<rgb> */
 		BOOLEAN active;
@@ -53,29 +53,32 @@ struct GRDIMAGE_CTRL {
 	struct M {	/* -M */
 		BOOLEAN active;
 	} M;
+	struct N {	/* -N */
+		BOOLEAN active;
+	} N;
 	struct Q {	/* -Q */
 		BOOLEAN active;
 	} Q;
 	struct S {	/* -S[-]b|c|l|n[/<threshold>] */
 		BOOLEAN active;
 		BOOLEAN antialias;
-		int interpolant;
+		GMT_LONG interpolant;
 		double threshold;
 	} S;
 };
 
 int main (int argc, char **argv)
 {
-	BOOLEAN error = FALSE, do_rgb = FALSE, need_to_project, normal_x, normal_y, resampled = FALSE;
+	BOOLEAN error = FALSE, do_rgb = FALSE, done, need_to_project, normal_x, normal_y, resampled = FALSE;
 
-	unsigned char *bitimage_8 = NULL, *bitimage_24 = NULL;
+	unsigned char *bitimage_8 = NULL, *bitimage_24 = NULL, *rgb_used = NULL;
 
 	char *grdfile[3], *c_method[2] = {"colorimage", "colortiles",};
 
-	int i, j, k, kk, rgb[3], nm, nm2 = 0, byte, nx_f, ny_f, grid_type = 0;
-	int nx, ny, nx_proj = 0, ny_proj = 0, node, n_grids = 0, index;
-	int PS_colormask_off = 0, PS_colormask, PS_interpolate;
-
+	GMT_LONG i, j, k, byte, nx_f, ny_f, nx, ny, nx_proj = 0, ny_proj = 0, index, grid_type = 0, n_grids = 0;
+	GMT_LONG PS_colormask_off = 0, PS_colormask, PS_interpolate, nm, nm2 = 0, node, kk;
+	int rgb[3], try;
+	
 	float *tmp1[3], *tmp2 = NULL, *map[3], *intensity = NULL;
 
 	double  dx, dy, x_side, y_side, x0 = 0.0, y0 = 0.0;
@@ -85,7 +88,6 @@ int main (int argc, char **argv)
 	struct GMT_EDGEINFO edgeinfo;
 	struct GRDIMAGE_CTRL *Ctrl;
 
-	void GMT_limits (struct GRD_HEADER *r, struct GRD_HEADER *g);
 	void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g);
 	void *New_Grdimage_Ctrl (), Free_Grdimage_Ctrl (struct GRDIMAGE_CTRL *C);
 
@@ -170,6 +172,9 @@ int main (int argc, char **argv)
 				case 'M':
 					Ctrl->M.active = TRUE;
 					break;
+				case 'N':
+					Ctrl->N.active = TRUE;
+					break;
 				case '0':	/* Backwards compatibility */
 					gmtdefs.color_image = 0;
 					break;
@@ -232,7 +237,7 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"grdimage %s - Plot grid files in 2-D\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdimage <grd_z|grd_r grd_g grd_b> %s [%s] [-C<cpt_file>] [-Ei|<dpi>] [-G[f|b]<rgb>]\n", GMT_J_OPT, GMT_B_OPT);
-		fprintf (stderr, "\t[-I<intensity_file>] [-K] [-M] [-O] [-P] [-Q] [%s] [-S[-]b|c|l|n[/<threshold>]] [-T]\n", GMT_Rgeo_OPT);
+		fprintf (stderr, "\t[-I<intensity_file>] [-K] [-M] [-N] [-O] [-P] [-Q] [%s] [-S[-]b|c|l|n[/<threshold>]] [-T]\n", GMT_Rgeo_OPT);
 		fprintf (stderr, "\t[%s] [-V] [%s] [%s] [%s]\n\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -251,9 +256,10 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-I use illumination.  Append name of intensity grid file\n");
 		GMT_explain_option ('K');
 		fprintf (stderr, "\t-M force monochrome image\n");
+		fprintf (stderr, "\t-N Do not clip image at the map boundary\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr, "\t-Q use PS Level 3 colormasking for nodes with z = NaN.\n");
+		fprintf (stderr, "\t-Q use PS Level 3 colormasking to make nodes with z = NaN transparent.\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\t-S Determines the interpolation mode (b = B-spline, c = bicubic, l = bilinear,\n");
 		fprintf (stderr, "\t   n = nearest-neighbor) [Default: bicubic]\n");
@@ -307,7 +313,6 @@ int main (int argc, char **argv)
 	PS_interpolate = (Ctrl->E.device_dpi) ? -1 : +1;
 	PS_colormask   = (Ctrl->Q.active) ? -1 : +1;;
 
-
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Allocates memory and read data file\n", GMT_program);
 
 	for (i = 0; i < n_grids; i++) {
@@ -339,36 +344,47 @@ int main (int argc, char **argv)
 	}
 
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
+	
+	/* Determine if grid is to be projected */
+
+	need_to_project = (project_info.projection != GMT_LINEAR || project_info.xyz_projection[0] == GMT_LOG10 || project_info.xyz_projection[0] == GMT_POW || project_info.xyz_projection[1] == GMT_LOG10 || project_info.xyz_projection[1] == GMT_POW || Ctrl->E.dpi > 0);
 
 	/* Determine the wesn to be used to read the grid file; or exit if file is outside -R */
 
-	if (GMT_grd_setregion (&g_head[0], &data_west, &data_east, &data_south, &data_north)) {	/* No grid to plot; just do empty map and exit */
+	if (GMT_grd_setregion (&g_head[0], &data_west, &data_east, &data_south, &data_north, need_to_project * Ctrl->S.interpolant)) {
+		/* No grid to plot; just do empty map and exit */
 		GMT_plotinit (argc, argv);
 		GMT_map_basemap ();
 		GMT_plotend ();
+		Free_Grdimage_Ctrl (Ctrl);	/* Deallocate control structure */
 		GMT_end (argc, argv);
+		exit (EXIT_SUCCESS);
 	}
 
 	nx_f = g_head[0].nx;
 	ny_f = g_head[0].ny;
 
-	/* Read data */
-
 	nx = GMT_get_n (data_west, data_east, g_head[0].x_inc, g_head[0].node_offset);
 	ny = GMT_get_n (data_south, data_north, g_head[0].y_inc, g_head[0].node_offset);
+#if 0
+	fprintf (stderr, "%g %g %g %g / %g %g\n", g_head[0].x_min, g_head[0].x_max, g_head[0].y_min, g_head[0].y_max, g_head[0].x_inc, g_head[0].y_inc);
+	fprintf (stderr, "%g %g %g %g\n", data_west, data_east, data_south, data_north);
+	fprintf (stderr, "%li %li\n", nx, ny);
+#endif
 
 	GMT_boundcond_init (&edgeinfo);
 
-	need_to_project = (project_info.projection != GMT_LINEAR || project_info.xyz_projection[0] == GMT_LOG10 || project_info.xyz_projection[0] == GMT_POW || project_info.xyz_projection[1] == GMT_LOG10 || project_info.xyz_projection[1] == GMT_POW);
-	if (Ctrl->E.dpi > 0 || need_to_project) {
-		nm = (4 + nx) * (4 + ny);
+	if (need_to_project) {
+		nm = GMT_get_nm (4 + nx, 4 + ny);
 		GMT_pad[0] = GMT_pad[1] = GMT_pad[2] = GMT_pad[3] = 2;
 	}
 	else
-		nm = nx * ny;
+		nm = GMT_get_nm (nx, ny);
+
+	/* Read data */
 
 	for (i = 0; i < n_grids; i++) {
-		tmp1[i] = (float *) GMT_memory (VNULL, (size_t)(nm), sizeof (float), GMT_program);
+		tmp1[i] = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
 		GMT_err_fail (GMT_read_grd (grdfile[i], &g_head[i], tmp1[i], data_west, data_east, data_south, data_north, GMT_pad, FALSE), grdfile[i]);
 	}
 
@@ -394,7 +410,7 @@ int main (int argc, char **argv)
 
 	for (i = 0; i < n_grids; i++) GMT_set_proj_limits (&r_head[i], &g_head[i]);
 
-	if (Ctrl->E.dpi > 0 || need_to_project) {	/* Need to resample the grd file */
+	if (need_to_project) {	/* Need to resample the grd file */
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: project grid files\n", GMT_program);
 
@@ -405,8 +421,8 @@ int main (int argc, char **argv)
 		for (i = 0; i < n_grids; i++) {
 			grid_type = (Ctrl->E.dpi > 0) ? 1 : g_head[i].node_offset;	/* Force pixel if dpi is set */
 			GMT_err_fail (GMT_grdproject_init (&r_head[i], 0.0, 0.0, nx_proj, ny_proj, Ctrl->E.dpi, grid_type), grdfile[i]);
-			nm2 = r_head[i].nx * r_head[i].ny;
-			map[i] = (float *) GMT_memory (VNULL, (size_t)nm2, sizeof (float), "grdimage");
+			nm2 = ((size_t)r_head[i].nx) * ((size_t)r_head[i].ny);
+			map[i] = (float *) GMT_memory (VNULL, nm2, sizeof (float), "grdimage");
 			GMT_grd_project (tmp1[i], &g_head[i], map[i], &r_head[i], &edgeinfo, Ctrl->S.antialias, Ctrl->S.interpolant, Ctrl->S.threshold, FALSE);
 			GMT_free ((void *)tmp1[i]);
 		}
@@ -419,7 +435,7 @@ int main (int argc, char **argv)
 				ny_proj = i_head.ny;
 			}
 			GMT_err_fail (GMT_grdproject_init (&j_head, 0.0, 0.0, nx_proj, ny_proj, Ctrl->E.dpi, grid_type), Ctrl->I.file);
-			intensity = (float *) GMT_memory (VNULL, (size_t)nm2, sizeof (float), "grdimage");
+			intensity = (float *) GMT_memory (VNULL, nm2, sizeof (float), "grdimage");
 			GMT_grd_project (tmp2, &i_head, intensity, &j_head, &edgeinfo, Ctrl->S.antialias, Ctrl->S.interpolant, Ctrl->S.threshold, FALSE);
 			GMT_free ((void *)tmp2);
 		}
@@ -442,13 +458,14 @@ int main (int argc, char **argv)
 
 	GMT_plotinit (argc, argv);
 
-	GMT_map_clip_on (GMT_no_rgb, 3);
+	if (!Ctrl->N.active) GMT_map_clip_on (GMT_no_rgb, 3);
 
 	if (gmtdefs.verbose) {
 		if (GMT_cpt_pattern) fprintf (stderr, "%s: Warning: Patterns in cpt file only apply to -T\n", GMT_program);
 		fprintf (stderr, "%s: Evaluate pixel colors\n", GMT_program);
 	}
 
+	if (Ctrl->Q.active) rgb_used = (unsigned char *) GMT_memory (VNULL, (size_t)(256*256*256), sizeof (unsigned char), GMT_program);
 	if (Ctrl->M.active || GMT_gray)
 		bitimage_8 = (unsigned char *) GMT_memory (VNULL, (size_t)nm, sizeof (char), GMT_program);
 	else {
@@ -462,39 +479,67 @@ int main (int argc, char **argv)
 	}
 	normal_x = !(project_info.projection == GMT_LINEAR && !project_info.xyz_pos[0] && !resampled);
 	normal_y = !(project_info.projection == GMT_LINEAR && !project_info.xyz_pos[1] && !resampled);
-	for (j = 0, byte = PS_colormask_off; j < r_head[0].ny; j++) {
-		kk = r_head[0].nx * (normal_y ? j : r_head[0].ny - j - 1);
-		for (i = 0; i < r_head[0].nx; i++) {	/* Compute rgb for each pixel */
-			node = kk + (normal_x ? i : r_head[0].nx - i - 1);
-			if (do_rgb) {
-				for (k = 0; k < 3; k++) {
-					if (GMT_is_fnan (map[k][node])) {	/* If one is NaN they are all assumed to be NaN */
-						k = 3;
-						memcpy ((void *)rgb, (void *)GMT_bfn[GMT_NAN].rgb, 3 * sizeof (int));
-						index = -1;	/* Ensures no illumination done later */
-					}
-					else {				/* Set color, let index = 0 so illuminate test will work */
-						rgb[k] = irint (map[k][node]);	if (rgb[k] < 0) rgb[k] = 0; else if (rgb[k] > 255) rgb[k] = 255;	/* Clip */
-						index = 0;
+	
+	for (try = 0, done = FALSE; !done && try < 2; try++) {	/* Evaluate colors at least once, or twice if -Q and we need to select another NaN color */
+		for (j = 0, byte = PS_colormask_off; j < r_head[0].ny; j++) {
+			kk = ((size_t)r_head[0].nx) * ((size_t)(normal_y ? j : r_head[0].ny - j - 1));
+			for (i = 0; i < r_head[0].nx; i++) {	/* Compute rgb for each pixel */
+				node = kk + (normal_x ? i : r_head[0].nx - i - 1);
+				if (do_rgb) {
+					for (k = 0; k < 3; k++) {
+						if (GMT_is_fnan (map[k][node])) {	/* If one is NaN they are all assumed to be NaN */
+							k = 3;
+							memcpy ((void *)rgb, (void *)GMT_bfn[GMT_NAN].rgb, 3 * sizeof (int));
+							index = -1;	/* Ensures no illumination done later */
+						}
+						else {				/* Set color, let index = 0 so illuminate test will work */
+							rgb[k] = irint (map[k][node]);	if (rgb[k] < 0) rgb[k] = 0; else if (rgb[k] > 255) rgb[k] = 255;	/* Clip */
+							index = 0;
+						}
 					}
 				}
-			}
-			else
-				index = GMT_get_rgb_from_z (map[0][node], rgb);
-			if (Ctrl->I.active && index != -1) GMT_illuminate (intensity[node], rgb);
+				else
+					index = GMT_get_rgb_from_z (map[0][node], rgb);
+				if (Ctrl->I.active && index != -1) GMT_illuminate (intensity[node], rgb);
 
-			if (GMT_gray)	/* Color table only has grays, pick r */
-				bitimage_8[byte++] = (unsigned char) rgb[0];
-			else if (Ctrl->M.active)	/* Convert rgb to gray using the GMT_YIQ transformation */
-				bitimage_8[byte++] = (unsigned char) GMT_YIQ (rgb);
-			else {
-				bitimage_24[byte++] = (unsigned char) rgb[0];
-				bitimage_24[byte++] = (unsigned char) rgb[1];
-				bitimage_24[byte++] = (unsigned char) rgb[2];
+				if (GMT_gray)	/* Color table only has grays, pick r */
+					bitimage_8[byte++] = (unsigned char) rgb[0];
+				else if (Ctrl->M.active)	/* Convert rgb to gray using the GMT_YIQ transformation */
+					bitimage_8[byte++] = (unsigned char) GMT_YIQ (rgb);
+				else {
+					bitimage_24[byte++] = (unsigned char) rgb[0];
+					bitimage_24[byte++] = (unsigned char) rgb[1];
+					bitimage_24[byte++] = (unsigned char) rgb[2];
+					if (Ctrl->Q.active && index != -1) rgb_used[(rgb[0]*256 + rgb[1])*256+rgb[2]] = TRUE;	/* Keep track of all r/g/b combinations used except for NaN */
+				}
 			}
 		}
-	}
 
+		if (Ctrl->Q.active) {	/* Check that colormasking will work OK */
+			index = (GMT_bfn[GMT_NAN].rgb[0]*256 + GMT_bfn[GMT_NAN].rgb[1])*256+GMT_bfn[GMT_NAN].rgb[2];
+			if (rgb_used[index]) {	/* This r/g/b also appears in the image as a non-NaN color; find a replacement color */
+				for (index = 0, k = -1; k == -1 && index < 256*256*256; index++) if (!rgb_used[index]) k = index;
+				if (k == -1) {
+					fprintf (stderr, "%s: Warning: Colormasking will fail as there is no unused color that can represent transparency\n", GMT_program);
+					done = TRUE;
+				}
+				else {	/* Pick the first unused color and let it play the role of the NaN color for transparency */
+					bitimage_24[0] = (unsigned char)(k >> 16);
+					bitimage_24[1] = (unsigned char)((k >> 8) & 255);
+					bitimage_24[2] = (unsigned char)(k & 255);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: transparency color reset from %d/%d/%d to color %d/%d/%d\n", GMT_program, \
+						GMT_bfn[GMT_NAN].rgb[0], GMT_bfn[GMT_NAN].rgb[1], GMT_bfn[GMT_NAN].rgb[2], (int)bitimage_24[0], (int)bitimage_24[1], (int)bitimage_24[2]);
+					GMT_bfn[GMT_NAN].rgb[0] = (int)bitimage_24[0];
+					GMT_bfn[GMT_NAN].rgb[1] = (int)bitimage_24[1];
+					GMT_bfn[GMT_NAN].rgb[2] = (int)bitimage_24[2];
+				}	
+			}
+		}
+		else
+			done = TRUE;
+	}
+	if (Ctrl->Q.active) GMT_free ((void *)rgb_used);
+	
 	for (i = 0; i < n_grids; i++) GMT_free ((void *)map[i]);
 	if (Ctrl->I.active) GMT_free ((void *)intensity);
 
@@ -516,15 +561,15 @@ int main (int argc, char **argv)
 
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Creating PostScript image ", GMT_program);
 
-	if (GMT_gray) for (k = 0, GMT_b_and_w = TRUE; GMT_b_and_w && k < nm; k++) if (!(bitimage_8[k] == 0 || bitimage_8[k] == 255)) GMT_b_and_w = FALSE;
+	if (GMT_gray) for (kk = 0, GMT_b_and_w = TRUE; GMT_b_and_w && kk < nm; kk++) if (!(bitimage_8[kk] == 0 || bitimage_8[kk] == 255)) GMT_b_and_w = FALSE;
 
 	if (GMT_b_and_w) {	/* Can get away with 1 bit image */
-		int nx8, shift, byte, b_or_w, k8, nx_pixels;
+		GMT_LONG nx8, shift, byte, b_or_w, nx_pixels, k8;
 		unsigned char *bit;
 
 		if (gmtdefs.verbose) fprintf (stderr, "[1-bit B/W image]\n");
 
-		nx8 = (int)ceil (r_head[0].nx / 8.0);
+		nx8 = (GMT_LONG)ceil (r_head[0].nx / 8.0);
 		nx_pixels = nx8 * 8;
 		bit = (unsigned char *) GMT_memory (VNULL, (size_t)(nx8 * r_head[0].ny), sizeof (char), GMT_program);
 
@@ -569,7 +614,7 @@ int main (int argc, char **argv)
 		GMT_free ((void *)bitimage_24);
 	}
 
-	GMT_map_clip_off();
+	if (!Ctrl->N.active) GMT_map_clip_off();
 
 	GMT_map_basemap ();
 
@@ -578,6 +623,7 @@ int main (int argc, char **argv)
 	Free_Grdimage_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
+	exit (EXIT_SUCCESS);
 
 	return(0);
 }
@@ -589,7 +635,7 @@ void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g)
 	 * nx/ny are set accordingly.  Not that some of these may change
 	 * if GMT_grdproject_init is called at a later stage */
 
-	int i, j;
+	GMT_LONG i, j;
 	BOOLEAN all_lats = FALSE, all_lons = FALSE;
 	double lon, lat, x, y;
 
@@ -603,17 +649,16 @@ void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g)
 		return;
 	}
 
-	if (GMT_IS_AZIMUTHAL) {	/* Applies only to Azimuthal projections */
+	if (GMT_IS_MAPPING) {
 		all_lats = GMT_180_RANGE (g->y_max, g->y_min);
 		all_lons = GMT_360_RANGE (g->x_max, g->x_min);
-
-		if (all_lons && all_lats) {	/* Whole globe, get square box */
+		if (all_lons && all_lats) {	/* Whole globe, get rectangular box */
 			r->x_min = project_info.xmin;	r->x_max = project_info.xmax;
 			r->y_min = project_info.ymin;	r->y_max = project_info.ymax;
 			return;
 		}
 	}
-
+	
 	/* Must search for extent along perimeter */
 
 	r->x_min = r->y_min = +DBL_MAX;
@@ -648,7 +693,7 @@ void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g)
 void *New_Grdimage_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDIMAGE_CTRL *C;
 
-	C = (struct GRDIMAGE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDIMAGE_CTRL), "New_Grdimage_Ctrl");
+	C = (struct GRDIMAGE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDIMAGE_CTRL), "New_Grdimage_Ctrl");
 
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 
