@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdmask.c,v 1.60 2007/09/30 04:16:48 guru Exp $
+ *	$Id: grdmask.c,v 1.76 2009/07/11 03:16:30 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -43,7 +43,7 @@
 struct GRDMASK_CTRL {
 	struct A {	/* -A[m|p|step] */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 		double step;
 	} A;
 	struct F {	/* -F */
@@ -72,16 +72,17 @@ int main (int argc, char **argv)
 {
 	BOOLEAN	error = FALSE, done, nofile = TRUE, periodic = FALSE, resample = FALSE;
 
-	char line[BUFSIZ], ptr[BUFSIZ];
+	char line[BUFSIZ], ptr[BUFSIZ], *not_used = NULL;
 
-	int	i, j, k, ij, nm, n_path, side, fno, n_files = 0, n_alloc = GMT_CHUNK, n_args;
-	int	one_or_zero, di, dj, i0, j0, n_fields, n_read, n_expected_fields, pos;
-	int 	distance_flag = 0, n_pol = 0;
+	GMT_LONG	i, j, side, fno, n_files = 0, n_args;
+	GMT_LONG	di, dj, i0, j0, n_fields, n_expected_fields, pos;
+	GMT_LONG 	distance_flag = 0, n_pol = 0;
+	GMT_LONG	k, ij, nm, n_path, n_read, n_alloc = GMT_CHUNK;
 
 	float *data;
 
 	double	*x, *y, xx, yy, xmin, xmax, ymin, ymax, lon_sum, lat_sum, dlon, step;
-	double xinc2, yinc2, distance, shrink = 1.0, idx, idy, x0, y0, *in;
+	double distance, shrink = 1.0, idx, idy, x0, y0, *in;
 	double xmin1, xmin2, xmax1, xmax2, lon_w;
 
 	FILE *fp;
@@ -107,11 +108,13 @@ int main (int argc, char **argv)
 				/* Common parameters */
 
 				case 'H':
+				case 'M':
 				case 'R':
 				case 'V':
 				case ':':
 				case 'b':
 				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &header.x_min, &header.x_max, &header.y_min, &header.y_max);
 					break;
@@ -155,9 +158,6 @@ int main (int argc, char **argv)
 					GMT_io.in_col_type[GMT_Y] = GMT_io.out_col_type[GMT_Y] = GMT_IS_LAT;
 					fprintf (stderr, "%s: Option -L is obsolete (but is processed correctly).  Please use -f instead\n", GMT_program);
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'S':
 					Ctrl->S.active = TRUE;
 					Ctrl->S.radius = GMT_getradius (&argv[i][2]);
@@ -177,7 +177,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "grdmask %s - Create mask grid file from polygons or point coverage\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdmask [<xyfiles>] -G<mask_grd_file> %s\n", GMT_I_OPT);
 		fprintf (stderr, "\t%s [-A[m|p]] [-F] [%s] [-N<out>/<edge>/<in>]\n", GMT_Rgeo_OPT, GMT_H_OPT);
-		fprintf (stderr, "\t[%s] [-S<radius>[m|c|k|K] [-V] [%s] [%s] [%s]\n\n", GMT_Mo_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
+		fprintf (stderr, "\t[-S<radius>[m|c|k|K] [-V] [%s] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -190,7 +190,6 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   unless m or p is appended to first follow meridian then parallel, or vice versa.\n");
 		fprintf (stderr, "\t-F Force pixel registration for output grid [Default is gridline orientation]\n");
 		GMT_explain_option ('H');
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N sets values to use if point is outside, on the path, or inside.\n");
 		fprintf (stderr, "\t   NaN is a valid entry.  Default values are 0/0/1.\n");
 		fprintf (stderr, "\t-S sets search radius in -R, -I units; append m or c for minutes or seconds.\n");
@@ -198,7 +197,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   whether they are inside a circle of specified radius [0] from the nearest data point.\n");
 		fprintf (stderr, "\t   Append k for km (implies -R,-I in degrees), use flat Earth approximation.\n");
 		fprintf (stderr, "\t   Append K for km (implies -R,-I in degrees), use exact geodesic distances.\n");
-		fprintf (stderr, "\t   If the current ELLIPSOID is Sphere then spherical great circle distances are used.\n");
+		fprintf (stderr, "\t   If the current ELLIPSOID is spherical then great circle distances are used.\n");
 		fprintf (stderr, "\t    [Default is to treat xyfiles as polygons and use inside/outside searching].\n");
 		GMT_explain_option ('V');
 		GMT_explain_option (':');
@@ -206,10 +205,13 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t    [Default is 2 input columns].\n");
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
@@ -239,7 +241,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (Ctrl->S.unit == 'k') distance_flag = 1;
@@ -252,12 +254,9 @@ int main (int argc, char **argv)
 	GMT_err_fail (GMT_grd_RI_verify (&header, 1), Ctrl->G.file);
 
 	header.xy_off = 0.5 * header.node_offset;
-	xinc2 = header.xy_off * header.x_inc;
-	yinc2 = header.xy_off * header.y_inc;
-	one_or_zero = 1 - header.node_offset;
 	idx = 1.0 / header.x_inc;
 	idy = 1.0 / header.y_inc;
-	nm = header.nx * header.ny;
+	nm = GMT_get_nm (header.nx, header.ny);
 	data = (float *) GMT_memory (VNULL, (size_t)nm, sizeof(float), GMT_program);
 	x = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(double), GMT_program);
 	y = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(double), GMT_program);
@@ -273,13 +272,15 @@ int main (int argc, char **argv)
 	}
 
 	if (distance_flag > 0) {
+		double width;
 		shrink = cosd (MAX(fabs(header.y_min), fabs(header.y_max)));
-		di = (shrink == 0.0) ? header.nx : (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.x_inc * shrink));
-		dj = (int)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.y_inc));
+		width = (shrink < GMT_CONV_LIMIT) ? DBL_MAX : ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.x_inc * shrink));
+		di = (width < (double)header.nx) ? (GMT_LONG) width : header.nx;
+		dj = (GMT_LONG)ceil (Ctrl->S.radius / (project_info.KM_PR_DEG * header.y_inc));
 	}
 	else {
-		di = (int)ceil (Ctrl->S.radius * idx);
-		dj = (int)ceil (Ctrl->S.radius * idy);
+		di = (GMT_LONG)ceil (Ctrl->S.radius * idx);
+		dj = (GMT_LONG)ceil (Ctrl->S.radius * idy);
 	}
 	periodic = (GMT_io.in_col_type[GMT_X] == GMT_IS_LON);	/* Dealing with geographic coordinates */
 	resample = (!Ctrl->A.active && periodic);
@@ -304,9 +305,9 @@ int main (int argc, char **argv)
 	}
 
 	memset ((void *)&P, 0, sizeof (struct GMT_LINE_SEGMENT));
-	P.coord = (double **) GMT_memory (VNULL, 2, sizeof (double *), GMT_program);	/* Needed as pointers below */
-	P.min = (double *) GMT_memory (VNULL, 2, sizeof (double), GMT_program);		/* Needed to hold min lon/lat */
-	P.max = (double *) GMT_memory (VNULL, 2, sizeof (double), GMT_program);		/* Needed to hold max lon/lat */
+	P.coord = (double **) GMT_memory (VNULL, (size_t)2, sizeof (double *), GMT_program);	/* Needed as pointers below */
+	P.min = (double *) GMT_memory (VNULL, (size_t)2, (size_t)sizeof (double), GMT_program);		/* Needed to hold min lon/lat */
+	P.max = (double *) GMT_memory (VNULL, (size_t)2, (size_t)sizeof (double), GMT_program);		/* Needed to hold max lon/lat */
 	
 	/* Initialize all nodes to the 'outside' value */
 
@@ -337,7 +338,7 @@ int main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		n_read = 0;
 		n_fields = GMT_input (fp,  &n_expected_fields, &in);
@@ -355,7 +356,7 @@ int main (int argc, char **argv)
 
 			while (!(GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE  */
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_read);
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_read);
 					exit (EXIT_FAILURE);
 				}
 				x[n_path] = in[GMT_X];
@@ -378,8 +379,8 @@ int main (int argc, char **argv)
 						dlon = x[n_path] - x[n_path-1];
 						if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
 						lon_sum += dlon;
-						lat_sum += y[n_path];
 					}
+					lat_sum += y[n_path];
 				}
 				else {	/* Cartesian x */
 					if (x[n_path] > xmax) xmax = x[n_path];
@@ -388,7 +389,7 @@ int main (int argc, char **argv)
 				n_path++;
 
 				if (n_path == (n_alloc-1)) {	/* n_alloc-1 since we may need 1 more to close polygon */
-					n_alloc += GMT_CHUNK;
+					n_alloc <<= 1;
 					x = (double *) GMT_memory ((void *)x, (size_t)n_alloc, sizeof(double), GMT_program);
 					y = (double *) GMT_memory ((void *)y, (size_t)n_alloc, sizeof(double), GMT_program);
 				}
@@ -438,7 +439,7 @@ int main (int argc, char **argv)
 						if (j < 0 || j >= header.ny) continue;
 						for (i = i0 - di; i <= (i0 + di); i++) {
 							if (i < 0 || i >= header.nx) continue;
-							ij = j * header.nx + i;
+							ij = GMT_IJ (j, i, header.nx);
 							x0 = GMT_i_to_x (i, header.x_min, header.x_max, header.x_inc, header.xy_off, header.nx);
 							y0 = GMT_j_to_y (j, header.y_min, header.y_max, header.y_inc, header.xy_off, header.ny);
 							distance = (GMT_distance_func) (x[k], y[k], x0, y0);
@@ -455,7 +456,6 @@ int main (int argc, char **argv)
 					x[n_path] = x[0];
 					y[n_path] = y[0];
 					dlon = x[n_path] - x[n_path-1];
-					lat_sum += y[n_path];
 					n_path++;
 					if (periodic) {
 						if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
@@ -515,10 +515,10 @@ int main (int argc, char **argv)
 
 						/* Here, point is inside or on edge, we must assign value */
 
-						ij = j*header.nx+i;
+						ij = GMT_IJ (j, i, header.nx);
 						data[ij] = (float)Ctrl->N.mask[side];
 					}
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Polygon %d scanning row %5.5d\r", GMT_program, n_pol, j);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Polygon %ld scanning row %5.5ld\r", GMT_program, n_pol, j);
 				}
 			}
 		}
@@ -544,7 +544,7 @@ int main (int argc, char **argv)
 void *New_Grdmask_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDMASK_CTRL *C;
 	
-	C = (struct GRDMASK_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDMASK_CTRL), "New_Grdmask_Ctrl");
+	C = (struct GRDMASK_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDMASK_CTRL), "New_Grdmask_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->A.step = gmtdefs.line_step;
