@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: grdpaste.c,v 1.25 2009/01/09 04:02:34 guru Exp $
+ *	$Id: grdpaste.c,v 1.26 2009/05/13 21:06:42 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -43,7 +43,7 @@ int main (int argc, char **argv)
 
 	char *grd_a, *grd_b, format[BUFSIZ];
 
-	int	i, way, one_or_zero, n_in = 0;
+	GMT_LONG	i, way, one_or_zero, n_in = 0;
 	
 	GMT_LONG nm;
 
