@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: grdreformat.c,v 1.47 2009/01/09 04:02:34 guru Exp $
+ *	$Id: grdreformat.c,v 1.50 2009/05/13 21:06:42 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -36,7 +36,7 @@ int main (int argc, char **argv)
 #include "grdreformat.h"	/* Defines N_GRDTXT_LINES and char *grd_formats[N_GRDTXT_LINES] array used in usage message */
 	BOOLEAN error = FALSE, global = FALSE;
 
-	int i, nfiles = 0, nx, ny, no_header, type[2];
+	GMT_LONG i, nfiles = 0, nx, ny, no_header, type[2];
 
 	size_t nm;
 	
@@ -132,7 +132,7 @@ int main (int argc, char **argv)
 	if (gmtdefs.verbose) {
 		if (file[0][0] == '=') strcpy (fname[0], "<stdin>");
 		if (file[1][0] == '=') strcpy (fname[1], "<stdout>");
-		fprintf (stderr, "%s: Translating file %s (format = %d) to file %s (format = %d)\n", GMT_program, fname[0], type[0], fname[1], type[1]);
+		fprintf (stderr, "%s: Translating file %s (format = %ld) to file %s (format = %ld)\n", GMT_program, fname[0], type[0], fname[1], type[1]);
 		if (no_header && GMT_grdformats[type[1]][0] != 'c' && GMT_grdformats[type[1]][0] != 'n') fprintf (stderr, "%s: No grd header will be written\n", GMT_program);
 	}
 
