@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: grdvolume.c,v 1.36 2007/10/07 03:26:15 guru Exp $
+ *	$Id: grdvolume.c,v 1.46 2009/05/13 21:06:43 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -51,8 +51,8 @@ struct GRDVOLUME_CTRL {
 	} Z;
 };
 
-double vol_prism_frac_x (float *z, size_t ij, size_t nx, double x0, double x1, double a, double b, double c, double d);
-double vol_prism_frac_y (float *z, size_t ij, size_t nx, double y0, double y1, double a, double b, double c, double d);
+double vol_prism_frac_x (float *z, GMT_LONG ij, GMT_LONG nx, double x0, double x1, double a, double b, double c, double d);
+double vol_prism_frac_y (float *z, GMT_LONG ij, GMT_LONG nx, double y0, double y1, double a, double b, double c, double d);
 
 int main (int argc, char **argv)
 {
@@ -60,11 +60,8 @@ int main (int argc, char **argv)
 
 	char *grdfile = CNULL, format[BUFSIZ];
 
-	int i, j, n = 0, c, k, pos, neg, nc, n_contours, mode = 0, nz = 0; 
-
-	size_t ij, nm, nx, ny;
-	
-	long ij_inc[4];
+	GMT_LONG i, j, n = 0, c, k, pos, neg, nc, n_contours, nx, ny, mode = 0, nz = 0; 
+	GMT_LONG ij, nm, ij_inc[4];
 	
 	float *f;
 
@@ -75,13 +72,13 @@ int main (int argc, char **argv)
 
 	struct GRD_HEADER grd;
 
-	void SW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
-	void NE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
-	void SE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
-	void NW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da);
-	void NS_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN right, double *dv, double *da);
-	void EW_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN top, double *dv, double *da);
-	int ors_find_kink (double y[], int n, int mode);
+	void SW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da);
+	void NE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da);
+	void SE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da);
+	void NW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da);
+	void NS_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN right, double *dv, double *da);
+	void EW_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN top, double *dv, double *da);
+	GMT_LONG ors_find_kink (double y[], GMT_LONG n, GMT_LONG mode);
 	void *New_Grdvolume_Ctrl (), Free_Grdvolume_Ctrl (struct GRDVOLUME_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -191,6 +188,7 @@ int main (int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
+	GMT_grd_init (&grd, argc, argv, FALSE);
 	GMT_err_fail (GMT_read_grd_info (grdfile, &grd), grdfile);
 
 	if (!project_info.region_supplied) {	/* No subset asked for */
@@ -202,9 +200,9 @@ int main (int argc, char **argv)
 	else if (!project_info.region)	/* Got w/s/e/n, make into w/e/s/n */
 		d_swap (south, east);
 
-	nx = (size_t)GMT_get_n (west, east, grd.x_inc, grd.node_offset);
-	ny = (size_t)GMT_get_n (south, north, grd.y_inc, grd.node_offset);
-	nm = nx * ny;
+	nx = GMT_get_n (west, east, grd.x_inc, grd.node_offset);
+	ny = GMT_get_n (south, north, grd.y_inc, grd.node_offset);
+	nm = GMT_get_nm (nx, ny);
 
 	f = (float *) GMT_memory (VNULL, nm, sizeof (float), GMT_program);
 
@@ -221,9 +219,9 @@ int main (int argc, char **argv)
 
 	n_contours = (Ctrl->C.active) ? irint ((Ctrl->C.high - Ctrl->C.low) / Ctrl->C.inc) + 1 : 1;
 
-	height = (double *) GMT_memory (VNULL, (size_t)n_contours, sizeof (double), GMT_program);
-	vol = (double *) GMT_memory (VNULL, (size_t)n_contours, sizeof (double), GMT_program);
-	area = (double *) GMT_memory (VNULL, (size_t)n_contours, sizeof (double), GMT_program);
+	height = (double *) GMT_memory (VNULL, n_contours, sizeof (double), GMT_program);
+	vol = (double *) GMT_memory (VNULL, n_contours, sizeof (double), GMT_program);
+	area = (double *) GMT_memory (VNULL, n_contours, sizeof (double), GMT_program);
 
 	if (!(Ctrl->Z.scale == 1.0 && Ctrl->Z.offset == 0.0)) {
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Subtracting %g and multiplying by %g\n", GMT_program, Ctrl->Z.offset, Ctrl->Z.scale);
@@ -388,14 +386,14 @@ int main (int argc, char **argv)
 	GMT_free ((void *)vol);
 	GMT_free ((void *)height);
 
-	GMT_end (argc, argv);
-
 	Free_Grdvolume_Ctrl (Ctrl);	/* Deallocate control structure */
 
+	GMT_end (argc, argv);
+
 	exit (EXIT_SUCCESS);
 }
 
-void SW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
+void SW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a SW-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x1, y0, frac;
@@ -413,7 +411,7 @@ void SW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv,
 	}
 }
 
-void NE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
+void NE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a NE-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x0, y1, a, x0_1, y1_1, frac = 0.0;
@@ -436,7 +434,7 @@ void NE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv,
 	}
 }
 
-void SE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
+void SE_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a SE-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x0, y1, c, x0_1, frac = 0.0;
@@ -458,7 +456,7 @@ void SE_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv,
 	}
 }
 
-void NW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv, double *da)
+void NW_triangle (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN triangle, double *dv, double *da)
 {	/* Calculates area of a NW-corner triangle */
 	/* triangle = TRUE gets triangle, FALSE gives the complementary area */
 	double x1, y0, y0_1, frac;
@@ -477,7 +475,7 @@ void NW_triangle (float f[], size_t ij, size_t nx, BOOLEAN triangle, double *dv,
 	}
 }
 
-void NS_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN right, double *dv, double *da)
+void NS_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN right, double *dv, double *da)
 {	/* Calculates area of a NS trapezoid */
 	/* right = TRUE gets the right trapezoid, FALSE gets the left */
 	double x0, x1;
@@ -494,7 +492,7 @@ void NS_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN right, double *dv, d
 	}
 }
 
-void EW_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN top, double *dv, double *da)
+void EW_trapezoid (float f[], GMT_LONG ij, GMT_LONG nx, BOOLEAN top, double *dv, double *da)
 {	/* Calculates area of a EW trapezoid */
 	/* top = TRUE gets the top trapezoid, FALSE gets the bottom */
 	double y0, y1;
@@ -515,7 +513,7 @@ void EW_trapezoid (float f[], size_t ij, size_t nx, BOOLEAN top, double *dv, dou
  * lines x0 and x1 and two horizontal lines y0 = ax +b and y1 = cx + d
  */
 
-double vol_prism_frac_x (float *z, size_t ij, size_t nx, double x0, double x1, double a, double b, double c, double d)
+double vol_prism_frac_x (float *z, GMT_LONG ij, GMT_LONG nx, double x0, double x1, double a, double b, double c, double d)
 {
 	double dzdx, dzdy, dzdxy, ca, db, c2a2, d2b2, cdab, v, x02, x12, x03, x04, x13, x14;
 
@@ -543,7 +541,7 @@ double vol_prism_frac_x (float *z, size_t ij, size_t nx, double x0, double x1, d
  * lines y0 and y1 and two vertical lines x0 = ay +b and x1 = cy + d
  */
 
-double vol_prism_frac_y (float *z, size_t ij, size_t nx, double y0, double y1, double a, double b, double c, double d)
+double vol_prism_frac_y (float *z, GMT_LONG ij, GMT_LONG nx, double y0, double y1, double a, double b, double c, double d)
 {
 	double dzdx, dzdy, dzdxy, ca, db, c2a2, d2b2, cdab, v, y02, y03, y04, y12, y13, y14;
 
@@ -567,9 +565,9 @@ double vol_prism_frac_y (float *z, size_t ij, size_t nx, double y0, double y1, d
 	return (v);
 }
 
-int ors_find_kink (double y[], int n, int mode)
+GMT_LONG ors_find_kink (double y[], GMT_LONG n, GMT_LONG mode)
 {	/* mode: 0 = find maximum, 1 = find curvature kink */
-	int i, ic, im;
+	GMT_LONG i, ic, im;
 	double *c, *f;
 	double median3 (double x[]);
 
@@ -581,7 +579,7 @@ int ors_find_kink (double y[], int n, int mode)
 
 	/* Calculate curvatures */
 
-	c = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
+	c = (double *) GMT_memory (VNULL, n, sizeof (double), GMT_program);
 
 	for (i = 1; i < (n-1); i++) c[i] = y[i+1] - 2.0 * y[i] + y[i-1];
 	c[0] = c[1];
@@ -589,7 +587,7 @@ int ors_find_kink (double y[], int n, int mode)
 
 	/* Apply 3-point median filter to curvatures  */
 
-	f = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
+	f = (double *) GMT_memory (VNULL, n, sizeof (double), GMT_program);
 	for (i = 1; i < (n-1); i++) f[i] = median3 (&c[i-1]);
 
 	/* Find maximum negative filtered curvature */
