@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: greenspline.c,v 1.35 2009/01/15 01:22:55 guru Exp $
+ *	$Id: greenspline.c,v 1.42 2009/07/11 03:16:30 guru Exp $
  *
  *	Copyright (c) 2008-2009 by P. Wessel
  *	See COPYING file for copying and redistribution conditions.
@@ -72,7 +72,7 @@ typedef double REAL;	/* Change this to float and recompile if you only want sing
 struct GREENSPLINE_CTRL {
 	struct A {	/* -A<gradientfile> */
 		BOOLEAN active;
-		int mode;	/* 0 = azimuths, 1 = directions, 2 = dx,dy components, 3 = dx, dy, dz components */
+		GMT_LONG mode;	/* 0 = azimuths, 1 = directions, 2 = dx,dy components, 3 = dx, dy, dz components */
 		char *file;
 	} A	;
 	struct C {	/* -C<cutoff> */
@@ -82,7 +82,7 @@ struct GREENSPLINE_CTRL {
 	} C;
 	struct D {	/* -D<distflag> */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 	} D;
 	struct F {	/* -F */
 		BOOLEAN active;
@@ -109,7 +109,7 @@ struct GREENSPLINE_CTRL {
 	} Q;
 	struct S {	/* -S<mode>[/args] */
 		BOOLEAN active, fast;
-		int mode;
+		GMT_LONG mode;
 		double value[2];
 		char *arg;
 	} S;
@@ -120,12 +120,12 @@ struct GREENSPLINE_CTRL {
 };
 
 struct ZGRID {
-	int nz;
+	GMT_LONG nz;
 	double z_min, z_max, z_inc;
 };
 
 #ifdef TEST
-void dump_green (PFD G, PFD D, double par[], double x0, double x1, int N, double *zz, double *gg);
+void dump_green (PFD G, PFD D, double par[], double x0, double x1, GMT_LONG N, double *zz, double *gg);
 #endif
 
 /* Functions for complex math */
@@ -160,10 +160,9 @@ PFD GMT_azimuth_func;
 
 int main (int argc, char **argv)
 {
-	int i, j, k, p, ii, n, m, nm, fno, n_files = 0, n_fields, n_args, dimension = 0;
-	int error = 0, n_expected_fields, normalize = 1, unit = 0, n_items;
-	size_t old_n_alloc, n_alloc;
-	GMT_LONG ij, ji, nxy, n_ok = 0;
+	GMT_LONG i, j, k, p, ii, n, m, nm, fno, n_files = 0, n_fields, n_args, dimension = 0;
+	GMT_LONG error = 0, n_expected_fields, normalize = 1, unit = 0, n_items;
+	GMT_LONG old_n_alloc, n_alloc, ij, ji, nxy, n_ok = 0;
 	BOOLEAN do_grid, done, nofile = TRUE;
 	char line[BUFSIZ];
 	char *method[N_METHODS] = {"minimum curvature Cartesian spline [1-D]",
@@ -176,7 +175,7 @@ int main (int argc, char **argv)
 		"regularized Cartesian spline in tension [3-D]",
 		"minimum curvature spherical spline",
 		"continuous curvature spherical spline in tension"};
-	char *mem_unit[3] = {"kb", "Mb", "Gb"}, txt[6][GMT_TEXT_LEN];
+	char *mem_unit[3] = {"kb", "Mb", "Gb"}, txt[6][GMT_TEXT_LEN], *not_used = NULL;
 	float *w = NULL;
 	double *obs, **D = NULL, **X, *alpha, *in, r, par[11], norm[7], az, grad;
 	double *WB_z = NULL, *WB_g = NULL, mem, r_min = -1.0, r_max = 1.0, part, C, p_val;
@@ -218,10 +217,10 @@ int main (int argc, char **argv)
 	double gradspline3d_sandwell (double r, double par[], double *G);
 	double gradspline3d_Wessel_Bercovici (double r, double par[], double *G);
 	double gradspline3d_Mitasova_Mitas (double r, double par[], double *G);
-	double undo_normalization (double *X, double q, int mode, double *coeff);
-	void do_normalization (double **X, double *obs, int n, int mode, double *coeff);
-	double get_radius (double *X0, double *X1, int dim);
-	double get_dircosine (double *D, double *X0, double *X1, int dim, BOOLEAN baz);
+	double undo_normalization (double *X, double q, GMT_LONG mode, double *coeff);
+	void do_normalization (double **X, double *obs, GMT_LONG n, GMT_LONG mode, double *coeff);
+	double get_radius (double *X0, double *X1, GMT_LONG dim);
+	double get_dircosine (double *D, double *X0, double *X1, GMT_LONG dim, BOOLEAN baz);
 	
 	PFD G = NULL, dGdr = NULL;
 	
@@ -473,8 +472,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t      Optionally, specify a length-scale [Default is grid spacing].\n");
 		fprintf (stderr, "\t   -Sr<tension> is a regularized spline in tension (Mitasova & Mitas, 1993).\n");
 		fprintf (stderr, "\t      Optionally, specify a length-scale [Default is grid spacing].\n");
-		fprintf (stderr, "\t   -Sp is a spherical surface spline (Parker, 1994); automatically sets -D3.\n");
-		fprintf (stderr, "\t   -Sq is a spherical surface spline in tension (Wessel & Becker, 2008); automatically sets -D3.\n");
+		fprintf (stderr, "\t   -Sp is a spherical surface spline (Parker, 1994); automatically sets -D4.\n");
+		fprintf (stderr, "\t   -Sq is a spherical surface spline in tension (Wessel & Becker, 2008); automatically sets -D4.\n");
 		fprintf (stderr, "\t      Use -SQ to speed up calculations by using precalculated lookup tables.\n");
 		fprintf (stderr, "\t      Append /n to set the (odd) number of points in the spline [%d]\n", N_X);
 		fprintf (stderr, "\t-T Mask grid file whose values are NaN or 0; its header implicitly sets -R, -I (and -F).\n");
@@ -499,7 +498,7 @@ int main (int argc, char **argv)
 	}
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = dimension + 1;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] <= dimension) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %d columns\n", GMT_program, dimension);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least %ld columns\n", GMT_program, dimension);
 		error++;
 	}
 	if (Ctrl->S.value[0] < 0.0 || Ctrl->S.value[0] >= 1.0) {
@@ -539,7 +538,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -587,7 +586,7 @@ int main (int argc, char **argv)
 			GMT_azimuth_func = NULL;
 			break;
 		default:	/* Cannot happen unless we make a bug */
-			fprintf (stderr, "%s: BUG since D (=%d) cannot be outside 0-5 range\n", GMT_program, Ctrl->D.mode+1);
+			fprintf (stderr, "%s: BUG since D (=%ld) cannot be outside 0-5 range\n", GMT_program, Ctrl->D.mode+1);
 			break;
 	}
 
@@ -610,7 +609,7 @@ int main (int argc, char **argv)
 
 	n_alloc = GMT_CHUNK;
 	X = (double **) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double *), GMT_program);
-	for (k = 0; k < (int)n_alloc; k++) X[k] = (double *) GMT_memory (VNULL, (size_t)dimension, sizeof (double), GMT_program);
+	for (k = 0; k < n_alloc; k++) X[k] = (double *) GMT_memory (VNULL, dimension, sizeof (double), GMT_program);
 	obs = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 
 	n = m = 0;
@@ -631,12 +630,12 @@ int main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Reading file %s\n", GMT_program, argv[fno]);
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 				exit (EXIT_FAILURE);
 			}
 
@@ -644,12 +643,12 @@ int main (int argc, char **argv)
 			obs[n] = in[dimension];
 			n++;
 
-			if (n == (int)n_alloc) {	/* Get more memory */
+			if (n == n_alloc) {	/* Get more memory */
 				old_n_alloc = n_alloc;
 				n_alloc <<= 1;
-				X = (double **) GMT_memory ((void *)X, (size_t)n_alloc, sizeof (double *), GMT_program);
-				for (k = old_n_alloc; k < (int)n_alloc; k++) X[k] = (double *) GMT_memory (VNULL, (size_t)dimension, sizeof (double), GMT_program);
-				obs = (double *) GMT_memory ((void *)obs, (size_t)n_alloc, sizeof (double), GMT_program);
+				X = (double **) GMT_memory ((void *)X, n_alloc, sizeof (double *), GMT_program);
+				for (k = (int)old_n_alloc; k < (int)n_alloc; k++) X[k] = (double *) GMT_memory (VNULL, dimension, sizeof (double), GMT_program);
+				obs = (double *) GMT_memory ((void *)obs, n_alloc, sizeof (double), GMT_program);
 			}
 		}
 
@@ -657,8 +656,8 @@ int main (int argc, char **argv)
 	}
 
 	for (k = n; k < (int)n_alloc; k++) GMT_free ((void *)X[k]);	/* Remove what was not used */
-	X = (double **) GMT_memory ((void *)X, (size_t)n, sizeof (double *), GMT_program);
-	obs = (double *) GMT_memory ((void *)obs, (size_t)n, sizeof (double), GMT_program);
+	X = (double **) GMT_memory ((void *)X, n, sizeof (double *), GMT_program);
+	obs = (double *) GMT_memory ((void *)obs, n, sizeof (double), GMT_program);
 	nm = n;
 
 	if (Ctrl->A.active) {	/* Read gradient constraints from file */
@@ -682,7 +681,7 @@ int main (int argc, char **argv)
 								obs[p] = S->segment[i]->coord[dimension][j];
 								break;
 							default:
-								fprintf (stderr, "%s: Bad gradient mode selected for 1-D data (%d) - aborting!\n", GMT_program, Ctrl->A.mode);
+								fprintf (stderr, "%s: Bad gradient mode selected for 1-D data (%ld) - aborting!\n", GMT_program, Ctrl->A.mode);
 								exit (EXIT_FAILURE);
 								break;
 						}
@@ -710,7 +709,7 @@ int main (int argc, char **argv)
 								obs[p] = S->segment[i]->coord[4][j];	/* Magnitude of gradient */
 								break;
 							default:
-								fprintf (stderr, "%s: Bad gradient mode selected for 2-D data (%d) - aborting!\n", GMT_program, Ctrl->A.mode);
+								fprintf (stderr, "%s: Bad gradient mode selected for 2-D data (%ld) - aborting!\n", GMT_program, Ctrl->A.mode);
 								exit (EXIT_FAILURE);
 								break;
 						}
@@ -728,13 +727,13 @@ int main (int argc, char **argv)
 								obs[p] = S->segment[i]->coord[6][j];	/* This is the gradient magnitude */
 								break;
 							default:
-								fprintf (stderr, "%s: Bad gradient mode selected for 3-D data (%d) - aborting!\n", GMT_program, Ctrl->A.mode);
+								fprintf (stderr, "%s: Bad gradient mode selected for 3-D data (%ld) - aborting!\n", GMT_program, Ctrl->A.mode);
 								exit (EXIT_FAILURE);
 								break;
 						}
 						break;
 					default:
-						fprintf (stderr, "%s: Bad dimension selected (%d) - aborting!\n", GMT_program, dimension);
+						fprintf (stderr, "%s: Bad dimension selected (%ld) - aborting!\n", GMT_program, dimension);
 						exit (EXIT_FAILURE);
 						break;
 				}
@@ -750,9 +749,9 @@ int main (int argc, char **argv)
 	
 	if (gmtdefs.verbose) {
 		if (m == 0)
-			fprintf (stderr, "%s: Found %d data points, yielding a %d by %d set of linear equations\n", GMT_program, n, nm, nm);
+			fprintf (stderr, "%s: Found %ld data points, yielding a %ld by %ld set of linear equations\n", GMT_program, n, nm, nm);
 		else
-			fprintf (stderr, "%s: Found %d data points and %d gradients, yielding a %d by %d set of linear equations\n",
+			fprintf (stderr, "%s: Found %ld data points and %ld gradients, yielding a %ld by %ld set of linear equations\n",
 				GMT_program, n, m, nm, nm);
 	}
 		
@@ -781,9 +780,9 @@ int main (int argc, char **argv)
 			h.y_inc = Ctrl->I.inc[GMT_Y];
 			GMT_RI_prepare (&h);	/* Ensure -R -I consistency and set nx, ny */
 			GMT_err_fail (GMT_grd_RI_verify (&h, 1), Ctrl->G.file);
-			if (dimension == 3) {
+			if (dimension == 3) {	/* Also set nz */
 				Z.z_inc = Ctrl->I.inc[GMT_Z];
-				Z.nz = irint ((Z.z_max - Z.z_min) / Z.z_inc) + h.node_offset;
+				Z.nz = GMT_get_n (Z.z_min, Z.z_max, Z.z_inc, h.node_offset);
 			}
 		}
 		else
@@ -885,13 +884,13 @@ int main (int argc, char **argv)
 			par[2] = (M_PI / par[4]) - M_LOG_2;
 			par[6] = 1.0 / (par[3] - par[2]);
 			if (Ctrl->S.fast) {
-				int nx;
+				GMT_LONG nx;
 				par[7] = Ctrl->S.value[1];
 				par[8] = (r_max - r_min) / (par[7] - 1.0);
 				par[9] = 1.0 / par[8];
 				par[10] = r_min;
-				nx = (int)irint (par[7]);
-				if (gmtdefs.verbose) fprintf (stderr, "Precalculate -SQ lookup table with %d items from %g to %g...", nx, r_min, r_max);
+				nx = (GMT_LONG)irint (par[7]);
+				if (gmtdefs.verbose) fprintf (stderr, "Precalculate -SQ lookup table with %ld items from %g to %g...", nx, r_min, r_max);
 				WB_z = (double *) GMT_memory (VNULL, nx, sizeof (double), GMT_program);
 				if (Ctrl->A.active) WB_g = (double *) GMT_memory (VNULL, nx, sizeof (double), GMT_program);
 				spline2d_Wessel_Becker_init (par, WB_z, WB_g, Ctrl->A.active);
@@ -970,11 +969,11 @@ int main (int argc, char **argv)
 	}
 
 	if (Ctrl->C.active) {		/* Solve using svd decomposition */
-		int n_use;
+		GMT_LONG n_use;
 		REAL *v, *s, *b, eig_max = 0.0;
 		char format[GMT_LONG_TEXT];
-		void svdcmp (REAL *a, int m, int n, REAL *w, REAL *v);
-		int solve_svd (REAL *u, int m, int n, REAL *v, REAL *w, REAL *b, int k, REAL *x, double cutoff);
+		void svdcmp (REAL *a, GMT_LONG m, GMT_LONG n, REAL *w, REAL *v);
+		GMT_LONG solve_svd (REAL *u, GMT_LONG m, GMT_LONG n, REAL *v, REAL *w, REAL *b, GMT_LONG k, REAL *x, double cutoff);
 		
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Solve linear equations by SVD ", GMT_program);
 	
@@ -1015,14 +1014,14 @@ int main (int argc, char **argv)
 			for (i = 0; i < nm; i++) obs[i] = (double)z4[i];
 			GMT_free ((void *)z4);
 		}
-		if (gmtdefs.verbose) fprintf (stderr, "[%d of %d eigen-values used]\n", n_use, nm);
+		if (gmtdefs.verbose) fprintf (stderr, "[%ld of %ld eigen-values used]\n", n_use, nm);
 			
 		GMT_free ((void *)s);
 		GMT_free ((void *)v);
 		GMT_free ((void *)b);
 	}
 	else {				/* Gauss-Jordan elimination */
-		int gaussj (REAL *a, int n, int ndim, REAL *b, int m, int mdim);
+		GMT_LONG gaussj (REAL *a, GMT_LONG n, GMT_LONG ndim, REAL *b, GMT_LONG m, GMT_LONG mdim);
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Solve linear equations by Gauss-Jordan elimination\n", GMT_program);
 		if (sizeof (REAL) == sizeof (double))
 			gaussj (A, nm, nm, obs, 1, 1);
@@ -1040,7 +1039,7 @@ int main (int argc, char **argv)
 	GMT_free ((void *)A);
 
 	if (Ctrl->N.file) {	/* Specified nodes only */
-		int n_output;
+		GMT_LONG n_output;
 		double out[4];
 		FILE *fp;
 	
@@ -1081,7 +1080,7 @@ int main (int argc, char **argv)
 		if (fp != GMT_stdout) GMT_fclose (fp);
 	}
 	else {
-		int nz_off, nxy, n_output;
+		GMT_LONG nz_off, nxy, n_output;
 		double *xp, *yp = NULL, wp, V[4];
 		FILE *fp = NULL;
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Evaluate spline at %ld equidistant output locations\n", GMT_program, n_ok);
@@ -1158,9 +1157,9 @@ int main (int argc, char **argv)
 
 #ifdef TEST
 /* Dump a table of x, G, dGdx for test purposes */
-void dump_green (PFD G, PFD D, double par[], double x0, double x1, int N, double *zz, double *gg)
+void dump_green (PFD G, PFD D, double par[], double x0, double x1, GMT_LONG N, double *zz, double *gg)
 {
-	int i;
+	GMT_LONG i;
 	double x, dx, dy, y, t, ry, rdy;
 	double min_y, max_y, min_dy, max_dy;
 	
@@ -1170,7 +1169,7 @@ void dump_green (PFD G, PFD D, double par[], double x0, double x1, int N, double
 	dx = (x1 - x0) / (N - 1);
 	for (i = 0; i < N; i++) {
 		x = x0 + i * dx;
-		t = (x0 < 0.0) ? R2D * acos (x) : x;
+		t = (x0 < 0.0) ? acosd (x) : x;
 		y = G(x, par, zz);
 		dy = D(x, par, gg);
 		if (y < min_y) min_y = y;
@@ -1182,7 +1181,7 @@ void dump_green (PFD G, PFD D, double par[], double x0, double x1, int N, double
 	rdy = max_dy - min_dy;
 	for (i = 0; i < N; i++) {
 		x = x0 + i * dx;
-		t = (x0 < 0.0) ? R2D * acos (x) : x;
+		t = (x0 < 0.0) ? acosd (x) : x;
 		y = G(x, par, zz);
 		dy = D(x, par, gg);
 		dy = (rdy > 0.0) ? (dy - min_dy)/rdy : 1.0;
@@ -1395,7 +1394,7 @@ double gradspline2d_Parker (double x, double par[], double *G)
  
 double spline2d_Wessel_Becker (double x, double par[], double *G)
 {	/* g = M_PI * Pv(-x)/sin (v*x) - log (1-x) normalized to 0-1 */
-	int n;
+	GMT_LONG n;
 	double z[2], pq[4];
 	
 	if (x == +1.0) return (1.0);
@@ -1414,7 +1413,7 @@ double spline2d_Wessel_Becker (double x, double par[], double *G)
 
 double gradspline2d_Wessel_Becker (double x, double par[], double *G)
 {	/* g = -M_PI * (v+1)*[x*Pv(-x)+Pv+1(-x)]/(sin (v*x)*sin(theta)) - 1/(1-x), normalized to 0-1 */
-	int n;
+	GMT_LONG n;
 	double z[2], v1[2], pq[4], s;
 	
 	if (x == +1.0 || x == -1.0) return (0.0);
@@ -1446,13 +1445,13 @@ double gradspline2d_Wessel_Becker (double x, double par[], double *G)
 
 double spline2d_lookup (double x, double par[], double *y)
 {
-	int k;
+	GMT_LONG k;
 	double f, f0, df;
 	
 	f = (x - par[10]) * par[9];	/* Floating point index */
 	f0 = floor (f);
 	df = f - f0;
-	k = (int)f0;
+	k = (GMT_LONG)f0;
 	if (df == 0.0) return (y[k]);
 	return (y[k]*(1.0 - df) + y[k+1] * df);
 }
@@ -1478,7 +1477,7 @@ void spline2d_Wessel_Becker_init (double par[], double *z, double *g, BOOLEAN gr
 	fp = fopen ("greenspline.b", "wb");
 	n_out = (grad) ? 3 : 2;
 #endif
-	nx = (int)irint (par[7]);
+	nx = (GMT_LONG)irint (par[7]);
 	for (i = 0; i < nx; i++) {
 		x = par[10] + i * par[8];
 		z[i] = spline2d_Wessel_Becker (x, par, NULL);
@@ -1566,14 +1565,14 @@ double gradspline3d_Mitasova_Mitas (double r, double par[], double *G)
 
 /* Modified from similar function in Numerical Recipes */
 
-int gaussj (REAL *a, int n, int ndim, REAL *b, int m, int mdim)
+GMT_LONG gaussj (REAL *a, GMT_LONG n, GMT_LONG ndim, REAL *b, GMT_LONG m, GMT_LONG mdim)
 {
-	int i, j, k, l, ll, *ipiv, *indxc, *indxr, irow = 0, icol = 0;
+	GMT_LONG i, j, k, l, ll, *ipiv, *indxc, *indxr, irow = 0, icol = 0;
 	REAL big, dum, pivinv;
 	
-	ipiv  = (int *) GMT_memory (VNULL, (size_t)n, sizeof (int), "gaussj");
-	indxc = (int *) GMT_memory (VNULL, (size_t)n, sizeof (int), "gaussj");
-	indxr = (int *) GMT_memory (VNULL, (size_t)n, sizeof (int), "gaussj");
+	ipiv  = (GMT_LONG *) GMT_memory (VNULL, (size_t)n, sizeof (GMT_LONG), "gaussj");
+	indxc = (GMT_LONG *) GMT_memory (VNULL, (size_t)n, sizeof (GMT_LONG), "gaussj");
+	indxr = (GMT_LONG *) GMT_memory (VNULL, (size_t)n, sizeof (GMT_LONG), "gaussj");
 	
 	for (i = 0; i < n; i++) {
 		big = 0.0;
@@ -1665,11 +1664,11 @@ int gaussj (REAL *a, int n, int ndim, REAL *b, int m, int mdim)
 
 #define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))
 
-void svdcmp (REAL *a, int m, int n, REAL *w, REAL *v)
+void svdcmp (REAL *a, GMT_LONG m, GMT_LONG n, REAL *w, REAL *v)
 {
 	/* void svdcmp(REAL *a,int m,int n,REAL *w,REAL *v) */
 	
-	int flag,i,its,j,jj,k,l=0,nm;
+	GMT_LONG flag,i,its,j,jj,k,l=0,nm;
 	REAL c,f,h,s,x,y,z;
 	REAL anorm=0.0,tnorm, g=0.0,scale=0.0;
 	REAL *rv1;
@@ -1953,13 +1952,13 @@ void svdcmp (REAL *a, int m, int n, REAL *w, REAL *v)
 */
 	
 
-int solve_svd (REAL *u, int m, int n, REAL *v, REAL *w, REAL *b, int k, REAL *x, double cutoff)
+GMT_LONG solve_svd (REAL *u, GMT_LONG m, GMT_LONG n, REAL *v, REAL *w, REAL *b, GMT_LONG k, REAL *x, double cutoff)
 {
 	REAL *ut,sing_max;
-	int i,j, n_use = 0;
+	GMT_LONG i,j, n_use = 0;
 
-	void mat_trans (REAL a[], int mrow, int ncol, REAL at[]);
-	void mat_mult (REAL a[], int mrow, int ncol, REAL b[], int kcol, REAL c[]);
+	void mat_trans (REAL a[], GMT_LONG mrow, GMT_LONG ncol, REAL at[]);
+	void mat_mult (REAL a[], GMT_LONG mrow, GMT_LONG ncol, REAL b[], GMT_LONG kcol, REAL c[]);
 	   
 		/* allocate work space */
 		
@@ -2022,10 +2021,10 @@ int solve_svd (REAL *u, int m, int n, REAL *v, REAL *w, REAL *b, int k, REAL *x,
  * coeff[6]:	The normalizing scale for the detrended data
  */
 
-void do_normalization_1d (double **X, double *obs, int n, int mode, double *coeff)
+void do_normalization_1d (double **X, double *obs, GMT_LONG n, GMT_LONG mode, double *coeff)
 {	/* mode == 1 norm w; mode == 2: also remove linear trend, norm & 4: also normalize result by range */
 
-	int	i;
+	GMT_LONG	i;
 	double	d;
 	
 	memset ((void *)coeff, 0, 5*sizeof(double));
@@ -2069,10 +2068,10 @@ void do_normalization_1d (double **X, double *obs, int n, int mode, double *coef
 
 }
 
-void do_normalization (double **X, double *obs, int n, int mode, double *coeff)
+void do_normalization (double **X, double *obs, GMT_LONG n, GMT_LONG mode, double *coeff)
 {	/* mode == 1 norm z; mode == 2: also remove plane, norm & 4: also normalize result by range */
 
-	int	i;
+	GMT_LONG	i;
 	double	d;
 	
 	memset ((void *)coeff, 0, 5*sizeof(double));
@@ -2129,7 +2128,7 @@ void do_normalization (double **X, double *obs, int n, int mode, double *coeff)
 
 }
 
-double undo_normalization (double *X, double w_norm, int mode, double *coeff)
+double undo_normalization (double *X, double w_norm, GMT_LONG mode, double *coeff)
 {
 	double w;
 	w = w_norm;
@@ -2139,18 +2138,18 @@ double undo_normalization (double *X, double w_norm, int mode, double *coeff)
 	return (w);
 }
 
-void mat_trans (REAL a[], int mrow, int ncol, REAL at[])
+void mat_trans (REAL a[], GMT_LONG mrow, GMT_LONG ncol, REAL at[])
 {
 	/* Return the transpose of a */
-	int i, j;
+	GMT_LONG i, j;
 	for (i = 0; i < ncol; i++) for (j = 0; j < mrow; j++) at[mrow*i+j] = a[ncol*j+i];
 }
 
-void mat_mult (REAL a[], int mrow, int ncol, REAL b[], int kcol, REAL c[])
+void mat_mult (REAL a[], GMT_LONG mrow, GMT_LONG ncol, REAL b[], GMT_LONG kcol, REAL c[])
 {
 	/* Matrix multiplication a * b = c */
 	
-	int i, j, k, ij;
+	GMT_LONG i, j, k, ij;
 	
 	for (i = 0; i < kcol; i++) {
 		for (j = 0; j < mrow; j++) {
@@ -2162,7 +2161,7 @@ void mat_mult (REAL a[], int mrow, int ncol, REAL b[], int kcol, REAL c[])
 }
 
 
-double get_radius (double *X0, double *X1, int dim)
+double get_radius (double *X0, double *X1, GMT_LONG dim)
 {
 	double r = 0.0;
 	/* Get distance between the two points */
@@ -2181,14 +2180,14 @@ double get_radius (double *X0, double *X1, int dim)
 	return (r);
 }
 
-double get_dircosine (double *D, double *X0, double *X1, int dim, BOOLEAN baz)
+double get_dircosine (double *D, double *X0, double *X1, GMT_LONG dim, BOOLEAN baz)
 {
 	/* D, the directional cosines of the observed gradient:
 	 * X0: Observation point.
 	 * X1: Prediction point.
 	 * Compute N, the direction cosine of X1-X2, then C = D dot N.
 	 */
-	int ii;
+	GMT_LONG ii;
 	double az, C = 0.0, N[3];
 	
 	switch (dim) {
