@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: minmax.c,v 1.49 2008/03/24 08:58:31 guru Exp $
+ *    $Id: minmax.c,v 1.60 2009/07/11 03:16:30 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -36,17 +36,22 @@ struct MINMAX_CTRL {	/* All control options for this program (except common args
 	struct E {	/* -E<L|l|H|h><col> */
 		BOOLEAN active;
 		BOOLEAN abs;
-		int mode;
-		int col;
+		GMT_LONG mode;
+		GMT_LONG col;
 	} E;
 	struct I {	/* -Idx[/dy[/<dz>..] */
 		BOOLEAN active;
-		int ncol;
+		GMT_LONG ncol;
 		double inc[BUFSIZ];
 	} I;
-	struct T {	/* -T<dz> */
+	struct S {	/* -S[x|y] */
+		BOOLEAN active;
+		BOOLEAN xbar, ybar;
+	} S;
+	struct T {	/* -T<dz>[/<col>] */
 		BOOLEAN active;
 		double inc;
+		GMT_LONG col;
 	} T;
 };
 
@@ -55,11 +60,9 @@ int main (int argc, char **argv)
 	BOOLEAN  error = FALSE, nofile = TRUE, done = FALSE, got_stuff = FALSE, first, give_r_string = FALSE;
 	BOOLEAN  dateline = FALSE, brackets = FALSE, work_on_abs_value, special = FALSE, quad[4] = {FALSE, FALSE, FALSE, FALSE};
 
-	char line[BUFSIZ], file[BUFSIZ], chosen[BUFSIZ];
+	char line[BUFSIZ], file[BUFSIZ], chosen[BUFSIZ], *not_used = NULL;
 
-	int i, j, ncol, n_files = 0, fno, n_args, n_fields, n_expected_fields, quad_no;
-
-	GMT_LONG n;
+	GMT_LONG i, j, ncol, n_files = 0, fno, n_args, n_fields, n_expected_fields, quad_no, n;
 	
 	double *xyzmin, *xyzmax, west, east, south, north, low, high, value, e_min = DBL_MAX, e_max = -DBL_MAX, *in;
 	double xmin1 = 360.0, xmin2 = 360.0, xmax1 = -360.0, xmax2 = -360.0;
@@ -68,7 +71,7 @@ int main (int argc, char **argv)
 
 	struct MINMAX_CTRL *Ctrl;
 
-	int strip_blanks_and_output (double x, int col);
+	GMT_LONG strip_blanks_and_output (double x, GMT_LONG col);
 	void *New_Minmax_Ctrl (), Free_Minmax_Ctrl (struct MINMAX_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -81,9 +84,11 @@ int main (int argc, char **argv)
 				/* Common parameters */
                       
 				case 'H':
+				case 'M':
 				case ':':
   				case 'b':
   				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], 0, 0, 0, 0);
 					break;
@@ -117,9 +122,6 @@ int main (int argc, char **argv)
 					}
 					Ctrl->E.col = atoi (&argv[i][3]);
                                         break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'I':
 					Ctrl->I.active = TRUE;
 					if (argv[i][2] == 'p') special = TRUE;
@@ -133,7 +135,18 @@ int main (int argc, char **argv)
 					break;
 				case 'T':
 					Ctrl->T.active = TRUE;
-					Ctrl->T.inc = atof (&argv[i][2]);
+					j = sscanf (&argv[i][2], "%lf/%ld", &Ctrl->T.inc, &Ctrl->T.col);
+					if (j == 1) Ctrl->T.col = 0;
+					break;
+				case 'S':
+					Ctrl->S.active = TRUE;
+					j = 2;
+					while (argv[i][j]) {
+						if (argv[i][j] == 'x') Ctrl->S.xbar = TRUE;
+						if (argv[i][j] == 'y') Ctrl->S.ybar = TRUE;
+						j++;
+					}
+					if (j == 2) Ctrl->S.xbar = Ctrl->S.ybar = TRUE;
 					break;
 				default:
 					error = TRUE;
@@ -147,8 +160,8 @@ int main (int argc, char **argv)
 
 	if (error || GMT_give_synopsis_and_exit) {	/* Because it's ok to give no arguments */
 		fprintf (stderr, "minmax %s - Find extreme values in ASCII tables\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: minmax [files] [-C] [-E<L|l|H|h><col>] [%s] [-I[p]<dx>[/<dy>[/<dz>..]] [%s]\n", GMT_H_OPT, GMT_M_OPT);
-		fprintf (stderr, "\t[-T<dz>] [%s] [%s] [%s]\n", GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
+		fprintf (stderr, "usage: minmax [files] [-C] [-E<L|l|H|h><col>] [%s] [-I[p]<dx>[/<dy>[/<dz>..]]\n", GMT_H_OPT);
+		fprintf (stderr, "\t[-S[x][y]] [-T<dz>[/<col>]] [%s] [%s] [%s] [%s]\n", GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT, GMT_m_OPT);
              
               	if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
  
@@ -162,17 +175,23 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   given determines how many columns are rounded off to the nearest multiple.\n");
 		fprintf (stderr, "\t   If only one increment is given we also use it for the second column (for backwards compatibility).\n");
 		fprintf (stderr, "\t   To override this behaviour, use -Ip<dx>.\n");
-		GMT_explain_option ('M');
+		fprintf (stderr, "\t-S adds extra space for error bars. Useful together with -I.\n");
+		fprintf (stderr, "\t   -Sx leaves space for horizontal error bar using value in third (2) column.\n");
+		fprintf (stderr, "\t   -Sy leaves space for vertical error bar using value in third (2) column.\n");
+		fprintf (stderr, "\t   -S or -Sxy leaves space for both error bars using values in third&fourth (2&3) columns.\n");
 		fprintf (stderr, "\t-T returns textstring -Tzmin/zmax/dz to nearest multiple of the given dz\n");
-		fprintf (stderr, "\t   Calculations are based on the first column only\n");
+		fprintf (stderr, "\t   Calculations are based on the first (0) column only.  Append /<col> to use another column\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
+	
+	GMT_check_lattice (&Ctrl->I.inc[0], &Ctrl->I.inc[1], NULL, &Ctrl->I.active);
 	if (Ctrl->I.active && !special && Ctrl->I.ncol == 1) {		/* Special case of dy = dx if not given */
 		Ctrl->I.inc[1] = Ctrl->I.inc[0];
 		Ctrl->I.ncol = 2;
@@ -218,7 +237,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (n_files > 0)
@@ -274,7 +293,7 @@ int main (int argc, char **argv)
 			}
 		}
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		n = ncol = 0;
 		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
@@ -290,17 +309,27 @@ int main (int argc, char **argv)
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 				exit (EXIT_FAILURE);
 			}
 
 			if (first) {	/* First time, allocate # of columns */
 
 				ncol = n_expected_fields;
-				if (Ctrl->E.active && Ctrl->E.col >= ncol) {
-	  				fprintf (stderr, "%s: GMT SYNTAX ERROR -E option: Chosen column exceeds column range\n", GMT_program);
+				if (Ctrl->S.active && 2 + Ctrl->S.xbar + Ctrl->S.ybar > ncol) {
+					fprintf (stderr, "%s: Not enough columns to support the -S option\n", GMT_program);
 					exit (EXIT_FAILURE);
 				}
+				if (Ctrl->E.active && Ctrl->E.col >= ncol) {
+	  				fprintf (stderr, "%s: GMT SYNTAX ERROR -E option: Chosen column exceeds column range (0-%ld)\n", GMT_program, ncol-1);
+					exit (EXIT_FAILURE);
+				}
+				if (Ctrl->T.active && Ctrl->T.col >= ncol) {
+					fprintf (stderr, "%s: GMT SYNTAX ERROR -T option: Chosen column exceeds column range (0-%ld)\n", GMT_program, ncol-1);
+					exit (EXIT_FAILURE);
+				}
+				if (Ctrl->T.active) ncol = Ctrl->T.col+1;
+				if (give_r_string) ncol = 2;
 
 				/* Now we know # of columns, so allocate memory */
 
@@ -319,7 +348,7 @@ int main (int argc, char **argv)
 
 			/* Decode all fields and update minmax arrays */
 
-			while (dateline && in[0] > 180.0) in[0] -= 360.0;
+			while (dateline && in[GMT_X] > 180.0) in[GMT_X] -= 360.0;
 
 			if (Ctrl->E.active && !GMT_is_dnan (in[Ctrl->E.col])) {
 				value = (work_on_abs_value) ? fabs (in[Ctrl->E.col]) : in[Ctrl->E.col];
@@ -346,6 +375,13 @@ int main (int argc, char **argv)
 						xmin2 = MIN (in[i], xmin2);
 						xmax2 = MAX (in[i], xmax2);
 					}
+					else if ((i == 0 && Ctrl->S.xbar) || (i == 1 && Ctrl->S.ybar)) {
+						/* Add/subtract value from error bar column */
+						j = (i == 1 && Ctrl->S.xbar) ? 3 : 2;
+						value = fabs(in[j]);
+						if (in[i] - value < xyzmin[i]) xyzmin[i] = in[i] - value;
+						if (in[i] + value > xyzmax[i]) xyzmax[i] = in[i] + value;
+					}
 					else {
 						if (in[i] < xyzmin[i]) xyzmin[i] = in[i];
 						if (in[i] > xyzmax[i]) xyzmax[i] = in[i];
@@ -364,7 +400,7 @@ int main (int argc, char **argv)
 		if (GMT_io.in_col_type[0] == GMT_IS_LON) {	/* Now determine longitude range */
 			int n_quad;
 			n_quad = quad[0] + quad[1] + quad[2] + quad[3];	/* How many quadrants had data */
-			if (quad[0] && quad[3]) {	/* Longitudes on either side of Greenwhich only, must use -180/+180 notation */
+			if (quad[0] && quad[3]) {	/* Longitudes on either side of Greenwich only, must use -180/+180 notation */
 				xyzmin[0] = xmin2;
 				xyzmax[0] = xmax2;
 				GMT_io.geo.range = 2;	/* Override this setting explicitly */
@@ -394,11 +430,14 @@ int main (int argc, char **argv)
 			if (xyzmin[0] < 0.0 && xyzmax[0] < 0.0) xyzmin[0] += 360.0, xyzmax[0] += 360.0;
 		}
 		if (give_r_string && got_stuff) {
-			west  = MIN (west, xyzmin[0]);		east  = MAX (east, xyzmax[0]);
-			south = MIN (south, xyzmin[1]);		north = MAX (north, xyzmax[1]);
+			west  = MIN (west, xyzmin[0]);
+			east  = MAX (east, xyzmax[0]);
+			south = MIN (south, xyzmin[1]);
+			north = MAX (north, xyzmax[1]);
 		}
 		else if (Ctrl->T.active && got_stuff) {
-			west  = MIN (west, xyzmin[0]);		east  = MAX (east, xyzmax[0]);
+			west  = MIN (west, xyzmin[Ctrl->T.col]);
+			east  = MAX (east, xyzmax[Ctrl->T.col]);
 		}
 		else if (!Ctrl->E.active && n > 0) {
 			if (!Ctrl->C.active) fprintf (GMT_stdout, "%s: N = %ld\t", file, n);
@@ -426,8 +465,10 @@ int main (int argc, char **argv)
 	}
 	if (got_stuff) {
 		if (give_r_string) {
-			west  = floor (west / Ctrl->I.inc[0]) * Ctrl->I.inc[0];	east  = ceil (east / Ctrl->I.inc[0]) * Ctrl->I.inc[0];
-			south = floor (south / Ctrl->I.inc[1]) * Ctrl->I.inc[1];	north = ceil (north / Ctrl->I.inc[1]) * Ctrl->I.inc[1];
+			west  = floor (west / Ctrl->I.inc[0]) * Ctrl->I.inc[0];
+			east  = ceil (east / Ctrl->I.inc[0]) * Ctrl->I.inc[0];
+			south = floor (south / Ctrl->I.inc[1]) * Ctrl->I.inc[1];
+			north = ceil (north / Ctrl->I.inc[1]) * Ctrl->I.inc[1];
 			if (east < west) east += 360.0;
 			fprintf (GMT_stdout, "-R");
 			strip_blanks_and_output (west, 0);	fputc ('/', GMT_stdout);
@@ -436,7 +477,8 @@ int main (int argc, char **argv)
 			strip_blanks_and_output (north, 1);	fputc ('\n', GMT_stdout);
 		}
 		else if (Ctrl->T.active) {	/* -T option */
-			west  = floor (west / Ctrl->T.inc) * Ctrl->T.inc;	east  = ceil (east / Ctrl->T.inc) * Ctrl->T.inc;
+			west  = floor (west / Ctrl->T.inc) * Ctrl->T.inc;
+			east  = ceil (east / Ctrl->T.inc) * Ctrl->T.inc;
 			fprintf (GMT_stdout, "-T");
 			strip_blanks_and_output (west, 0);	fputc ('/', GMT_stdout);
 			strip_blanks_and_output (east, 0);	fputc ('/', GMT_stdout);
@@ -458,11 +500,11 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-int strip_blanks_and_output (double x, int col)
+GMT_LONG strip_blanks_and_output (double x, GMT_LONG col)
 {
 	/* Alternative to GMT_ascii_output_one that strips off leading blanks first */
 
-	int k;
+	GMT_LONG k;
 	char text[GMT_TEXT_LEN];
 
 	GMT_ascii_format_one (text, x, GMT_io.out_col_type[col]);
