@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: nearneighbor.c,v 1.48 2007/09/30 04:16:49 guru Exp $
+ *	$Id: nearneighbor.c,v 1.63 2009/07/11 03:16:30 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -56,7 +56,7 @@ struct NEARNEIGHBOR_CTRL {	/* All control options for this program (except commo
 	} L;
 	struct N {	/* -N<sectors> */
 		BOOLEAN active;
-		int sectors, min_sectors;
+		GMT_LONG sectors, min_sectors;
 	} N;
 	struct S {	/* -S<radius>[m|c|k|K] */
 		BOOLEAN active;
@@ -70,7 +70,7 @@ struct NEARNEIGHBOR_CTRL {	/* All control options for this program (except commo
 
 struct NEARNEIGHBOR_NODE {	/* Structure with point id and distance pairs for all sectors */
 	float *distance;	/* Distance of nearest datapoint to this node per sector */
-	int *datum;		/* Point id of this data point */
+	GMT_LONG *datum;		/* Point id of this data point */
 };
 
 struct NEARNEIGHBOR_POINT {	/* Structure with input data constraints */
@@ -79,9 +79,10 @@ struct NEARNEIGHBOR_POINT {	/* Structure with input data constraints */
 
 int main (int argc, char **argv)
 {
-	int i, j, k, ij, i0, j0, *di, dj, sector, n, n_alloc = 5 * GMT_CHUNK, n_fields, nx_2;
-	int n_set, n_filled, n_almost, n_none, n_files = 0, n_args, fno, one_or_zero, n_expected_fields, distance_flag = 0;
-	int max_di, actual_max_di, ii, jj, n_req, x_wrap, y_wrap;
+	GMT_LONG i, j, k, i0, j0, ij, n, nm, n_alloc = GMT_CHUNK, n_set, n_filled, n_almost, n_none;
+	GMT_LONG *di, dj, sector, n_fields, ii, jj;
+	GMT_LONG n_files = 0, n_args, fno, n_expected_fields, distance_flag = 0;
+	GMT_LONG max_di, actual_max_di, n_req, x_wrap, y_wrap;
 
 	BOOLEAN error = FALSE, done = FALSE, nofile = TRUE;
 	BOOLEAN wrap_180, replicate_x, replicate_y;
@@ -92,7 +93,7 @@ int main (int argc, char **argv)
 
 	float *grd;
 
-	char line[BUFSIZ];
+	char line[BUFSIZ], *not_used = NULL;
 
 	FILE *fp = NULL;
 
@@ -102,7 +103,8 @@ int main (int argc, char **argv)
 	struct NEARNEIGHBOR_POINT  *point;
 	struct NEARNEIGHBOR_CTRL *Ctrl;
 
-	void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, double distance, int id);
+	void assign_node (struct NEARNEIGHBOR_NODE **node, GMT_LONG n_sector, GMT_LONG sector, double distance, GMT_LONG id);
+	void free_node (struct NEARNEIGHBOR_NODE *node);
 	void *New_Nearneighbor_Ctrl (), Free_Nearneighbor_Ctrl (struct NEARNEIGHBOR_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -156,7 +158,7 @@ int main (int argc, char **argv)
 				case 'L':
 					if (argv[i][2]) {
 						Ctrl->L.active = TRUE;
-						strncpy (Ctrl->L.mode, &argv[i][2], 4);
+						strncpy (Ctrl->L.mode, &argv[i][2], (size_t)4);
 					}
 					else {
 						GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
@@ -166,7 +168,7 @@ int main (int argc, char **argv)
 					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
-					n = sscanf (&argv[i][2], "%d/%d", &Ctrl->N.sectors, &Ctrl->N.min_sectors);
+					n = sscanf (&argv[i][2], "%ld/%ld", &Ctrl->N.sectors, &Ctrl->N.min_sectors);
 					if (n < 1) Ctrl->N.sectors = 4;
 					if (n < 2) Ctrl->N.min_sectors = Ctrl->N.sectors;
 					break;
@@ -201,7 +203,7 @@ int main (int argc, char **argv)
 		fprintf(stderr, "\t-S sets search radius in -R, -I units; append m or c for minutes or seconds.\n");
 		fprintf(stderr, "\t   Append k for km (implies -R,-I in degrees), use flat Earth approximation.\n");
 		fprintf(stderr, "\t   Append K for km (implies -R,-I in degrees), use exact geodesic distances.\n");
-		fprintf(stderr, "\t   If the current ELLIPSOID is Sphere then spherical great circle distances are used.\n");
+		fprintf(stderr, "\t   If the current ELLIPSOID is sperical then great circle distances are used.\n");
 		fprintf(stderr, "\n\tOPTIONS:\n");
 		fprintf(stderr, "\t-E value to use for empty nodes [Default is NaN].\n");
 		fprintf(stderr, "\t-F Force pixel registration [Default is gridline registration].\n");
@@ -222,6 +224,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
@@ -249,7 +253,7 @@ int main (int argc, char **argv)
 	n_req = (Ctrl->W.active) ? 4 : 3;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_req;
 	if (GMT_io.binary[GMT_IN] && n_req > GMT_io.ncol[GMT_IN]) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %d columns\n", GMT_program, n_req);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  binary input data must have at least %ld columns\n", GMT_program, n_req);
 		error++;
 	}
 	if (Ctrl->L.active && GMT_boundcond_parse (&edgeinfo, Ctrl->L.mode)) error++;
@@ -258,7 +262,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 	if (Ctrl->S.unit == 'k') distance_flag = 1;
 	if (Ctrl->S.unit == 'K') distance_flag = 2;
@@ -277,7 +281,6 @@ int main (int argc, char **argv)
         	n_files = 1;
         n_args = (argc > 1) ? argc : 2;
         
-	one_or_zero = !header.node_offset;
 	header.xy_off = 0.5 * header.node_offset;
 	xinc2 = header.xy_off * header.x_inc;
 	yinc2 = header.xy_off * header.y_inc;
@@ -286,17 +289,17 @@ int main (int argc, char **argv)
 
 	GMT_boundcond_param_prep (&header, &edgeinfo);
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Grid dimensions are nx = %d, ny = %d\n", GMT_program,
-		header.nx, header.ny);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Grid dimensions are nx = %d, ny = %d\n", GMT_program, header.nx, header.ny);
 
-	grid_node = (struct NEARNEIGHBOR_NODE **) GMT_memory (VNULL, (size_t)(header.nx * header.ny), sizeof (struct NEARNEIGHBOR_NODE *), GMT_program);
+	nm = GMT_get_nm (header.nx, header.ny);
+	grid_node = (struct NEARNEIGHBOR_NODE **) GMT_memory (VNULL, (size_t)nm, sizeof (struct NEARNEIGHBOR_NODE *), GMT_program);
 	point = (struct NEARNEIGHBOR_POINT *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct NEARNEIGHBOR_POINT), GMT_program);
 
-	di = (int *) GMT_memory (VNULL, (size_t)header.ny, sizeof (int), GMT_program);
-	shrink = (double *) GMT_memory (VNULL, (size_t)header.ny, sizeof (double), GMT_program);
+	di = (GMT_LONG *) GMT_memory (VNULL, header.ny, sizeof (GMT_LONG), GMT_program);
+	shrink = (double *) GMT_memory (VNULL, header.ny, sizeof (double), GMT_program);
 
-	x0 = (double *) GMT_memory (VNULL, (size_t)header.nx, sizeof (double), GMT_program);
-	y0 = (double *) GMT_memory (VNULL, (size_t)header.ny, sizeof (double), GMT_program);
+	x0 = (double *) GMT_memory (VNULL, header.nx, sizeof (double), GMT_program);
+	y0 = (double *) GMT_memory (VNULL, header.ny, sizeof (double), GMT_program);
 	for (i = 0; i < header.nx; i++) x0[i] = header.x_min + i * header.x_inc + xinc2;
 	for (j = 0; j < header.ny; j++) y0[j] = header.y_max - j * header.y_inc - yinc2;
 	if (distance_flag) {	/* Input data is geographical */
@@ -339,7 +342,6 @@ int main (int argc, char **argv)
 	y_top = header.y_max + dj * header.y_inc;	y_bottom = header.y_min - dj * header.y_inc;
 	x_width = header.x_max - header.x_min;		y_width = header.y_max - header.y_min;
 	half_x_width = 0.5 * x_width;			half_y_width = 0.5 * y_width;
-	nx_2 = edgeinfo.nxp / 2;
 	n = 0;
 	replicate_x = (edgeinfo.nxp && !header.node_offset);	/* Gridline registration has duplicate column */
 	replicate_y = (edgeinfo.nyp && !header.node_offset);	/* Gridline registration has duplicate row */
@@ -363,48 +365,48 @@ int main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 				exit (EXIT_FAILURE);
 			}
 
-			if (in[0] < x_left || in[0] > x_right) continue;
-			if (in[1] < y_bottom || in[1] > y_top) continue;
+			if (in[GMT_X] < x_left || in[GMT_X] > x_right) continue;
+			if (in[GMT_Y] < y_bottom || in[GMT_Y] > y_top) continue;
 
-			point[n].x = (float)in[0];
-			point[n].y = (float)in[1];
-			point[n].z = (float)in[2];
+			point[n].x = (float)in[GMT_X];
+			point[n].y = (float)in[GMT_Y];
+			point[n].z = (float)in[GMT_Z];
 			if (Ctrl->W.active) point[n].w = (float)in[3];
 
 			/* Find indices of the node closest to this data point */
 
-			i0 = GMT_x_to_i(in[0], header.x_min, header.x_inc, header.xy_off, header.nx);
-			j0 = GMT_y_to_j(in[1], header.y_min, header.y_inc, header.xy_off, header.ny);
+			i0 = GMT_x_to_i(in[GMT_X], header.x_min, header.x_inc, header.xy_off, header.nx);
+			j0 = GMT_y_to_j(in[GMT_Y], header.y_min, header.y_inc, header.xy_off, header.ny);
 
 			/* Loop over all nodes within radius of this node */
 
 			for (j = j0 - dj; j <= (j0 + dj); j++) {
 
-				jj = j;
+				jj = (int)j;
 				if (GMT_y_out_of_bounds (&jj, &header, &edgeinfo, &wrap_180)) continue;	/* Outside y-range */
 
 				for (i = i0 - di[jj]; i <= (i0 + di[jj]); i++) {
 
-					ii = i;
+					ii = (int)i;
 					if (GMT_x_out_of_bounds (&ii, &header, &edgeinfo, wrap_180)) continue;	/* Outside x-range */ 
 
 					/* Here, (ii,jj) is index of a node (k) inside the grid */
 
-					distance = (GMT_distance_func) (x0[ii], y0[jj], in[0], in[1]);
+					distance = (GMT_distance_func) (x0[ii], y0[jj], in[GMT_X], in[GMT_Y]);
 
 					if (distance > Ctrl->S.radius) continue;	/* Data constraint is too far from this node */
 
-					k = jj * header.nx + ii;
-					dx = in[0] - x0[ii];	dy = in[1] - y0[jj];
+					k = GMT_IJ (jj, ii, header.nx);
+					dx = in[GMT_X] - x0[ii];	dy = in[GMT_Y] - y0[jj];
 
 					/* Check for wrap-around in x or y.  This should only occur if the
 					   search radius is larger than 1/2 the grid width/height so that
@@ -440,15 +442,15 @@ int main (int argc, char **argv)
 				}
 			}
 			n++;
-			if (gmtdefs.verbose && !(n%1000)) fprintf (stderr, "%s: Processed record %10d\r", GMT_program, n);
+			if (gmtdefs.verbose && !(n%1000)) fprintf (stderr, "%s: Processed record %10ld\r", GMT_program, n);
 			if (n == n_alloc) {
-				n_alloc += GMT_CHUNK;
+				n_alloc <<= 1;
 				point = (struct NEARNEIGHBOR_POINT *) GMT_memory ((void *)point, (size_t)n_alloc, sizeof (struct NEARNEIGHBOR_POINT), GMT_program);
 			}
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Processed record %10d\n", GMT_program, n);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Processed record %10ld\n", GMT_program, n);
 
 	point = (struct NEARNEIGHBOR_POINT *) GMT_memory ((void *)point, (size_t)n, sizeof (struct NEARNEIGHBOR_POINT), GMT_program);
 	grd = (float *) GMT_memory (VNULL, (size_t)(header.nx * header.ny), sizeof (float), GMT_program);
@@ -473,6 +475,7 @@ int main (int argc, char **argv)
 			if (n_filled < Ctrl->N.min_sectors) { 	/* Not minimum set of neighbors in all sectors, set to empty and goto next node */
 				n_almost++;
 				grd[ij] = (float)Ctrl->E.value;
+				free_node (grid_node[ij]);
 				continue;
 			}
 
@@ -490,17 +493,18 @@ int main (int argc, char **argv)
 				}
 			}
 			grd[ij] = (float)(grd_sum / weight_sum);
+			free_node (grid_node[ij]);
 		}
-		if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %6d\r", GMT_program, j);
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %10ld\r", GMT_program, j);
 	}
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %6d\n", GMT_program, j);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Gridded row %10ld\n", GMT_program, j);
 
 	GMT_err_fail (GMT_write_grd (Ctrl->G.file, &header, grd, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 
 	if (gmtdefs.verbose) {
 		sprintf (line, "%s)\n", gmtdefs.d_format);
-		fprintf (stderr, "%s: %d nodes were assigned an average value\n", GMT_program, n_set);
-		fprintf (stderr, "%s: %d nodes failed sector criteria and %d nodes had no neighbor points (all set to ", GMT_program, n_almost, n_none);
+		fprintf (stderr, "%s: %ld nodes were assigned an average value\n", GMT_program, n_set);
+		fprintf (stderr, "%s: %ld nodes failed sector criteria and %ld nodes had no neighbor points (all set to ", GMT_program, n_almost, n_none);
 		(GMT_is_dnan (Ctrl->E.value)) ? fprintf (stderr, "NaN)\n") : fprintf (stderr, line, Ctrl->E.value);
 	}
 
@@ -519,20 +523,22 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-struct NEARNEIGHBOR_NODE *add_new_node(int n)
+struct NEARNEIGHBOR_NODE *add_new_node (GMT_LONG n)
 {
-	struct NEARNEIGHBOR_NODE *new, *add_new_node(int n);
-
+	struct NEARNEIGHBOR_NODE *new;
+	
 	new = (struct NEARNEIGHBOR_NODE *) GMT_memory (VNULL, (size_t)1, sizeof (struct NEARNEIGHBOR_NODE), GMT_program);
 	new->distance = (float *) GMT_memory (VNULL, (size_t)n, sizeof (float), GMT_program);
-	new->datum = (int *) GMT_memory (VNULL, (size_t)n, sizeof (int), GMT_program);
+	new->datum = (GMT_LONG *) GMT_memory (VNULL, (size_t)n, sizeof (GMT_LONG), GMT_program);
 	while (n > 0) new->datum[--n] = -1;
 
 	return (new);
 }
 
-void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, double distance, int id)
+void assign_node (struct NEARNEIGHBOR_NODE **node, GMT_LONG n_sector, GMT_LONG sector, double distance, GMT_LONG id)
 {
+	struct NEARNEIGHBOR_NODE *add_new_node(GMT_LONG n);
+
 	/* Allocates node space if not already used and updates the value if closer to node */
 
 	if (!(*node)) *node = add_new_node (n_sector);
@@ -542,10 +548,20 @@ void assign_node (struct NEARNEIGHBOR_NODE **node, int n_sector, int sector, dou
 	}
 }
 
+void free_node (struct NEARNEIGHBOR_NODE *node)
+{
+	/* Frees allocated node space */
+
+	if (!node) return;	/* Nothing to do */
+	GMT_free ((void *)node->distance);
+	GMT_free ((void *)node->datum);
+	GMT_free ((void *)node);
+}
+
 void *New_Nearneighbor_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct NEARNEIGHBOR_CTRL *C;
 
-	C = (struct NEARNEIGHBOR_CTRL *) GMT_memory (VNULL, 1, sizeof (struct NEARNEIGHBOR_CTRL), "New_Nearneighbor_Ctrl");
+	C = (struct NEARNEIGHBOR_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct NEARNEIGHBOR_CTRL), "New_Nearneighbor_Ctrl");
 
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.sectors = 4;
