@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: ps2raster.c,v 1.50 2009/02/09 18:46:59 jluis Exp $
+ *	$Id: ps2raster.c,v 1.65 2009/07/11 03:16:31 guru Exp $
  *
  *	Copyright (c) 2006-2009 by J. Luis
  *	See README file for copying and redistribution conditions.
@@ -34,6 +34,9 @@
  *		17-NOV-2008 New options to control text and graphics antialiasing
  *		28-DEC-2008 Added -F to select a different output file name
  *		17-JAN-2009 Added -W option to output world files
+ *		05-MAR-2009 Added -Wg option to output a simple KML file
+ *		28-MAR-2009 PW: Extended the -Wg option to select different altitude modes and to
+ *					modify the document and overlay names
  */
 
 #define GMT_WITH_NO_PS
@@ -58,10 +61,17 @@
 #define GS_DEV_TIFG		10
 #define GS_DEV_BMPG		11
 
+#define KML_GROUND_ABS		0
+#define KML_GROUND_REL		1
+#define KML_ABS				2
+#define KML_SEAFLOOR_REL	3
+#define KML_SEAFLOOR_ABS	4
+
 struct PS2RASTER_CTRL {
-	struct A {	/* -A[u] [Adjust boundingbox] */
+	struct A {	/* -A[u][-] [Adjust boundingbox] */
 		BOOLEAN active;
 		BOOLEAN strip;	/* Remove the -U time-stamp */
+		BOOLEAN reset;	/* The -A- turns -A off, overriding any automode in effect */
 	} A;
 	struct C {	/* -C<option> */
 		BOOLEAN active;
@@ -72,7 +82,7 @@ struct PS2RASTER_CTRL {
 	} D;
 	struct E {	/* -E<resolution> */
 		BOOLEAN active;
-		int dpi;
+		GMT_LONG dpi;
 	} E;
 	struct F {	/* -F<out_name> */
 		BOOLEAN active;
@@ -95,7 +105,7 @@ struct PS2RASTER_CTRL {
 	struct Q {	/* -Q[g|t]<bits> */
 		BOOLEAN active;
 		BOOLEAN on[2];	/* [0] for graphics, [1] for text antialiasing */
-		int bits[2];
+		GMT_LONG bits[2];
 	} Q;
 	struct S {	/* -S */
 		BOOLEAN active;
@@ -103,7 +113,7 @@ struct PS2RASTER_CTRL {
 	struct T {	/* -T */
 		BOOLEAN active;
 		BOOLEAN eps;	/* TRUE if we want to make EPS (possibly in addition to another format) */
-		int device;
+		GMT_LONG device;
 	} T;
 	struct V2 {	/* -V */
 		BOOLEAN active;
@@ -111,25 +121,34 @@ struct PS2RASTER_CTRL {
 	struct W {	/* -W -- for world file production */
 		BOOLEAN active;
 		BOOLEAN warp;
+		BOOLEAN kml;
+		GMT_LONG mode;	/* 0 = clamp at ground, 1 is relative to ground, 2 is absolute 3 is relative to seafloor, 4 is clamp at seafloor */
+		GMT_LONG min_lod, max_lod;	/* minLodPixels and maxLodPixels settings */
+		GMT_LONG min_fade, max_fade;	/* minFadeExtent and maxFadeExtent settings */
+		char *doctitle;		/* Name of KML document */
+		char *overlayname;	/* Name of the image overlay */
+		char *URL;		/* URL of remote site */
+		double altitude;
 	} W;
 };
 
 int main (int argc, char **argv) {
 	BOOLEAN error = FALSE, grayscale = FALSE, found_proj = FALSE;
 
-	int i, j, k, len, r, n_files = 0, pos_file, pos_ext, nr, nl, pix_w = 0, pix_h = 0, mode;
-	int got_BB, got_HRBB, got_BBatend, file_has_HRBB, got_orient, landscape, setup;
+	GMT_LONG i, j, k, len, r, n_files = 0, pos_file, pos_ext, nr, nl, pix_w = 0, pix_h = 0, mode;
+	GMT_LONG got_BB, got_HRBB, got_BBatend, file_has_HRBB, got_orient, landscape, setup, i_unused = 0;
 	double xt, yt, w, h, x0 = 0.0, x1 = 612.0, y0 = 0.0, y1 = 828.0;
 	double	west = 0.0, east = 0.0, south = 0.0, north = 0.0;
 
 	size_t n_alloc = GMT_SMALL_CHUNK;
 
 	char ps_file[BUFSIZ];
-	char **ps_names, *no_U_file = NULL, *clean_PS_file = NULL, *tmp_file = NULL, *out_file = NULL, *BB_file = NULL;
+	char **ps_names, *no_U_file = NULL, *clean_PS_file = NULL, *tmp_file = NULL, out_file[BUFSIZ], *BB_file = NULL;
 	char line[BUFSIZ], c[BUFSIZ], *p, c1[20], c2[20], c3[20], c4[20], cmd[BUFSIZ], proj4_name[20], *quiet = NULL;
-	char *gs_params = NULL, *gs_BB = NULL, *anti = NULL, text[BUFSIZ], gs_extra[BUFSIZ], *proj4_cmd = NULL;
+	char *gs_params = NULL, *gs_BB = NULL, *anti = NULL, text[BUFSIZ], gs_extra[BUFSIZ], *proj4_cmd = NULL, *not_used = NULL;
 	char *device[N_GS_DEVICES] = {"", "pdfwrite", "jpeg", "png16m", "ppmraw", "tiff24nc", "bmp16m", "pngalpha", "jpeggray", "pnggray", "tiffgray", "bmpgray"};
 	char *ext[N_GS_DEVICES] = {".eps", ".pdf", ".jpg", ".png", ".ppm", ".tif", ".bmp", ".png", ".jpg", ".png", ".tif", ".bmp"};
+	char *RefLevel[5] = {"clampToGround", "relativeToGround", "absolute", "relativeToSeaFloor", "clampToSeaFloor"};
 #ifdef WIN32
 	char at_sign[2] = "@";
 #else
@@ -141,6 +160,7 @@ int main (int argc, char **argv) {
 	struct PS2RASTER_CTRL *Ctrl;
 
 	void *New_Ps2raster_Ctrl (), Free_Ps2raster_Ctrl (struct PS2RASTER_CTRL *C);
+	BOOLEAN parse_GE_settings (char *arg, struct PS2RASTER_CTRL *C);
 	
 	Ctrl = (struct PS2RASTER_CTRL *)New_Ps2raster_Ctrl ();	/* Allocate and initialize a new control structure */
 
@@ -164,6 +184,7 @@ int main (int argc, char **argv) {
 				case 'A':	/* Adjust BoundingBox */
 					Ctrl->A.active = TRUE;
 					if (argv[i][2] == 'u') Ctrl->A.strip = TRUE;
+					if (argv[i][strlen(argv[i])-1] == '-') Ctrl->A.reset = TRUE;
 					break;
 				case 'C':	/* Append extra custom GS options */
 					strcat (gs_extra, " ");		/* Append to list of extra GS options */
@@ -185,6 +206,7 @@ int main (int argc, char **argv) {
 				case 'F':	/* Set explicitly the output file name */
 					Ctrl->F.active = TRUE;
 					Ctrl->F.file = strdup (&argv[i][2]);
+					GMT_chop_ext(Ctrl->F.file);	/* Make sure file name has no extension */
 					break;
 				case 'L':	/* Give list of files to convert */
 					Ctrl->L.active = TRUE;
@@ -214,7 +236,7 @@ int main (int argc, char **argv) {
 					}
 					Ctrl->Q.on[mode] = TRUE;
 					Ctrl->Q.bits[mode] = (argv[i][3]) ? atoi (&argv[i][3]) : 4;
-					sprintf (text, " %s%d", anti, Ctrl->Q.bits[mode]);
+					sprintf (text, " %s%ld", anti, Ctrl->Q.bits[mode]);
 					strcat (gs_extra, text);	/* Append to list of extra GS options */
 					break;
 				case 'S':	/* Write the GS command to STDOUT */
@@ -222,7 +244,7 @@ int main (int argc, char **argv) {
 					break;
 				case 'T':	/* Select output format (optionally also request EPS) */
 					Ctrl->T.active = TRUE;
-					grayscale = ((j = strlen(argv[i])) > 3 && argv[i][j-1] == '-');
+					grayscale = ((j = (int)strlen(argv[i])) > 3 && argv[i][j-1] == '-');
 					for (j = 2; argv[i][j]; j++) {
 						switch (argv[i][j]) {
 							case 'e':	/* EPS */
@@ -262,11 +284,9 @@ int main (int argc, char **argv) {
 					Ctrl->V.active = TRUE;
 					break;
 				case 'W':	/* Save world file */
-					Ctrl->W.active = TRUE;
-					if (argv[i][2] == '+') Ctrl->W.warp = TRUE;
+					error = parse_GE_settings (&argv[i][2], Ctrl);
 					break;
-				/* Options not recognized */
-				default:
+				default:	/* Options not recognized */
 					error = TRUE;
 					break;
 			}
@@ -279,9 +299,9 @@ int main (int argc, char **argv) {
 
 	if (argc == 1 || GMT_give_synopsis_and_exit || error) {
 		fprintf (stderr,"ps2raster %s - Converts one or several [E]PS file(s) to raster formats using GhostScript.\n\n", GMT_VERSION);
-		fprintf(stderr,"usage: ps2raster <psfile1> <psfile2> <...> [-A[u]] [-D<dir>] [-E<resolution>] [-F<out_name>]\n");
-		fprintf(stderr,"       [-G<ghost_path>] [-L<listfile>] [-N] [-P] [-Q[g|t]1|2|4]\n");
-		fprintf(stderr,"       [-Tb|e|f|g|G|j|m|t] [-V] [-W[+]]\n\n");
+		fprintf(stderr,"usage: ps2raster <psfile1> <psfile2> <...> [-A[u][-]] [-D<dir>] [-E<resolution>] [-F<out_name>]\n");
+		fprintf(stderr,"       [-G<ghost_path>] [-L<listfile>] [-N] [-P] [-Q[g|t]1|2|4] [-Tb|e|f|g|G|j|m|t] [-V]\n");
+		fprintf(stderr,"       [-W[+g][+k][+t<title>][+n<name>][+a<mode>[<alt]][+l<lodmin>/<lodmax>][+f<minfade>/<maxfade>][+<URL>]]\n\n");
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr,"Works by modifying the page size in order that the resulting\n");
@@ -291,6 +311,7 @@ int main (int argc, char **argv) {
 		fprintf(stderr,"\n\tOPTIONS:\n");
 		fprintf (stderr,"\t-A Adjust the BoundingBox to the minimum required by the image contents\n");
 		fprintf (stderr,"\t   Append u to strip out time-stamps (produced by GMT -U options)\n");
+		fprintf (stderr,"\t   Append - to make sure -A is NOT activated by -W\n");
 		fprintf (stderr,"\t-D Sets an alternative output directory (which must exist) [Default is same directory as PS files]\n");
 		fprintf (stderr,"\t   Use -D. to place the output in the current directory.\n");
 		fprintf (stderr,"\t-E Set raster resolution in dpi [default = 720 for PDF, 300 for others]\n");
@@ -326,7 +347,7 @@ int main (int argc, char **argv) {
 		fprintf (stderr,"\t   For example, -Tef creates both an EPS and PDF file.\n");
 		fprintf (stderr,"\t-V Provides progress report [default is silent] and shows the gdal_translate\n");
 		fprintf (stderr,"\t   command, in case you want to use this program to create a geoTIFF file.\n");
-		fprintf (stderr,"\t-W Write a ESRI type world file suitable to make (e.g) .tif files be\n");
+		fprintf (stderr,"\t-W Write a ESRI type world file suitable to make (e.g.) .tif files be\n");
 		fprintf (stderr,"\t   recognized as geotiff by softwares that know how to do it. Be aware,\n");
 		fprintf (stderr,"\t   however, that different results are obtained depending on the image\n");
 		fprintf (stderr,"\t   contents and if the -B option has been used or not. The trouble with\n");
@@ -338,9 +359,27 @@ int main (int argc, char **argv) {
 		fprintf (stderr,"\t   The world file naming follows the convention of jamming a 'w' in the\n");
 		fprintf (stderr,"\t   file extension. So, if the output is tif (-Tt) the world file is a .tfw,\n");
 		fprintf (stderr,"\t   for jpeg a .jgw, and so on.\n");
-		fprintf (stderr,"\t   Use -W+ to do a system call to gdal_translate and produce a true geoTIFF\n");
+		fprintf (stderr,"\t   Use -W+g to do a system call to gdal_translate and produce a true geoTIFF\n");
 		fprintf (stderr,"\t   image right away. The output file will have the extension .tiff\n");
 		fprintf (stderr,"\t   See the man page for other 'gotchas'. Automatically sets -A -P.\n");
+		fprintf (stderr,"\t   Use -W+k to create a minimalist KML file that allows loading the image in\n");
+		fprintf (stderr,"\t   Google Earth. Note that for this option the image must be in geographical\n");
+		fprintf (stderr,"\t   coordinates. If not, a warning is issued but the KML file is created anyway.\n");
+		fprintf (stderr,"\t   Several modifiers allow you to specify the content in the KML file:\n");
+		fprintf (stderr,"\t   +t<doctitle> sets the document name [\"GMT KML Document\"]\n");
+		fprintf (stderr,"\t   +n<layername> sets the name of this particular layer [\"GMT Image Overlay\"]\n");
+		fprintf (stderr,"\t   +a<altmode>[<altitude>] sets the altitude mode of this layer, where\n");
+		fprintf (stderr,"\t      <altmode> is one of 5 recognized by Google Earth:\n");
+		fprintf (stderr,"\t      G clamped to the ground [Default]\n");
+		fprintf (stderr,"\t      g Append altitude (in m) relative to ground\n");
+		fprintf (stderr,"\t      A Append absolute altitude (in m)\n");
+		fprintf (stderr,"\t      s Append altitude (in m) relative to seafloor\n");
+		fprintf (stderr,"\t      S clamped to the seafloor\n");
+		fprintf (stderr,"\t   +l<minLOD>/<maxLOD>] sets Level Of Detail when layer should be active [always active]\n");
+		fprintf (stderr,"\t     Image goes inactive when there are fewer than minLOD pixels or more\n");
+		fprintf (stderr,"\t     than maxLOD pixels visible.  -1 means never invisible.\n");
+		fprintf (stderr,"\t   +f<minfade>/<maxfade>] sets distances over which we fade from opaque to transparent [no fading]\n");
+		fprintf (stderr,"\t   +u<URL> prepands this URL to the name of the image referenced in the KML [local file]\n");
 
 		exit (EXIT_FAILURE);
 	}
@@ -382,7 +421,16 @@ int main (int argc, char **argv) {
 		Ctrl->F.active = FALSE;
 	}
 
-	if (Ctrl->W.active) Ctrl->A.active = Ctrl->P.active = TRUE;
+	if (Ctrl->W.kml && !(Ctrl->T.device == GS_DEV_JPG || Ctrl->T.device == GS_DEV_JPGG || Ctrl->T.device == GS_DEV_TIF || 
+				Ctrl->T.device == GS_DEV_TIFG || Ctrl->T.device == GS_DEV_PNG ||
+				Ctrl->T.device == GS_DEV_TPNG || Ctrl->T.device == GS_DEV_PNGG) ) {
+		fprintf (stderr, "%s: WARNERROR: As far as we know selected raster type is unsuported by GE\n", GMT_program);
+	}
+
+	if (Ctrl->W.active) {	/* Implies -P and -A (unless -A- is set ) */
+		Ctrl->P.active = TRUE;
+		if (!Ctrl->A.reset) Ctrl->A.active = TRUE;
+	}
 
 	/* Use default DPI if not already set */
 	if (Ctrl->E.dpi <= 0) Ctrl->E.dpi = (Ctrl->T.device == GS_DEV_PDF) ? 720 : 300;
@@ -427,6 +475,7 @@ int main (int argc, char **argv) {
 	/* Loop over all input files */
 
 	for (k = 0; k < n_files; k++) {
+		memset ((void *)out_file, 0, BUFSIZ);
 		strcpy(ps_file,ps_names[k]);
 		if ((fp = fopen (ps_file, "r")) == NULL) {
 			fprintf (stderr, "%s: Cannot to open file %s\n", GMT_program, ps_file);
@@ -435,7 +484,7 @@ int main (int argc, char **argv) {
 		if (Ctrl->V.active) fprintf (stderr, "%s: Processing %s:", GMT_program, ps_file);
 
 		/* Check for JUNKY Illustrator EPS crap with mix of \r and \n */
-		fgets (line, BUFSIZ, fp);	/* Get a single line - this will be a full buffer if crap */
+		not_used = fgets (line, BUFSIZ, fp);	/* Get a single line - this will be a full buffer if crap */
 		for (j = nr = nl = 0; j < (int)strlen (line); j++) {
 			if (line[j] == '\r') nr++;	/* Number of CR found */
 			if (line[j] == '\n') nl++;	/* Number of LF found */
@@ -480,7 +529,7 @@ int main (int argc, char **argv) {
 
 		got_BB = got_HRBB = file_has_HRBB = got_BBatend = got_orient = landscape = setup = FALSE;
 
-		len = strlen(ps_file);
+		len = (GMT_LONG)strlen(ps_file);
 		j = len - 1;
 		pos_file = -1;
 		pos_ext = -1;	/* In case file has no extension */
@@ -497,10 +546,10 @@ int main (int argc, char **argv) {
 			char *psfile_to_use;
 			if (Ctrl->V.active) fprintf (stderr, " Find HiResBoundingBox ");
 			BB_file = (char *) GMT_memory (VNULL, (size_t)BUFSIZ, sizeof(char), GMT_program);
-			sprintf (BB_file, "%s%cps2raster_%dc.bb", Ctrl->D.dir, DIR_DELIM, (int)getpid());
+			sprintf (BB_file, "%s%cps2raster_%ldc.bb", Ctrl->D.dir, DIR_DELIM, (GMT_LONG)getpid());
 			psfile_to_use = (Ctrl->A.strip) ? no_U_file : ((clean_PS_file) ? clean_PS_file : ps_file);
 			sprintf (cmd, "%s%s %s %s 2> %s", at_sign, Ctrl->G.file, gs_BB, psfile_to_use, BB_file);
-			system (cmd);		/* Execute the command that computes the tight BB */
+			i_unused = system (cmd);		/* Execute the command that computes the tight BB */
 			if ((fpb = fopen (BB_file, "r")) == NULL) {
 				fprintf (stderr, "%s: Unable to open file %s\n", GMT_program, BB_file);
 				exit (EXIT_FAILURE);
@@ -537,7 +586,7 @@ int main (int argc, char **argv) {
 			}
 		}
 		else {
-			sprintf (tmp_file, "%s%cps2raster_%dd.eps", Ctrl->D.dir, DIR_DELIM, (int)getpid());
+			sprintf (tmp_file, "%s%cps2raster_%ldd.eps", Ctrl->D.dir, DIR_DELIM, (GMT_LONG)getpid());
 			if ((fpo = fopen (tmp_file, "w+")) == NULL) {
 				fprintf (stderr, "%s: Unable to create a temporary file\n", GMT_program);
 				exit (EXIT_FAILURE);
@@ -614,6 +663,12 @@ int main (int argc, char **argv) {
 					sscanf (&line[8], "%s %s %s %s %s %s %s %s %s",proj4_name,xx1,xx2,yy1,yy2,c1,c2,c3,c4);
 					west = atof (c1);		east = atof (c2);
 					south = atof (c3);		north = atof (c4);
+					project_info.w = west = atof (xx1);		project_info.e = east = atof (xx2);
+					if (project_info.w > 180.0 && project_info.e > 180.0) {
+						project_info.w -= 360.0;
+						project_info.e -= 360.0;
+					}
+					project_info.s = south = atof (yy1);	project_info.n = north = atof (yy2);
 					found_proj = TRUE;
 					if ((ptmp = strstr(&line[2], "+proj")) != NULL) {  /* Search for the +proj in the comment line */
 						proj4_cmd = strdup(&line[(int)(ptmp - &line[0])]);
@@ -623,12 +678,30 @@ int main (int argc, char **argv) {
 						!strcmp(proj4_name,"eqc") ) {		/* Linear case, use original coords */
 						west  = atof(xx1);		east  = atof(xx2);
 						south = atof(yy1);		north = atof(yy2);
+						/* One further test. +xy was found, but have we geog coords? Check that */
+						if (!strcmp(proj4_name,"xy") && 
+								(west >= -180) && ((east <= 360) && ((east - west) <= 360)) &&
+								(south >= -90) && (north <= 90) ) {
+							proj4_cmd = strdup("latlon");
+							fprintf (stderr, "%s: WARNING: An unknown projection setting was found but since "
+									"image coordinates seam to be geographical, a linear transformation "
+									"will be used.\n", GMT_program);
+						}
+						else if (!strcmp(proj4_name,"xy") && Ctrl->W.warp) {	/* Do not operate on a twice unknown setting */
+							fprintf (stderr, "%s: WARNERROR: You requested an automatic geotiff generation, but "
+									"no recognized projection was used for the PS creation. Maybe you "
+									"should reconsider your wishes, or prove that I'm wrong.\n", GMT_program); 
+						}
+					}
+					else if (Ctrl->W.kml) {
+						fprintf (stderr, "%s: WARNERROR: To GE images must be in geographical coords. Very likely "
+									"this won't work as you wish inside GE.\n", GMT_program); 
 					}
 				}
 			}
 			sscanf (line, "%s",c);
 			if (!strncmp(c, "%%BoundingBox:", (size_t)14)) {
-				if (got_BB) fprintf (fpo, "%%%%BoundingBox: 0 0 %d %d\n", (int)ceil(w), (int)ceil(h));
+				if (got_BB) fprintf (fpo, "%%%%BoundingBox: 0 0 %ld %ld\n", (GMT_LONG)ceil(w), (GMT_LONG)ceil(h));
 				got_BB = FALSE;
 				if (file_has_HRBB) continue;	/* High-res BB will be put elsewhere */
 				if (got_HRBB) fprintf (fpo, "%%%%HiResBoundingBox: 0 0 %g %g\n", w, h);
@@ -651,7 +724,7 @@ int main (int argc, char **argv) {
 				setup=FALSE;
 			else if (!strncmp(c, "%%EndComments", (size_t)13)) {
 				fprintf (fpo, "%s", line);
-				if (r != 0) fprintf (fpo, "%d rotate\n", r);
+				if (r != 0) fprintf (fpo, "%ld rotate\n", r);
 				if (!GMT_IS_ZERO(xt) || !GMT_IS_ZERO(yt)) fprintf (fpo, "%g %g translate\n", xt, yt);
 				xt = yt = 0.0;
 				r = 0;
@@ -672,19 +745,18 @@ int main (int argc, char **argv) {
 			if (Ctrl->V.active) fprintf (stderr, " Convert to %s...", tag);
 
 			if (!Ctrl->F.active) {
-				out_file = (char *) GMT_memory (VNULL, (size_t)BUFSIZ, sizeof(char), GMT_program);
 				if (Ctrl->D.active) sprintf (out_file, "%s%c", Ctrl->D.dir, DIR_DELIM);		/* Use specified output directory */
 				strncat (out_file, &ps_file[pos_file], (size_t)(pos_ext - pos_file));
 			}
 			else
-				out_file = Ctrl->F.file;
+				strcpy (out_file, Ctrl->F.file);
 			strcat (out_file, ext[Ctrl->T.device]);
-			pix_w = (int)ceil (w * Ctrl->E.dpi / 72.0);
-			pix_h = (int)ceil (h * Ctrl->E.dpi / 72.0);
-			sprintf (cmd, "%s%s %s %s -sDEVICE=%s -g%dx%d -r%d -sOutputFile=%s -f%s", 
+			pix_w = (GMT_LONG)ceil (w * Ctrl->E.dpi / 72.0);
+			pix_h = (GMT_LONG)ceil (h * Ctrl->E.dpi / 72.0);
+			sprintf (cmd, "%s%s %s %s -sDEVICE=%s -g%ldx%ld -r%ld -sOutputFile=%s -f%s", 
 				at_sign, Ctrl->G.file, gs_params, gs_extra, device[Ctrl->T.device],
 				pix_w, pix_h, Ctrl->E.dpi, out_file, tmp_file);
-			system (cmd);		/* Execute the GhostScript command */
+			i_unused = system (cmd);		/* Execute the GhostScript command */
 			if (Ctrl->S.active) fprintf (stdout, "%s\n", cmd);
 		}
 		if (Ctrl->V.active) fprintf (stderr, " Done.\n");
@@ -696,13 +768,13 @@ int main (int argc, char **argv) {
 		if (clean_PS_file) GMT_free ((void *)clean_PS_file);
 		if (no_U_file) GMT_free ((void *)no_U_file);
 
-		if ( Ctrl->W.active && found_proj ) {	/* Write a world file */
+		if ( Ctrl->W.active && found_proj && !Ctrl->W.kml ) {	/* Write a world file */
 			double x_inc, y_inc;
 			char *world_file, *wext, *s;
 
 			x_inc = (east  - west)  / pix_w; 
 			y_inc = (north - south) / pix_h; 
-			if (Ctrl->V.active) fprintf(stderr, "width = %d\theight = %d\tX res = %f\tY res = %f\n", pix_w, pix_h, x_inc, y_inc);
+			if (Ctrl->V.active) fprintf(stderr, "width = %ld\theight = %ld\tX res = %f\tY res = %f\n", pix_w, pix_h, x_inc, y_inc);
 
 			/* West and North of the world file contain the coordinates of the center of the pixel
 			   but our current values are of the NW corner of the pixel (pixel registration). So 
@@ -713,7 +785,7 @@ int main (int argc, char **argv) {
 			world_file = (char *) GMT_memory (VNULL, (size_t)BUFSIZ, sizeof(char), GMT_program);
 			if (Ctrl->D.active) sprintf (world_file, "%s%c", Ctrl->D.dir, DIR_DELIM);	/* Use specified output directory */
 			if (Ctrl->F.active) {		/* Must rip the raster file extension before adding the world one */
-				for (i = strlen(out_file) - 1; i > 0; i--) {
+				for (i = (GMT_LONG)strlen(out_file) - 1; i > 0; i--) {
 					if (out_file[i] == '.') { 	/* Beginning of file extension */
 						pos_ext = i;
 						break;
@@ -734,7 +806,7 @@ int main (int argc, char **argv) {
 				fprintf (stderr, "%s: Unable to open file %s for writing\n", GMT_program, world_file);
 			}
 			else {
-				fprintf(fpw, "%.12f\n0.0\n0.0\n%.12f\n%.12f\n%.12f", x_inc, -y_inc, west, north);
+				fprintf (fpw, "%.12f\n0.0\n0.0\n%.12f\n%.12f\n%.12f", x_inc, -y_inc, west, north);
 				fclose(fpw);
 				if (Ctrl->V.active) {
 					fprintf(stderr, "Wrote world file %s\n", world_file);
@@ -748,7 +820,7 @@ int main (int argc, char **argv) {
 			if (Ctrl->W.warp && proj4_cmd && proj4_cmd[1] == 'p') {	/* We got a usable Proj4 string. Run it (if gdal is around) */
 				/* The true geotiff file will have the same base name plus a .tiff extension.
 				   We will reuse the world_file variable because all it is need is to replace the extension */
-				for (i = strlen(world_file) - 1; i > 0; i--) {
+				for (i = (GMT_LONG)strlen(world_file) - 1; i > 0; i--) {
 					if (world_file[i] == '.') { 	/* Beginning of file extension */
 						pos_ext = i;
 						break;
@@ -770,15 +842,84 @@ int main (int argc, char **argv) {
 						proj4_cmd, quiet, out_file, world_file); 
 #endif
 				free(proj4_cmd);
-				system (cmd);		/* Execute the gdal_translate command */
+				i_unused = system (cmd);		/* Execute the gdal_translate command */
 				if (Ctrl->V.active) fprintf(stderr, "\nThe gdal_translate command: \n%s\n", cmd);
 			}
 			else if (Ctrl->W.warp && !proj4_cmd)
 				fprintf (stderr, "%s: Could not find the Proj4 command in the PS file. No conversion performed.\n", GMT_program);
 
 			GMT_free ((void *)world_file);	
-			if (!Ctrl->F.active) GMT_free ((void *)out_file);
 		}
+
+		else if ( Ctrl->W.kml ) {	/* Write a basic kml file */
+			char *kml_file;
+			kml_file = (char *) GMT_memory (VNULL, (size_t)BUFSIZ, sizeof(char), GMT_program);
+			if (Ctrl->D.active) sprintf (kml_file, "%s%c", Ctrl->D.dir, DIR_DELIM);	/* Use specified output directory */
+			if (Ctrl->F.active) {		/* Must rip the raster file extension before adding the kml one */
+				for (i = (GMT_LONG)strlen(out_file) - 1; i > 0; i--) {
+					if (out_file[i] == '.') { 	/* Beginning of file extension */
+						pos_ext = i;
+						break;
+					}
+				}
+				out_file[pos_ext] = '\0';
+				strcat(kml_file, out_file);
+				out_file[pos_ext] = '.';	/* Reset the extension */
+			}
+			else
+				strncat (kml_file, &ps_file[pos_file], (size_t)(pos_ext - pos_file));
+
+			strcat (kml_file, ".kml");
+
+			if ((fpw = fopen (kml_file, "w")) == NULL) {
+				fprintf (stderr, "%s: Unable to open file %s for writing\n", GMT_program, kml_file);
+			}
+			else {
+				fprintf (fpw, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
+				fprintf (fpw, "<kml xmlns=\"http://earth.google.com/kml/2.1\">\n");
+				fprintf (fpw, "<Document>\n\t<name>%s</name>\n", Ctrl->W.doctitle);
+				fprintf (fpw, "\t<GroundOverlay>\n\t\t<name>%s</name>\n", Ctrl->W.overlayname);
+				fprintf (fpw, "\t\t<Icon>\n");
+				fprintf (fpw, "\t\t\t<href>");
+				if (Ctrl->W.URL) fprintf (fpw, "%s/", Ctrl->W.URL);
+				fprintf (fpw, "%s</href>\n", out_file);
+				fprintf (fpw, "\t\t</Icon>\n");
+				fprintf (fpw, "\t\t<altitudeMode>%s</altitudeMode>\n", RefLevel[Ctrl->W.mode]);
+				if (Ctrl->W.mode > KML_GROUND_ABS && Ctrl->W.mode < KML_SEAFLOOR_ABS)
+					fprintf (fpw, "\t\t<altitude>%g</altitude>\n", Ctrl->W.altitude);
+				fprintf (fpw, "\t\t<LatLonBox>\n");
+				fprintf (fpw, "\t\t\t<north>%f</north>\n", project_info.n);
+				fprintf (fpw, "\t\t\t<south>%f</south>\n", project_info.s);
+				fprintf (fpw, "\t\t\t<east>%f</east>\n", project_info.e);
+				fprintf (fpw, "\t\t\t<west>%f</west>\n", project_info.w);
+				fprintf (fpw, "\t\t</LatLonBox>\n");
+				fprintf (fpw, "\t\t<Region>\n");
+				fprintf (fpw, "\t\t<LatLonAltBox>\n");
+				fprintf (fpw, "\t\t\t<north>%f</north>\n", north);
+				fprintf (fpw, "\t\t\t<south>%f</south>\n", south);
+				fprintf (fpw, "\t\t\t<east>%f</east>\n", east);
+				fprintf (fpw, "\t\t\t<west>%f</west>\n", west);
+				fprintf (fpw, "\t\t</LatLonAltBox>\n");
+				if (Ctrl->W.min_lod != Ctrl->W.max_lod) {	/* Control layer visibility */
+ 					fprintf (fpw, "\t\t<Lod>\n");
+ 					fprintf (fpw, "\t\t\t<minLodPixels>%ld</minLodPixels>\n", Ctrl->W.min_lod);
+ 					fprintf (fpw, "\t\t\t<maxLodPixels>%ld</maxLodPixels>\n", Ctrl->W.max_lod);
+					if (Ctrl->W.min_fade) fprintf (fpw, "\t\t\t<minFadeExtent>%ld</minFadeExtent>\n", Ctrl->W.min_fade);
+	 				if (Ctrl->W.max_fade) fprintf (fpw, "\t\t\t<maxFadeExtent>%ld</maxFadeExtent>\n", Ctrl->W.max_fade);
+ 					fprintf (fpw, "\t\t</Lod>\n");
+				}
+				fprintf (fpw, "\t\t</Region>\n");
+				fprintf (fpw, "\t</GroundOverlay>\n");
+				fprintf (fpw, "</Document>\n</kml>\n");
+				fclose(fpw);
+				if (Ctrl->V.active) {
+					fprintf(stderr, "Wrote KML file %s\n", kml_file);
+				}
+			
+			}
+			GMT_free ((void *)kml_file);	
+		}
+
 		else if (Ctrl->W.active && !found_proj)
 			fprintf (stderr, "%s: Could not find the 'PROJ' tag in the PS file. No world file created.\n", GMT_program);
 	}
@@ -790,6 +931,77 @@ int main (int argc, char **argv) {
 	exit (EXIT_SUCCESS);
 }
 
+BOOLEAN parse_GE_settings (char *arg, struct PS2RASTER_CTRL *C)
+{
+	/* Syntax: -W[+g][+k][+t<doctitle>][+n<layername>][+a<altmode>][+l<lodmin>/<lodmax>] */
+	
+	BOOLEAN error = FALSE;
+	GMT_LONG pos = 0;
+	char txt[BUFSIZ], p[BUFSIZ];
+	
+	C->W.active = TRUE;
+	strcpy (txt, arg);
+	while (!error && (GMT_strtok (txt, "+", &pos, p))) {
+		switch (p[0]) {
+			case 'a':	/* Altitude setting */
+				switch (p[1]) {	/* Check which altitude mode we selected */
+					case 'G':
+						C->W.mode = KML_GROUND_ABS;
+						break;
+					case 'g':
+						C->W.mode = KML_GROUND_REL;
+						C->W.altitude = atof (&p[2]);
+						break;
+					case 'A':
+						C->W.mode = KML_ABS;
+						C->W.altitude = atof (&p[2]);
+						break;
+					case 's':
+						C->W.mode = KML_SEAFLOOR_REL;
+						C->W.altitude = atof (&p[2]);
+						break;
+					case 'S':
+						C->W.mode = KML_SEAFLOOR_ABS;
+						break;
+					default:
+						fprintf (stderr, "%s: GMT ERROR -W+a<mode>[par]: Unrecognized altitude mode %c\n", GMT_program, p[1]);
+						error = TRUE;
+						break;
+				}
+				break;
+			case 'f':	/* Set fading options in KML */
+				sscanf (&p[1], "%ld/%ld", &C->W.min_fade, &C->W.max_fade);
+				break;
+			case 'g':	/* Use gdal to make geotiff */
+				C->W.warp = TRUE;
+				break;
+			case 'k':	/* Produce a KML file */
+				C->W.kml = TRUE;
+				break;
+			case 'l':	/* Set KML level of detail for image */
+				sscanf (&p[1], "%ld/%ld", &C->W.min_lod, &C->W.max_lod);
+				break;
+			case 'n':	/* Set KML document layer name */
+				if (C->W.overlayname) free (C->W.overlayname);	/* Already set, free then reset */
+				C->W.overlayname = strdup (&p[1]);
+				break;
+			case 't':	/* Set KML document title */
+				if (C->W.doctitle) free (C->W.doctitle);	/* Already set, free then reset */
+				C->W.doctitle = strdup (&p[1]);
+				break;
+			case 'u':	/* Specify a remote address for image */
+				if (C->W.URL) free (C->W.URL);	/* Already set, free then reset */
+				C->W.URL = strdup (&p[1]);
+				break;
+			default:
+				fprintf (stderr, "%s: GMT ERROR -W+<opt>: Unrecognized option selection %c\n", GMT_program, p[1]);
+				error = TRUE;
+				break;
+		}
+	}
+	return (error);
+}
+
 void *New_Ps2raster_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PS2RASTER_CTRL *C;
 
@@ -803,6 +1015,9 @@ void *New_Ps2raster_Ctrl () {	/* Allocate and initialize a new control structure
 #endif
 	C->D.dir = strdup (".");
 
+	C->W.doctitle = strdup ("GMT KML Document");
+	C->W.overlayname = strdup ("GMT Image Overlay");
+
 	return ((void *)C);
 }
 
@@ -811,5 +1026,8 @@ void Free_Ps2raster_Ctrl (struct PS2RASTER_CTRL *C) {	/* Deallocate control stru
 	if (C->F.file) free ((void *)C->F.file);
 	if (C->G.file) free ((void *)C->G.file);
 	if (C->L.file) free ((void *)C->L.file);
+	free ((void *)C->W.doctitle);
+	free ((void *)C->W.overlayname);
+	if (C->W.URL) free ((void *)C->W.URL);
 	GMT_free ((void *)C);
 }
