@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pscoast.c,v 1.73 2007/09/29 23:44:14 remko Exp $
+ *	$Id: pscoast.c,v 1.96 2009/06/05 00:25:11 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -54,19 +54,21 @@
 #include "gmt.h"
 #include "pslib.h"
 
+#define LAKE	0
+#define RIVER	1
+
 struct PSCOAST_CTRL {
 	struct A {	/* -A<min_area>[/<min_level>/<max_level>] */
 		BOOLEAN active;
-		int low;	/* Lowest hierarchical level to use [0] */
-		int high;	/* Highest hierarchical level to use [4] */
-		double area;	/* Area of smallest geographical feature to include [0] */
+		struct GMT_SHORE_SELECT info;
 	} A;
 	struct C {	/* -C<fill> */
 		BOOLEAN active;
-		struct GMT_FILL fill;
+		struct GMT_FILL fill[2];	/* lake and riverlake fill */
 	} C;
 	struct D {	/* -D<resolution> */
 		BOOLEAN active;
+		BOOLEAN force;	/* if TRUE, select next highest level if current set is not avaialble */
 		char set;	/* One of f, h, i, l, c */
 	} D;
 	struct E {	/* -Eazim/elev */
@@ -80,7 +82,7 @@ struct PSCOAST_CTRL {
 	} G;
 	struct I {	/* -I<feature>[/<pen>] */
 		BOOLEAN active;
-		BOOLEAN use[GMT_N_RLEVELS];
+		GMT_LONG use[GMT_N_RLEVELS];
 		struct GMT_PEN pen[GMT_N_RLEVELS];
 	} I;
 	struct L {	/* -L */
@@ -89,7 +91,7 @@ struct PSCOAST_CTRL {
 	} L;
 	struct N {	/* -N<feature>[/<pen>] */
 		BOOLEAN active;
-		BOOLEAN use[GMT_N_BLEVELS];
+		GMT_LONG use[GMT_N_BLEVELS];
 		struct GMT_PEN pen[GMT_N_BLEVELS];
 	} N;
 	struct Q {	/* -Q */
@@ -104,9 +106,10 @@ struct PSCOAST_CTRL {
 		BOOLEAN active;
 		struct GMT_MAP_ROSE item;
 	} T;
-	struct W {	/* -W<pen> */
+	struct W {	/* -W[<feature>/]<pen> */
 		BOOLEAN active;
-		struct GMT_PEN pen;
+		GMT_LONG use[GMT_MAX_GSHHS_LEVEL];
+		struct GMT_PEN pen[GMT_MAX_GSHHS_LEVEL];
 	} W;
 	struct Z {	/* -Z<z_level> */
 		BOOLEAN active;
@@ -122,14 +125,16 @@ struct PSCOAST_CTRL {
 
 int main (int argc, char **argv)
 {
-	int i, n, np, ind, bin, base, anti_bin = -1, level, flag, np_new, k, last_k, err;
-	int level_to_be_painted = 0, direction, start_direction, stop_direction;
+	int i, np, ind, bin, base, anti_bin = -1, level, flag, np_new, k, last_k, err;
+	int level_to_be_painted = 0, direction, start_direction, stop_direction, last_pen_level;
 	int n_blevels = 0, n_rlevels = 0, bin_trouble, lowest_level = 1, n_pieces = 0;
 
+	GMT_LONG n;
+	
 	BOOLEAN	error = FALSE, shift = FALSE, need_coast_base, recursive, fill_ocean, fill_land;
 	BOOLEAN	greenwich = FALSE, possibly_donut_hell = FALSE, fill_in_use = FALSE;
 	BOOLEAN clobber_background, paint_polygons = FALSE, donut, dumping = FALSE;
-	BOOLEAN donut_hell = FALSE, world_map_save, did_clip = FALSE, clipping = FALSE;
+	BOOLEAN donut_hell = FALSE, world_map_save, clipping = FALSE;
 
 	double	west = 0.0, east = 0.0, south = 0.0, north = 0.0, bin_x[5], bin_y[5], out[2];
 	double west_border, east_border, anti_lon = 0.0, anti_lat = -90.0, edge = 720.0;
@@ -138,7 +143,7 @@ int main (int argc, char **argv)
 	char *string, comment[GMT_LONG_TEXT];
 	char *shore_resolution[5] = {"full", "high", "intermediate", "low", "crude"};
 
-	struct GMT_FILL fill[5];	/* Colors for (0) water, (1) land, (2) lakes, (3) islands in lakes, (4) lakes in islands in lakes */
+	struct GMT_FILL fill[6];	/* Colors for (0) water, (1) land, (2) lakes, (3) islands in lakes, (4) lakes in islands in lakes, (5) riverlakes */
 	struct GMT_PEN pen;
 	struct GMT_SHORE c;
 	struct GMT_BR b, r;
@@ -164,6 +169,7 @@ int main (int argc, char **argv)
 				case 'B':	/* Get tickmark information */
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -175,6 +181,7 @@ int main (int argc, char **argv)
 				case 'y':
 				case 'c':
 				case 'b':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
 					break;
@@ -183,23 +190,33 @@ int main (int argc, char **argv)
 
 				case 'A':
 					Ctrl->A.active = TRUE;
-					n = sscanf (&argv[i][2], "%lf/%d/%d", &Ctrl->A.area, &Ctrl->A.low, &Ctrl->A.high);
-					if (n == 1) Ctrl->A.low = 0, Ctrl->A.high = GMT_MAX_GSHHS_LEVEL;
+					GMT_set_levels (&argv[i][2], &Ctrl->A.info);
 					break;
-				case 'C':	/* Lake color */
+				case 'C':	/* Lake colors */
 					Ctrl->C.active = TRUE;
-					if (argv[i][2] && GMT_getfill (&argv[i][2], &Ctrl->C.fill)) {
-						GMT_fill_syntax ('C', " ");
-						error++;
+					if ((argv[i][2] == 'l' || argv[i][2] == 'r') && argv[i][3] == '/') {	/* Specific lake or river-lake fill */
+						k = (argv[i][2] == 'l') ? LAKE : RIVER;
+						if (argv[i][4] && GMT_getfill (&argv[i][4], &Ctrl->C.fill[k])) {
+							GMT_fill_syntax ('C', " ");
+							error++;
+						}
+					}
+					else if (argv[i][2]) {
+						if (GMT_getfill (&argv[i][2], &Ctrl->C.fill[LAKE])) {
+							GMT_fill_syntax ('C', " ");
+							error++;
+						}
+						Ctrl->C.fill[RIVER] = Ctrl->C.fill[LAKE];
 					}
 					break;
 				case 'D':
 					Ctrl->D.active = TRUE;
 					Ctrl->D.set = argv[i][2];
+					if (argv[i][3] == '+') Ctrl->D.force = TRUE;
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
-					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
+					error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'G':		/* Set Gray shade, pattern, or clipping */
 					Ctrl->G.active = TRUE;
@@ -214,9 +231,6 @@ int main (int argc, char **argv)
 					Ctrl->L.active = TRUE;
 					error += GMT_getscale (&argv[i][2], &Ctrl->L.item);
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
 					if (!argv[i][2]) {
@@ -274,19 +288,19 @@ int main (int argc, char **argv)
 							for (k = 0; k < GMT_N_RLEVELS; k++) Ctrl->I.pen[k] = pen;
 							break;
 						case 'r':
-							for (k = 0; k < 4; k++) Ctrl->I.use[k] = TRUE;
-							for (k = 0; k < 4; k++) Ctrl->I.pen[k] = pen;
+							for (k = 0; k < GMT_RIV_INTERMITTENT; k++) Ctrl->I.use[k] = TRUE;
+							for (k = 0; k < GMT_RIV_INTERMITTENT; k++) Ctrl->I.pen[k] = pen;
 							break;
 						case 'i':
-							for (k = 4; k < 7; k++) Ctrl->I.use[k] = TRUE;
-							for (k = 4; k < 7; k++) Ctrl->I.pen[k] = pen;
+							for (k = GMT_RIV_INTERMITTENT; k < GMT_RIV_CANALS; k++) Ctrl->I.use[k] = TRUE;
+							for (k = GMT_RIV_INTERMITTENT; k < GMT_RIV_CANALS; k++) Ctrl->I.pen[k] = pen;
 							break;
 						case 'c':
-							for (k = 7; k < 10; k++) Ctrl->I.use[k] = TRUE;
-							for (k = 7; k < 10; k++) Ctrl->I.pen[k] = pen;
+							for (k = GMT_RIV_CANALS; k < GMT_N_RLEVELS; k++) Ctrl->I.use[k] = TRUE;
+							for (k = GMT_RIV_CANALS; k < GMT_N_RLEVELS; k++) Ctrl->I.pen[k] = pen;
 							break;
 						default:
-							k = atoi (&argv[i][2]) - 1;
+							k = atoi (&argv[i][2]);
 							if (k < 0 || k >= GMT_N_RLEVELS) {
 								fprintf (stderr, "%s: GMT SYNTAX ERROR -I option: Feature not in list!\n", GMT_program);
 								error++;
@@ -313,10 +327,30 @@ int main (int argc, char **argv)
 					error += GMT_getrose (&argv[i][2], &Ctrl->T.item);
 					break;
 				case 'W':
-					Ctrl->W.active = TRUE;
-					if (argv[i][2] && GMT_getpen (&argv[i][2], &Ctrl->W.pen)) {
-						GMT_pen_syntax ('W', " ");
-						error++;
+					Ctrl->W.active = TRUE;	/* Want to draw shorelines */
+					if ((argv[i][2] >= '1' && argv[i][2] <= '4') && argv[i][3] == '/') {	/* Specific pen for this feature */
+						k = (GMT_LONG)(argv[i][2] - '1');
+						if (argv[i][4] && GMT_getpen (&argv[i][4], &Ctrl->W.pen[k])) {
+							GMT_pen_syntax ('W', " ");
+							error++;
+						}
+						Ctrl->W.use[k] = TRUE;
+					}
+					else if (argv[i][2]) {	/* Same pen for all features */
+						Ctrl->W.use[0] = TRUE;
+						if (GMT_getpen (&argv[i][2], &Ctrl->W.pen[0])) {
+							GMT_pen_syntax ('W', " ");
+							error++;
+						}
+						else {
+							for (k = 1; k < GMT_MAX_GSHHS_LEVEL; k++) {
+								Ctrl->W.pen[k] = Ctrl->W.pen[0];
+								Ctrl->W.use[k] = TRUE;
+							}
+						}
+					}
+					else {	/* Accept default pen for all features */
+						for (k = 0; k < GMT_MAX_GSHHS_LEVEL; k++) Ctrl->W.use[k] = TRUE;
 					}
 					break;
 				case 'Q':
@@ -349,12 +383,12 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
 		fprintf (stderr,"pscoast %s - Plot continents, shorelines, rivers, and borders on maps\n\n", GMT_VERSION);
-		fprintf (stderr,"usage: pscoast %s %s\n\t[-A<min_area>[/<min_level>/<max_level>]] [%s] [-C[<fill>]]\n", GMT_B_OPT, GMT_J_OPT, GMT_Rgeoz_OPT);
-		fprintf (stderr, "\t[-D<resolution>] [-Eaz/el] [-G[<fill>]] [-I<feature>[/<pen>]] [%s] [-K]\n", GMT_Jz_OPT);
-		fprintf (stderr, "\t[%s]\n", GMT_SCALE);
-		fprintf (stderr, "\t[%s] [-N<feature>[/<pen>]] [-O] [-P] [-Q] [-S<fill>]\n", GMT_Mo_OPT);
+		fprintf (stderr,"usage: pscoast %s %s [%s] [%s]\n", GMT_B_OPT, GMT_J_OPT, GMT_A_OPT, GMT_Rgeoz_OPT);
+		fprintf (stderr, "\t[-C[<feature>/]<fill>] [-D<resolution>][+] [%s] [-G[<fill>]] [-I<feature>[/<pen>]]\n", GMT_E_OPT);
+		fprintf (stderr, "\t[%s] [-K] [%s]\n", GMT_Jz_OPT, GMT_SCALE);
+		fprintf (stderr, "\t[-N<feature>[/<pen>]] [-O] [-P] [-Q] [-S<fill>]\n");
 		fprintf (stderr, "\t[%s] [%s]\n", GMT_TROSE, GMT_U_OPT);
-		fprintf (stderr, "\t[-V] [-W[<pen>]] [%s] [%s] [-Z<zlevel>] [%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT, GMT_bo_OPT, GMT_c_OPT);
+		fprintf (stderr, "\t[-V] [-W[<feature>/][<pen>]] [%s] [%s] [-Z<zlevel>] [%s] [%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT, GMT_bo_OPT, GMT_c_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -362,21 +396,25 @@ int main (int argc, char **argv)
 		GMT_explain_option ('Z');
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-A features smaller than <min_area> (in km^2) or of levels (0-4) outside the min-max levels\n");
-		fprintf (stderr, "\t   will be skipped [0/4 (4 means lake inside island inside lake)]\n");
+		GMT_GSHHS_syntax ('A', "Place limits on coastline features from the GSHHS data base.");
 		GMT_explain_option ('b');
-		GMT_fill_syntax ('C', "Sets separate color for lakes [Default is same as ocean].");
+		GMT_fill_syntax ('C', "Sets separate color for lakes and riverlakes [Default is same as ocean].");
+		fprintf (stderr, "\t   Alternatively, set custom fills below.  Repeat the -C option as needed\n");
+		fprintf (stderr, "\t      l = Lakes\n");
+		fprintf (stderr, "\t      r = River-lakes\n");
 		fprintf (stderr, "\t-D Choose one of the following resolutions:\n");
 		fprintf (stderr, "\t   f - full resolution (may be very slow for large regions)\n");
 		fprintf (stderr, "\t   h - high resolution (may be slow for large regions)\n");
 		fprintf (stderr, "\t   i - intermediate resolution\n");
 		fprintf (stderr, "\t   l - low resolution [Default]\n");
 		fprintf (stderr, "\t   c - crude resolution, for busy plots that need crude continent outlines only\n");
-		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t   Append + to use a lower resolution should the chosen one not be available [abort].\n");
+		GMT_explain_option ('E');
 		GMT_fill_syntax ('G', "Paint or clip \"dry\" areas.");
 		fprintf (stderr, "\t   6) c to issue clip paths for land areas.\n");
 		GMT_pen_syntax ('I', "draws rivers.  Specify feature and optionally append pen [Default is solid line of unit thickness].");
 		fprintf (stderr, "\t   Choose from the features below.  Repeat the -I option as many times as needed\n");
+		fprintf (stderr, "\t      0 = Double-lined rivers (river-lakes)\n");
 		fprintf (stderr, "\t      1 = Permanent major rivers\n");
 		fprintf (stderr, "\t      2 = Additional major rivers\n");
 		fprintf (stderr, "\t      3 = Additional rivers\n");
@@ -387,21 +425,12 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t      8 = Major canals\n");
 		fprintf (stderr, "\t      9 = Minor canals\n");
 		fprintf (stderr, "\t     10 = Irrigation canals\n");
-		fprintf (stderr, "\t      a = All rivers and canals (1-10)\n");
-		fprintf (stderr, "\t      r = All permanent rivers (1-4)\n");
+		fprintf (stderr, "\t      a = All rivers and canals (0-10)\n");
+		fprintf (stderr, "\t      r = All permanent rivers (0-4)\n");
 		fprintf (stderr, "\t      i = All intermittent rivers (5-7)\n");
 		fprintf (stderr, "\t      c = All canals (8-10)\n");
 		GMT_explain_option ('K');
-		fprintf (stderr, "\t-L draws a simple map scaLe centered on <lon0>/<lat0>.  Use -Lx to specify Cartesian\n");
-		fprintf (stderr, "\t   coordinates instead.  Scale is calculated at latitude <slat>; optionally give longitude\n");
-		fprintf (stderr, "\t   <slon> [Default is central longitude].  <length> is in km, or [nautical] miles if [n] m\n");
-		fprintf (stderr, "\t   is appended.  -Lf draws a \"fancy\" scale [Default is plain]. By default, the label is set\n");
-		fprintf (stderr, "\t   to the distance unit and placed on top [t].  Use the :label:<just> mechanism to specify another\n");
-		fprintf (stderr, "\t   label (or - to keep the default) and placement (t,b,l,r, and u - to use the label as a unit.\n");
-		fprintf (stderr, "\t   Append +p<pen> and/or +f<fill> to draw/paint a rectangle beneath the scale [no rectangle]\n");
-		fprintf (stderr, "\t-M Dump a multisegment ascii (or binary, see -bo) file to standard output.  No plotting occurs\n");
-		fprintf (stderr, "\t   Specify any combination of -W, -I, -N.  Optionally, append 1-char\n");
-		fprintf (stderr, "\t   segment header <flag> [%c]\n", GMT_io.EOF_flag[GMT_OUT]);
+		GMT_mapscale_syntax ('L', "Draws a simple map scale centered on <lon0>/<lat0>");
 		GMT_pen_syntax ('N', "draws boundaries.  Specify feature and optionally append pen [Default is solid line of unit thickness].");
 		fprintf (stderr, "\t   Choose from the features below.  Repeat the -N option as many times as needed\n");
 		fprintf (stderr, "\t     1 = National boundaries\n");
@@ -413,24 +442,24 @@ int main (int argc, char **argv)
 		fprintf (stderr,"\t-Q means terminate previously set clip-paths\n");
 		GMT_fill_syntax ('S', "Paint of clip \"wet\" areas.");
 		fprintf (stderr, "\t   6) c to issue clip paths for water areas.\n");
-		fprintf (stderr, "\t-T draws a north-poinTing rose centered on <lon0>/<lat0>.  Use -Tx to specify Cartesian\n");
-		fprintf (stderr, "\t   coordinates instead.  -Tf draws a \"fancy\" rose [Default is plain].  Give rose diameter\n");
-		fprintf (stderr, "\t   <size> and optionally the west, east, south, north labels desired [W,E,S,N].\n");
-		fprintf (stderr, "\t   For fancy rose, specify as <info> the kind you want: 1 draws E-W, N-S directions [Default],\n");
-		fprintf (stderr, "\t   2 adds NW-SE and NE-SW, while 3 adds WNW-ESE, NNW-SSE, NNE-SSW, and ENE-WSW.\n");
-		fprintf (stderr, "\t   For Magnetic compass rose, specify -Tm.  Use the optional <info> = <dec>/<dlabel> (where <dec> is\n");
-		fprintf (stderr, "\t   the magnetic declination and <dlabel> is a label for the magnetic compass needle) to plot\n");
-		fprintf (stderr, "\t   directions to both magnetic and geographic north [Default is just geographic].\n");
-		fprintf (stderr, "\t   If the North label = \'*\' then a north star is plotted instead of the label.\n");
-		fprintf (stderr, "\t   Append +<gints>/<mints> to override default annotation/tick interval(s) [10/5/1/30/5/1].\n");
+		GMT_maprose_syntax ('T', "Draws a north-pointing map rose centered on <lon0>/<lat0>");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
-		GMT_pen_syntax ('W', "draw shorelines.  Append pen [Default is solid line of unit thickness].");
+		GMT_pen_syntax ('W', "draw shorelines.  Append pen [Default is solid black line of unit thickness for all levels].");
+		fprintf (stderr, "\t   Alternatively, set custom pens below.  Repeat the -W option as many times as needed\n");
+		fprintf (stderr, "\t      1 = Coastline\n");
+		fprintf (stderr, "\t      2 = Lake shores\n");
+		fprintf (stderr, "\t      3 = Island in lakes shores\n");
+		fprintf (stderr, "\t      4 = Lake in island in lake shores\n");
+		fprintf (stderr, "\t   When feature-specific pens are used, those not set are deactivated\n");
 		GMT_explain_option ('X');
 		fprintf (stderr, "\t-Z For 3-D plots: Set the z-level of map [0]\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
 		GMT_explain_option ('c');
+		fprintf (stderr, "\t-m Dump a multisegment ascii (or binary, see -bo) file to standard output.  No plotting occurs\n");
+		fprintf (stderr, "\t   Specify any combination of -W, -I, -N.  Optionally, append 1-char\n");
+		fprintf (stderr, "\t   segment header <flag> [%c]\n", GMT_io.EOF_flag[GMT_OUT]);
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -438,24 +467,28 @@ int main (int argc, char **argv)
 	fill_land = (Ctrl->G.active && !Ctrl->G.clip);
 	fill_ocean = (Ctrl->S.active && !Ctrl->S.clip);
 	if (Ctrl->C.active && !(fill_land || fill_ocean || Ctrl->W.active)) {	/* Just lakes, fix -A */
-		if (Ctrl->A.low < 2) Ctrl->A.low = 2;
+		if (Ctrl->A.info.low < 2) Ctrl->A.info.low = 2;
 	}
 
 	/* Check that the options selected are mutually consistent */
 
 	dumping = (GMT_io.multi_segments[GMT_IN] || GMT_io.multi_segments[GMT_OUT]);	/* -M was set */
 	clipping = (Ctrl->G.clip || Ctrl->S.clip);
-	if (!(Ctrl->Q.active || project_info.region_supplied)) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
-		error++;
+	if (!Ctrl->Q.active) {	/* Need -R -J */
+		if (!project_info.region_supplied) {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
+			error++;
+		}
+		if (!GMT_IS_MAPPING) {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR:  Geographic coastline data require the use of a map projection (use -Jx<scale>d if linear scaling is desired).\n", GMT_program);
+			error++;
+		}
 	}
-	if (!GMT_IS_MAPPING) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Geographic coastline data require the use of a map projection (use -Jx<scale>d if linear scaling is desired).\n", GMT_program);
-		error++;
-	}
-	if (Ctrl->W.pen.width < 0.0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -W option:  Pen thickness cannot be negative\n", GMT_program);
-		error++;
+	for (k = 0; k < GMT_MAX_GSHHS_LEVEL; k++) {
+		if (Ctrl->W.pen[k].width < 0.0) {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR -W option:  Pen thickness for feature %d cannot be negative\n", GMT_program, k);
+			error++;
+		}
 	}
 	if (!(fill_land || fill_ocean || Ctrl->C.active || Ctrl->W.active || Ctrl->N.active || Ctrl->I.active || clipping || Ctrl->Q.active)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify at least one of -C, -G, -S, -I, -N, and -W\n", GMT_program);
@@ -489,7 +522,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: Must specify at least one of -I, -N, and -W\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
-
+		
 	if (Ctrl->I.active) {
 		for (k = n_rlevels = 0; k < GMT_N_RLEVELS; k++) {
 			if (!Ctrl->I.use[k]) continue;
@@ -497,7 +530,7 @@ int main (int argc, char **argv)
 				fprintf (stderr, "%s: GMT SYNTAX ERROR -I option:  Pen thickness cannot be negative\n", GMT_program);
 				error++;
 			}
-			Ctrl->I.use[n_rlevels] = k + 1;
+			Ctrl->I.use[n_rlevels] = k;
 			Ctrl->I.pen[n_rlevels] = Ctrl->I.pen[k];
 			n_rlevels++;
 		}
@@ -518,13 +551,16 @@ int main (int argc, char **argv)
 
 	if (error) exit (EXIT_FAILURE);
 
+	if (Ctrl->D.force) Ctrl->D.set = GMT_shore_adjust_res (Ctrl->D.set);
 	base = GMT_set_resolution (&Ctrl->D.set, 'D');
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
 	fill[0] = Ctrl->S.fill;
 	fill[1] = fill[3] = Ctrl->G.fill;
-	fill[2] = fill[4] = ((Ctrl->C.active) ? Ctrl->C.fill : Ctrl->S.fill);
+	fill[2] = fill[4] = ((Ctrl->C.active) ? Ctrl->C.fill[LAKE] : Ctrl->S.fill);
+	fill[5] = (Ctrl->C.active) ? Ctrl->C.fill[RIVER] : fill[2];
 	need_coast_base = (fill_land || fill_ocean || Ctrl->C.active || Ctrl->W.active || Ctrl->G.clip || Ctrl->S.clip);
+	if (Ctrl->Q.active) need_coast_base = FALSE;	/* Since we just end clipping */
 	clobber_background = (fill_land && fill_ocean);
 	recursive = (((fill_land + (fill_ocean || Ctrl->C.active)) == 1) || clipping);
 	paint_polygons = (fill_land || fill_ocean || Ctrl->C.active) || clipping;
@@ -534,7 +570,7 @@ int main (int argc, char **argv)
 		east -= 360.0;
 	}
 
-	if (Ctrl->Q.active && project_info.projection == -1) {	/* Fake area and linear projection */
+	if (Ctrl->Q.active && project_info.projection == GMT_NO_PROJ) {	/* Fake area and linear projection */
 		GMT_parse_J_option ("x1d");
 		east = north = 1.0;
 	}
@@ -546,7 +582,7 @@ int main (int argc, char **argv)
 
 	world_map_save = GMT_world_map;
 
-	if (need_coast_base && GMT_init_shore (Ctrl->D.set, &c, project_info.w, project_info.e, project_info.s, project_info.n))  {
+	if (need_coast_base && GMT_init_shore (Ctrl->D.set, &c, project_info.w, project_info.e, project_info.s, project_info.n, &Ctrl->A.info))  {
 		fprintf (stderr, "%s: %s resolution shoreline data base not installed\n", GMT_program, shore_resolution[base]);
 		need_coast_base = FALSE;
 	}
@@ -555,6 +591,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: %s resolution political boundary data base not installed\n", GMT_program, shore_resolution[base]);
 		Ctrl->N.active = FALSE;
 	}
+	if (need_coast_base && gmtdefs.verbose == 2) fprintf (stderr, "GSHHS version %s\n%s\n%s\n", c.version, c.title, c.source);
 
 	if (Ctrl->I.active && GMT_init_br ('r', Ctrl->D.set, &r, project_info.w, project_info.e, project_info.s, project_info.n)) {
 		fprintf (stderr, "%s: %s resolution river data base not installed\n", GMT_program, shore_resolution[base]);
@@ -588,14 +625,17 @@ int main (int argc, char **argv)
 			GMT_plotend ();
 
 			if (gmtdefs.verbose) fprintf (stderr, "%s: Done!\n", GMT_program);
+			
+			Free_Pscoast_Ctrl (Ctrl);	/* Deallocate control structure */
 
 			GMT_end (argc, argv);
+			exit (EXIT_SUCCESS);
 		}
 
 		if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
 	}
 
-	if (fill_ocean && !Ctrl->C.active) Ctrl->C.fill = fill[4] = Ctrl->S.fill;	/* Use ocean fill by default for lakes */
+	if (fill_ocean && !Ctrl->C.active) Ctrl->C.fill[LAKE] = fill[4] = Ctrl->S.fill;	/* Use ocean fill by default for lakes */
 
 	for (i = 0; i < 5; i++) if (fill[i].use_pattern) fill_in_use = TRUE;
 
@@ -635,8 +675,8 @@ int main (int argc, char **argv)
 	if (clobber_background) {	/* Paint entire map as ocean first, then lay land on top */
 		n = GMT_map_clip_path (&xtmp, &ytmp, &donut);
 		if (donut) {
-			ps_line (xtmp, ytmp, n, 1, TRUE, FALSE);
-			ps_line (&xtmp[n], &ytmp[n], n, -1, TRUE, FALSE);
+			ps_line (xtmp, ytmp, n, 1, TRUE);
+			ps_line (&xtmp[n], &ytmp[n], n, -1, TRUE);
 			GMT_fill (xtmp, ytmp, n, &Ctrl->S.fill, -9);
 		}
 		else
@@ -661,10 +701,11 @@ int main (int argc, char **argv)
 		lowest_level = (fill_ocean) ? 0 : 1;
 	}
 
-	if (west < 0.0 && east > 0.0 && project_info.projection > 5) greenwich = TRUE;
+	if (west < 0.0 && east > 0.0 && !GMT_IS_LINEAR) greenwich = TRUE;
 	if ((360.0 - fabs (project_info.e - project_info.w) ) < c.bsize) {
 		GMT_world_map = TRUE;
 		if (project_info.projection == GMT_GNOMONIC || project_info.projection == GMT_GENPER) GMT_world_map = FALSE;
+		if (GMT_IS_AZIMUTHAL) GMT_world_map = FALSE;
 	}
 	if (GMT_world_map && greenwich)
 		edge = project_info.central_meridian;
@@ -682,12 +723,13 @@ int main (int argc, char **argv)
 	west_border = floor (project_info.w / c.bsize) * c.bsize;
 	east_border = ceil (project_info.e / c.bsize) * c.bsize;
 
-	if (!dumping && Ctrl->W.active) GMT_setpen (&Ctrl->W.pen);
+	if (!dumping && Ctrl->W.active) GMT_setpen (&Ctrl->W.pen[0]);
 
 	if (Ctrl->Z.active) project_info.z_level = Ctrl->Z.level;
 
 	/* Maximum step size (in degrees) used for interpolation of line segments along great circles */
 	step = gmtdefs.line_step / project_info.x_scale / project_info.M_PR_DEG;
+	last_pen_level = -1;
 
 	for (ind = 0; need_coast_base && ind < c.nb; ind++) {	/* Loop over necessary bins only */
 
@@ -695,7 +737,7 @@ int main (int argc, char **argv)
 #ifdef DEBUG
 		if (Ctrl->debug.bin >= 0 && bin != Ctrl->debug.bin) continue;
 #endif
-		if ((err = GMT_get_shore_bin (ind, &c, Ctrl->A.area, Ctrl->A.low, Ctrl->A.high))) {
+		if ((err = GMT_get_shore_bin (ind, &c))) {
 			fprintf (stderr, "%s: %s [%s resolution shoreline]\n", GMT_program, GMT_strerror(err), shore_resolution[base]);
 			exit (EXIT_FAILURE);
 		}
@@ -720,7 +762,7 @@ int main (int argc, char **argv)
 			bin_y[2] = bin_y[3] = c.lat_sw + c.bsize;
 			GMT_geo_to_xy (c.lon_sw + 0.5 * c.bsize, c.lat_sw + 0.5 * c.bsize, &x_c, &y_c);
 			dist = hypot (x_c - x_0, y_c - y_0);
-			donut_hell = GMT_non_zero_winding (anti_lon, anti_lat, bin_x, bin_y, 5);
+			donut_hell = GMT_non_zero_winding (anti_lon, anti_lat, bin_x, bin_y, (GMT_LONG)5);
 			if (!donut_hell) donut_hell = (dist > 0.8 * project_info.r);
 		}
 
@@ -728,17 +770,14 @@ int main (int argc, char **argv)
 
 			/* Assemble one or more segments into polygons */
 
-			if ((np = GMT_assemble_shore (&c, direction, Ctrl->A.low, TRUE, shift, west_border, east_border, &p)) == 0) continue;
+			if ((np = GMT_assemble_shore (&c, direction, TRUE, shift, west_border, east_border, &p)) == 0) continue;
 
 			/* Get clipped polygons in x,y inches that can be plotted */
 
 			np_new = GMT_prep_polygons (&p, np, donut_hell, step, bin_trouble);
 
 			if (clipping) {
-				for (k = level_to_be_painted; k < GMT_MAX_GSHHS_LEVEL - 1; k++) {
-					recursive_clipping (-1, np_new, p, k, k, &n_pieces);
-					did_clip = TRUE;
-				}
+				for (k = level_to_be_painted; k < GMT_MAX_GSHHS_LEVEL - 1; k++) recursive_clipping (-1, np_new, p, k, k, &n_pieces);
 
 				for (k = 0; k < np_new; k++) {	/* Do any remaining interior polygons */
 					if (p[k].n == 0) continue;
@@ -755,14 +794,14 @@ int main (int argc, char **argv)
 					level = p[k].level;
 					if (donut_hell && GMT_non_zero_winding (anti_x, anti_y, p[k].lon, p[k].lat, p[k].n)) {	/* Antipode inside polygon, must do donut */
 						n = GMT_map_clip_path (&xtmp, &ytmp, &donut);
-						ps_line (xtmp, ytmp, n, 1, TRUE, FALSE);
-						ps_line (p[k].lon, p[k].lat, p[k].n, -1, TRUE, FALSE);
-						GMT_fill (xtmp, ytmp, n, &fill[level], -9);
+						ps_line (xtmp, ytmp, n, 1, TRUE);
+						ps_line (p[k].lon, p[k].lat, p[k].n, -1, TRUE);
+						GMT_fill (xtmp, ytmp, n, &fill[p[k].fid], -9);
 						GMT_free ((void *)xtmp);
 						GMT_free ((void *)ytmp);
 					}
 					else
-						GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[level], FALSE);
+						GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[p[k].fid], FALSE);
 				}
 			}
 			else {	/* Must avoid pointing anything but the polygons inside */
@@ -774,7 +813,7 @@ int main (int argc, char **argv)
 				for (k = 0; k < np_new; k++) {	/* Do any remaining interior polygons */
 					if (p[k].n == 0) continue;
 					if (p[k].level%2 == level_to_be_painted || p[k].level > 2)
-						GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[p[k].level], FALSE);
+						GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[p[k].fid], FALSE);
 				}
 
 			}
@@ -784,21 +823,26 @@ int main (int argc, char **argv)
 		}
 
 		if (Ctrl->W.active && c.ns) {	/* Draw or dump shorelines, no need to assemble polygons */
-			if ((np = GMT_assemble_shore (&c, direction, Ctrl->A.low, FALSE, shift, west_border, east_border, &p)) == 0) continue;
+			if ((np = GMT_assemble_shore (&c, direction, FALSE, shift, west_border, east_border, &p)) == 0) continue;
 
 			for (i = 0; i < np; i++) {
 				if (dumping) {
-					sprintf (GMT_io.segment_header, "%c Shore Bin # %d, Level %d\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
+					sprintf (GMT_io.segment_header, "%c Shore Bin # %d, Level %ld\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
 					GMT_write_segmentheader (GMT_stdout, 2);
 					for (k = 0; k < p[i].n; k++) {
-						out[0] = p[i].lon[k];
-						out[1] = p[i].lat[k];
+						out[GMT_X] = p[i].lon[k];
+						out[GMT_Y] = p[i].lat[k];
 						GMT_output (GMT_stdout, 2, out);
 					}
 				}
-				else {
+				else if (Ctrl->W.use[p[i].level-1]) {
 					if (donut_hell) p[i].n = GMT_fix_up_path (&p[i].lon, &p[i].lat, p[i].n, step, 0);
 					GMT_n_plot = GMT_geo_to_xy_line (p[i].lon, p[i].lat, p[i].n);
+					k = p[i].level - 1;
+					if (k != last_pen_level) {
+						GMT_setpen (&Ctrl->W.pen[k]);
+						last_pen_level = k;
+					}
 					if (GMT_n_plot) GMT_plot_line (GMT_x_plot, GMT_y_plot, GMT_pen, GMT_n_plot);
 				}
 			}
@@ -812,7 +856,10 @@ int main (int argc, char **argv)
 	}
 	if (need_coast_base) GMT_shore_cleanup (&c);
 
-	if (clipping && did_clip) ps_clipon ((double *)NULL, (double *)NULL, 0, GMT_no_rgb, 2);	/* Dummy to end path */
+	if (clipping) {
+		if (n_pieces == 0) ps_clipon ((double *)NULL, (double *)NULL, 0, GMT_no_rgb, 1);	/* Just in case no piece was found */
+		ps_clipon ((double *)NULL, (double *)NULL, 0, GMT_no_rgb, 2);	/* Dummy to end path */
+	}
 
 	if (gmtdefs.verbose) fprintf (stderr, "\n");
 
@@ -834,15 +881,18 @@ int main (int argc, char **argv)
 				shift = ((left - edge) <= 180.0 && (right - edge) > 180.0);
 			}
 
-			if ((np = GMT_assemble_br (&r, shift, edge, &p)) == 0) continue;
+			if ((np = GMT_assemble_br (&r, shift, edge, &p)) == 0) {
+				GMT_free_br (&r);
+				continue;
+			}
 
 			for (i = 0; i < np; i++) {
 				if (dumping) {
-					sprintf (GMT_io.segment_header, "%c River Bin # %d, Level %d\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
+					sprintf (GMT_io.segment_header, "%c River Bin # %d, Level %ld\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
 					GMT_write_segmentheader (GMT_stdout, 2);
 					for (k = 0; k < p[i].n; k++) {
-						out[0] = p[i].lon[k];
-						out[1] = p[i].lat[k];
+						out[GMT_X] = p[i].lon[k];
+						out[GMT_Y] = p[i].lat[k];
 						GMT_output (GMT_stdout, 2, out);
 					}
 				}
@@ -850,7 +900,7 @@ int main (int argc, char **argv)
 					GMT_n_plot = GMT_geo_to_xy_line (p[i].lon, p[i].lat, p[i].n);
 					if (!GMT_n_plot) continue;
 
-					k = p[i].level - 1;
+					k = p[i].level;
 					if (k != last_k) {
 						GMT_setpen (&Ctrl->I.pen[k]);
 						last_k = k;
@@ -862,10 +912,7 @@ int main (int argc, char **argv)
 			/* Free up memory */
 
 			GMT_free_br (&r);
-			for (k = 0; k < np; k++) {
-				GMT_free ((void *)p[k].lon);
-				GMT_free ((void *)p[k].lat);
-			}
+			GMT_free_polygons (p, np);
 			GMT_free ((void *)p);
 		}
 		GMT_br_cleanup (&r);
@@ -896,15 +943,18 @@ int main (int argc, char **argv)
 				shift = ((left - edge) <= 180.0 && (right - edge) > 180.0);
 			}
 
-			if ((np = GMT_assemble_br (&b, shift, edge, &p)) == 0) continue;
+			if ((np = GMT_assemble_br (&b, shift, edge, &p)) == 0) {
+				GMT_free_br (&b);
+				continue;
+			}
 
 			for (i = 0; i < np; i++) {
 				if (dumping) {
-					sprintf (GMT_io.segment_header, "%c Border Bin # %d, Level %d\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
+					sprintf (GMT_io.segment_header, "%c Border Bin # %d, Level %ld\n", GMT_io.EOF_flag[GMT_OUT], bin, p[i].level);
 					GMT_write_segmentheader (GMT_stdout, 2);
 					for (k = 0; k < p[i].n; k++) {
-						out[0] = p[i].lon[k];
-						out[1] = p[i].lat[k];
+						out[GMT_X] = p[i].lon[k];
+						out[GMT_Y] = p[i].lat[k];
 						GMT_output (GMT_stdout, 2, out);
 					}
 				}
@@ -925,10 +975,7 @@ int main (int argc, char **argv)
 			/* Free up memory */
 
 			GMT_free_br (&b);
-			for (k = 0; k < np; k++) {
-				GMT_free ((void *)p[k].lon);
-				GMT_free ((void *)p[k].lat);
-			}
+			GMT_free_polygons (p, np);
 			GMT_free ((void *)p);
 		}
 		GMT_br_cleanup (&b);
@@ -951,6 +998,8 @@ int main (int argc, char **argv)
 		GMT_plotend ();
 	}
 
+	if (gmtdefs.verbose) fprintf (stderr, "Done\n");
+	
 	Free_Pscoast_Ctrl (Ctrl);	/* Deallocate control structure */
 
 	GMT_end (argc, argv);
@@ -983,8 +1032,8 @@ void recursive_painting (int k0, int np, struct GMT_GSHHS_POL p[], int level, in
 		}
 
 		if (p[k].level == start_level) {	/* Done nesting, time to paint the assembled swiss cheese polygon */
-			ps_clipon (p[k].lon, p[k].lat, 0, GMT_no_rgb, 4);	/* Dummy call with no points just to end current path */
-			GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[p[k].level], -1);
+			ps_clipon (p[k].lon, p[k].lat, (GMT_LONG)0, GMT_no_rgb, 4);	/* Dummy call with no points just to end current path */
+			GMT_fill (p[k].lon, p[k].lat, p[k].n, &fill[p[k].fid], -1);
 		}
 	}
 }
@@ -1013,7 +1062,7 @@ BOOLEAN add_this_polygon_to_path (int k0, struct GMT_GSHHS_POL *p, int level, in
 {
 	/* Determines if we should add the current polygon pol[k] to the growing path we are constructing */
 
-	int j;
+	GMT_LONG j;
 	if (p[k].level != level) return (FALSE);	/* Sorry, this polygon does not have the correct level */
 	if (k0 == -1) return (TRUE);			/* The first one is always used */
 	/* Must make sure the polygon is inside the mother polygon.  Because numerical noise can trick us we try up to two points */
@@ -1028,22 +1077,23 @@ void *New_Pscoast_Ctrl () {	/* Allocate and initialize a new control structure *
 	int k;
 	struct PSCOAST_CTRL *C;
 
-	C = (struct PSCOAST_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSCOAST_CTRL), "New_Pscoast_Ctrl");
+	C = (struct PSCOAST_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSCOAST_CTRL), "New_Pscoast_Ctrl");
 
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 
-	C->A.high = GMT_MAX_GSHHS_LEVEL;			/* Include all GSHHS levels */
+	C->A.info.high = GMT_MAX_GSHHS_LEVEL;			/* Include all GSHHS levels */
 	C->D.set = 'l';						/* Low-resolution coastline data */
 	C->E.azimuth = 180.0;
 	C->E.elevation = 90.0;
 	GMT_init_fill (&C->S.fill, 255, 255, 255);		/* Default Ocean color */
 	GMT_init_fill (&C->G.fill, 0, 0, 0);			/* Default Land color */
-	GMT_init_fill (&C->C.fill, 255, 255, 255);		/* Default Lake color */
-	for (k = 0; k < GMT_N_RLEVELS; k++) GMT_init_pen (&C->I.pen[k], GMT_PENWIDTH);	/* Default river pens */
-	for (k = 0; k < GMT_N_BLEVELS; k++) GMT_init_pen (&C->N.pen[k], GMT_PENWIDTH);	/* Default border pens */
+	GMT_init_fill (&C->C.fill[LAKE], 255, 255, 255);	/* Default Lake color */
+	GMT_init_fill (&C->C.fill[RIVER], 255, 255, 255);	/* Default Riverlake color */
+	for (k = 0; k < GMT_N_RLEVELS; k++) GMT_init_pen (&C->I.pen[k], GMT_PENWIDTH);		/* Default river pens */
+	for (k = 0; k < GMT_N_BLEVELS; k++) GMT_init_pen (&C->N.pen[k], GMT_PENWIDTH);		/* Default border pens */
+	for (k = 0; k < GMT_MAX_GSHHS_LEVEL; k++) GMT_init_pen (&C->W.pen[k], GMT_PENWIDTH);	/* Default coastline pens */
 	memset ((void *)&C->L.item, 0, sizeof (struct GMT_MAP_SCALE));
 	memset ((void *)&C->T.item, 0, sizeof (struct GMT_MAP_ROSE));
-	GMT_init_pen (&C->W.pen, GMT_PENWIDTH);				/* Default coastline pen */
 #ifdef DEBUG
 	C->debug.bin = -1;
 #endif
