@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pscontour.c,v 1.86 2007/10/03 00:35:13 guru Exp $
+ *	$Id: pscontour.c,v 1.112 2009/07/11 03:16:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -84,7 +84,7 @@ struct PSCONTOUR_LINE {	/* Beginning and end of straight contour segment */
 };
 
 struct PSCONTOUR {
-	int n_alloc, nl;
+	GMT_LONG n_alloc, nl;
 	double val;
 	double angle;
 	char type;
@@ -104,16 +104,19 @@ struct PSCONTOUR_CHAIN {
 
 int main(int argc, char **argv)
 {
-	int n, np, nx, i, ij, k, k2, k3, node1, node2, c, rgb[3], PSCONTOUR_SUM;
-	int n_alloc, section = 0, *ind, *vert, *cind, n_fields, n_contours;
-	int add, close, bad, n_expected_fields, low, high, last_entry, last_exit;
-
-	BOOLEAN error = FALSE, more, skip = FALSE;
+	GMT_LONG nx, k2, k3, node1, node2, c, PSCONTOUR_SUM;
+	GMT_LONG n_alloc, section = 0, *vert, *cind, n_fields, n_contours = 0;
+	GMT_LONG add, close, bad, n_expected_fields, last_entry, last_exit;
+	GMT_LONG ij, n, np, k, i, low, high;
+	
+	int rgb[3], *ind = NULL;
+	
+	BOOLEAN error = FALSE, more, skip = FALSE, advance = FALSE;
 
 	double xx[3], yy[3], zz[3], xout[5], yout[5], *in, west, east, south, north;
 	double *xc, *yc, *zc, *x, *y, *z, *xp, *yp, current_contour = -DBL_MAX, xyz[2][3];
 
-	char line[BUFSIZ], cont_label[GMT_LONG_TEXT], format[GMT_LONG_TEXT];
+	char line[BUFSIZ], cont_label[GMT_LONG_TEXT], format[GMT_LONG_TEXT], *not_used = NULL;
 #ifdef TRIANGLE_D
 	char *algorithm = "Shewchuk";
 #else
@@ -122,11 +125,11 @@ int main(int argc, char **argv)
 
 	FILE *fp = NULL, *fp_d = NULL;
 
-	struct PSCONTOUR *cont;
+	struct PSCONTOUR *cont = NULL;
 	struct PSCONTOUR_CTRL *Ctrl;
 
-	int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, int **v, int **cindex);
-	void paint_it (double x[], double y[], int n, double z);
+	GMT_LONG get_triangle_crossings (double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, GMT_LONG **v, GMT_LONG **cindex);
+	void paint_it (double x[], double y[], GMT_LONG n, double z);
 	void *New_Pscontour_Ctrl (), Free_Pscontour_Ctrl (struct PSCONTOUR_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -145,6 +148,7 @@ int main(int argc, char **argv)
 				case 'H':
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -156,6 +160,7 @@ int main(int argc, char **argv)
 				case 'y':
 				case 'b':
 				case 'c':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
@@ -190,7 +195,7 @@ int main(int argc, char **argv)
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
-					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
+					error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'G':
 					Ctrl->G.active = TRUE;
@@ -206,10 +211,6 @@ int main(int argc, char **argv)
 						error++;
 					}
 					break;
-				case 'M':	/* with -D, create one multiple line segments */
-					GMT_multisegment (&argv[i][2]);
-					GMT_io.multi_segments[GMT_OUT] = 2;
-					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
 					break;
@@ -245,10 +246,10 @@ int main(int argc, char **argv)
 	if (GMT_give_synopsis_and_exit || argc == 1) {	/* Display usage */
 		fprintf (stderr,"pscontour %s - Contour xyz-data by triangulation [%s]\n\n", GMT_VERSION, algorithm);
 		fprintf (stderr,"usage: pscontour <xyzfile> -C<cpt_file> %s %s\n", GMT_J_OPT, GMT_Rgeoz_OPT);
-		fprintf (stderr, "\t[-A[-|<annot_int>][<labelinfo>] [%s] [-D<dumpfile>] [-E<az>/<el>]\n", GMT_B_OPT);
-		fprintf (stderr, "\t[%s] [%s] [-I] [%s] [-K] [-L<pen>] [%s] [-N] [-O]\n", GMT_CONTG, GMT_H_OPT, GMT_Jz_OPT, GMT_Mo_OPT);
- 		fprintf (stderr, "\t[-P] [S-] [-T<indexfile>] [-U] [-V] [-W[+]<pen>] [%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT);
-		fprintf (stderr, "\t[%s] [%s] [%s]\n\n", GMT_c_OPT, GMT_t_OPT, GMT_b_OPT);
+		fprintf (stderr, "\t[-A[-|<annot_int>][<labelinfo>] [%s] [-D<dumpfile>] [%s]\n", GMT_B_OPT, GMT_E_OPT);
+		fprintf (stderr, "\t[%s] [%s] [-I] [%s] [-K] [-L<pen>] [-N] [-O]\n", GMT_CONTG, GMT_H_OPT, GMT_Jz_OPT);
+ 		fprintf (stderr, "\t[-P] [-S] [-T<indexfile>] [-U] [-V] [-W[+]<pen>] [%s] [%s]\n", GMT_X_OPT, GMT_Y_OPT);
+		fprintf (stderr, "\t[%s] [%s] [%s] [%s]\n\n", GMT_c_OPT, GMT_t_OPT, GMT_b_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -262,16 +263,14 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t   <labelinfo> controls the specifics of the labels.  Append what you need:\n");
 		GMT_label_syntax (5, 0);
 		GMT_explain_option ('b');
-		fprintf (stderr, "\t-D to Dump contour lines to individual files (but see -M)\n");
-		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t-D to Dump contour lines to individual files (but see -m)\n");
+		GMT_explain_option ('E');
 		fprintf (stderr, "\t-G Controls placement of labels along contours.  Choose among five algorithms:\n");
 		GMT_cont_syntax (3, 0);
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-I Color triangles using the cpt file\n");
 		GMT_explain_option ('K');
 		GMT_pen_syntax ('L', "draws the triangular mesh with the specified pen.");
-		fprintf (stderr, "\t-M Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
-		fprintf (stderr, "\t   whose first character is <flag> ['>'].  This header also has the contour level value\n");
 		fprintf (stderr,"\t-N do NOT clip contours/image at the border [Default clips]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
@@ -290,6 +289,8 @@ int main(int argc, char **argv)
 		fprintf (stderr,"\t   Default is 3 input columns.\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('n');
+		fprintf (stderr, "\t-m Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
+		fprintf (stderr, "\t   whose first character is <flag> ['>'].  This header also has the contour level value\n");
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -330,11 +331,11 @@ int main(int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (GMT_io.binary[GMT_OUT] && GMT_io.multi_segments[GMT_OUT]) {
-		fprintf (stderr, "%s: GMT Warning.  -M for output ignored with -D, -bo\n", GMT_program);
+		fprintf (stderr, "%s: GMT Warning.  -m for output ignored with -D, -bo\n", GMT_program);
 		error++;
 	}
 	z_project.view_azimuth = Ctrl->E.azimuth;
@@ -350,6 +351,7 @@ int main(int argc, char **argv)
 		fprintf (stderr, "%s: contours will be written to file contour\n", GMT_program);
 		strcpy (Ctrl->D.file,"contour");
 	}
+	if (GMT_io.multi_segments[GMT_OUT]) GMT_io.multi_segments[GMT_OUT] = 2;
 
 	GMT_read_cpt (Ctrl->C.file);
 	if (Ctrl->I.active && GMT_continuous) {
@@ -382,7 +384,7 @@ int main(int argc, char **argv)
 	y = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	z = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 
-	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 	xyz[0][2] = DBL_MAX;	xyz[1][2] = -DBL_MAX;
 	n = 0;
@@ -392,20 +394,20 @@ int main(int argc, char **argv)
 	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 			exit (EXIT_FAILURE);
 		}
 
 		if (Ctrl->S.active) {	/* Must check if points are inside plot region */
-			GMT_map_outside (in[0], in[1]);
+			GMT_map_outside (in[GMT_X], in[GMT_Y]);
 			skip = ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1);
 		}
 
-		if (!(skip || GMT_is_dnan (in[2]))) {	/* Unless outside or z = NaN */
+		if (!(skip || GMT_is_dnan (in[GMT_Z]))) {	/* Unless outside or z = NaN */
 
-			x[n] = in[0];
-			y[n] = in[1];
-			z[n] = in[2];
+			x[n] = in[GMT_X];
+			y[n] = in[GMT_Y];
+			z[n] = in[GMT_Z];
 			if (z[n] < xyz[1][2]) {	/* New minimum */
 				xyz[0][0] = x[n];
 				xyz[0][1] = y[n];
@@ -419,11 +421,18 @@ int main(int argc, char **argv)
 			n++;
 
 			if (n == n_alloc) {
-				n_alloc += GMT_CHUNK;
+				n_alloc <<= 1;
 				x = (double *) GMT_memory ((void *)x, (size_t)n_alloc, sizeof (double), GMT_program);
 				y = (double *) GMT_memory ((void *)y, (size_t)n_alloc, sizeof (double), GMT_program);
 				z = (double *) GMT_memory ((void *)z, (size_t)n_alloc, sizeof (double), GMT_program);
 			}
+			if (n == INT_MAX) {
+				fprintf (stderr, "%s: ERROR: Cannot triangulate more than %d points\n", GMT_program, INT_MAX);
+				GMT_free ((void *)x);
+				GMT_free ((void *)y);
+				GMT_free ((void *)z);
+				exit (EXIT_FAILURE);
+			}	
 		}
 
 		n_fields = GMT_input (fp, &n_expected_fields, &in);
@@ -440,9 +449,9 @@ int main(int argc, char **argv)
 
 	for (i = 0; i < n; i++) GMT_geo_to_xy (x[i], y[i], &x[i], &y[i]);
 
-	if (fp_d) {
-		n_alloc = 3 * GMT_CHUNK;
-		ind = (int *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (int), GMT_program);
+	if (Ctrl->T.active) {	/* Read precalculated triangulation indices */
+		n_alloc = GMT_CHUNK;
+		ind = (int *) GMT_memory (VNULL, (size_t)(3*n_alloc), sizeof (int), GMT_program);
 
 		ij = np = 0;
 
@@ -452,12 +461,24 @@ int main(int argc, char **argv)
 			more = (GMT_fgets (line, BUFSIZ, fp_d) != CNULL);
 
 		while (more) {
-			if (!GMT_io.binary[GMT_IN] && sscanf (line, "%d %d %d", &ind[ij], &ind[ij+1], &ind[ij+2]) != 3) continue;
-			ij += 3;
-			np++;
-			if (ij == n_alloc) {
-				n_alloc += (3 * GMT_CHUNK);
-				ind = (int *) GMT_memory ((void *)ind, (size_t)n_alloc, sizeof (int), GMT_program);
+			if (!GMT_io.binary[GMT_IN]) {	/* Ascii input, must be more careful */
+				advance = FALSE;
+				if (!GMT_is_a_blank_line (line)) {	/* Got a record to process */
+					i = sscanf (line, "%d %d %d", &ind[ij], &ind[ij+1], &ind[ij+2]);
+				 	if (i != 3) {	/* Mangled record? */
+						fprintf (stderr, "%s: Error reading triangulation indices near line %ld\n", GMT_program, np);
+						exit (EXIT_FAILURE);
+					}
+					advance = TRUE;
+				}
+			}
+			if (advance) {	/* All is well if we get here */
+				ij += 3;
+				np++;
+			}
+			if (np == n_alloc) {
+				n_alloc <<= 1;
+				ind = (int *) GMT_memory ((void *)ind, (size_t)(3*n_alloc), sizeof (int), GMT_program);
 			}
 			if (GMT_io.binary[GMT_IN])	/* Binary input */
 				more = (GMT_fread ((void *)&ind[ij], sizeof (int), (size_t)3, fp_d) == 3);
@@ -469,7 +490,7 @@ int main(int argc, char **argv)
 	}
 	else	/* Do Delaunay triangulation */
 
-		np = GMT_delaunay (x, y, n, &ind);
+		np = GMT_delaunay (x, y, (int)n, &ind);
 
 	if (Ctrl->L.active) {	/* Draw triangular mesh */
 
@@ -482,28 +503,29 @@ int main(int argc, char **argv)
 			xx[2] = x[ind[k]];	yy[2] = y[ind[k++]];
 
 			if (project_info.three_D) for (c = 0; c < 3; c++) GMT_xy_do_z_to_xy (xx[c], yy[c], project_info.z_level, &xx[c], &yy[c]);
-			ps_line (xx, yy, 3, 3, 1, FALSE);
+			ps_line (xx, yy, (GMT_LONG)3, 3, TRUE);
 		}
 	}
 
 	/* Get PSCONTOUR structs */
 
-	n_contours = GMT_n_colors + 1;
-	cont = (struct PSCONTOUR *) GMT_memory (VNULL, (size_t)n_contours, sizeof (struct PSCONTOUR), GMT_program);
+	if (Ctrl->W.active) {
+		n_contours = GMT_n_colors + 1;
+		cont = (struct PSCONTOUR *) GMT_memory (VNULL, (size_t)n_contours, sizeof (struct PSCONTOUR), GMT_program);
 
-	for (i = 0; i < GMT_n_colors; i++) {
-		cont[i].val = GMT_lut[i].z_low;
-		cont[i].type = (GMT_lut[i].annot && !Ctrl->A.off) ? 'A' : 'C';
-		cont[i].angle = (Ctrl->contour.angle_type == 2) ? Ctrl->contour.label_angle : GMT_d_NaN;
+		for (i = 0; i < GMT_n_colors; i++) {
+			cont[i].val = GMT_lut[i].z_low;
+			cont[i].type = (GMT_lut[i].annot && !Ctrl->A.off) ? 'A' : 'C';
+			cont[i].angle = (Ctrl->contour.angle_type == 2) ? Ctrl->contour.label_angle : GMT_d_NaN;
+		}
+		cont[GMT_n_colors].val = GMT_lut[GMT_n_colors-1].z_high;
+		cont[GMT_n_colors].type = ((GMT_lut[GMT_n_colors-1].annot & 2) && !Ctrl->A.off) ? 'A' : 'C';
+		cont[GMT_n_colors].angle = (Ctrl->contour.angle_type == 2) ? Ctrl->contour.label_angle : GMT_d_NaN;
+		for (i = 0; i < n_contours; i++) {
+			cont[i].n_alloc = GMT_SMALL_CHUNK;
+			cont[i].L = (struct PSCONTOUR_LINE *) GMT_memory (VNULL, (size_t)GMT_SMALL_CHUNK, sizeof (struct PSCONTOUR_LINE), GMT_program);
+		}
 	}
-	cont[GMT_n_colors].val = GMT_lut[GMT_n_colors-1].z_high;
-	cont[GMT_n_colors].type = ((GMT_lut[GMT_n_colors-1].annot & 2) && !Ctrl->A.off) ? 'A' : 'C';
-	cont[GMT_n_colors].angle = (Ctrl->contour.angle_type == 2) ? Ctrl->contour.label_angle : GMT_d_NaN;
-	for (i = 0; i < n_contours; i++) {
-		cont[i].n_alloc = GMT_SMALL_CHUNK;
-		cont[i].L = (struct PSCONTOUR_LINE *) GMT_memory (VNULL, (size_t)GMT_SMALL_CHUNK, sizeof (struct PSCONTOUR_LINE), GMT_program);
-	}
-
 	Ctrl->contour.line_pen = Ctrl->W.pen;
 
 	for (i = ij = 0; i < np; i++, ij += 3) {	/* For all triangles */
@@ -525,7 +547,7 @@ int main(int argc, char **argv)
 					zzz += zz[k];
 					kzz++;
 				}
-				if (kzz) paint_it (xx, yy, 3, zzz / kzz);
+				if (kzz) paint_it (xx, yy, (GMT_LONG)3, zzz / kzz);
 			}
 			else {	/* Must paint all those slices separately */
 
@@ -636,10 +658,10 @@ int main(int argc, char **argv)
 				cont[c].L[n].y1 = yc[k2++];
 				n++;
 				if (n >= cont[c].n_alloc) {
-					cont[c].n_alloc += GMT_SMALL_CHUNK;
+					cont[c].n_alloc <<= 1;
 					cont[c].L = (struct PSCONTOUR_LINE *) GMT_memory ((void *)cont[c].L, (size_t)cont[c].n_alloc, sizeof (struct PSCONTOUR_LINE), GMT_program);
 				}
-				cont[c].nl = n;
+				cont[c].nl = (int)n;
 			}
 		}
 
@@ -740,7 +762,7 @@ int main(int argc, char **argv)
 					p = p->next;
 					GMT_free ((void *)q);
 					if (n == n_alloc) {
-						n_alloc += GMT_SMALL_CHUNK;
+						n_alloc <<= 1;
 						xp = (double *) GMT_memory ((void *)xp, (size_t)n_alloc, sizeof (double), GMT_program);
 						yp = (double *) GMT_memory ((void *)yp, (size_t)n_alloc, sizeof (double), GMT_program);
 					}
@@ -782,12 +804,13 @@ int main(int argc, char **argv)
 				GMT_free ((void *)xp);
 				GMT_free ((void *)yp);
 			}
-
+			GMT_free ((void *)head_c);
 		}
 		GMT_contlabel_plot (&Ctrl->contour);
+		GMT_contlabel_free (&Ctrl->contour);
 	}
 
-        if (!Ctrl->N.active) GMT_map_clip_off ();
+    if (!Ctrl->N.active) GMT_map_clip_off ();
 
 	GMT_map_basemap ();
 
@@ -798,7 +821,23 @@ int main(int argc, char **argv)
 	GMT_free ((void *)x);
 	GMT_free ((void *)y);
 	GMT_free ((void *)z);
-	GMT_free ((void *)ind);
+	if (Ctrl->W.active) GMT_free ((void *)cont);
+	if (Ctrl->T.active)
+		GMT_free ((void *)ind);	/* Allocated above by GMT_memory */
+	else {
+#ifdef TRIANGLE_D
+#ifdef DEBUG
+	/* Shewchuk's function allocated the memory separately */
+		GMT_memtrack_off (GMT_mem_keeper);
+#endif
+#endif
+		GMT_free ((void *) ind);
+#ifdef TRIANGLE_D
+#ifdef DEBUG
+		GMT_memtrack_on (GMT_mem_keeper);
+#endif
+#endif
+	}
 
 	Free_Pscontour_Ctrl (Ctrl);	/* Deallocate control structure */
 
@@ -808,13 +847,13 @@ int main(int argc, char **argv)
 }
 
 
-int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, int **v, int **cindex)
+GMT_LONG get_triangle_crossings (double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, GMT_LONG **v, GMT_LONG **cindex)
 {
 	/* This routine finds all the contour crossings for this triangle.  Each contour consists of
 	 * linesegments made up of two points, with coordinates xc, yc, and contour level zc.
 	 */
 	 
-	int i, j, k, k2, i1, nx, n_alloc, *vout, *cind;
+	GMT_LONG i, j, k, k2, i1, nx, n_alloc, *vout, *cind;
 	double xx[3], yy[3], zz[3], zmin, zmax, dz, frac, *xout, *yout, *zout;
 
 	xx[0] = x[ind[0]];	yy[0] = y[ind[0]];	zz[0] = z[ind[0]];
@@ -838,8 +877,8 @@ int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **
 	xout = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	yout = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	zout = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-	vout = (int *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (int), GMT_program);
-	cind = (int *) GMT_memory (VNULL, (size_t)nx, sizeof (int), GMT_program);
+	vout = (GMT_LONG *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (GMT_LONG), GMT_program);
+	cind = (GMT_LONG *) GMT_memory (VNULL, (size_t)nx, sizeof (GMT_LONG), GMT_program);
 
 	/* Fill out array zout which holds the nx contour levels */
 
@@ -874,17 +913,26 @@ int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **
 		if (j%2) j--;	/* Contour went through a single vertex only, skip this */
 	}
 
-	*xc = xout;
-	*yc = yout;
-	*zc = zout;
-	*v = vout;
-	*cindex = cind;
-
-	return (j/2);
+	nx = j / 2;	/* Since j might have changed */
+	if (nx) {
+		*xc = xout;
+		*yc = yout;
+		*zc = zout;
+		*v = vout;
+		*cindex = cind;
+	} else {
+		GMT_free ((void *)xout);
+		GMT_free ((void *)yout);
+		GMT_free ((void *)zout);
+		GMT_free ((void *)vout);
+		GMT_free ((void *)cind);
+	}
+	return (nx);
 }
 
-void paint_it (double x[], double y[], int n, double z) {
-	int index, k, rgb[3];
+void paint_it (double x[], double y[], GMT_LONG n, double z) {
+	GMT_LONG index, k;
+	int rgb[3];
 	struct GMT_FILL *f;
 
 	if (n < 3) return;	/* Need at least 3 points to make a polygon */
@@ -897,16 +945,16 @@ void paint_it (double x[], double y[], int n, double z) {
 	if (project_info.three_D)
 		for (k = 0; k < n; k++) GMT_xy_do_z_to_xy (x[k], y[k], project_info.z_level, &x[k], &y[k]);
 	if ((index >= 0 && (f = GMT_lut[index].fill)) || (index < 0 && (f = GMT_bfn[index+3].fill))) {
-		GMT_fill (x, y, n, f, FALSE);	/* Contours drawn separately later if desired */
+		GMT_fill (x, y, (GMT_LONG)n, f, FALSE);	/* Contours drawn separately later if desired */
 	}
 	else
-		ps_patch (x, y, n, rgb, FALSE);	/* Contours drawn separately later if desired */
+		ps_patch (x, y, (GMT_LONG)n, rgb, FALSE);	/* Contours drawn separately later if desired */
 }
 
 void *New_Pscontour_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSCONTOUR_CTRL *C;
 	
-	C = (struct PSCONTOUR_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSCONTOUR_CTRL), "New_Pscontour_Ctrl");
+	C = (struct PSCONTOUR_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSCONTOUR_CTRL), "New_Pscontour_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	GMT_contlabel_init (&C->contour, 1);
