@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pshistogram.c,v 1.55 2007/09/30 04:16:49 guru Exp $
+ *	$Id: pshistogram.c,v 1.68 2009/07/11 03:16:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@
  * Updated to v3.1 13-Jun-1998 Paul Wessel
  * Updated to v3.3 23-Apr-1999 Paul Wessel
  *		     Now uses two passes to get accurate mean and stdev
- * 27-JAN-2005: Added -T option to select column of interrest
+ * 27-JAN-2005: Added -T option to select column of interest
  * Version:	4.1.2 Free of global variables, and now uses double precision
  *		4.2.1 -C is now -F (-C will still work) and -C<cpt> paints bars
  */
@@ -56,7 +56,7 @@ struct PSHISTOGRAM_CTRL {
 	} G;
 	struct I {	/* -I[o] */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 	} I;
 	struct L {	/* -L<pen> */
 		BOOLEAN active;
@@ -70,7 +70,7 @@ struct PSHISTOGRAM_CTRL {
 	} S;
 	struct T {	/* -T<col> */
 		BOOLEAN active;
-		int col;
+		GMT_LONG col;
 	} T;
 	struct W {	/* -W<width> */
 		BOOLEAN active;
@@ -78,7 +78,7 @@ struct PSHISTOGRAM_CTRL {
 	} W;
 	struct Z {	/* -Z<type> */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 	} Z;
 };
 
@@ -91,22 +91,23 @@ struct PSHISTOGRAM_CTRL {
 
 struct PSHISTOGRAM_INFO {	/* Control structure for pshistogram */
 	double	yy0, yy1;
-	int	*boxh;
-	int	n_boxes;
-	int	n_counted;
+	GMT_LONG	*boxh;
+	GMT_LONG	n_boxes;
+	GMT_LONG	n_counted;
 	double	box_width;
 	double	west, east, south, north;
-	int	center_box, cumulative;
-	int	hist_type;
+	GMT_LONG	center_box, cumulative;
+	GMT_LONG	hist_type;
 };
 
 int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, automatic = FALSE;
 
-	char string[BUFSIZ], text[BUFSIZ], buffer[BUFSIZ], format[BUFSIZ];
+	char buffer[BUFSIZ], format[BUFSIZ], *not_used = NULL;
 	
-	int i, n_alloc = GMT_CHUNK, n_expected_fields, n_fields, pos, n;
+	GMT_LONG i, n_alloc = GMT_CHUNK, n;
+	GMT_LONG n_expected_fields, n_fields;
 	
 	double	*data, stats[6];
 
@@ -117,9 +118,9 @@ int main (int argc, char **argv)
 	struct PSHISTOGRAM_INFO F;
 	struct PSHISTOGRAM_CTRL *Ctrl;
 	
-	int fill_boxes (struct PSHISTOGRAM_INFO *F, double *x, int n);
-	int plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, BOOLEAN draw_outline, struct GMT_PEN *pen, struct GMT_FILL *fill, BOOLEAN cpt);
-	int get_loc_scl (double *x, int n, double *stats);
+	GMT_LONG fill_boxes (struct PSHISTOGRAM_INFO *F, double *x, GMT_LONG n);
+	GMT_LONG plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, BOOLEAN draw_outline, struct GMT_PEN *pen, struct GMT_FILL *fill, BOOLEAN cpt);
+	GMT_LONG get_loc_scl (double *x, GMT_LONG n, double *stats);
 	void *New_Pshistogram_Ctrl (), Free_Pshistogram_Ctrl (struct PSHISTOGRAM_CTRL *C);
 
 	argc = GMT_begin (argc, argv);
@@ -141,6 +142,7 @@ int main (int argc, char **argv)
 				case 'K':
 				case 'O':
 				case 'P':
+				case 'R':
 				case 'U':
 				case 'V':
 				case 'X':
@@ -198,19 +200,6 @@ int main (int argc, char **argv)
 				case 'Q':
 					Ctrl->Q.active = TRUE;
 					break;
-				case 'R':
-					project_info.region_supplied = TRUE;
-					strcpy (string, &argv[i][2]);
-					pos = 0;
-					GMT_strtok (string, "/", &pos, text);
-					GMT_scanf (text, GMT_IS_GEO, &F.west);
-					GMT_strtok (string, "/", &pos, text);
-					GMT_scanf (text, GMT_IS_GEO, &F.east);
-					GMT_strtok (string, "/", &pos, text);
-					GMT_scanf (text, GMT_IS_GEO, &F.south);
-					GMT_strtok (string, "/", &pos, text);
-					GMT_scanf (text, GMT_IS_GEO, &F.north);
-					break;
 				case 'S':
 					Ctrl->S.active = TRUE;
 					break;
@@ -271,7 +260,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-Q plot a cumulative histogram\n");
 		GMT_explain_option ('r');
 		fprintf (stderr, "\t   If neither -R nor -I are set, w/e/s/n will be based on input data\n");
-		fprintf (stderr, "\t-S Draws a stairs-step diagram instead of histogram.\n");
+		fprintf (stderr, "\t-S Draws a stairs-step diagram [Default is bar histogram].\n");
 		fprintf (stderr, "\t-T Append column to use (0 is first) [0]\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
@@ -293,7 +282,7 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	if (!Ctrl->I.active && (project_info.projection < 0 || project_info.projection > 9)) {
+	if (!Ctrl->I.active && !GMT_IS_LINEAR) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -J option:  Only linear projection supported.\n", GMT_program);
 		error++;
 	}
@@ -329,12 +318,12 @@ int main (int argc, char **argv)
 		error++;
 	}
         if (GMT_io.binary[GMT_IN] && Ctrl->T.col >= GMT_io.ncol[GMT_IN]) {
-                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Chosen column (%d) exceeds the columns (%d) in binary input data (-bi)\n", \
+                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Chosen column (%ld) exceeds the columns (%ld) in binary input data (-bi)\n", \
 		GMT_program, Ctrl->T.col+1, GMT_io.ncol[GMT_IN]);
 		error++;
 	}
         if (Ctrl->T.col < 0) {
-                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Chosen column (%d) must be >= 0\n", GMT_program, Ctrl->T.col);
+                fprintf (stderr, "%s: GMT SYNTAX ERROR.  Chosen column (%ld) must be >= 0\n", GMT_program, Ctrl->T.col);
 		error++;
 	}
 
@@ -342,7 +331,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	F.hist_type = Ctrl->Z.mode;
@@ -350,7 +339,7 @@ int main (int argc, char **argv)
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
 	F.cumulative = Ctrl->Q.active;
-	F.center_box = Ctrl->C.active;
+	F.center_box = Ctrl->F.active;
 	if (Ctrl->I.active) gmtdefs.verbose = TRUE;
 	if (!Ctrl->I.active && !project_info.region_supplied) automatic = TRUE;
 
@@ -369,23 +358,23 @@ int main (int argc, char **argv)
 	x_min = DBL_MAX;	x_max = -DBL_MAX;
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : BUFSIZ;
 
-	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp_in);
+	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (buffer, BUFSIZ, fp_in);
 
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, 2, n);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%d) fields near line %ld\n", GMT_program, n_fields, 2, n);
 			exit (EXIT_FAILURE);
 		}
 		if (Ctrl->T.col >= n_fields) {
-			fprintf (stderr, "%s: Expected at least %d fields but only found %d near line %d\n", GMT_program, Ctrl->T.col+1, n_fields, n);
+			fprintf (stderr, "%s: Expected at least %ld fields but only found %ld near line %ld\n", GMT_program, Ctrl->T.col+1, n_fields, n);
 			exit (EXIT_FAILURE);
 		}
 		data[n] = in[Ctrl->T.col];
 		if (!GMT_is_dnan (data[n])) n++;
 
 		if (n == n_alloc) {
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			data = (double *) GMT_memory ((void *)data, (size_t) n_alloc, sizeof (double), GMT_program);
 		}
 		x_min = MIN (x_min, data[n-1]);
@@ -397,7 +386,7 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
-	if (gmtdefs.verbose) fprintf (stderr,"%s: %d points read\n", GMT_program, n);
+	if (gmtdefs.verbose) fprintf (stderr,"%s: %ld points read\n", GMT_program, n);
 
 	data = (double *) GMT_memory ((void *)data, (size_t)n , sizeof (double), GMT_program);
 	if (fp_in != GMT_stdin) GMT_fclose (fp_in);
@@ -440,7 +429,7 @@ int main (int argc, char **argv)
 
 	if (Ctrl->I.active) {	/* Only info requested, quit before plotting */
 		if (Ctrl->I.mode) {
-			int ibox;
+			GMT_LONG ibox;
 			double xx, yy;
 			sprintf (format, "%s\t%s\n", gmtdefs.d_format, gmtdefs.d_format);
 			for (ibox = 0; ibox < F.n_boxes; ibox++) {
@@ -538,12 +527,12 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-int fill_boxes (struct PSHISTOGRAM_INFO *F, double *data, int n) {
+GMT_LONG fill_boxes (struct PSHISTOGRAM_INFO *F, double *data, GMT_LONG n) {
 
 	double	add_half = 0.0;
-	int	b0, b1, i, ibox, count_sum;
+	GMT_LONG	b0, b1, i, ibox, count_sum;
 
-	F->n_boxes = (int)ceil( (F->east - F->west) / F->box_width);
+	F->n_boxes = (GMT_LONG)ceil( (F->east - F->west) / F->box_width);
 
 	if (F->center_box) {
 		F->n_boxes++;
@@ -552,14 +541,14 @@ int fill_boxes (struct PSHISTOGRAM_INFO *F, double *data, int n) {
 
 	if (F->n_boxes <= 0) return (-1);
 
-	F->boxh = (int *) GMT_memory (VNULL, (size_t)F->n_boxes , sizeof (int), GMT_program);
+	F->boxh = (GMT_LONG *) GMT_memory (VNULL, (size_t)F->n_boxes , sizeof (GMT_LONG), GMT_program);
 
 	F->n_counted = 0;
 
 	/* First fill boxes with counts  */
 
 	for (i = 0; i < n; i++) {
-		ibox = (int)floor( ( (data[i] - F->west) / F->box_width) + add_half);
+		ibox = (GMT_LONG)floor( ( (data[i] - F->west) / F->box_width) + add_half);
 		if (ibox < 0 || ibox >= F->n_boxes) continue;
 		F->boxh[ibox] ++;
 		F->n_counted++;
@@ -569,7 +558,7 @@ int fill_boxes (struct PSHISTOGRAM_INFO *F, double *data, int n) {
 		count_sum = 0;
 		for (ibox = 0; ibox < F->n_boxes; ibox++) {
 			count_sum += F->boxh[ibox];
-			F->boxh[ibox] = count_sum;
+			F->boxh[ibox] = (GMT_LONG)count_sum;
 		}
 		b0 = 0;
 		b1 = count_sum;
@@ -597,10 +586,10 @@ int fill_boxes (struct PSHISTOGRAM_INFO *F, double *data, int n) {
 		else if (F->hist_type == PSHISTOGRAM_LOG10_FREQ_PCT)
 			F->yy0 = d_log101p( 100.0 * b0 / F->n_counted );
 		else
-			F->yy0 = b0;
+			F->yy0 = (double)b0;
 	}
 	else {
-		F->yy0 = 0;
+		F->yy0 = 0.0;
 	}
 	if (b1) {
 		if (F->hist_type == PSHISTOGRAM_LOG_COUNTS)
@@ -614,17 +603,18 @@ int fill_boxes (struct PSHISTOGRAM_INFO *F, double *data, int n) {
 		else if (F->hist_type == PSHISTOGRAM_LOG10_FREQ_PCT)
 			F->yy1 = d_log101p( 100.0 * b1 / F->n_counted );
 		else
-			F->yy1 = b1;
+			F->yy1 = (double)b1;
 	}
 	else {
-		F->yy1 = 0;
+		F->yy1 = 0.0;
 	}
 	return (0);
 }
 
-int plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, BOOLEAN draw_outline, struct GMT_PEN *pen, struct GMT_FILL *fill, BOOLEAN cpt)
+GMT_LONG plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, BOOLEAN draw_outline, struct GMT_PEN *pen, struct GMT_FILL *fill, BOOLEAN cpt)
 {
-	int	i, ibox, first = TRUE, index, rgb[3];
+	GMT_LONG	i, ibox, first = TRUE, index;
+	int rgb[3];
 	double	x[4], y[4], xx, yy, xval, *px, *py;
 	struct GMT_FILL *f;
 
@@ -681,12 +671,12 @@ int plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, B
 			else if (cpt) {
 				index = GMT_get_rgb_from_z (xval, rgb);
 				if ((index >= 0 && (f = GMT_lut[index].fill)) || (index < 0 && (f = GMT_bfn[index+3].fill)))	/* Pattern */
-					GMT_fill (px, py, 4, f, draw_outline);
+					GMT_fill (px, py, (GMT_LONG)4, f, draw_outline);
 				else
-					ps_patch (px, py, 4, rgb, draw_outline);
+					ps_patch (px, py, (GMT_LONG)4, rgb, draw_outline);
 			}
 			else
-				GMT_fill (px, py, 4, fill, draw_outline);
+				GMT_fill (px, py, (GMT_LONG)4, fill, draw_outline);
 		}
 	}
 
@@ -695,11 +685,12 @@ int plot_boxes (struct PSHISTOGRAM_INFO *F, BOOLEAN stairs, BOOLEAN flip_to_y, B
 	return (0);
 }
 
-int get_loc_scl (double *data, int n, double *stats)
+GMT_LONG get_loc_scl (double *data, GMT_LONG n, double *stats)
 {
 	/* Returns stats[] = L2, L1, LMS location, L2, L1, LMS scale  */
 
-	int	i, j, n_multiples;
+	GMT_LONG	i, j;
+	GMT_LONG n_multiples;
 	double	dx;
 
 	if (n < 3) return (-1);
@@ -712,16 +703,16 @@ int get_loc_scl (double *data, int n, double *stats)
 
 	/* Get mode */
 
-	GMT_mode (data, (size_t)n, (size_t)j, 0, 0, &n_multiples, &stats[2]);
-	if (n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %d multiple modes found\n", GMT_program, n_multiples);
+	GMT_mode (data, n, j, 0, 0, &n_multiples, &stats[2]);
+	if (n_multiples > 0 && gmtdefs.verbose) fprintf (stderr, "%s: WARNING: %ld multiple modes found\n", GMT_program, n_multiples);
 
 	/* Get MAD for L1 */
 
-	GMT_getmad (data, (size_t)n, stats[1], &stats[4]);
+	GMT_getmad (data, n, stats[1], &stats[4]);
 
 	/* Get LMSscale for mode */
 
-	GMT_getmad (data, (size_t)n, stats[2], &stats[5]);
+	GMT_getmad (data, n, stats[2], &stats[5]);
 
 	/* Calculate mean and stdev in two passes to minimize risk of overflow */
 
@@ -740,7 +731,7 @@ int get_loc_scl (double *data, int n, double *stats)
 void *New_Pshistogram_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSHISTOGRAM_CTRL *C;
 	
-	C = (struct PSHISTOGRAM_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSHISTOGRAM_CTRL), "New_Pshistogram_Ctrl");
+	C = (struct PSHISTOGRAM_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSHISTOGRAM_CTRL), "New_Pshistogram_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.azimuth = 180.0;
