@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psscale.c,v 1.82 2007/10/04 23:13:49 guru Exp $
+ *	$Id: psscale.c,v 1.99 2009/07/10 22:22:06 remko Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -44,6 +44,7 @@
 struct PSSCALE_CTRL {
 	struct A {	/* -A */
 		BOOLEAN active;
+		GMT_LONG mode;
 	} A;
 	struct C {	/* -C<cptfile> */
 		BOOLEAN active;
@@ -56,7 +57,7 @@ struct PSSCALE_CTRL {
 	} D;
 	struct E {	/* -E[b|f][<length>] */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 		double length;
 	} E;
 	struct I {	/* -I[<intens>|<min_i>/<max_i>] */
@@ -68,7 +69,7 @@ struct PSSCALE_CTRL {
 	} M;
 	struct N {	/* -N<dpi> */
 		BOOLEAN active;
-		int dpi;
+		GMT_LONG dpi;
 	} N;
 	struct L {	/* -L[i][<gap>] */
 		BOOLEAN active;
@@ -94,7 +95,7 @@ int main(int argc, char **argv)
 	char flag, line[BUFSIZ], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
 	char txt_c[GMT_LONG_TEXT], txt_d[GMT_LONG_TEXT], text[GMT_LONG_TEXT];
 
-	int i, j, n;
+	GMT_LONG i, j, n;
 
 	double max_intens[2], dz, *z_width, x_origin = 0.0, y_origin = 0.0;
 	double start_val, stop_val;
@@ -103,8 +104,8 @@ int main(int argc, char **argv)
 
 	struct PSSCALE_CTRL *Ctrl;
 
-	void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dpi, BOOLEAN flip_anot, BOOLEAN B_set, BOOLEAN equi, 
-	   BOOLEAN horizontal, BOOLEAN logscl, BOOLEAN intens, double *max_intens, BOOLEAN skip_lines, int extend, double e_length, double gap, BOOLEAN interval_annot, BOOLEAN monochrome);
+	void GMT_draw_colorbar (double length, double width, double *z_width, GMT_LONG bit_dpi, GMT_LONG flip, BOOLEAN B_set, BOOLEAN equi, 
+	   BOOLEAN horizontal, BOOLEAN logscl, BOOLEAN intens, double *max_intens, BOOLEAN skip_lines, GMT_LONG extend, double e_length, double gap, BOOLEAN interval_annot, BOOLEAN monochrome);
 
 	void *New_Psscale_Ctrl (), Free_Psscale_Ctrl (struct PSSCALE_CTRL *C);
 	
@@ -140,6 +141,14 @@ int main(int argc, char **argv)
 
 				case 'A':
 					Ctrl->A.active = TRUE;
+					if (!argv[i][2]) Ctrl->A.mode |= 3;
+					for (j = 2; argv[i][j]; j++) {
+						switch (argv[i][j]) {
+							case 'a': Ctrl->A.mode |= 1; break;
+							case 'l': Ctrl->A.mode |= 2; break;
+							case 'c': Ctrl->A.mode |= 4; break;
+						}
+					}
 					break;
 				case 'C':
 					Ctrl->C.active = TRUE;
@@ -147,7 +156,7 @@ int main(int argc, char **argv)
 					break;
 				case 'D':
 					Ctrl->D.active = TRUE;
-					n = strlen(argv[i]) - 1;
+					n = (GMT_LONG)strlen(argv[i]) - 1;
 					flag = argv[i][n];
 					if (flag == 'h' || flag == 'H') {
 						Ctrl->D.horizontal = TRUE;
@@ -224,7 +233,7 @@ int main(int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "%s %s - To create grayscale or colorscale for maps\n\n", GMT_program, GMT_VERSION);
-		fprintf(stderr, "usage: psscale -D<xpos/ypos/length/width>[h] [-A] [-C<cpt_file>] [-E[b|f][<length>]] [%s] [-I[<max_intens>|<low_i>/<high_i>]\n", GMT_B_OPT);
+		fprintf(stderr, "usage: psscale -D<xpos/ypos/length/width>[h] [-A[a|l|c]] [-C<cpt_file>] [-E[b|f][<length>]] [%s] [-I[<max_intens>|<low_i>/<high_i>]\n", GMT_B_OPT);
 		fprintf(stderr, "\t[-K] [-L[i][<gap>]] [-M] [-N<dpi>] [-O] [-P] [-Q] [-S] [%s] [-V] [%s]\n\t[%s] [-Z<zfile>] [%s]\n\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT);
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -232,7 +241,9 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t   Give negative length to reverse the scalebar\n");
 		fprintf (stderr, "\t   Append h for horizontal scale\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "\t-A Place the desired annotations on the other side of the colorscale instead\n");
+		fprintf (stderr, "\t-A Place the desired annotations/labels on the other side of the colorscale instead\n");
+		fprintf (stderr, "\t   Append a or l to move only the annotations or labels to the other side\n");
+		fprintf (stderr, "\t   Append c to plot vertical labels as columns\n");
 		fprintf (stderr, "\t-B Set scale annotation interval and label. Use y-label to set unit label\n");
 		fprintf (stderr, "\t   If no annotation interval is set it is taken from the cpt file\n");
 		fprintf (stderr, "\t-C Color palette file. If not set, stdin is read.\n");
@@ -242,7 +253,7 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t   If a categorical CPT file is given the -Li is set automatically\n");
 		fprintf (stderr, "\t-E add sidebar triangles for back- and foreground colors.\n");
 		fprintf (stderr, "\t   Specify b(ackground) or f(oreground) to get one only [Default is both]\n");
-		fprintf (stderr, "\t   Optionally, append triangle height [Default scales with barwidth]\n");
+		fprintf (stderr, "\t   Optionally, append triangle height [Default is half the barwidth]\n");
 		fprintf (stderr, "\t-I add illumination for +-<max_intens> or <low_i> to <high_i> [-1.0/1.0]\n");
 		fprintf (stderr, "\t   Alternatively, specify <lower>/<upper> intensity values\n");
 		GMT_explain_option ('K');
@@ -270,17 +281,19 @@ int main(int argc, char **argv)
 
 	/* Check that the options selected are mutually consistent */
 
-	if (z_project.view_azimuth > 360.0 || z_project.view_elevation <= 0.0 || z_project.view_elevation > 90.0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Enter azimuth in 0-360 range, elevation in 0-90 range\n", GMT_program);
+	if (!Ctrl->D.active) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR: -D is required and must be specified\n", GMT_program);
 		error++;
 	}
-	if (fabs (Ctrl->D.length) < GMT_SMALL ) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: scale length must be nonzero\n", GMT_program);
-		error++;
-	}
-	if (Ctrl->D.width <= 0.0) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: scale width must be positive\n", GMT_program);
-		error++;
+	else {
+		if (fabs (Ctrl->D.length) < GMT_SMALL ) {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: scale length must be nonzero\n", GMT_program);
+			error++;
+		}
+		if (Ctrl->D.width <= 0.0) {
+			fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: scale width must be positive\n", GMT_program);
+			error++;
+		}
 	}
 	if (Ctrl->N.dpi < 1) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -N option: dpi must be > 0\n", GMT_program);
@@ -345,7 +358,7 @@ int main(int argc, char **argv)
 			if (line[0] == '#' || line[0] == '\n') continue;	/* Skip comments and blank lines */
 			n = sscanf (line, "%lf", &z_width[i++]);
 			if (n != 1) {
-				fprintf (stderr, "%s: Error reading file %s near line %d\n", GMT_program, Ctrl->Z.file, i);
+				fprintf (stderr, "%s: Error reading file %s near line %ld\n", GMT_program, Ctrl->Z.file, i);
 				exit (EXIT_FAILURE);
 			}
 		}
@@ -418,11 +431,11 @@ int main(int argc, char **argv)
 
 	if (GMT_ps.unix_time) {	/* Extra shift/unshift since psscale does things differently */
 		ps_transrotate (-GMT_ps.x_origin, -GMT_ps.y_origin, 0.0);
-		GMT_timestamp (GMT_ps.unix_time_pos[0], GMT_ps.unix_time_pos[1], GMT_ps.unix_time_label);
+		GMT_timestamp (GMT_ps.unix_time_pos[0], GMT_ps.unix_time_pos[1], GMT_ps.unix_time_just, GMT_ps.unix_time_label);
 		ps_transrotate (GMT_ps.x_origin, GMT_ps.y_origin, 0.0);
 	}
 
-	GMT_draw_colorbar (Ctrl->D.length, Ctrl->D.width, z_width, Ctrl->N.dpi, Ctrl->A.active, B_set, Ctrl->L.active, Ctrl->D.horizontal, Ctrl->Q.active, Ctrl->I.active, max_intens, Ctrl->S.active, Ctrl->E.mode, Ctrl->E.length, Ctrl->L.spacing, Ctrl->L.interval, Ctrl->M.active);
+	GMT_draw_colorbar (Ctrl->D.length, Ctrl->D.width, z_width, Ctrl->N.dpi, Ctrl->A.mode, B_set, Ctrl->L.active, Ctrl->D.horizontal, Ctrl->Q.active, Ctrl->I.active, max_intens, Ctrl->S.active, Ctrl->E.mode, Ctrl->E.length, Ctrl->L.spacing, Ctrl->L.interval, Ctrl->M.active);
 
 	if (must_shift_back) ps_transrotate (-x_origin, -y_origin, 0.0);
 
@@ -439,16 +452,17 @@ int main(int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dpi, BOOLEAN flip_anot, BOOLEAN B_set, BOOLEAN equi, BOOLEAN horizontal, BOOLEAN logscl, BOOLEAN intens, double *max_intens, BOOLEAN skip_lines, int extend, double e_length, double gap, BOOLEAN interval_annot, BOOLEAN monochrome)
+void GMT_draw_colorbar (double length, double width, double *z_width, GMT_LONG bit_dpi, GMT_LONG flip, BOOLEAN B_set, BOOLEAN equi, BOOLEAN horizontal, BOOLEAN logscl, BOOLEAN intens, double *max_intens, BOOLEAN skip_lines, GMT_LONG extend, double e_length, double gap, BOOLEAN interval_annot, BOOLEAN monochrome)
 {
-	int i, ii, id, j, ndec = -1, dec, p_val, depth, rgb[3], rrggbb[3];
-	int nx = 0, ny = 0, nm, barmem, k, justify, l_justify, this_just, use_labels = 0, Label_justify;
+	GMT_LONG i, ii, id, j, nb, ndec = -1, dec, p_val, depth;
+	GMT_LONG nx = 0, ny = 0, nm, barmem, k, justify, l_justify, this_just, use_labels = 0, Label_justify;
+	int rgb[3], rrggbb[3];
 	BOOLEAN reverse, all = TRUE, use_image, center = FALSE, const_width = TRUE, do_annot;
-	char format[GMT_LONG_TEXT], text[GMT_LONG_TEXT], test[GMT_LONG_TEXT], unit[GMT_LONG_TEXT];
+	char format[GMT_LONG_TEXT], text[GMT_LONG_TEXT], test[GMT_LONG_TEXT], unit[GMT_LONG_TEXT], label[GMT_LONG_TEXT];
 	unsigned char *bar = 0, *tmp = 0;
-	double off, annot_off, label_off, len, len2, size, x0, x1, dx, xx, dir, y_base;
+	double off, annot_off, label_off, len, len2, size, x0, x1, dx, xx, dir, y_base, y_annot, y_label;
 	double z, xleft, xright, inc_i, inc_j, start_val, stop_val;
-	double get_z (double x, double *width, int n);
+	double get_z (double x, double *width, GMT_LONG n);
 	double xp[4], yp[4];
 	struct GMT_FILL *f;
 	void fix_format (char *unit, char *format);
@@ -555,23 +569,32 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 
 	GMT_setpen (&gmtdefs.frame_pen);
 
-	unit[0] = 0;
-	if (!strcmp (frame_info.axis[0].label, frame_info.axis[1].label)) frame_info.axis[1].label[0] = 0;	/* Defeat the auto-repeat of axis info */
-	strcpy (unit, frame_info.axis[1].label);	/* Save possible unit here since we need to copy x-axis info to the y-axis for vertical axis */
+	unit[0] = label[0] = 0;
+	/* Defeat the auto-repeat of axis info */
+	if (!strcmp (frame_info.axis[0].label, frame_info.axis[1].label)) frame_info.axis[1].label[0] = 0;
+	/* Save label and unit, because we are going to switch them off in frame_info and do it ourselves */
+	strcpy (label, frame_info.axis[0].label);
+	strcpy (unit, frame_info.axis[1].label);
+	frame_info.axis[0].label[0] = frame_info.axis[1].label[1] = 0;
 
-	if (flip_anot) {
-		justify = (horizontal) ? 2 : 5;
-		l_justify = (horizontal) ? 2 : 7;
-		Label_justify = 2;
+	if (flip & 1) {
+		justify = l_justify = (horizontal) ? 2 : 7;
 		y_base = width;
-		dir = -1.0;
+		dir = 1.0;
 	}
 	else {
 		justify = (horizontal) ? 10 : 7;
-		Label_justify = 10;
 		l_justify = (horizontal) ? 10 : 5;
 		y_base = 0.0;
-		dir = 1.0;
+		dir = -1.0;
+	}
+	if (flip & 2) {
+		Label_justify = 2;
+		y_label = width + gmtdefs.label_offset;
+	}
+	else {
+		Label_justify = 10;
+		y_label = -gmtdefs.label_offset;
 	}
 
 	/* Current point (0,0) is now at lower left location of bar */
@@ -589,18 +612,19 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 				x1 += z_width[ii];
 				if ((f = GMT_lut[ii].fill)) {	/* Using pattern fills */
 					xp[0] = xp[3] = x0 + gap;	xp[1] = xp[2] = x1 - gap;
-					GMT_fill (xp, yp, 4, f, center);	/* Contours drawn separately below if desired */
+					GMT_fill (xp, yp, (GMT_LONG)4, f, center);	/* Contours drawn separately below if desired */
 				}
 				else if (intens) {
-					ii = (GMT_gray || monochrome) ? 1 : 3;
-					tmp = (unsigned char *) GMT_memory (VNULL, (size_t)ny*ii, sizeof (char), GMT_program);
-					for (j = 0, k = i*ii; j < ny*ii; k+=(nx-1)*ii) {
+					nb = (GMT_gray || monochrome) ? 1 : 3;
+					tmp = (unsigned char *) GMT_memory (VNULL, (size_t)ny*nb, sizeof (char), GMT_program);
+					for (j = 0, k = i*nb; j < ny*nb; k+=(nx-1)*nb) {
 						tmp[j++] = bar[k++];
 						tmp[j++] = bar[k++];
 						tmp[j++] = bar[k++];
 					}
-					GMT_color_image (x0 + gap, 0.0, z_width[i] - 2*gap, width, tmp, 1, ny, depth);
+					GMT_color_image (x0 + gap, 0.0, z_width[ii] - 2*gap, width, tmp, 1, ny, depth);
 					GMT_free ((void *)tmp);
+					ps_rect (x0+gap, 0.0, x1-gap, width, GMT_no_rgb, center);
 				}
 				else {
 					memcpy ((void *)rgb, (void *)GMT_lut[ii].rgb_low, (size_t)(3 * sizeof (int)));
@@ -612,31 +636,36 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 			}
 		}
 
-		if (extend & 1) {	/* Add color triangles for background color  */
+		annot_off = ((len > 0.0 && !center) ? len : 0.0) + gmtdefs.annot_offset[0];
+		label_off = annot_off + gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH] + gmtdefs.label_offset;
+		y_annot = y_base + dir * annot_off;
+		if ((flip & 1) == (flip & 2) / 2) y_label = y_base + dir * label_off;
+
+		if (extend & (reverse + 1)) {	/* Add color triangle on left side */
 			xp[0] = xp[1] = xleft - gap;	xp[2] = xp[0] - e_length;
 			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
 			id = (reverse) ? GMT_FGD : GMT_BGD;
 			if ((f = GMT_bfn[id].fill))
-				GMT_fill (xp, yp, 3, f, FALSE);	/* Contours drawn separately below if desired */
+				GMT_fill (xp, yp, (GMT_LONG)3, f, FALSE);	/* Contours drawn separately below if desired */
 			else {
 				memcpy ((void *)rgb, (void *)GMT_bfn[id].rgb, (size_t)(3 * sizeof (int)));
 				if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb);
-				ps_polygon (xp, yp, 3, rgb, FALSE);
+				ps_polygon (xp, yp, (GMT_LONG)3, rgb, FALSE);
 			}
-			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
+			ps_polygon (xp, yp, (GMT_LONG)3, GMT_no_rgb, TRUE);
 		}
-		if (extend & 2) {	/* Add color triangles for foreground color  */
+		if (extend & (2 - reverse)) {	/* Add color triangle on right side */
 			xp[0] = xp[1] = xright + gap;	xp[2] = xp[0] + e_length;
 			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
 			id = (reverse) ? GMT_BGD : GMT_FGD;
 			if ((f = GMT_bfn[id].fill))
-				GMT_fill (xp, yp, 3, f, FALSE);	/* Contours drawn separately below if desired */
+				GMT_fill (xp, yp, (GMT_LONG)3, f, FALSE);	/* Contours drawn separately below if desired */
 			else {
 				memcpy ((void *)rgb, (void *)GMT_bfn[id].rgb, (size_t)(3 * sizeof (int)));
 				if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb);
-				ps_polygon (xp, yp, 3, rgb, FALSE);
+				ps_polygon (xp, yp, (GMT_LONG)3, rgb, FALSE);
 			}
-			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
+			ps_polygon (xp, yp, (GMT_LONG)3, GMT_no_rgb, TRUE);
 		}
 
 		if (gap == 0.0) {
@@ -645,18 +674,13 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 			ps_segment (xleft, 0.0, xleft, width);
 			ps_segment (xright, 0.0, xright, width);
 		}
+
 		if (B_set) {	/* Used -B */
-			if (flip_anot)
-				GMT_xy_axis (xleft, width, length, start_val, stop_val, &frame_info.axis[0], FALSE, frame_info.side[0]-1);
-			else
-				GMT_xy_axis (xleft, 0.0, length, start_val, stop_val, &frame_info.axis[0], TRUE, frame_info.side[0]-1);
+			GMT_xy_axis (xleft, y_base, length, start_val, stop_val, &frame_info.axis[0], !(flip & 1), frame_info.side[0]-1);
 			dx =  GMT_get_map_interval (0, GMT_GRID_UPPER); 
 			if (dx > 0.0) GMT_linearx_grid (GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, 0.0, width, dx);
 		}
 		else {
-			annot_off = (((len > 0.0 && !center) ? len : 0.0) + gmtdefs.annot_offset[0]);
-			label_off = -(annot_off + 1.5 * gmtdefs.annot_offset[0] + (gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH]) * GMT_font[gmtdefs.annot_font[0]].height);
-
 			/* First draw gridlines, unless skip_lines is TRUE */
 
 			if (!skip_lines) {
@@ -680,7 +704,7 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 				xx = (reverse) ? xright - x1 : x1;
 				if (!center) ps_plot (xx, y_base, PSL_PEN_MOVE);
 				if (all || (GMT_lut[i].annot & 1)) {	/* Annotate this */
-					if (!center) ps_plotr (0.0, -(len * dir), PSL_PEN_DRAW_AND_STROKE);
+					if (!center) ps_plotr (0.0, dir * len, PSL_PEN_DRAW_AND_STROKE);
 					this_just = justify;
 					do_annot = TRUE;
 					if (use_labels && GMT_lut[i].label) {
@@ -692,48 +716,48 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 					else if (logscl) {
 						p_val = irint (GMT_lut[i].z_low);
 						if (GMT_IS_ZERO (GMT_lut[i].z_low - (double)p_val))
-							sprintf (text, "10@+%d@+", irint (GMT_lut[i].z_low));
+							sprintf (text, "10@+%ld@+", (GMT_LONG)irint (GMT_lut[i].z_low));
 						else
 							do_annot = FALSE;
 					}
 					else
 						sprintf (text, format, GMT_lut[i].z_low);
-					if (do_annot) ps_text (xx, y_base - (dir * annot_off), gmtdefs.annot_font_size[0], text, 0.0, -this_just, 0);
+					if (do_annot) ps_text (xx, y_annot, gmtdefs.annot_font_size[0], text, 0.0, -this_just, 0);
 				}
 				else
-					if (!center) ps_plotr (0.0, -(len2 * dir), PSL_PEN_DRAW_AND_STROKE);
+					if (!center) ps_plotr (0.0, dir * len2, PSL_PEN_DRAW_AND_STROKE);
 				x1 += z_width[i];
 			}
-			xx = (reverse) ? xleft : xright;
 			if (!center && !use_labels) {
-				ps_plot (xx, 0.0, PSL_PEN_MOVE);
+				xx = (reverse) ? xleft : xright;
+				ps_plot (xx, y_base, PSL_PEN_MOVE);
 				i = GMT_n_colors-1;
 				if (all || (GMT_lut[i].annot & 2)) {
-					ps_plotr (0.0, -(len * dir), PSL_PEN_DRAW_AND_STROKE);
+					ps_plotr (0.0, dir * len, PSL_PEN_DRAW_AND_STROKE);
 					this_just = justify;
 					do_annot = TRUE;
 					if (logscl) {
 						p_val = irint (GMT_lut[i].z_high);
 						if (GMT_IS_ZERO (GMT_lut[i].z_high - (double)p_val))
-							sprintf (text, "10@+%d@+", p_val);
+							sprintf (text, "10@+%ld@+", p_val);
 						else
 							do_annot = FALSE;
 					}
 					else
 						sprintf (text, format, GMT_lut[i].z_high);
-					if (do_annot) ps_text (xx, y_base - (dir * annot_off), gmtdefs.annot_font_size[0], text, 0.0, -this_just, 0);
+					if (do_annot) ps_text (xx, y_annot, gmtdefs.annot_font_size[0], text, 0.0, -this_just, 0);
 				}
 				else
-					ps_plotr (0.0, -(len2 * dir), PSL_PEN_DRAW_AND_STROKE);
-			}
-			if (frame_info.axis[0].label[0]) {
-				ps_setfont (gmtdefs.label_font);
-				ps_text (xleft+0.5*length, y_base + (dir * label_off), gmtdefs.label_font_size, frame_info.axis[0].label, 0.0, Label_justify, 0);
+					ps_plotr (0.0, dir * len2, PSL_PEN_DRAW_AND_STROKE);
 			}
 		}
+		if (label[0]) {	/* Add label */
+			ps_setfont (gmtdefs.label_font);
+			ps_text (xleft + 0.5 * length, y_label, gmtdefs.label_font_size, label, 0.0, Label_justify, 0);
+		}
 		if (unit[0]) {	/* Add unit label */
 			ps_setfont (gmtdefs.annot_font[0]);
-			ps_text (xright+e_length+gmtdefs.annot_offset[0], 0.5*width, gmtdefs.annot_font_size[0], unit, 0.0, 5, 0);
+			ps_text (xright + e_length + gmtdefs.annot_offset[0], 0.5 * width, gmtdefs.annot_font_size[0], unit, 0.0, 5, 0);
 		}
 	}
 	else {	/* Vertical scale */
@@ -745,24 +769,26 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 			x0 = x1 = 0.0;
 			yp[0] = yp[1] = 0.0;	yp[2] = yp[3] = width;
 			for (i = 0; i < GMT_n_colors; i++) {
-				x1 += z_width[i];
-				if ((f = GMT_lut[i].fill)) {	/* Using pattern fills */
+				ii = (reverse) ? GMT_n_colors - i - 1 : i;
+				x1 += z_width[ii];
+				if ((f = GMT_lut[ii].fill)) {	/* Using pattern fills */
 					xp[0] = xp[3] = x0 + gap;	xp[1] = xp[2] = x1 - gap;
-					GMT_fill (xp, yp, 4, f, center);	/* Contours drawn separately below if desired */
+					GMT_fill (xp, yp, (GMT_LONG)4, f, center);	/* Contours drawn separately below if desired */
 				}
 				else if (intens) {
-					ii = (GMT_gray || monochrome) ? 1 : 3;
-					tmp = (unsigned char *) GMT_memory (VNULL, (size_t)ny*ii, sizeof (char), GMT_program);
-					for (j = 0, k = i*ii; j < ny*ii; k+=(nx-1)*ii) {
+					nb = (GMT_gray || monochrome) ? 1 : 3;
+					tmp = (unsigned char *) GMT_memory (VNULL, (size_t)ny*nb, sizeof (char), GMT_program);
+					for (j = 0, k = i*nb; j < ny*nb; k+=(nx-1)*nb) {
 						tmp[j++] = bar[k++];
 						tmp[j++] = bar[k++];
 						tmp[j++] = bar[k++];
 					}
-					GMT_color_image (x0 + gap, 0.0, z_width[i] - 2*gap, width, tmp, 1, ny, depth);
+					GMT_color_image (x0 + gap, 0.0, z_width[ii] - 2*gap, width, tmp, 1, ny, depth);
 					GMT_free ((void *)tmp);
+					ps_rect (x0 + gap, 0.0, x1 - gap, width, GMT_no_rgb, center);
 				}
 				else {
-					memcpy ((void *)rgb, (void *)GMT_lut[i].rgb_low, (size_t)(3 * sizeof (int)));
+					memcpy ((void *)rgb, (void *)GMT_lut[ii].rgb_low, (size_t)(3 * sizeof (int)));
 					if (intens) GMT_illuminate (max_intens[1], rgb);
 					if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb);
 					ps_rect (x0 + gap, 0.0, x1 - gap, width, rgb, center);
@@ -770,49 +796,53 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 				x0 = x1;
 			}
 		}
+
 		if (center && interval_annot) {
-			sprintf (text, "%d - %d", (int) (floor (GMT_lut[0].z_low)), (int) (ceil (GMT_lut[0].z_high)));
-			sprintf (test, "%d - %d", (int) (floor (GMT_lut[GMT_n_colors-1].z_low)), (int) (ceil (GMT_lut[GMT_n_colors-1].z_high)));
-			off = ((MAX ((int)strlen (text), (int)strlen (test)) + 2*ndec) * GMT_DEC_SIZE - 0.4 + 
+			sprintf (text, "%ld - %ld", (GMT_LONG) (floor (GMT_lut[0].z_low)), (GMT_LONG) (ceil (GMT_lut[0].z_high)));
+			sprintf (test, "%ld - %ld", (GMT_LONG) (floor (GMT_lut[GMT_n_colors-1].z_low)), (GMT_LONG) (ceil (GMT_lut[GMT_n_colors-1].z_high)));
+			off = ((MAX ((GMT_LONG)strlen (text), (GMT_LONG)strlen (test)) + 2*ndec) * GMT_DEC_SIZE - 0.4 + 
 				((ndec > 0) ? 2*GMT_PER_SIZE : 0.0))
 				* gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH];
 		}
 		else {
-			sprintf (text, "%d", (int) (floor (GMT_lut[0].z_low)));
-			sprintf (test, "%d", (int) (ceil (center ? GMT_lut[GMT_n_colors-1].z_low : GMT_lut[GMT_n_colors-1].z_high)));
-			off = ((MAX ((int)strlen (text), (int)strlen (test)) + ndec) * GMT_DEC_SIZE +
+			sprintf (text, "%ld", (GMT_LONG) (floor (GMT_lut[0].z_low)));
+			sprintf (test, "%ld", (GMT_LONG) (ceil (center ? GMT_lut[GMT_n_colors-1].z_low : GMT_lut[GMT_n_colors-1].z_high)));
+			off = ((MAX ((GMT_LONG)strlen (text), (GMT_LONG)strlen (test)) + ndec) * GMT_DEC_SIZE +
 				((ndec > 0) ? GMT_PER_SIZE : 0.0))
 				* gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH];
 		}
-		if (use_labels) off = 0.0;
-		annot_off = -(((len > 0.0 && !center) ? len : 0.0) + gmtdefs.annot_offset[0] + off);
-		if (logscl) annot_off = -(((len > 0.0 && !center) ? len : 0.0) + gmtdefs.annot_offset[0]);
-		label_off = annot_off - 0.7 * gmtdefs.annot_font_size[0] * GMT_u2u[GMT_PT][GMT_INCH] - 1.5 * gmtdefs.annot_offset[0];
-		if (extend & 1) {	/* Add color triangles for background */
+
+		annot_off = ((len > 0.0 && !center) ? len : 0.0) + gmtdefs.annot_offset[0] + off;
+		label_off = annot_off + gmtdefs.label_offset;
+		if (use_labels || (flip & 1) || logscl) annot_off -= off;
+		y_annot = y_base + dir * annot_off;
+		if ((flip & 1) == (flip & 2) / 2) y_label = y_base + dir * label_off;
+
+		if (extend & (reverse + 1)) {	/* Add color triangle at bottom */
 			xp[0] = xp[1] = xleft - gap;	xp[2] = xp[0] - e_length;
 			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
 			id = (reverse) ? GMT_FGD : GMT_BGD;
 			if ((f = GMT_bfn[id].fill))
-				GMT_fill (xp, yp, 3, f, FALSE);	/* Contours drawn separately below if desired */
+				GMT_fill (xp, yp, (GMT_LONG)3, f, FALSE);	/* Contours drawn separately below if desired */
 			else {
 				memcpy ((void *)rgb, (void *)GMT_bfn[id].rgb, (size_t)(3 * sizeof (int)));
 				if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb);
-				ps_polygon (xp, yp, 3, rgb, FALSE);
+				ps_polygon (xp, yp, (GMT_LONG)3, rgb, FALSE);
 			}
-			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
+			ps_polygon (xp, yp, (GMT_LONG)3, GMT_no_rgb, TRUE);
 		}
-		if (extend & 2) {	/* Add color triangles for foreground */
+		if (extend & (2 - reverse)) {	/* Add color triangle at top */
 			xp[0] = xp[1] = xright + gap;	xp[2] = xp[0] + e_length;
 			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
 			id = (reverse) ? GMT_BGD : GMT_FGD;
 			if ((f = GMT_bfn[id].fill))
-				GMT_fill (xp, yp, 3, f, FALSE);	/* Contours drawn separately below if desired */
+				GMT_fill (xp, yp, (GMT_LONG)3, f, FALSE);	/* Contours drawn separately below if desired */
 			else {
 				memcpy ((void *)rgb, (void *)GMT_bfn[id].rgb, (size_t)(3 * sizeof (int)));
 				if (monochrome) rgb[0] = rgb[1] = rgb[2] = GMT_YIQ (rgb);
-				ps_polygon (xp, yp, 3, rgb, FALSE);
+				ps_polygon (xp, yp, (GMT_LONG)3, rgb, FALSE);
 			}
-			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
+			ps_polygon (xp, yp, (GMT_LONG)3, GMT_no_rgb, TRUE);
 		}
 		if (gap == 0.0) {
 			ps_segment (xleft, 0.0, xleft + length, 0.0);
@@ -821,18 +851,17 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 			ps_segment (xright, 0.0, xright, width);
 		}
 		if (B_set) {	/* Used -B. Must kludge by copying x-axis and scaling to y since we must use GMT_xy_axis to draw a rotated x-axis... */
+			PFL tmp;
 			dx = GMT_get_map_interval (0, GMT_GRID_UPPER); 
 			if (dx > 0.0) GMT_linearx_grid (GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, 0.0, width, dx);
 			ps_transrotate (0.0, 0.0, -90.0);
 			memcpy ((void *)&frame_info.axis[1], (void *)&frame_info.axis[0], sizeof (struct GMT_PLOT_AXIS));
-			frame_info.axis[1].label[0] = 0;
 			d_swap (project_info.x_scale, project_info.y_scale);
 			d_swap (project_info.x0, project_info.y0);
+			i_swap (project_info.xyz_projection[0], project_info.xyz_projection[1]);
+			tmp = GMT_x_forward; GMT_y_forward = GMT_x_forward; GMT_x_forward = tmp;
 			for (i = 0; i < 5; i++) frame_info.axis[1].item[i].parent = 1;
-			if (flip_anot)
-				GMT_xy_axis (-width, 0.0, length, start_val, stop_val, &frame_info.axis[1], TRUE, frame_info.side[0]-1);
-			else
-				GMT_xy_axis (0.0, 0.0, length, start_val, stop_val, &frame_info.axis[1], FALSE, frame_info.side[0]-1);
+			GMT_xy_axis (-y_base, 0.0, length, start_val, stop_val, &frame_info.axis[1], (flip & 1), frame_info.side[0]-1);
 			ps_rotatetrans (0.0, 0.0, 90.0);
 		}
 		else {
@@ -857,7 +886,7 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 				xx = (reverse) ? xright - x1 : x1;
 				if (!center) ps_plot (xx, y_base, PSL_PEN_MOVE);
 				if (all || (GMT_lut[i].annot & 1)) {
-					if (!center) ps_plotr (0.0, -(len * dir), PSL_PEN_DRAW_AND_STROKE);
+					if (!center) ps_plotr (0.0, dir * len, PSL_PEN_DRAW_AND_STROKE);
 					this_just = justify;
 					do_annot = TRUE;
 					if (use_labels && GMT_lut[i].label) {
@@ -869,17 +898,17 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 					else if (logscl) {
 						p_val = irint (GMT_lut[i].z_low);
 						if (GMT_IS_ZERO (GMT_lut[i].z_low - (double)p_val))
-							sprintf (text, "10@+%d@+", p_val);
+							sprintf (text, "10@+%ld@+", p_val);
 						else
 							do_annot = FALSE;
 						this_just = l_justify;
 					}
 					else
 						sprintf (text, format, GMT_lut[i].z_low);
-					if (do_annot) ps_text (xx, y_base + (dir * annot_off), gmtdefs.annot_font_size[0], text, -90.0, -this_just, 0);
+					if (do_annot) ps_text (xx, y_annot, gmtdefs.annot_font_size[0], text, -90.0, -this_just, 0);
 				}
 				else
-					if (!center) ps_plotr (0.0, -(dir * len2), PSL_PEN_DRAW_AND_STROKE);
+					if (!center) ps_plotr (0.0, dir * len2, PSL_PEN_DRAW_AND_STROKE);
 				x1 += z_width[i];
 			}
 			if (!center && !use_labels) {
@@ -887,39 +916,40 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 				ps_plot (xx, y_base, PSL_PEN_MOVE);
 				i = GMT_n_colors-1;
 				if (all || (GMT_lut[i].annot & 2)) {
-					ps_plotr (0.0, -(len * dir), PSL_PEN_DRAW_AND_STROKE);
+					ps_plotr (0.0, dir * len, PSL_PEN_DRAW_AND_STROKE);
 					this_just = justify;
 					do_annot = TRUE;
 					if (logscl) {
 						p_val = irint (GMT_lut[i].z_high);
 						if (GMT_IS_ZERO (GMT_lut[i].z_high - (double)p_val))
-							sprintf (text, "10@+%d@+", p_val);
+							sprintf (text, "10@+%ld@+", p_val);
 						else
 							do_annot = FALSE;
 						this_just = l_justify;
 					}
 					else
 						sprintf (text, format, GMT_lut[i].z_high);
-					if (do_annot) ps_text (xx, y_base + (dir * annot_off), gmtdefs.annot_font_size[0], text, -90.0, -this_just, 0);
+					if (do_annot) ps_text (xx, y_annot, gmtdefs.annot_font_size[0], text, -90.0, -this_just, 0);
 				}
 				else
-					ps_plotr (0.0, -(len2 * dir), PSL_PEN_DRAW_AND_STROKE);
+					ps_plotr (0.0, dir * len2, PSL_PEN_DRAW_AND_STROKE);
 			}
 		}
 
-		if (frame_info.axis[0].label[0]) {
+		if (label[0]) {	/* Add label */
 			ps_setfont (gmtdefs.label_font);
-			if (strchr (frame_info.axis[0].label, '@') || strchr (frame_info.axis[0].label, '(')) {	/* Must set text horizontally */
-				ps_text (xleft+0.5*length, y_base + (dir * label_off), gmtdefs.label_font_size, frame_info.axis[0].label, 0.0, Label_justify, 0);
+			if (strchr (label, '@') || strchr (label, '(') || !(flip & 4)) { /* Must set text along-side color bar */
+				ps_text (xleft + 0.5 * length, y_label, gmtdefs.label_font_size, label, 0.0, Label_justify, 0);
 			}
-			else {
+			else {	/* Set label text as column (AARRGGHH) */
+				y_label += 0.5 * ((flip & 2) - 1) * gmtdefs.label_font_size * GMT_u2u[GMT_PT][GMT_INCH];
 				size = 0.9 * gmtdefs.label_font_size * GMT_u2u[GMT_PT][GMT_INCH];
-				x0 = 0.5 * (length + ((int)strlen (frame_info.axis[0].label) -1) * size);
+				x0 = 0.5 * (length + ((GMT_LONG)strlen (label) -1) * size);
 				text[1] = 0;
-				for (i = 0; i < (int)strlen (frame_info.axis[0].label); i++) {
+				for (i = 0; i < (GMT_LONG)strlen (label); i++) {
 					x1 = x0 - i * size;
-					text[0] = frame_info.axis[0].label[i];
-					ps_text (x1, y_base + (dir * label_off), gmtdefs.label_font_size, text, -90.0, 6, 0);
+					text[0] = label[i];
+					ps_text (x1, y_label, gmtdefs.label_font_size, text, -90.0, 6, 0);
 				}
 			}
 		}
@@ -930,12 +960,12 @@ void GMT_draw_colorbar (double length, double width, double *z_width, int bit_dp
 		ps_rotatetrans (-width, 0.0, -90.0);
 
 	}
-	if (use_image) GMT_free ((void *)bar);
+	if (use_image || intens) GMT_free ((void *)bar);
 }
 
-double get_z (double x, double *width, int n)
+double get_z (double x, double *width, GMT_LONG n)
 {
-	int i = 0;
+	GMT_LONG i = 0;
 	double tmp;
 
 	tmp = width[0];
@@ -946,16 +976,16 @@ double get_z (double x, double *width, int n)
 
 void fix_format (char *unit, char *format)
 {
-	int i, j;
+	GMT_LONG i, j;
 	char text[GMT_TEXT_LEN], new_format[BUFSIZ];
 
 	/* Check if annotation units should be added */
 
 	if (unit && unit[0]) {	/* Must append the unit string */
 		if (!strchr (unit, '%'))	/* No percent signs */
-			strncpy (text, unit, GMT_TEXT_LEN);
+			strncpy (text, unit, (size_t)GMT_TEXT_LEN);
 		else {
-			for (i = j = 0; i < (int)strlen (unit); i++) {
+			for (i = j = 0; i < (GMT_LONG)strlen (unit); i++) {
 				text[j++] = unit[i];
 				if (unit[i] == '%') text[j++] = unit[i];
 			}
@@ -972,7 +1002,7 @@ void fix_format (char *unit, char *format)
 void *New_Psscale_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSSCALE_CTRL *C;
 	
-	C = (struct PSSCALE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSSCALE_CTRL), "New_Psscale_Ctrl");
+	C = (struct PSSCALE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSSCALE_CTRL), "New_Psscale_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.dpi = 300;
