@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pstext.c,v 1.68 2007/05/19 00:29:26 pwessel Exp $
+ *	$Id: pstext.c,v 1.96 2009/07/11 03:16:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -25,9 +25,9 @@
  * Date:	21-JAN-1991-2000
  * Version:	2.0 based on old v1.x
  * Version:	3.1 based on old 3.0
- * Version:	3.3: PW: Introducing paragraph mode -M
+ * Version:	3.3: PW: Introducing paragraph mode -m
  * Version:	3.3.4: PW: Input x,y can be in dd:mm[:ss] format
- * Version:	4.2.x: Now can hande @_, @:, @; everywhere and @;colorname; is OK
+ * Version:	4.2.x: Now can handle @_, @:, @; everywhere and @;colorname; is OK
  *
  */
 
@@ -72,22 +72,23 @@ struct PSTEXT_CTRL {
 		BOOLEAN active;
 		BOOLEAN outline;
 		BOOLEAN paint;
-		int mode;
+		GMT_LONG mode;
 		struct GMT_PEN pen;
 		struct GMT_FILL fill;
 	} W;
 	struct Z {	/* -Z<z_level> */
 		BOOLEAN active;
+		GMT_LONG mode;	/* 0 for common z_level, 1 for optional z-value in 3rd col for each record */
 		double level;
 	} Z;
 };
 
 struct PSTEXT_INFO {
-	int text_justify;
-	int paragraph_font;
-	int block_justify;
-	int boxflag;
-	int space_flag;
+	GMT_LONG text_justify;
+	GMT_LONG paragraph_font;
+	GMT_LONG block_justify;
+	GMT_LONG boxflag;
+	GMT_LONG space_flag;
 	double x_offset, y_offset;	/* Offset from reference point */
 	double line_spacing;
 	double paragraph_width;
@@ -103,27 +104,27 @@ struct PSTEXT_INFO {
 
 int main(int argc, char **argv)
 {
-	int i, j, k, last_font, fno, n_files = 0, n_args, n_words = 0, nscan;
-	int ix, iy, *use_font, n_processed = 0, n_read = 0, n_paragraphs = 0, pos;
-	size_t n_alloc = 0;
+	GMT_LONG i, j, k, last_font, fno, n_files = 0, n_args, nscan;
+	GMT_LONG ix, iy, *use_font, n_paragraphs = 0, pos;
+	GMT_LONG n_words = 0, n_read = 0, n_processed = 0, n_alloc = 0;
 
-	double xy[2], west = 0., east = 0.0, south = 0.0, north = 0.0, plot_x, plot_y;
-	double z_level = 0.0, tmp, dx, dy, xx[2], yy[2];
+	double xy[2], west = 0.0, east = 0.0, south = 0.0, north = 0.0, plot_x = 0.0, plot_y = 0.0;
+	double z_level = 0.0, tmp, dx, dy, xx[2], yy[2], z_level_orig = 0.0;
 
 	BOOLEAN error = FALSE, nofile = TRUE, master_record = FALSE;
 	BOOLEAN do_paragraphs = FALSE, done = FALSE, old_GMT_world_map, add;
 	BOOLEAN skip_text_records = FALSE;
 
 	char text[BUFSIZ], line[BUFSIZ], this_font[GMT_LONG_TEXT], just_key[5], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
-	char txt_x[GMT_LONG_TEXT], txt_y[GMT_LONG_TEXT], p[BUFSIZ];
-	char **word = (char **)NULL, pjust;
+	char txt_x[GMT_LONG_TEXT], txt_y[GMT_LONG_TEXT], txt_z[GMT_LONG_TEXT], p[BUFSIZ];
+	char **word = (char **)NULL, pjust, *not_used = NULL;
 
 	FILE *fp = NULL;
 
 	struct PSTEXT_INFO T;
 	struct PSTEXT_CTRL *Ctrl;
 
-	void GMT_putwords (double x, double y, char **text, int n_words, struct PSTEXT_INFO *info);
+	void GMT_putwords (double x, double y, char **text, GMT_LONG n_words, struct PSTEXT_INFO *info);
 	void *New_Pstext_Ctrl (), Free_Pstext_Ctrl (struct PSTEXT_CTRL *C);
 	void load_parameters (struct PSTEXT_INFO *T, struct PSTEXT_CTRL *C);
 	
@@ -145,6 +146,7 @@ int main(int argc, char **argv)
 				case 'H':
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -155,6 +157,7 @@ int main(int argc, char **argv)
 				case 'Y':
 				case 'y':
 				case 'c':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
@@ -167,10 +170,14 @@ int main(int argc, char **argv)
 					break;
 				case 'C':
 					Ctrl->C.active = TRUE;
-					k = sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b);
-					Ctrl->C.dx = GMT_convert_units (txt_a, GMT_INCH);
-					Ctrl->C.dy = (k == 2) ? GMT_convert_units (txt_b, GMT_INCH) : Ctrl->C.dx;
-					Ctrl->C.percent = (strchr (argv[i], '%')) ? TRUE : FALSE;
+					if (argv[i][2]) {	/* Replace default settings with user settings */
+						Ctrl->C.percent = (strchr (argv[i], '%')) ? TRUE : FALSE;
+						k = sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b);
+						for (j = 0; txt_a[j]; j++) if (txt_a[j] == '%') txt_a[j] = '\0';	/* Remove % signs before processing values */
+						for (j = 0; k == 2 && txt_b[j]; j++) if (txt_b[j] == '%') txt_b[j] = '\0';
+						Ctrl->C.dx = GMT_convert_units (txt_a, GMT_INCH);
+						Ctrl->C.dy = (k == 2) ? GMT_convert_units (txt_b, GMT_INCH) : Ctrl->C.dx;
+					}
 					break;
 				case 'D':
 					Ctrl->D.active = TRUE;
@@ -185,13 +192,13 @@ int main(int argc, char **argv)
 						}
 						argv[i][j] = 0;
 					}
-					sscanf (&argv[i][k], "%[^/]/%s", txt_a, txt_b);
+					j = sscanf (&argv[i][k], "%[^/]/%s", txt_a, txt_b);
 					Ctrl->D.dx = GMT_convert_units (txt_a, GMT_INCH);
-					Ctrl->D.dy = GMT_convert_units (txt_b, GMT_INCH);
+					Ctrl->D.dy = (j == 2) ? GMT_convert_units (txt_b, GMT_INCH) : Ctrl->D.dx;
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
-					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
+					error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'G':
 					Ctrl->G.active = TRUE;
@@ -203,9 +210,6 @@ int main(int argc, char **argv)
 				case 'L':
 					Ctrl->L.active = TRUE;
 					break;
-				case 'M':               /* Multiple line segments input: Text blocks */
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':	/* Do not clip at border */
 					Ctrl->N.active = TRUE;
 					break;
@@ -224,12 +228,12 @@ int main(int argc, char **argv)
 						Ctrl->W.outline = TRUE;
 						Ctrl->W.mode = argv[i][j];
 						if (argv[i][j+1] && GMT_getpen (&argv[i][j+1], &Ctrl->W.pen)) {
-							fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Bad pen given after %c\n", GMT_program, Ctrl->W.mode);
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Bad pen given after %c\n", GMT_program, (int)Ctrl->W.mode);
 							error++;
 						}
 						if (j > 3) {	/* Gave fill */
 							char txt_a[GMT_LONG_TEXT];
-							strncpy (txt_a, &argv[i][2], j-3);
+							strncpy (txt_a, &argv[i][2], (size_t)(j-3));
 							txt_a[j-3] = '\0';
 							if (GMT_getfill (txt_a, &Ctrl->W.fill)) {
 								GMT_fill_syntax ('W', " ");
@@ -245,7 +249,7 @@ int main(int argc, char **argv)
 							int n;
 							n = 2;
 							while (!(argv[i][n] == '/' || argv[i][n] == '\0')) n++;
-							strncpy (txt_a, &argv[i][2], n-2);	txt_a[n-2] = '\0';
+							strncpy (txt_a, &argv[i][2], (size_t)(n-2));	txt_a[n-2] = '\0';
 							if (GMT_colorname2index (txt_a) >= 0) {	/* Found a colorname; thus this was actually a fill statement instead */
 								if (GMT_getfill (txt_a, &Ctrl->W.fill)) {
 									GMT_fill_syntax ('W', " ");
@@ -257,7 +261,7 @@ int main(int argc, char **argv)
 								Ctrl->W.outline = TRUE;	/* Want box outline */
 								Ctrl->W.mode = argv[i][j];
 								if (argv[i][j+1] && GMT_getpen (&argv[i][j+1], &Ctrl->W.pen)) {
-									fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Bad pen given after %c\n", GMT_program, Ctrl->W.mode);
+									fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Bad pen given after %c\n", GMT_program, (int)Ctrl->W.mode);
 									error++;
 								}
 								argv[i][j] = 0;
@@ -273,10 +277,11 @@ int main(int argc, char **argv)
 					}
 					break;
 				case 'Z':
-					if (argv[i][2]) {
+					Ctrl->Z.active = TRUE;
+					if (argv[i][2] == '+' && argv[i][3] == '\0') /* Read level for each item */
+						Ctrl->Z.mode = 1;
+					else /* Gave specific constant level */
 						Ctrl->Z.level = atof (&argv[i][2]);
-						Ctrl->Z.active = TRUE;
-					}
 					break;
 
 				/* Options not recognized */
@@ -294,12 +299,12 @@ int main(int argc, char **argv)
 	if (argc == 1 || Ctrl->L.active || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "pstext %s - To plot text on maps\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pstext <txtfile> %s %s\n", GMT_J_OPT, GMT_Rgeoz_OPT);
-		fprintf (stderr, "\t[-A] [%s] [-C<dx>/<dy>] [-D[j]<dx>/<dy>[v[<pen>]] [-Eaz/el] [-G<color>]\n", GMT_B_OPT);
-		fprintf (stderr, "\t[%s] [-K] [-L] [%s] [-N] [-O] [-P] [-S<pen>] [%s]\n", GMT_Ho_OPT, GMT_Mo_OPT, GMT_U_OPT);
+		fprintf (stderr, "\t[-A] [%s] [-C<dx>/<dy>] [-D[j]<dx>[/<dy>][v[<pen>]] [%s] [-G<color>]\n", GMT_B_OPT, GMT_E_OPT);
+		fprintf (stderr, "\t[%s] [-K] [-L] [-N] [-O] [-P] [-S<pen>] [%s]\n", GMT_Ho_OPT, GMT_U_OPT);
 		fprintf (stderr, "\t[-V] [-W[<fill>,][o|O|c|C[<pen>]]] [%s] [%s]\n", GMT_X_OPT, GMT_X_OPT);
-		fprintf (stderr, "\t[-Z<zlevel>] [%s] [%s]\n\n", GMT_t_OPT, GMT_c_OPT);
+		fprintf (stderr, "\t[-Z[<zlevel>|+]] [%s] [%s] [%s]\n\n", GMT_t_OPT, GMT_c_OPT, GMT_mo_OPT);
 		fprintf (stderr, "\tReads (x,y,size,angle,fontno,justify,text) from <txtfile> [or stdin]\n");
-		fprintf (stderr, "\tOR (with -M) one or more text paragraphs with formatting info in the segment header.\n");
+		fprintf (stderr, "\tOR (with -m) one or more text paragraphs with formatting info in the segment header.\n");
 		fprintf (stderr, "\tjustify is of the form [T|M|B][L|C|R] (top/middle/bottom/left/center/right)\n");
 		fprintf (stderr, "\tBuilt-in escape sequences:\n");
 		fprintf (stderr, "\t   @~ toggles between current font and Symbol font\n");
@@ -312,14 +317,14 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t   @_ toggles between normal and underlined text\n");
 		fprintf (stderr, "\t   @!<char1><char2> makes one composite character\n");
 		fprintf (stderr, "\t   @@ prints the @ sign itself\n");
-		fprintf (stderr, "\t   @e, @o, @a, @E, @O, @A give the accented Scandinavian characters\n");
+		fprintf (stderr, "\t   Use @a, @c, @e, @n, @o, @s, @u, @A, @C @E, @N, @O, @U for accented European characters\n");
 		fprintf (stderr, "\t(See manual page for more information)\n");
 
 		if (Ctrl->L.active) {	/* List fonts */
 			fprintf (stderr, "\n\tFont #	Font Name\n");
 			fprintf (stderr, "\t------------------------------------\n");
 			for (i = 0; i < GMT_N_FONTS; i++)
-				fprintf (stderr, "\t%3d\t%s\n", i, GMT_font[i].name);
+				fprintf (stderr, "\t%3ld\t%s\n", i, GMT_font[i].name);
 		}
 
 		if (Ctrl->L.active || GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
@@ -333,15 +338,11 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t   if -W has been set.  Append units {c,i,m,p} or %% of fontsize [15%%]\n");
 		fprintf (stderr, "\t-D adds <add_x>,<add_y> to the text origin AFTER projecting with -J. [0/0]\n");
 		fprintf (stderr, "\t   Use -Dj to move text origin away from point (direction determined by text's justification)\n");
-		fprintf (stderr, "\t   Append v[<pen>] to draw line from text to original point\n");
-		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t   Append v[<pen>] to draw line from text to original point.  If <add_y> is not given it equal <add_x>\n");
+		GMT_explain_option ('E');
 		fprintf (stderr, "\t-G set the color (red/green/blue (0-255)) for solid text [%d/%d/%d]\n", gmtdefs.basemap_frame_rgb[0], gmtdefs.basemap_frame_rgb[1], gmtdefs.basemap_frame_rgb[2]);
 		GMT_explain_option ('H');
 		GMT_explain_option ('K');
-		GMT_explain_option ('M');
-		fprintf (stderr, "\t   Expects (x y size angle fontno justify linespace parwidth parjust) in segment header\n");
-		fprintf (stderr, "\t   followed by lines with one or more paragraphs of text.\n");
-		fprintf (stderr, "\t   parjust is one of (l)eft, (c)enter, (r)ight, or (j)ustified.\n");
 		fprintf (stderr, "\t-N Do Not clip text that exceeds the map boundaries [Default will clip]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
@@ -352,12 +353,18 @@ int main(int argc, char **argv)
 		fprintf (stderr, "\t   Append fill color [none].  To draw outline, append o, O, or c, and optionally a pen [No outline]\n");
 		fprintf (stderr, "\t   Separate the fill color and the outline information by a comma if both are present.\n");
 		fprintf (stderr, "\t   Lower case o will draw rectangular outline\n");
-		fprintf (stderr, "\t   Upper case O will draw rectangle with rounded corners (-M only)\n");
-		fprintf (stderr, "\t   Lower case c will draw concave rectangle (-M only)\n");
-		fprintf (stderr, "\t   Upper case C will draw convex rectangle (-M only)\n");
+		fprintf (stderr, "\t   Upper case O will draw rectangle with rounded corners (-m only)\n");
+		fprintf (stderr, "\t   Lower case c will draw concave rectangle (-m only)\n");
+		fprintf (stderr, "\t   Upper case C will draw convex rectangle (-m only)\n");
 		GMT_explain_option ('X');
-		fprintf (stderr, "\t-Z For 3-D plots: Set the z-level of map [0]\n");
+		fprintf (stderr, "\t-Z For 3-D plots: Set the z-level of map [0].  If -Z+ is given then we expect\n");
+		fprintf (stderr, "\t   records to have an optional z value in the 3rd column (i.e., x y z size ...)\n");
+		fprintf (stderr, "\t   Note that -Z+ also sets -N\n");
 		GMT_explain_option ('c');
+		GMT_explain_option ('m');
+		fprintf (stderr, "\t   Expects (x y size angle fontno justify linespace parwidth parjust) in segment header\n");
+		fprintf (stderr, "\t   followed by lines with one or more paragraphs of text.\n");
+		fprintf (stderr, "\t   parjust is one of (l)eft, (c)enter, (r)ight, or (j)ustified.\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
@@ -379,7 +386,7 @@ int main(int argc, char **argv)
 		error++;
 	}
 	if (!(z_project.view_azimuth == 180.0 && z_project.view_elevation == 90.0) && do_paragraphs) {
-		fprintf (stderr, "%s: -E not implemented with -M yet\n", GMT_program);
+		fprintf (stderr, "%s: -E not implemented with -m yet\n", GMT_program);
 		error++;
 	}
 	if (Ctrl->D.dx == 0.0 && Ctrl->D.dy == 0.0 && Ctrl->D.line) {
@@ -401,7 +408,7 @@ int main(int argc, char **argv)
 	
 	z_project.view_azimuth = Ctrl->E.azimuth;
 	z_project.view_elevation = Ctrl->E.elevation;
-	use_font = (int *) GMT_memory (VNULL, (size_t)GMT_N_FONTS, sizeof (int), GMT_program);
+	use_font = (GMT_LONG *) GMT_memory (VNULL, (size_t)GMT_N_FONTS, sizeof (GMT_LONG), GMT_program);
 	add = !(T.x_offset == 0.0 && T.y_offset == 0.0);
 	if (add && Ctrl->D.justify) T.boxflag |= 64;
 	if (n_files > 0)
@@ -416,16 +423,17 @@ int main(int argc, char **argv)
 
 	if (project_info.three_D) ps_transrotate (-z_project.xmin, -z_project.ymin, 0.0);
 
+	if (Ctrl->Z.mode == 1) z_level_orig = project_info.z_level;
 	if (Ctrl->Z.active) {
 		project_info.z_level = Ctrl->Z.level;
 		GMT_z_to_zz (Ctrl->Z.level, &z_level);
 	}
 	GMT_setpen (&T.txtpen);
 
-	if (!Ctrl->N.active) GMT_map_clip_on (GMT_no_rgb, 3);
+	if (!(Ctrl->N.active || Ctrl->Z.mode == 1)) GMT_map_clip_on (GMT_no_rgb, 3);
 
 	last_font = 0;
-	ix = (gmtdefs.xy_toggle[0]);	iy = 1 - ix;
+	ix = (gmtdefs.xy_toggle[GMT_IN]);	iy = 1 - ix;
 
 	/* if (draw_box && project_info.three_D) draw_box = FALSE; */	/* Not implemented yet */
 
@@ -460,12 +468,10 @@ int main(int argc, char **argv)
 		}
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = fgets (line, BUFSIZ, fp);
 
 		while (GMT_fgets (line, BUFSIZ, fp)) {
 
-			if (line[0] == '\0' || (line[0] == '#' && GMT_io.EOF_flag[GMT_IN] != '#')) continue;	/* Skip blank lines or # comments */
-
 			GMT_enforce_rgb_triplets (line, BUFSIZ);	/* If @; is used, make sure the color information passed on to ps_text is in r/b/g format */
 			
 			if (do_paragraphs) {	/* Paragraph mode */
@@ -477,15 +483,25 @@ int main(int argc, char **argv)
 						n_processed = n_words = 0;
 						n_paragraphs++;
 					}
-
-					nscan = sscanf (&line[2], "%s %s %lf %lf %s %s %s %s %c\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, txt_a, txt_b, &pjust);
+					
+					if (Ctrl->Z.mode == 1) {	/* Expect z in 3rd column */
+						nscan = sscanf (&line[2], "%s %s %s %lf %lf %s %s %s %s %c\n", txt_x, txt_y, txt_z, &T.font_size, &T.paragraph_angle, this_font, just_key, txt_a, txt_b, &pjust);
+						if ((GMT_scanf (txt_z, GMT_io.in_col_type[2], &Ctrl->Z.level) == GMT_IS_NAN)) {
+							fprintf (stderr, "%s: Record %ld had bad z coordinate, must exit)\n", GMT_program, n_read);
+							exit (EXIT_FAILURE);
+						}
+						project_info.z_level = Ctrl->Z.level;
+						GMT_z_to_zz (Ctrl->Z.level, &z_level);
+					}
+					else
+						nscan = sscanf (&line[2], "%s %s %lf %lf %s %s %s %s %c\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, txt_a, txt_b, &pjust);
 					if ((GMT_scanf (txt_x, GMT_io.in_col_type[0], &xy[ix]) == GMT_IS_NAN) || (GMT_scanf (txt_y, GMT_io.in_col_type[1], &xy[iy]) == GMT_IS_NAN)) {
-						fprintf (stderr, "%s: Record %d had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
+						fprintf (stderr, "%s: Record %ld had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
 						exit (EXIT_FAILURE);
 					}
 
-					if (nscan != 9) {
-						fprintf (stderr, "%s: Record %d had incomplete paragraph information, must exit)\n", GMT_program, n_read);
+					if (nscan != (9 + Ctrl->Z.mode)) {
+						fprintf (stderr, "%s: Record %ld had incomplete paragraph information, must exit)\n", GMT_program, n_read);
 						exit (EXIT_FAILURE);
 					}
 					GMT_geo_to_xy (xy[0], xy[1], &plot_x, &plot_y);
@@ -500,19 +516,15 @@ int main(int argc, char **argv)
 						T.paragraph_angle = fmod (tmp + 360.0 + 90.0, 180.0) - 90.0;	/* Ensure usable angles for text plotting */
 					}
 					T.text_justify = (int)pjust;
-					T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key, -1, -1);
+					T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key, 12);
 					if (T.block_justify == -99) {
-						fprintf (stderr, "%s: Record %d had bad justification info (set to LB)\n", GMT_program, n_read);
+						fprintf (stderr, "%s: Record %ld had bad justification info (set to LB)\n", GMT_program, n_read);
 						T.block_justify = 1;
 					}
-					if (this_font[0] >= '0' && this_font[0] <= '9')	/* Gave a font # */
-						T.paragraph_font = atoi (this_font);
-					else {	/* Gave a font name */
-						T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
-						if (T.paragraph_font == GMT_N_FONTS) {
-							fprintf (stderr, "%s: Record %d had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
-							T.paragraph_font = 0;
-						}
+					T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
+					if (T.paragraph_font == GMT_N_FONTS) {
+						fprintf (stderr, "%s: Record %ld had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
+						T.paragraph_font = 0;
 					}
 					if (!use_font[T.paragraph_font] && strcmp (gmtdefs.encoding.name, "Standard") != 0) {	/* Must reencode this font */
 						ps_encode_font (T.paragraph_font);
@@ -525,27 +537,26 @@ int main(int argc, char **argv)
 				else {	/* Text block record */
 					if (skip_text_records) continue;	/* Skip all records for this paragraph */
 					if (!master_record) {
-						fprintf (stderr, "%s: Text record line %d not preceeded by paragraph information, must exit)\n", GMT_program, n_read);
+						fprintf (stderr, "%s: Text record line %ld not preceded by paragraph information, must exit)\n", GMT_program, n_read);
 						exit (EXIT_FAILURE);
 					}
 
 					line[strlen(line)-1] = 0;
 
-					if (line[0] == 0) {	/* Blank line */
-						word[n_words] = (char *) GMT_memory (VNULL, (size_t)(1), sizeof (char), GMT_program);
+					if (line[0] == 0) {	/* Blank line marked by single NULL character */
+						word[n_words] = (char *) calloc ((size_t)(1), sizeof (char));
 						n_words++;
-						if ((size_t)n_words == n_alloc) {
-							n_alloc += GMT_CHUNK;
+						if (n_words == n_alloc) {
+							n_alloc <<= 1;
 							word = (char **) GMT_memory (VNULL, n_alloc, sizeof (char *), GMT_program);
 						}
 					}
 					pos = 0;
 					while ((GMT_strtok (line, " ", &pos, p))) {
-						word[n_words] = (char *) GMT_memory (VNULL, (size_t)(strlen(p)+1), sizeof (char), GMT_program);
-						strcpy (word[n_words], p);
+						word[n_words] = strdup (p);
 						n_words++;
-						if ((size_t)n_words == n_alloc) {
-							n_alloc += GMT_CHUNK;
+						if (n_words == n_alloc) {
+							n_alloc <<= 1;
 							word = (char **) GMT_memory (VNULL, n_alloc, sizeof (char *), GMT_program);
 						}
 					}
@@ -555,13 +566,25 @@ int main(int argc, char **argv)
 				n_read++;
 			}
 			else {	/* Old-style pstext input */
-				nscan = sscanf (line, "%s %s %lf %lf %s %s %[^\n]\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, text);
+				if (GMT_is_a_blank_line (line)) continue;	/* Skip blank lines or # comments */
+
+				if (Ctrl->Z.mode == 1) {	/* Expect z in 3rd column */
+					nscan = sscanf (line, "%s %s %s %lf %lf %s %s %[^\n]\n", txt_x, txt_y, txt_z, &T.font_size, &T.paragraph_angle, this_font, just_key, text);
+					if ((GMT_scanf (txt_z, GMT_io.in_col_type[2], &Ctrl->Z.level) == GMT_IS_NAN)) {
+						fprintf (stderr, "%s: Record %ld had bad z coordinate, must exit)\n", GMT_program, n_read);
+						exit (EXIT_FAILURE);
+					}
+					project_info.z_level = Ctrl->Z.level;
+					GMT_z_to_zz (Ctrl->Z.level, &z_level);
+				}
+				else
+					nscan = sscanf (line, "%s %s %lf %lf %s %s %[^\n]\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, text);
 				if ((GMT_scanf (txt_x, GMT_io.in_col_type[0], &xy[ix]) == GMT_IS_NAN) || (GMT_scanf (txt_y, GMT_io.in_col_type[1], &xy[iy]) == GMT_IS_NAN)) {
-					fprintf (stderr, "%s: Record %d had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
+					fprintf (stderr, "%s: Record %ld had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
 					exit (EXIT_FAILURE);
 				}
-				if (nscan != 7) {
-					fprintf (stderr, "%s: Record %d is incomplete (skipped)\n", GMT_program, n_read);
+				if (nscan != (7+Ctrl->Z.mode)) {
+					fprintf (stderr, "%s: Record %ld is incomplete (skipped)\n", GMT_program, n_read);
 					continue;
 				}
 				n_read++;
@@ -572,9 +595,9 @@ int main(int argc, char **argv)
 					if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
 				}
 
-				T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key, -1, -1);
+				T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key, 12);
 				if (T.block_justify == -99) {
-					fprintf (stderr, "%s: Record %d had bad justification info (skipped)\n", GMT_program, n_read);
+					fprintf (stderr, "%s: Record %ld had bad justification info (skipped)\n", GMT_program, n_read);
 					continue;
 				}
 				if (Ctrl->A.active) {
@@ -590,14 +613,10 @@ int main(int argc, char **argv)
 					}
 					xx[1] = plot_x;	yy[1] = plot_y;
 				}
-				if (this_font[0] >= '0' && this_font[0] <= '9')	/* Gave a font # */
-					T.paragraph_font = atoi (this_font);
-				else {	/* Gave a font name */
-					T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
-					if (T.paragraph_font == GMT_N_FONTS) {
-						fprintf (stderr, "%s: Record %d had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
-						T.paragraph_font = 0;
-					}
+				T.paragraph_font = GMT_font_lookup (this_font, GMT_font, GMT_N_FONTS);
+				if (T.paragraph_font == GMT_N_FONTS) {
+					fprintf (stderr, "%s: Record %ld had bad font (set to %s (0))\n", GMT_program, n_read, GMT_font[0].name);
+					T.paragraph_font = 0;
 				}
 				n_paragraphs++;
 
@@ -632,7 +651,7 @@ int main(int argc, char **argv)
 				}
 				if (T.boxflag & 32) {	/* Draw line from original point to shifted location */
 					GMT_setpen (&T.vecpen);
-					if (project_info.three_D) GMT_2D_to_3D (xx, yy, project_info.z_level, 2);
+					if (project_info.three_D) GMT_2D_to_3D (xx, yy, project_info.z_level, (GMT_LONG)2);
 					ps_segment (xx[0], yy[0], xx[1], yy[1]);
 				}
 			}
@@ -648,10 +667,12 @@ int main(int argc, char **argv)
 
  	if (do_paragraphs) GMT_free ((void *)word);
 
-	if (!Ctrl->N.active) GMT_map_clip_off (); 
+	if (!(Ctrl->N.active || Ctrl->Z.mode == 1)) GMT_map_clip_off (); 
 
 	GMT_world_map = old_GMT_world_map;
 
+	if (Ctrl->Z.mode == 1) project_info.z_level = z_level_orig;
+
 	GMT_map_basemap ();
 
 	if (project_info.three_D) ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);
@@ -660,9 +681,9 @@ int main(int argc, char **argv)
 
 	if (gmtdefs.verbose) {
 		if (do_paragraphs)
-			fprintf (stderr, "pswords: Plotted %d textblocks\n", n_paragraphs);
+			fprintf (stderr, "pswords: Plotted %ld textblocks\n", n_paragraphs);
 		else
-			fprintf (stderr, "pstext: Plotted %d text-strings\n", n_paragraphs);
+			fprintf (stderr, "pstext: Plotted %ld text-strings\n", n_paragraphs);
 	}
 
 	GMT_free ((void *)use_font);
@@ -674,11 +695,10 @@ int main(int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void GMT_putwords (double x, double y, char **text, int n_words, struct PSTEXT_INFO *T) {
+void GMT_putwords (double x, double y, char **text, GMT_LONG n_words, struct PSTEXT_INFO *T) {
 	char *boxpen_texture = CNULL, *vecpen_texture = CNULL;
-	int boxpen_width, boxpen_offset, boxpen_rgb[3];
-	int vecpen_width, vecpen_offset, vecpen_rgb[3];
-	int i;
+	GMT_LONG boxpen_width, boxpen_offset, vecpen_width, vecpen_offset, i;
+	int boxpen_rgb[3], vecpen_rgb[3];
 	double dx, dy;
 
 	boxpen_texture = GMT_convertpen (&T->boxpen, &boxpen_width, &boxpen_offset, boxpen_rgb);
@@ -701,9 +721,7 @@ void GMT_putwords (double x, double y, char **text, int n_words, struct PSTEXT_I
 	if (boxpen_texture) GMT_free ((void *)boxpen_texture);
 	if (vecpen_texture) GMT_free ((void *)vecpen_texture);
 
-	for (i = 0; i < n_words; i++) {
-		GMT_free ((void *)text[i]);
-	}
+	for (i = 0; i < n_words; i++) free ((void *)text[i]);
 }
 
 void load_parameters (struct PSTEXT_INFO *T, struct PSTEXT_CTRL *C) {
@@ -741,7 +759,7 @@ void load_parameters (struct PSTEXT_INFO *T, struct PSTEXT_CTRL *C) {
 void *New_Pstext_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSTEXT_CTRL *C;
 	
-	C = (struct PSTEXT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSTEXT_CTRL), "New_Pstext_Ctrl");
+	C = (struct PSTEXT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSTEXT_CTRL), "New_Pstext_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 		
