@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: pswiggle.c,v 1.50 2007/09/30 04:16:49 guru Exp $
+ *	$Id: pswiggle.c,v 1.65 2009/07/11 03:16:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -89,19 +89,20 @@ int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, nofile = TRUE, paint_wiggle, done;
 
-	char line[BUFSIZ], *units = CNULL, txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
+	char line[BUFSIZ], *units = CNULL, txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], *not_used = NULL;
 
-	int i, j, n, k, n_files = 0, fno, n_alloc = GMT_CHUNK, n_args, n_expected_fields, n_fields, wantx, wanty;
+	GMT_LONG i, j, n, n_alloc = GMT_CHUNK;
+	GMT_LONG k, n_files = 0, fno, n_args, n_expected_fields, n_fields, wantx, wanty;
 
 	double west = 0.0, east = 360.0, south = 0.0, north = 0.0, x_2, y_2;
-	double dx, dy, dz, ds, *in, *xx, *yy, *zz, *lon, *lat, *z, start_az, stop_az;
+	double dx, dy, dz, ds, *in, *xx, *yy, *zz, *lon, *lat, *z, start_az, stop_az, fix_az;
 
 	FILE *fp = NULL;
 
 	struct PSWIGGLE_CTRL *Ctrl;
 
 	void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLEAN gave_xy, char *units);
-	void plot_wiggle (double *x, double *y, double *z, int np, double zscale, double start_az, double stop_az, BOOLEAN fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, BOOLEAN paint_wiggle, BOOLEAN negative, BOOLEAN outline, BOOLEAN track);
+	void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, double start_az, double stop_az, BOOLEAN fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, BOOLEAN paint_wiggle, BOOLEAN negative, BOOLEAN outline, BOOLEAN track);
 	void *New_Pswiggle_Ctrl (), Free_Pswiggle_Ctrl (struct PSWIGGLE_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -118,6 +119,7 @@ int main (int argc, char **argv)
 				case 'H':
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -130,6 +132,7 @@ int main (int argc, char **argv)
 				case 'b':
 				case 'c':
 				case 'f':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
@@ -151,7 +154,7 @@ int main (int argc, char **argv)
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
-					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
+					error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'G':
 					Ctrl->G.active = TRUE;
@@ -164,9 +167,6 @@ int main (int argc, char **argv)
 					Ctrl->I.value = atof (&argv[i][2]);
 					Ctrl->I.active = TRUE;
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':
 					Ctrl->N.active = TRUE;
 					break;
@@ -226,9 +226,9 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr,"pswiggle %s - Plot xyz-series along tracks\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pswiggle <xyz-files> %s %s -Z<scale>\n", GMT_J_OPT, GMT_Rgeo_OPT);
-		fprintf (stderr, "\t[-A<azimuth>] [%s] [-C<center>] [-D<gap>] [-E<az/el>] [-G<fill>] [%s]\n", GMT_B_OPT, GMT_Ho_OPT);
-		fprintf (stderr, "\t[-I<az>] -K] [%s] [-N] [-O] [-P] [-S[x]<lon0>/<lat0>/<length>/<units>] [-T<trackpen>]\n", GMT_Mo_OPT);
-		fprintf (stderr, "\t[%s] [-V] [-W<outlinepen>] [%s] [%s]\n\t[%s] [%s] [%s] [%s]\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
+		fprintf (stderr, "\t[-A<azimuth>] [%s] [-C<center>] [-D<gap>] [%s] [-G<fill>] [%s]\n", GMT_B_OPT, GMT_E_OPT, GMT_Ho_OPT);
+		fprintf (stderr, "\t[-I<az>] -K [-N] [-O] [-P] [-S[x]<lon0>/<lat0>/<length>/<units>] [-T<trackpen>]\n");
+		fprintf (stderr, "\t[%s] [-V] [-W<outlinepen>] [%s] [%s]\n\t[%s] [%s] [%s] [%s]] [%s]\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -241,12 +241,11 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-C sets center value to be removed from z before plotting. [0]\n");
 		fprintf (stderr, "\t-D means there is a datagap if 2 points are more than <gap> distance units apart\n");
 		fprintf (stderr, "\t   gap is in km for map projections, user-units otherwise\n");
-		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		GMT_explain_option ('E');
 		GMT_fill_syntax ('G', "Specify color/pattern for positive areas.");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-I set fixed projection azimuths for wiggles\n");
 		GMT_explain_option ('K');
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N Fill negative wiggles instead [Default is positive]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
@@ -264,6 +263,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 3 input columns\n");
 		GMT_explain_option ('f');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -298,7 +298,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	z_project.view_azimuth = Ctrl->E.azimuth;
@@ -339,7 +339,7 @@ int main (int argc, char **argv)
 	/* Now convert angles to radians and keep it that way */
 	start_az = (Ctrl->A.value - 90.0) * D2R;
 	stop_az  = (Ctrl->A.value + 90.0) * D2R;
-	Ctrl->I.value *= D2R;
+	fix_az = Ctrl->I.value * D2R;
 
 	GMT_map_clip_on (GMT_no_rgb, 3);
 
@@ -377,9 +377,9 @@ int main (int argc, char **argv)
 			ps_comment (line);
 		}
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 		if (GMT_io.multi_segments[GMT_IN]) {
-			GMT_fgets (line, BUFSIZ, fp);
+			not_used = GMT_fgets (line, BUFSIZ, fp);
 			if (gmtdefs.verbose) ps_comment (line);
 		}
 		n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3;
@@ -396,16 +396,16 @@ int main (int argc, char **argv)
 			n = 0;
 			while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 					exit (EXIT_FAILURE);
 				}
 
-				lon[n] = in[0];
-				lat[n] = in[1];
-				z[n] = in[2] - Ctrl->C.value;
+				lon[n] = in[GMT_X];
+				lat[n] = in[GMT_Y];
+				z[n] = in[GMT_Z] - Ctrl->C.value;
 				n++;
 				if (n >= n_alloc) {
-					n_alloc += GMT_CHUNK;
+					n_alloc <<= 1;
 					lon = (double *) GMT_memory ((void *)lon, (size_t)n_alloc, sizeof (double), GMT_program);
 					lat = (double *) GMT_memory ((void *)lat, (size_t)n_alloc, sizeof (double), GMT_program);
 					z = (double *) GMT_memory ((void *)z, (size_t)n_alloc, sizeof (double), GMT_program);
@@ -425,7 +425,7 @@ int main (int argc, char **argv)
 				dx = lon[i] - lon[i-1];
 				dy = lat[i] - lat[i-1];
 				if (GMT_IS_MAPPING) {	/* Distance in meters */
-					dx *= cos (0.5 * (lat[i] + lat[i-1]) * D2R);
+					dx *= cosd (0.5 * (lat[i] + lat[i-1]));
 					ds = hypot (dx, dy) * project_info.M_PR_DEG;
 				}
 				else
@@ -435,12 +435,12 @@ int main (int argc, char **argv)
 
 				if (j > 0 && (ds > Ctrl->D.value || GMT_is_dnan (z[i]))) {	/* Data gap, plot what we have */
 					paint_wiggle = (Ctrl->G.active && ((Ctrl->N.active && zz[j-1] <= 0.0) || (!Ctrl->N.active && zz[j-1] >= 0.0)));
-					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, Ctrl->I.value, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
+					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, fix_az, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
 					j = 0;
 				}
 				else if (j >= PSWIGGLE_MAX_POINTS) {
 					paint_wiggle = (Ctrl->G.active && ((Ctrl->N.active && zz[j-1] <= 0.0) || (!Ctrl->N.active && zz[j-1] >= 0.0)));
-					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, Ctrl->I.value, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
+					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, fix_az, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
 					xx[0] = xx[j-1];
 					yy[0] = yy[j-1];
 					zz[0] = zz[j-1];
@@ -453,7 +453,7 @@ int main (int argc, char **argv)
 					zz[j] = 0.0;
 					j++;
 					paint_wiggle = (Ctrl->G.active && ((Ctrl->N.active && zz[j-2] <= 0.0) || (!Ctrl->N.active && zz[j-2] >= 0.0)));
-					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, Ctrl->I.value, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
+					plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, fix_az, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
 					xx[0] = xx[j-1];
 					yy[0] = yy[j-1];
 					zz[0] = zz[j-1];
@@ -467,7 +467,7 @@ int main (int argc, char **argv)
 	
 			if (j > 1) {
 				paint_wiggle = (Ctrl->G.active && ((Ctrl->N.active && zz[j-1] <= 0.0) || (!Ctrl->N.active && zz[j-1] >= 0.0)));
-				plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, Ctrl->I.value, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
+				plot_wiggle (xx, yy, zz, j, Ctrl->Z.scale, start_az, stop_az, Ctrl->I.active, fix_az, &Ctrl->G.fill, &Ctrl->W.pen, &Ctrl->T.pen, paint_wiggle, Ctrl->N.active, Ctrl->W.active, Ctrl->T.active);
 			}
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
@@ -495,10 +495,10 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void plot_wiggle (double *x, double *y, double *z, int np, double zscale, double start_az, double stop_az, BOOLEAN fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, BOOLEAN paint_wiggle, BOOLEAN negative, BOOLEAN outline, BOOLEAN track)
+void plot_wiggle (double *x, double *y, double *z, GMT_LONG np, double zscale, double start_az, double stop_az, BOOLEAN fixed, double fix_az, struct GMT_FILL *fill, struct GMT_PEN *pen_o, struct GMT_PEN *pen_t, BOOLEAN paint_wiggle, BOOLEAN negative, BOOLEAN outline, BOOLEAN track)
 {
-	double dx, dy, len, az = 0.0, s, c, x_inc, y_inc;
-	int n = 0, i;
+	double dx, dy, len, az = 0.0, s = 0.0, c = 0.0, x_inc, y_inc;
+	GMT_LONG n = 0, i;
 	static char *name[2] = {"Positive", "Negative"}, comment[GMT_LONG_TEXT];
 
 	if (fixed) {
@@ -558,7 +558,7 @@ void plot_wiggle (double *x, double *y, double *z, int np, double zscale, double
 		sprintf (comment, "Wiggle line");
 		ps_comment (comment);
 		GMT_setpen (pen_o);
-		ps_line (&GMT_x_plot[1], &GMT_y_plot[1], np, 3, FALSE, TRUE);
+		ps_line (&GMT_x_plot[1], &GMT_y_plot[1], np, 3, FALSE);
 	}
 
 	if (track) {	/* Finally draw track line */
@@ -566,7 +566,7 @@ void plot_wiggle (double *x, double *y, double *z, int np, double zscale, double
 		sprintf (comment, "Track line");
 		ps_comment (comment);
 		GMT_setpen (pen_t);
-		ps_line (x, y, np, 3, FALSE, TRUE);
+		ps_line (x, y, np, 3, FALSE);
 	}
 }
 
@@ -591,7 +591,7 @@ void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLE
 	GMT_xyz_to_xy (x0, y0 - dy, 0.0, &xx[1], &yy[1]);
 	GMT_xyz_to_xy (x0, y0 + dy, 0.0, &xx[2], &yy[2]);
 	GMT_xyz_to_xy (x0 + gmtdefs.map_scale_height, y0 + dy, 0.0, &xx[3], &yy[3]);
-	ps_line (xx, yy, 4, 3, FALSE, TRUE);
+	ps_line (xx, yy, (GMT_LONG)4, 3, FALSE);
 	off = ((gmtdefs.map_scale_height > 0.0) ? gmtdefs.tick_length : 0.0) + gmtdefs.annot_offset[0];
 	GMT_text3D (x0 + off, y0, project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], txt, 0.0, 5, 0);
 }
@@ -599,7 +599,7 @@ void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLE
 void *New_Pswiggle_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSWIGGLE_CTRL *C;
 	
-	C = (struct PSWIGGLE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSWIGGLE_CTRL), "New_Pswiggle_Ctrl");
+	C = (struct PSWIGGLE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSWIGGLE_CTRL), "New_Pswiggle_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->D.value = DBL_MAX;
