@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------
- *	$Id: psxy.c,v 1.151 2009/01/11 02:52:38 remko Exp $
+ *	$Id: psxy.c,v 1.176 2009/07/11 03:16:31 guru Exp $
  *
  *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
@@ -34,7 +34,7 @@
 struct PSXY_CTRL {
 	struct A {	/* -A[m|p|step] */
 		BOOLEAN active;
-		int mode;
+		GMT_LONG mode;
 		double step;
 	} A;
 	struct C {	/* -C<cpt> */
@@ -47,8 +47,8 @@ struct PSXY_CTRL {
 	} D;
 	struct E {	/* -E[x|X][y|Y][cap][/[+|-]<pen>] */
 		BOOLEAN active;
-		int xbar, ybar;	/* 0 = not used, 1 = error bar, 2 = box-whisker, 3 notched box-whisker */
-		int mode;	/* 0 = normal, 1 = -C applies to error pen color, 2 = -C applies to symbol fill & error pen color */
+		GMT_LONG xbar, ybar;	/* 0 = not used, 1 = error bar, 2 = box-whisker, 3 notched box-whisker */
+		GMT_LONG mode;	/* 0 = normal, 1 = -C applies to error pen color, 2 = -C applies to symbol fill & error pen color */
 		double size;
 		struct GMT_PEN pen;
 	} E;
@@ -70,32 +70,35 @@ struct PSXY_CTRL {
 		BOOLEAN active;
 		char *arg;
 	} S;
+	struct T {	/* -T */
+		BOOLEAN active;
+	} T;
 	struct W {	/* -W<pen> */
 		BOOLEAN active;
-		int mode;	/* 0 = normal, 1 = -C applies to pen color only, 2 = -C applies to symbol fill & pen color */
+		GMT_LONG mode;	/* 0 = normal, 1 = -C applies to pen color only, 2 = -C applies to symbol fill & pen color */
 		struct GMT_PEN pen;
 	} W;
 };
 
 int main (int argc, char **argv)
 {
-	BOOLEAN	error = FALSE, nofile = TRUE, polygon = FALSE, read_symbol = FALSE, penset_OK = TRUE, outline_active;
+	BOOLEAN	error = FALSE, nofile, polygon = FALSE, read_symbol = FALSE, penset_OK = TRUE, outline_active;
 	BOOLEAN	default_outline = FALSE, done, get_rgb = FALSE, resample, bar_clip;
 	BOOLEAN old_GMT_world_map, error_x = FALSE, error_y = FALSE, def_err_xy = FALSE, clip_set = FALSE, fill_active;
 
-	char line[BUFSIZ], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
+	char line[BUFSIZ], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], *not_used = NULL;
 
-	GMT_LONG i, n, n_alloc = GMT_CHUNK, n_total_read = 0, plot_n;
-	
-	int j, j0, n_files = 0, fno, xy_errors[2], two, three, four, error_type[2] = {0,0};
-	int n_args, n_fields, n_cols_start = 2, this_outline, change;
-	int n_required, n_expected = 0, reset_cap = 0, error_cols[3] = {1,4,5};
+	GMT_LONG i, n, n_alloc = 0, n_total_read = 0, plot_n;
+	GMT_LONG j, j0, n_files = 0, fno, xy_errors[2], two, three, four, error_type[2] = {0,0};
+	GMT_LONG n_args, n_fields, n_cols_start = 2, this_outline, change;
+	GMT_LONG n_required, n_expected = 0, reset_cap = 0, error_cols[3] = {1,4,5};
 
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, s, c, step;
-	double plot_x, plot_y, x_1, x_2, y_1, y_2, x0, y0, *xx, *yy, *xp, *yp;
+	double plot_x, plot_y, x_1, x_2, y_1, y_2, x0, y0;
 	double direction = 0.0, length = 0.0, major = 0.0, minor = 0.0, x2, y2;
 	double dir1 = 0.0, dir2 = 0.0, x_len = 0.0, y_len = 0.0, dx, dy, size;
 	double v_w, h_l, h_w, tmp, *in, font_size, dim[3];
+	double *xx = NULL, *yy = NULL, *xp = NULL, *yp = NULL;
 
 	FILE *fp = NULL;
 
@@ -106,15 +109,15 @@ int main (int argc, char **argv)
 
 	void plot_x_errorbar (double x, double y, double delta_x, double error_width2, GMT_LONG line);
 	void plot_y_errorbar (double x, double y, double delta_y, double error_width2, GMT_LONG line);
-	void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, int notched);
-	void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, int notched);
+	void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, GMT_LONG notched);
+	void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, GMT_LONG notched);
 	void *New_Psxy_Ctrl (), Free_Psxy_Ctrl (struct PSXY_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
 
 	Ctrl = (struct PSXY_CTRL *)New_Psxy_Ctrl ();	/* Allocate and initialize a new control structure */
 
-	xy_errors[0] = xy_errors[1] = 0;	/* These will be col # of where to find this info in data */
+	xy_errors[GMT_X] = xy_errors[1] = 0;	/* These will be col # of where to find this info in data */
 
 	/* Initialize GMT_SYMBOL structure */
 
@@ -143,6 +146,7 @@ int main (int argc, char **argv)
 				case 'H':
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -155,6 +159,8 @@ int main (int argc, char **argv)
 				case 'b':
 				case 'c':
 				case 'f':
+				case 'g':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
@@ -236,9 +242,6 @@ int main (int argc, char **argv)
 				case 'L':		/* Close line segments */
 					Ctrl->L.active = TRUE;
 					break;
-				case 'M':		/* Multiple line segments */
-					GMT_multisegment (&argv[i][2]);
-					break;
 				case 'N':		/* Do not skip points outside border */
 					Ctrl->N.active = TRUE;
 					break;
@@ -246,6 +249,9 @@ int main (int argc, char **argv)
 					Ctrl->S.active = TRUE;
 					Ctrl->S.arg = &argv[i][2];
 					break;
+				case 'T':		/* Skip all input files */
+					Ctrl->T.active = TRUE;
+					break;
 				case 'W':		/* Set line attributes */
 					Ctrl->W.active = TRUE;
 					j = 2;
@@ -271,10 +277,10 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
 		fprintf (stderr,"psxy %s - Plot lines, polygons, and symbols on maps\n\n", GMT_VERSION);
-		fprintf (stderr,"usage: psxy <infiles> %s %s [-A[m|p]] [%s] [-C<cpt>]\n", GMT_J_OPT, GMT_Rgeo_OPT, GMT_B_OPT);
-		fprintf (stderr, "\t[-D<dx>/<dy>] [-E[x|y|X|Y][n][cap][/[+|-]<pen>]] [-G<fill>] [%s] [-I<intens>] [-K] [-L] [%s]\n", GMT_H_OPT, GMT_Mo_OPT);
-		fprintf (stderr, "\t[-N] [-O] [-P] [-S[<symbol>][<size>]] [%s] [-V] [-W[+|-][<pen>]] [%s]\n", GMT_U_OPT, GMT_X_OPT);
-		fprintf (stderr, "\t[%s] [%s] [%s] [%s] [%s]\n\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
+		fprintf (stderr,"usage: psxy <infiles> %s %s [-A[m|p]] [%s] [-C<cpt>] [-D<dx>/<dy>]\n", GMT_J_OPT, GMT_Rgeo_OPT, GMT_B_OPT);
+		fprintf (stderr, "\t[-E[x|y|X|Y][n][cap][/[+|-]<pen>]] [-G<fill>] [%s] [-I<intens>] [-K] [-L] [-N] [-O] [-P]\n", GMT_H_OPT);
+		fprintf (stderr, "\t[-S[<symbol>][<size>]] [-T] [%s] [-V] [-W[+|-][<pen>]] [%s] [%s]\n", GMT_U_OPT, GMT_X_OPT, GMT_Y_OPT);
+		fprintf (stderr, "\t[%s] [%s] [%s] [%s] [%s] [%s]\n\n", GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT, GMT_g_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
@@ -286,23 +292,22 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   m or p is appended to first follow meridian then parallel, or vice versa.\n");
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-C Use cpt-file to assign symbol colors based on z-value in 3rd column (requires -S)\n");
-		fprintf (stderr, "\t   Without -S, psxy excepts -M lines/polygons and looks for -Z<val> options in each multiheader\n");
+		fprintf (stderr, "\t   Without -S, psxy excepts -m lines/polygons and looks for -Z<val> options in each multiheader\n");
 		fprintf (stderr, "\t   Then, color is applied to polygon fill (-L) or polygon pen\n");
 		fprintf (stderr, "\t-D Offset symbol or line positions by <dx>/<dy> [no offset].\n");
 		fprintf (stderr, "\t-E means draw error bars for x, y, or both.  Add cap-width [%g]. Append pen attributes;\n", Ctrl->E.size);
 		fprintf (stderr, "\t   leading + applies cpt color (-C) to both symbol fill and error pen, - applies it to pen only.\n");
 		fprintf (stderr, "\t   If X or Y is used instead a box-and-whisker diagram is drawn instead, using data from 4\n");
-		fprintf (stderr, "\t   extra columns to get the 0 %%, 25 %%, 75 %% ,and 100%% quantiles (point value is assumed to be 50%%)\n");
+		fprintf (stderr, "\t   extra columns to get the 0 %%, 25 %%, 75 %%, and 100%% quantiles (point value is assumed to be 50%%)\n");
 		fprintf (stderr, "\t   If n is appended after X (or Y) we expect a 5th extra column with the sample size which is needed to draw\n");
 		fprintf (stderr, "\t   a notched box-and whisker diagram (notch width represents uncertainty in median).\n");
 		fprintf (stderr, "\t   Finally, use -W, -G to control the appearance of the 25-75%% box.\n");
 		GMT_fill_syntax ('G', "Specify color or pattern [Default is no fill].");
-		fprintf (stderr, "\t   -G option can be present in all subheaders (requires -M and no -S)\n");
+		fprintf (stderr, "\t   -G option can be present in all subheaders (requires -m and no -S)\n");
 		GMT_explain_option ('H');
 		fprintf (stderr, "\t-I sets a constant intensity used to modulate the fill color (requires -C or -G)\n");
 		GMT_explain_option ('K');
 		fprintf (stderr, "\t-L Force closed polygons.\n");
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N Do Not skip/clip symbols that fall outside map border [Default will ignore those outside]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
@@ -353,6 +358,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t     Upper case H, B, T, S will give double-headed vector [Default is t]\n");
 		fprintf (stderr, "\t   Wedges: the start and stop directions of pie wedge must be in input columns 3, 4.\n");
 		fprintf (stderr, "\t     If -SW rather than -Sw is selected, psxy will expect two azimuths instead.\n");
+		fprintf (stderr, "\t-T Ignores all input files.\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
 		GMT_pen_syntax ('W', "sets pen attributes:");
@@ -364,6 +370,8 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is the required number of columns\n");
 		GMT_explain_option ('f');
+		GMT_explain_option ('g');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -381,7 +389,7 @@ int main (int argc, char **argv)
 	}
 	if (Ctrl->C.active && S.symbol == 0 && !GMT_io.multi_segments[GMT_IN]) {
 		error++;
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option: Must also specify a symbol (see -S) or give polygon file with -M\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option: Must also specify a symbol (see -S) or give polygon file with -m\n", GMT_program);
 	}
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
@@ -407,40 +415,41 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Conflicting -E and -W options regarding -C option application\n", GMT_program);
 		error++;
 	}
+	if (Ctrl->T.active && n_files) fprintf (stderr, "%s: GMT WARNING.  Option -T ignores all input files\n", GMT_program);
 
 	if (error) exit (EXIT_FAILURE);
 
 	if (Ctrl->E.active) {	/* Set error bar parameters */
 		j = 2;	/* Normally, error bar related columns start in position 2 */
 		if (Ctrl->E.xbar == 1) {
-			xy_errors[0] = j++;
-			error_type[0] = 0;
+			xy_errors[GMT_X] = j++;
+			error_type[GMT_X] = 0;
 		}
 		else if (Ctrl->E.xbar == 2) {	/* Box-whisker instead */
-			xy_errors[0] = j++;
-			error_type[0] = 1;
+			xy_errors[GMT_X] = j++;
+			error_type[GMT_X] = 1;
 		}
 		else if (Ctrl->E.xbar == 3) {	/* Notched Box-whisker instead */
-			xy_errors[0] = j++;
-			error_type[0] = 2;
+			xy_errors[GMT_X] = j++;
+			error_type[GMT_X] = 2;
 		}
 		if (Ctrl->E.ybar == 1) {
-			xy_errors[1] = j++;
-			error_type[1] = 0;
+			xy_errors[GMT_Y] = j++;
+			error_type[GMT_Y] = 0;
 		}
 		else if (Ctrl->E.ybar == 2) {	/* Box-whisker instead */
-			xy_errors[1] = j++;
-			error_type[1] = 1;
+			xy_errors[GMT_Y] = j++;
+			error_type[GMT_Y] = 1;
 		}
 		else if (Ctrl->E.ybar == 3) {	/* Notched Box-whisker instead */
-			xy_errors[1] = j++;
-			error_type[1] = 2;
+			xy_errors[GMT_Y] = j++;
+			error_type[GMT_Y] = 2;
 		}
-		if (!(xy_errors[0] || xy_errors[1])) {	/* Default is plain error bars for both */
+		if (!(xy_errors[GMT_X] || xy_errors[GMT_Y])) {	/* Default is plain error bars for both */
 			def_err_xy = TRUE;
-			xy_errors[0] = 2;	/* Assumes xy input, later check for -: */
-			xy_errors[1] = 3;
-			error_type[0] = error_type[1] = 1;
+			xy_errors[GMT_X] = 2;	/* Assumes xy input, later check for -: */
+			xy_errors[GMT_Y] = 3;
+			error_type[GMT_X] = error_type[GMT_Y] = 1;
 		}
 	}
 
@@ -467,22 +476,16 @@ int main (int argc, char **argv)
 		if (get_rgb) n_cols_start++;
 	}
 
-	if (n_files > 0)
-		nofile = FALSE;
-	else
-		n_files = 1;
-	n_args = (argc > 1) ? argc : 2;
-
 	if (Ctrl->E.active) {
-		if (xy_errors[0] && xy_errors[1] && error_type[0] >= 1) xy_errors[1] += error_cols[error_type[0]] - 1;	/* Need 3 or 4 columns for whisker bars */
-		if (def_err_xy && gmtdefs.xy_toggle[0]) {	/* -E should be -Eyx */
-			i_swap (xy_errors[0], xy_errors[1]);
-			i_swap (error_type[0], error_type[1]);
+		if (xy_errors[GMT_X] && xy_errors[GMT_Y] && error_type[GMT_X] >= 1) xy_errors[GMT_Y] += error_cols[error_type[GMT_X]] - 1;	/* Need 3 or 4 columns for whisker bars */
+		if (def_err_xy && gmtdefs.xy_toggle[GMT_IN]) {	/* -E should be -Eyx */
+			i_swap (xy_errors[GMT_X], xy_errors[GMT_Y]);
+			i_swap (error_type[GMT_X], error_type[GMT_Y]);
 		}
-		if (xy_errors[0]) n_cols_start += error_cols[error_type[0]], error_x = TRUE;
-		if (xy_errors[1]) n_cols_start += error_cols[error_type[1]], error_y = TRUE;
-		xy_errors[0] += (S.read_size + get_rgb);	/* Move 0-2 columns over */
-		xy_errors[1] += (S.read_size + get_rgb);
+		if (xy_errors[GMT_X]) n_cols_start += error_cols[error_type[GMT_X]], error_x = TRUE;
+		if (xy_errors[GMT_Y]) n_cols_start += error_cols[error_type[GMT_Y]], error_y = TRUE;
+		xy_errors[GMT_X] += (S.read_size + get_rgb);	/* Move 0-2 columns over */
+		xy_errors[GMT_Y] += (S.read_size + get_rgb);
 	}
 	two   = (get_rgb) ? 3 : 2;
 	three = (get_rgb) ? 4 : 3;
@@ -492,13 +495,13 @@ int main (int argc, char **argv)
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN] = n_required;
 	n_expected = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : n_required;
 	if (n_expected < n_required) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %d columns\n", GMT_program, n_required);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %ld columns\n", GMT_program, n_required);
 		error++;
 	}
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	GMT_err_fail (GMT_map_setup (west, east, south, north), "");
@@ -547,12 +550,11 @@ int main (int argc, char **argv)
 	outline_active =  Ctrl->W.active;
 	if (S.symbol > 0 && !outline_active && !fill_active && !get_rgb) outline_active = TRUE;		/* If no fill nor outline for symbols then turn outline on */
 
-	if (S.symbol <= 0) {
-		xx = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-		yy = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
-	}
 	if (Ctrl->D.active) ps_transrotate (Ctrl->D.dx, Ctrl->D.dy, 0.0);	/* Shift plot a bit */
-	done = FALSE;
+	done = Ctrl->T.active;	/* When -T is used, skip all input */
+	n_args = (argc > 1) ? argc : 2;
+	nofile = (n_files == 0);
+
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 		if (!nofile && argv[fno][0] == '-') continue;
 		if (nofile) {
@@ -576,7 +578,7 @@ int main (int argc, char **argv)
 		}
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
-				GMT_fgets (line, BUFSIZ, fp);
+				not_used = GMT_fgets (line, BUFSIZ, fp);
 				if (i == 0 && S.G.label_type == 2) GMT_extract_label (&line[1], S.G.label);	/* Set initial header as potential label */
 			}
 		}
@@ -584,9 +586,9 @@ int main (int argc, char **argv)
 
 			GMT_world_map = (S.symbol == GMT_SYMBOL_ELLIPSE && S.convert_angles) ? FALSE : TRUE;
 			if (read_symbol) n_expected = BUFSIZ;
-			while ((n_fields = GMT_input (fp, &n_expected, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
+			while ((n_fields = GMT_input (fp, &n_expected, &in)) >= 0 && !GMT_REC_IS_EOF) {
 
-				while ((GMT_io.status & GMT_IO_SEGMENT_HEADER) && !(GMT_io.status & GMT_IO_EOF)) {	/* Process segment headers */
+				while (GMT_REC_IS_SEG_HEADER && !GMT_REC_IS_EOF) {	/* Process segment headers */
 					if (gmtdefs.verbose) ps_comment (GMT_io.segment_header);
 					
 					change = GMT_parse_multisegment_header (GMT_io.segment_header, Ctrl->C.active, &fill_active, &current_fill, &default_fill, &outline_active, &current_pen, &default_pen, default_outline);
@@ -599,10 +601,10 @@ int main (int argc, char **argv)
 					if (read_symbol) n_expected = BUFSIZ;
 					n_fields = GMT_input (fp, &n_expected, &in);
 				}
-				if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
+				if (GMT_REC_IS_EOF) continue;		/* At EOF */
 
-				if (GMT_io.status & GMT_IO_MISMATCH && !read_symbol) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
+				if (GMT_REC_IS_ERROR && !read_symbol) {
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
 					exit (EXIT_FAILURE);
 				}
 				n_total_read++;
@@ -623,7 +625,7 @@ int main (int argc, char **argv)
 				if (S.symbol == GMT_SYMBOL_BARX) GMT_geo_to_xy (S.base, south, &x0, &plot_y);	/* Zero x level for horizontal bars */
 
 				if (get_rgb) {
-					GMT_get_rgb_from_z (in[2], current_fill.rgb);
+					GMT_get_rgb_from_z (in[GMT_Z], current_fill.rgb);
 					if (Ctrl->I.active) GMT_illuminate (Ctrl->I.value, current_fill.rgb);
 					if (S.symbol == GMT_SYMBOL_CROSS || S.symbol == GMT_SYMBOL_PLUS || S.symbol == GMT_SYMBOL_POINT || S.symbol == GMT_SYMBOL_TEXT) ps_setpaint (current_fill.rgb);
 				}
@@ -706,13 +708,14 @@ int main (int argc, char **argv)
 					}
 				}
 				else if (!Ctrl->N.active) {
-					GMT_map_outside (in[0], in[1]);
+					GMT_map_outside (in[GMT_X], in[GMT_Y]);
 					if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
 				}
 
-				GMT_geo_to_xy (in[0], in[1], &plot_x, &plot_y);
-				if (GMT_is_dnan (plot_x)) {
-					fprintf (stderr, "%s: Data point with x = NaN near line %ld\n", GMT_program, n_total_read);
+				if (GMT_geo_to_xy (in[GMT_X], in[GMT_Y], &plot_x, &plot_y)) continue;	/* NaNs */
+				
+				if (GMT_is_dnan (plot_x)) {	/* Transformation yielded a NaN (e.g. log (-ve)) */
+					fprintf (stderr, "%s: Warning: Data point with x = NaN near line %ld\n", GMT_program, n_total_read);
 					continue;
 				}
 				if (GMT_is_dnan (plot_y)) {
@@ -725,16 +728,16 @@ int main (int argc, char **argv)
 					if (Ctrl->E.mode & 1) memcpy ((void *)current_fill.rgb, (void *)GMT_no_rgb, 3*sizeof (int));
 					GMT_setpen (&Ctrl->E.pen);
 					if (error_x) {
-						if (error_type[0] == 0)
-							plot_x_errorbar (in[0], in[1], in[xy_errors[0]], Ctrl->E.size, n_total_read);
+						if (error_type[GMT_X] == 0)
+							plot_x_errorbar (in[GMT_X], in[GMT_Y], in[xy_errors[GMT_X]], Ctrl->E.size, n_total_read);
 						else
-							plot_x_whiskerbar (plot_x, in[1], &in[xy_errors[0]], Ctrl->E.size, current_fill.rgb, n_total_read, error_type[0]);
+							plot_x_whiskerbar (plot_x, in[GMT_Y], &in[xy_errors[GMT_X]], Ctrl->E.size, current_fill.rgb, n_total_read, error_type[GMT_X]);
 					}
 					if (error_y) {
-						if (error_type[1] == 0)
-							plot_y_errorbar (in[0], in[1], in[xy_errors[1]], Ctrl->E.size, n_total_read);
+						if (error_type[GMT_Y] == 0)
+							plot_y_errorbar (in[GMT_X], in[GMT_Y], in[xy_errors[GMT_Y]], Ctrl->E.size, n_total_read);
 						else
-							plot_y_whiskerbar (in[0], plot_y, &in[xy_errors[1]], Ctrl->E.size, current_fill.rgb, n_total_read, error_type[1]);
+							plot_y_whiskerbar (in[GMT_X], plot_y, &in[xy_errors[GMT_Y]], Ctrl->E.size, current_fill.rgb, n_total_read, error_type[GMT_Y]);
 					}
 					if (!Ctrl->W.mode) GMT_setpen (&current_pen);
 				}
@@ -750,8 +753,8 @@ int main (int argc, char **argv)
 						break;
 					case GMT_SYMBOL_BARX:
 						if (S.user_unit) {	/* Width measured in y units */
-							GMT_geo_to_xy (S.base, in[1]-S.size_x2, &x_1, &y_1);
-							GMT_geo_to_xy (in[0], in[1]+S.size_x2, &x_2, &y_2);
+							GMT_geo_to_xy (S.base, in[GMT_Y]-S.size_x2, &x_1, &y_1);
+							GMT_geo_to_xy (in[GMT_X], in[GMT_Y]+S.size_x2, &x_2, &y_2);
 							dim[0] = x_2 - x_1;	dim[1] = y_2 - y_1;
 							GMT_rect (x_1, y_1, 0.0, dim, &current_fill, outline_active);
 						}
@@ -762,8 +765,8 @@ int main (int argc, char **argv)
 						break;
 					case GMT_SYMBOL_BARY:
 						if (S.user_unit) {	/* Width measured in x units */
-							GMT_geo_to_xy (in[0]-S.size_x2, S.base, &x_1, &y_1);
-							GMT_geo_to_xy (in[0]+S.size_x2, in[1], &x_2, &y_2);
+							GMT_geo_to_xy (in[GMT_X]-S.size_x2, S.base, &x_1, &y_1);
+							GMT_geo_to_xy (in[GMT_X]+S.size_x2, in[GMT_Y], &x_2, &y_2);
 							dim[0] = x_2 - x_1;	dim[1] = y_2 - y_1;
 							GMT_rect (x_1, y_1, 0.0, dim, &current_fill, outline_active);
 						}
@@ -829,7 +832,7 @@ int main (int argc, char **argv)
 						break;
 					case GMT_SYMBOL_ELLIPSE:
 						if (S.convert_angles == 2)
-							GMT_plot_ellipse (in[0], in[1], 0.0, major, minor, direction, current_fill, outline_active);
+							GMT_plot_ellipse (in[GMT_X], in[GMT_Y], 0.0, major, minor, direction, current_fill, outline_active);
 						else {
 							dim[0] = direction; dim[1] = major; dim[2] = minor;
 							GMT_ellipse (plot_x, plot_y, 0.0, dim, &current_fill, outline_active);
@@ -837,7 +840,7 @@ int main (int argc, char **argv)
 						break;
 					case GMT_SYMBOL_ROTATERECT:
 						if (S.convert_angles == 2)
-							GMT_plot_rectangle (in[0], in[1], 0.0, x_len, y_len, direction, current_fill, outline_active);
+							GMT_plot_rectangle (in[GMT_X], in[GMT_Y], 0.0, x_len, y_len, direction, current_fill, outline_active);
 						else {
 							dim[0] = direction; dim[1] = x_len; dim[2] = y_len;
 							GMT_rotrect (plot_x, plot_y, 0.0, dim, &current_fill, outline_active);
@@ -845,7 +848,7 @@ int main (int argc, char **argv)
 						break;
 					case GMT_SYMBOL_VECTOR:
 						if (S.convert_angles == 2) {
-							GMT_azim_to_angle (in[0], in[1], 0.1, direction, &tmp);
+							GMT_azim_to_angle (in[GMT_X], in[GMT_Y], 0.1, direction, &tmp);
 							direction = tmp;
 						}
 						else if (S.convert_angles == 1)	/* Cartesian azimuth */
@@ -872,7 +875,7 @@ int main (int argc, char **argv)
 						break;
 					case GMT_SYMBOL_VECTOR2:
 						if (S.convert_angles == 2) {
-							GMT_azim_to_angle (in[0], in[1], 1.0, direction, &tmp);
+							GMT_azim_to_angle (in[GMT_X], in[GMT_Y], 1.0, direction, &tmp);
 							direction = tmp;
 						}
 						else if (S.convert_angles == 1)	/* Cartesian azimuth */
@@ -906,9 +909,9 @@ int main (int argc, char **argv)
 						break;
 					case GMT_SYMBOL_PIE:
 						if (S.convert_angles == 2) {
-							GMT_azim_to_angle (in[0], in[1], 1.0, dir1, &tmp);
+							GMT_azim_to_angle (in[GMT_X], in[GMT_Y], 1.0, dir1, &tmp);
 							dir1 = tmp;
-							GMT_azim_to_angle (in[0], in[1], 1.0, dir2, &tmp);
+							GMT_azim_to_angle (in[GMT_X], in[GMT_Y], 1.0, dir2, &tmp);
 							dir2 = tmp;
 						}
 						dim[0] = S.size_x2; dim[1] = dir1; dim[2] = dir2;
@@ -935,65 +938,54 @@ int main (int argc, char **argv)
 
 			n_required = 2;
 			n_fields = GMT_input (fp, &n_expected, &in);
-			while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
-
-				while (GMT_io.status & GMT_IO_SEGMENT_HEADER) {	/* Process multisegment header flags */
-					if (gmtdefs.verbose) ps_comment (GMT_io.segment_header);
+			while (!GMT_REC_IS_EOF) {	/* Not yet EOF */
+				while (GMT_REC_IS_NEW_SEGMENT) {	/* Process multisegment header flags (or NaN gaps) */
+					if (GMT_REC_IS_SEG_HEADER) {
+						if (gmtdefs.verbose) ps_comment (GMT_io.segment_header);
 					
-					change = GMT_parse_multisegment_header (GMT_io.segment_header, Ctrl->C.active, &fill_active, &current_fill, &default_fill, &outline_active, &current_pen, &default_pen, default_outline);
-					if (Ctrl->I.active) {
-						GMT_illuminate (Ctrl->I.value, current_fill.rgb);
-						GMT_illuminate (Ctrl->I.value, default_fill.rgb);
+						change = GMT_parse_multisegment_header (GMT_io.segment_header, Ctrl->C.active, &fill_active, &current_fill, &default_fill, &outline_active, &current_pen, &default_pen, default_outline);
+						if (Ctrl->I.active) {
+							GMT_illuminate (Ctrl->I.value, current_fill.rgb);
+							GMT_illuminate (Ctrl->I.value, default_fill.rgb);
+						}
+						if ((change & 4) && penset_OK) GMT_setpen (&current_pen);
+						if ((change & 1)) polygon = TRUE;
+						if ((change & 2) && !Ctrl->L.active) {
+							polygon = FALSE;
+							ps_setpaint (current_fill.rgb);
+						}
+						if (S.G.label_type == 2) {	/* Update segment header */
+							GMT_extract_label (&GMT_io.segment_header[1], S.G.label);
+						}
 					}
-					if ((change & 4) && penset_OK) GMT_setpen (&current_pen);
-					if ((change & 1)) polygon = TRUE;
-					if ((change & 2) && !Ctrl->L.active) {
-						polygon = FALSE;
-						ps_setpaint (current_fill.rgb);
-					}
-					if (S.G.label_type == 2) {	/* Update segment header */
-						GMT_extract_label (&GMT_io.segment_header[1], S.G.label);
-					}
-			
 					n_fields = GMT_input (fp, &n_expected, &in);
 					n_total_read++;
 				}
-				if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
+				if (GMT_REC_IS_EOF) continue;		/* At EOF */
+				if (GMT_REC_IS_GAP) GMT_io.status = 0;	/* Done with gap */
 
 				n = 0;
-				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
-					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
+				while (!GMT_REC_IS_LINE_BREAK) {	/* Keep going until FALSE or = 2 segment header or gap */
+					if (GMT_REC_IS_ERROR) {
+						fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
 						exit (EXIT_FAILURE);
 					}
-
-					xx[n] = in[0];	yy[n] = in[1];
-					n++;
-					if (n == n_alloc) {
-						n_alloc <<= 1;
-						xx = (double *) GMT_memory ((void *)xx, (size_t)n_alloc, sizeof (double), GMT_program);
-						yy = (double *) GMT_memory ((void *)yy, (size_t)n_alloc, sizeof (double), GMT_program);
-					}
+					if (n == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&xx, (void **)&yy, n, n_alloc, sizeof (double), GMT_program);
+					xx[n] = in[GMT_X];	yy[n] = in[GMT_Y];	n++;
 					n_fields = GMT_input (fp, &n_expected, &in);
 				}
+				
+				/* Done reading a segment */
+				
 				if (polygon && GMT_polygon_is_open (xx, yy, n)) {	/* Explicitly close polygon so that arc will work */
-					xx[n] = xx[0];
-					yy[n] = yy[0];
-					n++;
-					if (n == n_alloc) {
-						n_alloc <<= 1;
-						xx = (double *) GMT_memory ((void *)xx, (size_t)n_alloc, sizeof (double), GMT_program);
-						yy = (double *) GMT_memory ((void *)yy, (size_t)n_alloc, sizeof (double), GMT_program);
-					}
+					if (n == n_alloc) n_alloc = GMT_alloc_memory2 ((void **)&xx, (void **)&yy, n, n_alloc, sizeof (double), GMT_program);
+					xx[n] = xx[0];	yy[n] = yy[0];	n++;
 				}
-
-				xx = (double *) GMT_memory ((void *)xx, (size_t)n, sizeof (double), GMT_program);
-				yy = (double *) GMT_memory ((void *)yy, (size_t)n, sizeof (double), GMT_program);
-				n_alloc = n;
+				n_alloc = GMT_alloc_memory2 ((void **)&xx, (void **)&yy, 0, n, sizeof (double), GMT_program);
 		
 				if (GMT_cpt_skip) continue;	/* CPT says skip this polygon */
 				
-				if (resample) n = GMT_fix_up_path (&xx, &yy, n, step, Ctrl->A.mode);	/* Resample if spacing is too coarse */
+				if (resample) n = n_alloc = GMT_fix_up_path (&xx, &yy, n, step, Ctrl->A.mode);	/* Resample if spacing is too coarse */
 
 				if (polygon)	/* Want a filled polygon */
 					GMT_fill_polygon (xx, yy, 0.0, n, &current_fill, outline_active);	/* NO outline since path may have been clipped */
@@ -1097,15 +1089,15 @@ void plot_y_errorbar (double x, double y, double delta_y, double error_width2, G
 	}
 }
 
-void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, int kind) {
-	int i;
-	static int q[4] = {0, 25, 75, 100};
+void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, GMT_LONG kind) {
+	GMT_LONG i;
+	static GMT_LONG q[4] = {0, 25, 75, 100};
 	double xx[4], yy[4];
 
 	for (i = 0; i < 4; i++) {	/* for 0, 25, 75, 100% hinges */
 		GMT_geo_to_xy (hinge[i], y, &xx[i], &yy[i]);
 		if (GMT_is_dnan (xx[i])) {
-			fprintf (stderr, "%s: Warning: X %d %% hinge exceeded domain near line %ld\n", GMT_program, q[i], line);
+			fprintf (stderr, "%s: Warning: X %ld %% hinge exceeded domain near line %ld\n", GMT_program, q[i], line);
 			xx[i] = (i <2 ) ? project_info.xmin :  project_info.xmax;
 		}
 	}
@@ -1139,15 +1131,15 @@ void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2,
 	}
 }
 
-void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, int kind) {
-	int i;
-	static int q[4] = {0, 25, 75, 100};
+void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, GMT_LONG kind) {
+	GMT_LONG i;
+	static GMT_LONG q[4] = {0, 25, 75, 100};
 	double xx[4], yy[4];
 
 	for (i = 0; i < 4; i++) {	/* for 0, 25, 75, 100% hinges */
 		GMT_geo_to_xy (x, hinge[i], &xx[i], &yy[i]);
 		if (GMT_is_dnan (yy[i])) {
-			fprintf (stderr, "%s: Warning: Y %d %% hinge exceeded domain near line %ld\n", GMT_program, q[i], line);
+			fprintf (stderr, "%s: Warning: Y %ld %% hinge exceeded domain near line %ld\n", GMT_program, q[i], line);
 			yy[i] = (i <2 ) ? project_info.ymin :  project_info.ymax;
 		}
 	}
