@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psxyz.c,v 1.130 2008/05/01 03:00:39 guru Exp $
+ *	$Id: psxyz.c,v 1.161 2009/07/11 03:16:31 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -55,6 +55,10 @@ struct PSXYZ_CTRL {
 		BOOLEAN active;
 		struct GMT_FILL fill;
 	} G;
+	struct I {	/* -I<intensity> */
+		BOOLEAN active;
+		double value;
+	} I;
 	struct L {	/* -L */
 		BOOLEAN active;
 	} L;
@@ -70,7 +74,7 @@ struct PSXYZ_CTRL {
 	} S;
 	struct W {	/* -W<pen> */
 		BOOLEAN active;
-		int mode;	/* 0 = normal, 1 = -C applies to pen color only, 2 = -C applies to symbol fill & pen color */
+		GMT_LONG mode;	/* 0 = normal, 1 = -C applies to pen color only, 2 = -C applies to symbol fill & pen color */
 		struct GMT_PEN pen;
 	} W;
 	struct Z {	/* -Z<z_level> */
@@ -81,7 +85,7 @@ struct PSXYZ_CTRL {
 
 struct PSXYZ_DATA1 {
 	char *string;
-	int symbol, flag;
+	GMT_LONG symbol, flag;
 	float x_size, y_size, z_size;
 	double lon, lat;
 	double x, y, z, value, dist;
@@ -100,31 +104,31 @@ int main (int argc, char **argv)
 	BOOLEAN	get_rgb = FALSE, read_symbol, clip_set = FALSE, fill_active, bar_clip;
 	BOOLEAN default_outline = FALSE, outline_active;
 
-	char line[BUFSIZ], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT];
+	char line[BUFSIZ], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT], *not_used = NULL;
 
-	GMT_LONG	i, n, n_alloc = GMT_CHUNK, n_total_read = 0;
+	GMT_LONG i, n, n_alloc = 0, n_total_read = 0, j, n_files = 0;
+	GMT_LONG fno, n_cols_start = 3, n_args, n_fields, this_outline;
+	GMT_LONG three, four, five, n_required = 3, n_expected = 0, change;
 	
-	int 	j, n_files = 0, fno, rgb[9], n_cols_start = 3, n_fields;
-	int	n_args, three, four, five, n_required = 3;
-	int	n_expected = 0, this_outline, change;
+	int rgb[9];
 
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, dim[3], DX, DY;
 	double lux[3], tmp, *in, x_1, x_2, y_1, y_2, xxx, yyy, font_size;
-	double x2, y2, v_w, h_l, h_w, dx, dy, xx[2], yy[2], s, c, size, *xp, *yp;
+	double x2, y2, v_w, h_l, h_w, dx, dy, xx[2], yy[2], s, c, size;
 
 	FILE *fp = NULL;
 
 	struct GMT_PEN default_pen, current_pen;
 	struct GMT_FILL default_fill, current_fill;
 	struct GMT_SYMBOL S;
-	struct PSXYZ_DATA1 *data1;
-	struct PSXYZ_DATA2 *data2;
+	struct PSXYZ_DATA1 *data1 = NULL;
+	struct PSXYZ_DATA2 *data2 = NULL;
 	struct PSXYZ_CTRL *Ctrl;
 
-	void column3D (double x, double y, double z, double base, double x_size, double y_size, int *rgb, int outline);
-	void cube3D (double x, double y, double z, double x_size, double y_size, int *rgb, int outline);
-	void barx3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, int outline);
-	void bary3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, int outline);
+	void column3D (double x, double y, double z, double base, double x_size, double y_size, int *rgb, GMT_LONG outline);
+	void cube3D (double x, double y, double z, double x_size, double y_size, int *rgb, GMT_LONG outline);
+	void barx3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, GMT_LONG outline);
+	void bary3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, GMT_LONG outline);
 	void sort_on_distance (struct PSXYZ_DATA1 *data, GMT_LONG n);
 	void *New_Psxyz_Ctrl (), Free_Psxyz_Ctrl (struct PSXYZ_CTRL *C);
 	
@@ -163,6 +167,7 @@ int main (int argc, char **argv)
 				case 'H':
 				case 'J':
 				case 'K':
+				case 'M':
 				case 'O':
 				case 'P':
 				case 'R':
@@ -175,6 +180,8 @@ int main (int argc, char **argv)
 				case 'b':
 				case 'c':
 				case 'f':
+				case 'g':
+				case 'm':
 				case ':':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &west, &east, &south, &north);
@@ -200,7 +207,7 @@ int main (int argc, char **argv)
 					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
-					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
+					error += GMT_get_proj3D (&argv[i][2], &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
 				case 'G':		/* Set color for symbol or polygon */
 					Ctrl->G.active = TRUE;
@@ -209,12 +216,13 @@ int main (int argc, char **argv)
 						error++;
 					}
 					break;
+				case 'I':	/* Adjust symbol color via intensity */
+					Ctrl->I.value = atof (&argv[i][2]);
+					Ctrl->I.active = TRUE;
+					break;
 				case 'L':		/* Force closed polygons */
 					Ctrl->L.active = TRUE;
 					break;
-				case 'M':		/* Multiple line segments */
-					GMT_multisegment (&argv[i][2]);
-					break;
                                 case 'N':               /* Do not clip to map */
                                         Ctrl->N.active = TRUE;
                                         break;
@@ -254,35 +262,35 @@ int main (int argc, char **argv)
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
 		fprintf (stderr,"psxyz %s - Plot lines, polygons, and symbols in 3-D\n\n", GMT_VERSION);
-		fprintf(stderr,"usage: psxyz <xyzfiles> %s %s [%s]\n", GMT_J_OPT, GMT_Rgeoz_OPT, GMT_B_OPT);
-		fprintf(stderr, "\t[%s] [-C<cpt>] [-D<dx>/<dy>[/<dz>]] [-E<az/el>] [-G<fill>] [%s] [-K] [-L] [%s] [-N] [-O]\n", GMT_Jz_OPT, GMT_Ho_OPT, GMT_Mo_OPT);
-		fprintf(stderr, "\t[-P] [-Q] [-S<symbol><size>[/size_y]] [%s] [-V] [-W[+|-][<pen>]] [%s]\n", GMT_U_OPT, GMT_X_OPT);
-		fprintf(stderr, "\t[%s] [%s] [%s] [%s] [%s]\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
+		fprintf(stderr,"usage: psxyz <xyzfiles> %s %s [%s] [%s] [-C<cpt>]\n", GMT_J_OPT, GMT_Rgeoz_OPT, GMT_B_OPT, GMT_Jz_OPT);
+		fprintf(stderr, "\t[-D<dx>/<dy>[/<dz>]] [%s] [-G<fill>] [%s] [-I<intens>] [-K] [-L]\n", GMT_E_OPT, GMT_Ho_OPT);
+		fprintf(stderr, "\t[-N] [-O] [-P] [-Q] [-S<symbol><size>[/size_y]] [%s] [-V] [-W[+|-][<pen>]] [%s]\n", GMT_U_OPT, GMT_X_OPT);
+		fprintf(stderr, "\t[%s] [%s] [%s] [%s] [%s] [%s] [%s]\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT, GMT_g_OPT, GMT_mo_OPT);
 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\t<xyzfiles> is one or more files.  If none, read standard input\n");
 		GMT_explain_option ('j');
+		GMT_explain_option ('Z');
 		GMT_explain_option ('r');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-C Use cpt-file to assign symbol colors based on t-value in 4th column (requires -S)\n");
-		fprintf (stderr, "\t   Without -S, psxyz expects -M lines/polygons and looks for -Z<tval> strings in each multiheader\n");
+		fprintf (stderr, "\t   Without -S, psxyz expects -m lines/polygons and looks for -Z<tval> strings in each multiheader\n");
 		fprintf (stderr, "\t-D Offset symbol or line positions by <dx>/<dy>[/<dy>] [no offset].\n");
-		fprintf (stderr, "\t-E set user viewpoint azimuth and elevation [180/90].\n");
+		GMT_explain_option ('E');
 		GMT_fill_syntax ('G', "Specify color or pattern [Default is no fill].");
 		fprintf (stderr, "\t   If -G is specified but not -S, psxyz draws a filled polygon.\n");
 		GMT_explain_option ('H');
-		GMT_explain_option ('Z');
+		fprintf (stderr, "\t-I sets a constant intensity used to modulate the fill color (requires -C or -G)\n");
 		GMT_explain_option ('K');
 		fprintf (stderr, "\t-L Force closed polygons\n");
-		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N Do Not skip symbols that fall outside map border [Default will ignore those outside]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
 		fprintf (stderr, "\t-Q Do NOT sort symbols based on distance to viewer before plotting\n");
 		fprintf (stderr, "\t-S to select symbol type and symbol size (in %s).  Choose between\n", GMT_unit_names[gmtdefs.measure_unit]);
-		fprintf (stderr, "\t   -(xdash), st(a)r, (b|B)ar, (c)ircle, (d)iamond, (e)llipse, (f)ront, octa(g)on, (h)exagon\n");
+		fprintf (stderr, "\t   -(xdash), +(plus), st(a)r, (b|B)ar, (c)ircle, (d)iamond, (e)llipse, (f)ront, octa(g)on, (h)exagon\n");
 		fprintf (stderr, "\t   (i)nvtriangle, (j)rotated rectangle, (k)ustom, (%c)etter, pe(n)tagon, c(o)lumn, (p)oint, (q)uoted line,\n", 'l');
 		fprintf (stderr, "\t   (r)ect, (s)quare, (t)riangle, c(u)be, (v)ector, (w)edge, (x)cross, (y)dash, (z)dash\n");
 		fprintf (stderr, "\t   If no size is specified, psxyz expects the 4th column to have sizes.\n");
@@ -341,6 +349,8 @@ int main (int argc, char **argv)
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is the required number of columns\n");
 		GMT_explain_option ('f');
+		GMT_explain_option ('g');
+		GMT_explain_option ('m');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -361,7 +371,7 @@ int main (int argc, char **argv)
 	}
 	if (Ctrl->C.active && S.symbol == 0 && !GMT_io.multi_segments[GMT_IN]) {
 		error++;
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option: Must also specify a symbol (see -S) or give polygon file with -M\n", GMT_program);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -C option: Must also specify a symbol (see -S) or give polygon file with -m\n", GMT_program);
 	}
 	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
@@ -389,6 +399,10 @@ int main (int argc, char **argv)
 	default_pen = current_pen = Ctrl->W.pen;
 	default_fill = current_fill = Ctrl->G.fill;
 	default_outline = Ctrl->W.active;
+	if (Ctrl->I.active) {
+		GMT_illuminate (Ctrl->I.value, current_fill.rgb);
+		GMT_illuminate (Ctrl->I.value, default_fill.rgb);
+	}
 
 	if (get_rgb) n_cols_start++;
 
@@ -400,13 +414,13 @@ int main (int argc, char **argv)
 	n_expected = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : n_required;
 	if (GMT_io.binary[GMT_IN] && GMT_io.ncol[GMT_IN] == 0) GMT_io.ncol[GMT_IN]=  n_required;
 	if (n_expected < n_required) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %d columns\n", GMT_program, n_required);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %ld columns\n", GMT_program, n_required);
 		error++;
 	}
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (Ctrl->C.active) GMT_read_cpt (Ctrl->C.file);
@@ -474,6 +488,7 @@ int main (int argc, char **argv)
 		GMT_xyz_to_xy (Ctrl->D.dx, Ctrl->D.dy, Ctrl->D.dz, &DX, &DY);
 		ps_transrotate (DX, DY, 0.0);	/* Shift plot a bit */
 	}
+	GMT_io.skip_if_NaN[GMT_Z] = TRUE;	/* Extend GMT NaN-handling to the z-coordinate */
 
 	done = FALSE;
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
@@ -499,7 +514,7 @@ int main (int argc, char **argv)
 		}
 		if (GMT_io.io_header[GMT_IN]) {
 			for (i = 0; i < GMT_io.n_header_recs; i++) {
-				GMT_fgets (line, BUFSIZ, fp);
+				not_used = GMT_fgets (line, BUFSIZ, fp);
 				if (i == 0 && S.G.label_type == 2) GMT_extract_label (&line[1], S.G.label);	/* Set initial header as potential label */
 			}
 		}
@@ -509,23 +524,26 @@ int main (int argc, char **argv)
 			GMT_world_map = (S.symbol == GMT_SYMBOL_ELLIPSE && S.convert_angles) ? FALSE : TRUE;
 			if (read_symbol) n_expected = BUFSIZ;
 
-			data1 = (struct PSXYZ_DATA1 *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct PSXYZ_DATA1), GMT_program);
 			n = 0;
-			while ((n_fields = GMT_input (fp, &n_expected, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
+			while ((n_fields = GMT_input (fp, &n_expected, &in)) >= 0 && !GMT_REC_IS_EOF) {
 
-				while ((GMT_io.status & GMT_IO_SEGMENT_HEADER) && !(GMT_io.status & GMT_IO_EOF)) {	/* Skip segment headers */
+				while (GMT_REC_IS_SEG_HEADER && !GMT_REC_IS_EOF) {	/* Skip segment headers */
 					if (gmtdefs.verbose) ps_comment (GMT_io.segment_header);
 					
 					change = GMT_parse_multisegment_header (GMT_io.segment_header, Ctrl->C.active, &fill_active, &current_fill, &default_fill, &outline_active, &current_pen, &default_pen, default_outline);
+					if (Ctrl->I.active) {
+						GMT_illuminate (Ctrl->I.value, current_fill.rgb);
+						GMT_illuminate (Ctrl->I.value, default_fill.rgb);
+					}
 					if (change & 4) GMT_setpen (&current_pen);
 
 					if (read_symbol) n_expected = BUFSIZ;
 					n_fields = GMT_input (fp, &n_expected, &in);
 				}
-				if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
+				if (GMT_REC_IS_EOF) continue;		/* At EOF */
 	
-				if (GMT_io.status & GMT_IO_MISMATCH && !read_symbol) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
+				if (GMT_REC_IS_ERROR && !read_symbol) {
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
 					exit (EXIT_FAILURE);
 				}
 				n_total_read++;
@@ -546,9 +564,11 @@ int main (int argc, char **argv)
 					S.size_y = (S.symbol == GMT_SYMBOL_COLUMN) ? in[four] * GMT_u2u[S.u][GMT_INCH] : S.size_x;
 				}
 
+				if (n == n_alloc) n_alloc = GMT_alloc_memory ((void **)&data1, n, n_alloc, sizeof (struct PSXYZ_DATA1), GMT_program);
 				data1[n].flag = 0;
-				data1[n].lon = in[0];
-				data1[n].lat = in[1];
+				data1[n].lon = in[GMT_X];
+				data1[n].lat = in[GMT_Y];
+				data1[n].string = NULL;
 				if (S.read_vector) {
 					data1[n].x_size = (float)in[three];	/* direction */
 					data1[n].y_size = (float)(in[four] * GMT_u2u[S.u][GMT_INCH]);	/* length */
@@ -635,18 +655,19 @@ int main (int argc, char **argv)
 						else if (in[GMT_Y] > north)
 							in[GMT_Y] = north;
 					}
-					if (in[2] < project_info.z_bottom || in[2] > project_info.z_top) continue;
+					if (in[GMT_Z] < project_info.z_bottom || in[GMT_Z] > project_info.z_top) continue;
 				}
 				else if (!Ctrl->N.active) {
-					if (in[2] < project_info.z_bottom || in[2] > project_info.z_top) continue;
-					GMT_map_outside (in[0], in[1]);
+					if (in[GMT_Z] < project_info.z_bottom || in[GMT_Z] > project_info.z_top) continue;
+					GMT_map_outside (in[GMT_X], in[GMT_Y]);
 					if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
 				}
 
-				GMT_project3D (in[0], in[1], in[2], &data1[n].x, &data1[n].y, &data1[n].z);
+				GMT_project3D (in[GMT_X], in[GMT_Y], in[GMT_Z], &data1[n].x, &data1[n].y, &data1[n].z);
 				if (get_rgb) {	/* Lookup z to get rgb */
 					data1[n].value = in[3];
 					GMT_get_rgb_from_z (data1[n].value, current_fill.rgb);
+					if (Ctrl->I.active) GMT_illuminate (Ctrl->I.value, current_fill.rgb);
 				}
 				if (S.symbol == GMT_SYMBOL_COLUMN) {
 					if (GMT_is_dnan (S.base))
@@ -669,10 +690,7 @@ int main (int argc, char **argv)
 						GMT_y_to_yy (S.base, &tmp);
 					data1[n].z_size = (float)tmp;
 				}
-				if (S.symbol == GMT_SYMBOL_TEXT) {
-					data1[n].string = (char *) GMT_memory (VNULL, (size_t) (strlen (S.string) + 1), sizeof (char), GMT_program);
-					strcpy (data1[n].string, S.string);
-				}
+				if (S.symbol == GMT_SYMBOL_TEXT) data1[n].string = strdup (S.string);
 				data1[n].symbol = S.symbol;
 				if (Ctrl->W.mode) {
 					memcpy ((void *)Ctrl->W.pen.rgb, (void *)current_fill.rgb, 3*sizeof (int));
@@ -682,15 +700,10 @@ int main (int argc, char **argv)
 				data1[n].f = current_fill;
 				data1[n].p = current_pen;
 				data1[n].outline = outline_active;
-
 				n++;
-				if (n == n_alloc) {
-					n_alloc <<= 1;
-					data1 = (struct PSXYZ_DATA1 *) GMT_memory ((void *)data1, (size_t)n_alloc, sizeof (struct PSXYZ_DATA1), GMT_program);
-				}
 				if (read_symbol) n_expected = BUFSIZ;
 			}
-			data1 = (struct PSXYZ_DATA1 *) GMT_memory ((void *)data1, (size_t)n, sizeof (struct PSXYZ_DATA1), GMT_program);
+			n_alloc = GMT_alloc_memory ((void **)&data1, 0, n, sizeof (struct PSXYZ_DATA1), GMT_program);
 	
 			/* Sort according to distance  from viewer */
 	
@@ -757,9 +770,13 @@ int main (int argc, char **argv)
 						dim[0] = (double)data1[i].x_size;
 						GMT_cross (data1[i].x, data1[i].y, data1[i].z, dim, NULL, FALSE);
 						break;
+					case GMT_SYMBOL_PLUS:
+						dim[0] = (double)data1[i].x_size;
+						GMT_plus (data1[i].x, data1[i].y, data1[i].z, dim, NULL, FALSE);
+						break;
 					case GMT_SYMBOL_POINT:
 						dim[0] = GMT_POINT_SIZE;
-						GMT_cross (data1[i].x, data1[i].y, data1[i].z, dim, NULL, FALSE);
+						GMT_plus (data1[i].x, data1[i].y, data1[i].z, dim, NULL, FALSE);
 						break;
 					case GMT_SYMBOL_CIRCLE:
 						size = (double)data1[i].x_size;
@@ -824,6 +841,7 @@ int main (int argc, char **argv)
 							GMT_text3D (data1[i].x, data1[i].y, data1[i].z, font_size, S.font_no, data1[i].string, 0.0, 6, FALSE);
 						else
 							GMT_text3D (data1[i].x, data1[i].y, data1[i].z, font_size, S.font_no, data1[i].string, 0.0, 6, TRUE);
+						free ((void *)data1[i].string);
 						break;
 					case GMT_SYMBOL_VECTOR:
 						if (data1[i].flag & 2) {
@@ -923,62 +941,64 @@ int main (int argc, char **argv)
 						GMT_draw_custom_symbol (data1[i].x, data1[i].y, data1[i].z, (double)data1[i].x_size, S.custom, &current_pen, &current_fill, data1[i].outline);
 						break;
 				}
-				if (data1[i].string) GMT_free ((void *)data1[i].string);
 			}
-			GMT_free ((void *)data1);
 		}
 		else {	/* Line/polygon part */
-			data2 = (struct PSXYZ_DATA2 *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (struct PSXYZ_DATA2), GMT_program);
 			n_required = 3;
 			n_fields = GMT_input (fp, &n_expected, &in);
-			while (! (GMT_io.status & GMT_IO_EOF)) {
-
-				while (GMT_io.status & GMT_IO_SEGMENT_HEADER) {
-					if (gmtdefs.verbose) ps_comment (GMT_io.segment_header);
+			while (!GMT_REC_IS_EOF) {
+				while (GMT_REC_IS_NEW_SEGMENT) {	/* Process multisegment header flags (or NaN gaps) */
+					if (GMT_REC_IS_SEG_HEADER) {
+						if (gmtdefs.verbose) ps_comment (GMT_io.segment_header);
 					
-					change = GMT_parse_multisegment_header (GMT_io.segment_header, Ctrl->C.active, &fill_active, &current_fill, &default_fill, &outline_active, &current_pen, &default_pen, default_outline);
-					if ((change & 4) && penset_OK) GMT_setpen (&current_pen);
-					if ((change & 1)) polygon = TRUE;
-					if ((change & 2) && !Ctrl->L.active) {
-						polygon = FALSE;
-						ps_setpaint (current_fill.rgb);
+						change = GMT_parse_multisegment_header (GMT_io.segment_header, Ctrl->C.active, &fill_active, &current_fill, &default_fill, &outline_active, &current_pen, &default_pen, default_outline);
+						if (Ctrl->I.active) {
+							GMT_illuminate (Ctrl->I.value, current_fill.rgb);
+							GMT_illuminate (Ctrl->I.value, default_fill.rgb);
+						}
+						if ((change & 4) && penset_OK) GMT_setpen (&current_pen);
+						if ((change & 1)) polygon = TRUE;
+						if ((change & 2) && !Ctrl->L.active) {
+							polygon = FALSE;
+							ps_setpaint (current_fill.rgb);
+						}
+						if (S.G.label_type == 2) {	/* Update segment header */
+							GMT_extract_label (&GMT_io.segment_header[1], S.G.label);
+						}
 					}
-					if (S.G.label_type == 2) {	/* Update segment header */
-						GMT_extract_label (&GMT_io.segment_header[1], S.G.label);
-					}
-
 					n_fields = GMT_input (fp, &n_expected, &in);
 					n_total_read++;
 				}
-				if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
+				if (GMT_REC_IS_EOF) continue;		/* At EOF */
+				if (GMT_REC_IS_GAP) GMT_io.status = 0;	/* Done with gap */
 
 				n = 0;
-				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
-					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
+				while (!GMT_REC_IS_LINE_BREAK) {	/* Keep going until FALSE or = 2 segment header or gap */
+					if (GMT_REC_IS_ERROR) {
+						fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
 						exit (EXIT_FAILURE);
 					}
 
-					data2[n].x = in[0];	data2[n].y = in[1];
-					data2[n].z = in[2];
+					if (n == n_alloc) n_alloc = GMT_alloc_memory ((void **)&data2, n, n_alloc, sizeof (struct PSXYZ_DATA2), GMT_program);
+					data2[n].x = in[GMT_X];	data2[n].y = in[GMT_Y];
+					data2[n].z = in[GMT_Z];
 					n++;
-					if (n == n_alloc) {
-						n_alloc <<= 1;
-						data2 = (struct PSXYZ_DATA2 *) GMT_memory ((void *)data2, (size_t)n_alloc, sizeof (struct PSXYZ_DATA2), GMT_program);
-					}
 					n_fields = GMT_input (fp, &n_expected, &in);
 				}
-
+				
+				/* Done reading a segment */
+				
 				if (polygon) {  /* Explicitly close polygon */
+					if (n == n_alloc) n_alloc = GMT_alloc_memory ((void **)&data2, n, n_alloc, sizeof (struct PSXYZ_DATA2), GMT_program);
 					data2[n].x = data2[0].x;
 					data2[n].y = data2[0].y;
 					data2[n].z = data2[0].z;
 					n++;
 				}
-				data2 = (struct PSXYZ_DATA2 *) GMT_memory ((void *)data2, (size_t)n, sizeof (struct PSXYZ_DATA2), GMT_program);
-				n_alloc = n;
+				n_alloc = GMT_alloc_memory ((void **)&data2, n, n_alloc, sizeof (struct PSXYZ_DATA2), GMT_program);
 		
 				if (!GMT_cpt_skip) {
+					double *xp, *yp;
 					xp = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
 					yp = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
 		
@@ -996,7 +1016,7 @@ int main (int argc, char **argv)
 					}
 					else {
 						for (i = 0; i < n; i++) GMT_geoz_to_xy (data2[i].x, data2[i].y, data2[i].z, &xp[i], &yp[i]);
-						ps_line (xp, yp, n, 3, FALSE, TRUE);
+						ps_line (xp, yp, n, 3, FALSE);
 					}
 					if (S.symbol == GMT_SYMBOL_FRONT) {
 						for (i = 0; i < n; i++) GMT_geo_to_xy (data2[i].x, data2[i].y, &xp[i], &yp[i]);
@@ -1006,13 +1026,12 @@ int main (int argc, char **argv)
 					GMT_free ((void *)xp);
 					GMT_free ((void *)yp);
 				}
-				n = 0;
 			}
-			GMT_free ((void *)data2);
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
-
 	}
+	if (data1) GMT_free ((void *)data1);
+	if (data2) GMT_free ((void *)data2);
 
 	if (S.symbol == GMT_SYMBOL_QUOTED_LINE) {
 		GMT_contlabel_plot (&S.G);
@@ -1036,9 +1055,9 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void column3D (double x, double y, double z, double base, double x_size, double y_size, int *rgb, int outline)
+void column3D (double x, double y, double z, double base, double x_size, double y_size, int *rgb, GMT_LONG outline)
 {
-	int i, j, k;
+	GMT_LONG i, j, k;
 	double zz, xp[4], yp[4], zp[4], plot_x[4], plot_y[4], top, sign;
 
 	x_size *= 0.5;
@@ -1078,9 +1097,9 @@ void column3D (double x, double y, double z, double base, double x_size, double
 	}
 }
 
-void cube3D (double x, double y, double z, double x_size, double y_size, int *rgb, int outline)
+void cube3D (double x, double y, double z, double x_size, double y_size, int *rgb, GMT_LONG outline)
 {
-	int i, j, k;
+	GMT_LONG i, j, k;
 	double xp[4], yp[4], zp[4], plot_x[4], plot_y[4], sign;
 
 	x_size *= 0.5;
@@ -1116,9 +1135,9 @@ void cube3D (double x, double y, double z, double x_size, double y_size, int *rg
 	}
 }
 
-void bary3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, int outline)
+void bary3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, GMT_LONG outline)
 {
-	int i;
+	GMT_LONG i;
 	double xp[4], yp[4], plot_x[4], plot_y[4];
 
 	size *= 0.5;
@@ -1128,9 +1147,9 @@ void bary3D (double x, double y, double z, double base, double size, struct GMT_
 	GMT_fill (plot_x, plot_y, (GMT_LONG)4, f, outline);
 }
 
-void barx3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, int outline)
+void barx3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, GMT_LONG outline)
 {
-	int i;
+	GMT_LONG i;
 	double xp[4], yp[4], plot_x[4], plot_y[4];
 
 	size *= 0.5;
@@ -1164,7 +1183,7 @@ void sort_on_distance (struct PSXYZ_DATA1 *data, GMT_LONG n)
 		c = x;
 	}
 	else {
-		a = -tan (z_project.view_azimuth);
+		a = -z_project.sin_az / z_project.cos_az;
 		b = -1.0;
 		c = y - x * a;
 	}
