@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: spectrum1d.c,v 1.35 2007/09/30 04:16:49 guru Exp $
+ *	$Id: spectrum1d.c,v 1.43 2009/07/11 03:16:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -87,7 +87,7 @@ struct SPECTRUM1D_CTRL {
 	} N;
 	struct S {	/* -S<segment_size> */
 		BOOLEAN active;
-		int size;
+		GMT_LONG size;
 	} S;
 	struct W {	/* -W */
 		BOOLEAN active;
@@ -96,7 +96,7 @@ struct SPECTRUM1D_CTRL {
 
 struct SPECTRUM1D_INFO {	/* Control structure for spectrum1d */
 	BOOLEAN	y_given;
-	int	n_spec, window, window_2;
+	GMT_LONG	n_spec, window, window_2;
 	float	*datac;
 	double	dt, x_variance, y_variance, d_n_windows, y_pow;
 	struct	SPEC {
@@ -113,9 +113,10 @@ int main (int argc, char **argv)
 {
 	BOOLEAN	error = FALSE;
 	
-	char	buffer[BUFSIZ];
+	char	buffer[BUFSIZ], *not_used = NULL;
 
-	int	i, k, n_alloc, n_expected_fields, n_fields, j, n_outputs, window_test = 2, n_data;
+	GMT_LONG	i, k, j, n_expected_fields, n_fields, n_outputs, window_test = 2;
+	GMT_LONG	n_alloc, n_data;
 
 	float	*x, *y = NULL;
 
@@ -127,8 +128,8 @@ int main (int argc, char **argv)
 	struct SPECTRUM1D_CTRL *Ctrl;
 
 	void alloc_arrays (struct SPECTRUM1D_INFO *C);
-	void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, int n_data);
-	void write_output (struct SPECTRUM1D_INFO *C, char *cols, int n_outputs, BOOLEAN write_wavelength, char *namestem);
+	void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, GMT_LONG n_data);
+	void write_output (struct SPECTRUM1D_INFO *C, char *cols, GMT_LONG n_outputs, BOOLEAN write_wavelength, char *namestem);
 	void free_space (struct SPECTRUM1D_INFO *C);
 	void *New_Spectrum1d_Ctrl (), Free_Spectrum1d_Ctrl (struct SPECTRUM1D_CTRL *C);
 	
@@ -240,7 +241,7 @@ int main (int argc, char **argv)
 		error++;
 	}
 	if (window_test != Ctrl->S.size) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option: Segment size not radix 2.  Try %d or %d\n", GMT_program,
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option: Segment size not radix 2.  Try %ld or %ld\n", GMT_program,
 			(window_test/2), window_test);
 		error++;
 	}
@@ -262,7 +263,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	C.dt = Ctrl->D.inc;
@@ -300,21 +301,21 @@ int main (int argc, char **argv)
 	if (C.y_given) y = (float *)GMT_memory (VNULL, (size_t)n_alloc, sizeof(float), GMT_program);
 	n_expected_fields = (GMT_io.ncol[GMT_IN]) ? GMT_io.ncol[GMT_IN] : C.y_given + 1;
 
-	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
+	if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (buffer, BUFSIZ, fp);
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n_data);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_data);
 			exit (EXIT_FAILURE);
 		}
 
-                x[n_data] = (float)in[0];
-		if (C.y_given) y[n_data] = (float)in[1];
+                x[n_data] = (float)in[GMT_X];
+		if (C.y_given) y[n_data] = (float)in[GMT_Y];
                 n_data++;
 
                 if (n_data == n_alloc) {
-                        n_alloc += GMT_CHUNK;
+                        n_alloc <<= 1;
 			x = (float *) GMT_memory ((void *)x, (size_t)n_alloc, sizeof(float), GMT_program);
 			if (C.y_given) y = (float *)GMT_memory ((void *)y, (size_t)n_alloc, sizeof(float), GMT_program);
                 }
@@ -322,7 +323,7 @@ int main (int argc, char **argv)
         if (fp != GMT_stdin) GMT_fclose(fp);
 	x = (float *) GMT_memory ((void *)x, (size_t)n_data, sizeof(float), GMT_program);
 	if (C.y_given) y = (float *)GMT_memory ((void *)y, (size_t)n_data, sizeof(float), GMT_program);
-	if (gmtdefs.verbose) fprintf(stderr,"Read %d data points.\n", n_data);
+	if (gmtdefs.verbose) fprintf(stderr,"Read %ld data points.\n", n_data);
 
 	compute_spectra (&C, x, y, n_data);
 
@@ -349,10 +350,10 @@ void alloc_arrays (struct SPECTRUM1D_INFO *C)
 	C->datac = (float *) GMT_memory (VNULL, (size_t)C->window_2, sizeof(float), GMT_program);
 }
 
-void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, int n_data)
+void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, GMT_LONG n_data)
 {
-	int	n_windows, w, i, t_start, t_stop, t, f;
-	int	narray;
+	GMT_LONG	n_windows, w, i, t_start, t_stop, t, f;
+	GMT_LONG	narray;
 	float	work = 0.0;
 	double	dw, spec_scale, x_varp, y_varp = 1.0, one_on_nw, co_quad;
 	double	xreal, ximag, yreal, yimag, xpower, ypower, co_spec, quad_spec;
@@ -377,7 +378,7 @@ void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, int n_data)
 	dw = (n_windows > 1) ? (double)(n_data - C->window) / (double)(n_windows - 1) : 1.0;
 
 	for (w = 0; w < n_windows; w++) {
-		t_start = (int)floor (0.5 + w * dw);
+		t_start = (GMT_LONG)floor (0.5 + w * dw);
 		t_stop = t_start + C->window;
 		if (C->y_given) {
 			for (t = t_start, i = 0; t < t_stop; t++, i+=2) {
@@ -436,7 +437,7 @@ void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, int n_data)
 
 		if (gmtdefs.verbose) {
 			C->y_pow = (C->y_given) ? C->y_variance/y_varp : 0.0;
-			fprintf(stderr,"Window %d from %d to %d\n", w, t_start, t_stop);
+			fprintf(stderr,"Window %ld from %ld to %ld\n", w, t_start, t_stop);
 			sprintf(format, "X var:  %s  X pow:  %s  ratio:  %s  Y var:  %s  Y pow:  %s  ratio:  %s\n",
 				gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 			fprintf(stderr, format, C->x_variance, x_varp, (C->x_variance/x_varp), C->y_variance, y_varp, C->y_pow);
@@ -465,9 +466,9 @@ void compute_spectra (struct SPECTRUM1D_INFO *C, float *x, float *y, int n_data)
 	}
 }
 
-void write_output (struct SPECTRUM1D_INFO *C, char *col, int n_outputs, BOOLEAN write_wavelength, char *namestem)
+void write_output (struct SPECTRUM1D_INFO *C, char *col, GMT_LONG n_outputs, BOOLEAN write_wavelength, char *namestem)
 {
-	int	i, j;
+	GMT_LONG	i, j;
 	double	delta_f, eps_pow, out[3], *f_or_w;
 	char	fname[GMT_LONG_TEXT];
 	FILE	*fpout;
@@ -489,8 +490,8 @@ void write_output (struct SPECTRUM1D_INFO *C, char *col, int n_outputs, BOOLEAN
 			}
 			if (gmtdefs.verbose) fprintf(stderr,"%s:  Writing %s\n", GMT_program, fname);
 			for (i = 0; i < C->n_spec; i++) {
-				out[0] = f_or_w[i];
-				out[1] = C->spec[i].xpow;
+				out[GMT_X] = f_or_w[i];
+				out[GMT_Y] = C->spec[i].xpow;
 				out[2] = eps_pow * C->spec[i].xpow;
 				GMT_output (fpout, 3, out);
 			}
@@ -505,8 +506,8 @@ void write_output (struct SPECTRUM1D_INFO *C, char *col, int n_outputs, BOOLEAN
 			}
 			if (gmtdefs.verbose) fprintf(stderr,"%s:  Writing %s\n", GMT_program, fname);
 			for (i = 0; i < C->n_spec; i++) {
-				out[0] = f_or_w[i];
-				out[1] = C->spec[i].ypow;
+				out[GMT_X] = f_or_w[i];
+				out[GMT_Y] = C->spec[i].ypow;
 				out[2] = eps_pow * C->spec[i].ypow;
 				GMT_output (fpout, 3, out);
 			}
@@ -520,9 +521,9 @@ void write_output (struct SPECTRUM1D_INFO *C, char *col, int n_outputs, BOOLEAN
 			}
 			if (gmtdefs.verbose) fprintf(stderr,"%s:  Writing %s\n", GMT_program, fname);
 			for (i = 0; i < C->n_spec; i++) {
-				out[0] = f_or_w[i];
-				out[1] = C->spec[i].ypow * C->spec[i].coh;
-				out[2] = out[1] * eps_pow * sqrt( (2.0 - C->spec[i].coh) / C->spec[i].coh);
+				out[GMT_X] = f_or_w[i];
+				out[GMT_Y] = C->spec[i].ypow * C->spec[i].coh;
+				out[2] = out[GMT_Y] * eps_pow * sqrt( (2.0 - C->spec[i].coh) / C->spec[i].coh);
 				GMT_output (fpout, 3, out);
 			}
 			GMT_fclose(fpout);
@@ -535,9 +536,9 @@ void write_output (struct SPECTRUM1D_INFO *C, char *col, int n_outputs, BOOLEAN
 			}
 			if (gmtdefs.verbose) fprintf(stderr,"%s:  Writing %s\n", GMT_program, fname);
 			for (i = 0; i < C->n_spec; i++) {
-				out[0] = f_or_w[i];
-				out[1] = C->spec[i].ypow * (1.0 - C->spec[i].coh);
-				out[2] = out[1] * eps_pow;
+				out[GMT_X] = f_or_w[i];
+				out[GMT_Y] = C->spec[i].ypow * (1.0 - C->spec[i].coh);
+				out[2] = out[GMT_Y] * eps_pow;
 				GMT_output (fpout, 3, out);
 			}
 			GMT_fclose(fpout);
@@ -550,9 +551,9 @@ void write_output (struct SPECTRUM1D_INFO *C, char *col, int n_outputs, BOOLEAN
 			}
 			if (gmtdefs.verbose) fprintf(stderr,"%s:  Writing %s\n", GMT_program, fname);
 			for (i = 0; i < C->n_spec; i++) {
-				out[0] = f_or_w[i];
-				out[1] = C->spec[i].gain;
-				out[2] = out[1] * eps_pow * sqrt( (1.0 - C->spec[i].coh) / (2.0 * C->spec[i].coh) );
+				out[GMT_X] = f_or_w[i];
+				out[GMT_Y] = C->spec[i].gain;
+				out[2] = out[GMT_Y] * eps_pow * sqrt( (1.0 - C->spec[i].coh) / (2.0 * C->spec[i].coh) );
 				GMT_output (fpout, 3, out);
 			}
 			GMT_fclose(fpout);
@@ -573,9 +574,9 @@ void write_output (struct SPECTRUM1D_INFO *C, char *col, int n_outputs, BOOLEAN
 			}
 			if (gmtdefs.verbose) fprintf(stderr,"%s:  Writing %s\n", GMT_program, fname);
 			for (i = 0; i < C->n_spec; i++) {
-				out[0] = f_or_w[i];
-				out[1] = C->spec[i].radmit;
-				out[2] = fabs (eps_pow * sqrt( (1.0 - C->spec[i].coh) / (2.0 * C->spec[i].coh) ) * out[1]);
+				out[GMT_X] = f_or_w[i];
+				out[GMT_Y] = C->spec[i].radmit;
+				out[2] = fabs (eps_pow * sqrt( (1.0 - C->spec[i].coh) / (2.0 * C->spec[i].coh) ) * out[GMT_Y]);
 				GMT_output (fpout, 3, out);
 			}
 			GMT_fclose(fpout);
@@ -588,8 +589,8 @@ void write_output (struct SPECTRUM1D_INFO *C, char *col, int n_outputs, BOOLEAN
 			}
 			if (gmtdefs.verbose) fprintf(stderr,"%s:  Writing %s\n", GMT_program, fname);
 			for (i = 0; i < C->n_spec; i++) {
-				out[0] = f_or_w[i];
-				out[1] = C->spec[i].phase;
+				out[GMT_X] = f_or_w[i];
+				out[GMT_Y] = C->spec[i].phase;
 				out[2] = eps_pow * sqrt( (1.0 - C->spec[i].coh) / (2.0 * C->spec[i].coh) );
 				GMT_output (fpout, 3, out);
 			}
@@ -602,9 +603,9 @@ void write_output (struct SPECTRUM1D_INFO *C, char *col, int n_outputs, BOOLEAN
 			}
 			if (gmtdefs.verbose) fprintf(stderr,"%s:  Writing %s\n", GMT_program, fname);
 			for (i = 0; i < C->n_spec; i++) {
-				out[0] = f_or_w[i];
-				out[1] = C->spec[i].coh;
-				out[2] = out[1] * eps_pow * (1.0 - C->spec[i].coh) * sqrt(2.0 / C->spec[i].coh);
+				out[GMT_X] = f_or_w[i];
+				out[GMT_Y] = C->spec[i].coh;
+				out[2] = out[GMT_Y] * eps_pow * (1.0 - C->spec[i].coh) * sqrt(2.0 / C->spec[i].coh);
 				GMT_output (fpout, 3, out);
 			}
 			GMT_fclose(fpout);
@@ -623,7 +624,7 @@ void free_space (struct SPECTRUM1D_INFO *C)
 
 void detrend_and_hanning (struct SPECTRUM1D_INFO *C)
 {
-	int	i, t;
+	GMT_LONG	i, t;
 	double	sumx, sumtx, sumy, sumty, sumt2, x_slope, x_mean, y_slope, y_mean;
 	double	t_factor, h_period, h_scale, hc, hw, tt;
 	sumx = 0.0;
@@ -689,7 +690,7 @@ void detrend_and_hanning (struct SPECTRUM1D_INFO *C)
 void *New_Spectrum1d_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct SPECTRUM1D_CTRL *C;
 	
-	C = (struct SPECTRUM1D_CTRL *) GMT_memory (VNULL, 1, sizeof (struct SPECTRUM1D_CTRL), "New_Spectrum1d_Ctrl");
+	C = (struct SPECTRUM1D_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct SPECTRUM1D_CTRL), "New_Spectrum1d_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->D.inc = 1.0;
