@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: splitxyz.c,v 1.39 2007/09/30 04:16:49 guru Exp $
+ *	$Id: splitxyz.c,v 1.49 2009/07/11 03:16:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -78,22 +78,22 @@ struct  DATA    {
 
 int main(int argc, char **argv)
 {
-	int     i, j, k, ndata, n_alloc, nprofiles, begin, end, n_expected_fields, n_outputs = 0, n_fields;
-	int	n_required_fields, output_choice[SPLITXYZ_N_OUTPUT_CHOICES];
+	GMT_LONG     i, ndata, n_alloc, nprofiles, begin, end;
+	GMT_LONG	j, k, n_required_fields, output_choice[SPLITXYZ_N_OUTPUT_CHOICES], n_expected_fields, n_outputs = 0, n_fields;
 
 	BOOLEAN	error = FALSE, ok, hilow = FALSE, z_selected = FALSE;
 
 	double	dy, dx, last_d, last_c, last_s, csum, ssum, this_c, this_s, dotprod;
 	double	mean_azim, fwork[SPLITXYZ_F_RES], *in, out[SPLITXYZ_N_OUTPUT_CHOICES];
 
- 	char    buffer[BUFSIZ], filename[BUFSIZ];
+ 	char    buffer[BUFSIZ], filename[BUFSIZ], *not_used = NULL;
 
 	struct  DATA *data;
 	struct SPLITXYZ_CTRL *Ctrl;
 
 	FILE    *fp = NULL, *fpout;
 
-	void	filterz(struct DATA *data, int ndata, double z_filter, double *fwork, int hilow), filterxy_setup(double *fwork), filterxy(struct DATA *data, int begin, int end, double xy_filter, double *fwork, int hilow);
+	void	filterz(struct DATA *data, GMT_LONG ndata, double z_filter, double *fwork, GMT_LONG hilow), filterxy_setup(double *fwork), filterxy(struct DATA *data, GMT_LONG begin, GMT_LONG end, double xy_filter, double *fwork, GMT_LONG hilow);
 	void *New_Splitxyz_Ctrl (), Free_Splitxyz_Ctrl (struct SPLITXYZ_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -101,7 +101,7 @@ int main(int argc, char **argv)
 	Ctrl = (struct SPLITXYZ_CTRL *)New_Splitxyz_Ctrl ();	/* Allocate and initialize a new control structure */
 
 	fpout = GMT_stdout;
-	memset ((void *)output_choice, 0, SPLITXYZ_N_OUTPUT_CHOICES * sizeof (int));
+	memset ((void *)output_choice, 0, SPLITXYZ_N_OUTPUT_CHOICES * sizeof (GMT_LONG));
 
         for (i = 1; i < argc; i++) {
                 if (argv[i][0] == '-') {
@@ -283,7 +283,7 @@ int main(int argc, char **argv)
 	else
 		n_expected_fields = n_required_fields;
 	if (GMT_io.binary[GMT_IN] && n_expected_fields < n_required_fields) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %d columns\n", GMT_program, n_required_fields);
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data must have at least %ld columns\n", GMT_program, n_required_fields);
 		error++;
 	}
 
@@ -291,7 +291,7 @@ int main(int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -328,8 +328,8 @@ int main(int argc, char **argv)
 	}
 	if (n_outputs == 0) {	/* Generate default -Q setting (all) */
 		n_outputs = 5 - Ctrl->Z.active;
-		for (i = 0; i < 2; i++) output_choice[i] = i;
-		for (i = 2; i < n_outputs; i++) output_choice[i] = i + Ctrl->Z.active;
+		for (i = 0; i < 2; i++) output_choice[i] = (int)i;
+		for (i = 2; i < n_outputs; i++) output_choice[i] = (int)i + Ctrl->Z.active;
 
 	}
 
@@ -350,30 +350,30 @@ int main(int argc, char **argv)
 
         data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 
-	if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
+	if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (buffer, BUFSIZ, fp);
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, i);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, i);
 			exit (EXIT_FAILURE);
 
 		}
 		i++;
                 if (ndata == n_alloc) {
-                        n_alloc += GMT_CHUNK;
+                        n_alloc <<= 1;
                         data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
                 }
 		if (!Ctrl->S.active) {
-			data[ndata].val[0] = in[0];
-			data[ndata].val[1] = in[1];
-			if (!Ctrl->Z.active) data[ndata].val[2] = in[2];
+			data[ndata].val[0] = in[GMT_X];
+			data[ndata].val[1] = in[GMT_Y];
+			if (!Ctrl->Z.active) data[ndata].val[2] = in[GMT_Z];
 			if (ndata) {
 				dy = (data[ndata].val[1] - data[ndata-1].val[1]);
 				dx = (data[ndata].val[0] - data[ndata-1].val[0]);
 				if (Ctrl->M.active) {
 					dy *= project_info.KM_PR_DEG;
-					dx *= (project_info.KM_PR_DEG * cos (0.5 * (data[ndata].val[1] + data[ndata-1].val[1]) * D2R) );
+					dx *= (project_info.KM_PR_DEG * cosd (0.5 * (data[ndata].val[1] + data[ndata-1].val[1])));
 				}
 				if (dy == 0.0 && dx == 0.0) {
 					data[ndata].val[3] = data[ndata-1].val[3];
@@ -390,9 +390,9 @@ int main(int argc, char **argv)
 			ndata++;
 		}
 		else {
-			data[ndata].val[0] = in[0];
-			data[ndata].val[1] = in[1];
-			data[ndata].val[2] = in[2];
+			data[ndata].val[0] = in[GMT_X];
+			data[ndata].val[1] = in[GMT_Y];
+			data[ndata].val[2] = in[GMT_Z];
 			data[ndata].val[3] = in[3];
 			data[ndata].val[4] = D2R * (90.0 - in[4]);	/* Angles are stored as CCW angles in radians */
 			ndata++;
@@ -486,14 +486,14 @@ int main(int argc, char **argv)
 					/* If (Ctrl->N.name) we write separate files, else GMT_stdout with > marks:  */
 
 					if (Ctrl->N.name) {
-						sprintf(filename, "%s.profile%d", Ctrl->N.name, nprofiles);
+						sprintf(filename, "%s.profile%ld", Ctrl->N.name, nprofiles);
 						if ( (fpout = GMT_fopen(filename, GMT_io.w_mode)) == NULL) {
 							fprintf(stderr,"%s:  Cannot create %s\n", GMT_program, argv[i]);
 							exit (EXIT_FAILURE);
 						}
 					}
 					else
-						fprintf (GMT_stdout, "> Start profile # %d\n", nprofiles);
+						fprintf (GMT_stdout, "> Start profile # %ld\n", nprofiles);
 
 					for (i = begin; i < end; i++) {
 						for (j = 0; j < n_outputs; j++) {	/* Remember to convert CCW angles back to azimuths */
@@ -504,7 +504,7 @@ int main(int argc, char **argv)
 
 					if (fpout != GMT_stdout) {
 						GMT_fclose (fpout);
-						if (gmtdefs.verbose) fprintf (stderr,"%s: Wrote %d points to file %s\n", GMT_program, end-begin, filename);
+						if (gmtdefs.verbose) fprintf (stderr,"%s: Wrote %ld points to file %s\n", GMT_program, end-begin, filename);
 					}
 				}
 			}
@@ -514,7 +514,7 @@ int main(int argc, char **argv)
 
 	/* Get here when all profiles have been found and written.  */
 
-	if (gmtdefs.verbose) fprintf(stderr,"%s:  Split %d data into %d files.\n", GMT_program, ndata, nprofiles);
+	if (gmtdefs.verbose) fprintf(stderr,"%s:  Split %ld data into %ld files.\n", GMT_program, ndata, nprofiles);
 	GMT_free ((void *)data);
 
 	Free_Splitxyz_Ctrl (Ctrl);	/* Deallocate control structure */
@@ -525,11 +525,11 @@ int main(int argc, char **argv)
 }
 
 
-void filterz (struct DATA *data, int ndata, double z_filter, double *fwork, int hilow)
+void filterz (struct DATA *data, GMT_LONG ndata, double z_filter, double *fwork, GMT_LONG hilow)
 {
 
 	double	half_width, sum, dt, tmp;
-	int	i, j, k, istart, istop;
+	GMT_LONG	i, j, k, istart, istop;
 
 	half_width = 0.5 * z_filter;
 	sum = 0.0;
@@ -552,7 +552,7 @@ void filterz (struct DATA *data, int ndata, double z_filter, double *fwork, int
 		sum = 0.0;
 		data[j].w = 0.0;
 		for(i = istart; i < istop; i++) {
-			k = (int)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
+			k = (GMT_LONG)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
 			sum += fwork[k];
 			data[j].w += (data[i].val[2] * fwork[k]);
 		}
@@ -570,7 +570,7 @@ void filterz (struct DATA *data, int ndata, double z_filter, double *fwork, int
 
 void filterxy_setup (double *fwork)
 {
-	int	i;
+	GMT_LONG	i;
 	double	tmp, sum = 0.0;
 
 	tmp = M_PI / SPLITXYZ_F_RES;
@@ -584,9 +584,9 @@ void filterxy_setup (double *fwork)
 	return;
 }
 
-void filterxy (struct DATA *data, int begin, int end, double xy_filter, double *fwork, int hilow)
+void filterxy (struct DATA *data, GMT_LONG begin, GMT_LONG end, double xy_filter, double *fwork, GMT_LONG hilow)
 {
-	int	i, j, k, istart, istop;
+	GMT_LONG	i, j, k, istart, istop;
 	double	half_width, dt, sum;
 
 	half_width = 0.5 * fabs(xy_filter);
@@ -601,7 +601,7 @@ void filterxy (struct DATA *data, int begin, int end, double xy_filter, double *
 		sum = 0.0;
 		data[j].w = 0.0;
 		for(i = istart; i < istop; i++) {
-			k = (int)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
+			k = (GMT_LONG)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
 			sum += fwork[k];
 			data[j].w += (data[i].val[0] * fwork[k]);
 		}
@@ -624,7 +624,7 @@ void filterxy (struct DATA *data, int begin, int end, double xy_filter, double *
 		sum = 0.0;
 		data[j].w = 0.0;
 		for(i = istart; i < istop; i++) {
-			k = (int)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
+			k = (GMT_LONG)floor(dt*fabs(data[i].val[3] - data[j].val[3]));
 			sum += fwork[k];
 			data[j].w += (data[i].val[1] * fwork[k]);
 		}
@@ -644,7 +644,7 @@ void filterxy (struct DATA *data, int begin, int end, double xy_filter, double *
 void *New_Splitxyz_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct SPLITXYZ_CTRL *C;
 	
-	C = (struct SPLITXYZ_CTRL *) GMT_memory (VNULL, 1, sizeof (struct SPLITXYZ_CTRL), "New_Splitxyz_Ctrl");
+	C = (struct SPLITXYZ_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct SPLITXYZ_CTRL), "New_Splitxyz_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
         C->A.azimuth = 90.0;
