@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: surface.c,v 1.46 2007/10/01 20:14:53 guru Exp $
+ *	$Id: surface.c,v 1.65 2009/07/14 05:10:48 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -79,6 +79,7 @@
  * Version:	4
  * Version:	4.1: 14-SEP-2005 by PW, Added enhanced -I
  * Version:	4.1.2 PW: Minimized (but not eliminated) global variables
+ * 64-bit Compliant: Yes
  *
  *
  */
@@ -106,11 +107,11 @@ struct SURFACE_CTRL {
 		BOOLEAN active;
 		char *low, *high;
 		double min, max;
-		int lmode, hmode;
+		GMT_LONG lmode, hmode;
 	} L;
 	struct N {	/* -N<max_iterations> */
 		BOOLEAN active;
-		int value;
+		GMT_LONG value;
 	} N;
 	struct Q {	/* -Q */
 		BOOLEAN active;
@@ -130,13 +131,13 @@ struct SURFACE_CTRL {
 	} Z;
 };
 
-#define SURFACE_OUTSIDE 2000000000	/* Index number indicating data is outside usable area */
+#define SURFACE_OUTSIDE LONG_MAX	/* Index number indicating data is outside usable area */
 
 struct SURFACE_DATA {	/* Data point and index to node it currently constrains  */
 	float x;
 	float y;
 	float z;
-	int index;
+	GMT_LONG index;
 };
 
 struct SURFACE_BRIGGS {		/* Coefficients in Taylor series for Laplacian(z) a la I. C. Briggs (1974)  */
@@ -144,7 +145,7 @@ struct SURFACE_BRIGGS {		/* Coefficients in Taylor series for Laplacian(z) a la
 };
 
 struct SURFACE_GLOBAL {		/* Things needed inside compare function must be global for now */
-	int block_ny;			/* Number of nodes in y-dir for a given grid factor */
+	GMT_LONG block_ny;			/* Number of nodes in y-dir for a given grid factor */
 	double grid_xinc, grid_yinc;	/* size of each grid cell for a given grid factor */
 	double x_min, y_min;		/* Lower left corner of grid */
 } GMT_Surface_Global;
@@ -155,27 +156,29 @@ struct SURFACE_INFO {	/* Control structure for surface setup and execution */
 					 * I means just interpolate from larger grid */
 	char format[BUFSIZ];
 	char *low_file, *high_file;	/* Pointers to grids with low and high limits, if selected */
-	int n_alloc;
-	int npoints;			/* Number of data points */
-	int nx;				/* Number of nodes in x-dir. */
-	int ny;				/* Number of nodes in y-dir. (Final grid) */
-	int mx;
-	int my;
-	int ij_sw_corner, ij_se_corner,ij_nw_corner, ij_ne_corner;
-	int block_nx;			/* Number of nodes in x-dir for a given grid factor */
-	int block_ny;			/* Number of nodes in y-dir for a given grid factor */
-	int max_iterations;		/* Max iter per call to iterate */
-	int total_iterations;
-	int grid, old_grid;		/* Node spacings  */
-	int grid_east;
-	int n_fact;			/* Number of factors in common (ny-1, nx-1) */
-	int factors[32];		/* Array of common factors */
-	int long_verbose;
-	int n_empty;			/* No of unconstrained nodes at initialization  */
-	int set_low;			/* 0 unconstrained,1 = by min data value, 2 = by user value */
-	int set_high;			/* 0 unconstrained,1 = by max data value, 2 = by user value */
-	int constrained;		/* TRUE if set_low or set_high is TRUE */
-	int offset[25][12];		/* Indices of 12 nearby points in 25 cases of edge conditions  */
+	GMT_LONG grid, old_grid;	/* Node spacings  */
+	GMT_LONG n_fact;		/* Number of factors in common (ny-1, nx-1) */
+	GMT_LONG factors[32];		/* Array of common factors */
+	GMT_LONG long_verbose;
+	GMT_LONG set_low;		/* 0 unconstrained,1 = by min data value, 2 = by user value */
+	GMT_LONG set_high;		/* 0 unconstrained,1 = by max data value, 2 = by user value */
+	GMT_LONG n_alloc;
+	GMT_LONG npoints;		/* Number of data points */
+	GMT_LONG ij_sw_corner, ij_se_corner,ij_nw_corner, ij_ne_corner;
+	GMT_LONG n_empty;		/* No of unconstrained nodes at initialization  */
+	GMT_LONG nx;			/* Number of nodes in x-dir. */
+	GMT_LONG ny;			/* Number of nodes in y-dir. (Final grid) */
+	GMT_LONG nxny;			/* Total number of grid nodes without boundaries  */
+	GMT_LONG mx;
+	GMT_LONG my;
+	GMT_LONG mxmy;			/* Total number of grid nodes with boundaries  */
+	GMT_LONG block_nx;		/* Number of nodes in x-dir for a given grid factor */
+	GMT_LONG block_ny;		/* Number of nodes in y-dir for a given grid factor */
+	GMT_LONG max_iterations;	/* Max iter per call to iterate */
+	GMT_LONG total_iterations;
+	GMT_LONG grid_east;
+	GMT_LONG offset[25][12];	/* Indices of 12 nearby points in 25 cases of edge conditions  */
+	BOOLEAN constrained;		/* TRUE if set_low or set_high is TRUE */
 	float *lower, *upper;		/* arrays for minmax values, if set */
 	float *u;			/* Pointer to grid array */
 	double low_limit, high_limit;	/* Constrains on range of solution */
@@ -204,34 +207,34 @@ struct SURFACE_INFO {	/* Control structure for surface setup and execution */
 };
 
 struct SURFACE_SUGGESTION {	/* Used to find top ten list of faster grid dimensions  */
-	int nx;
-	int ny;
+	GMT_LONG nx;
+	GMT_LONG ny;
 	double factor;	/* Speed up by a factor of factor  */
 };
 
 int compare_points (const void *point_1v, const void *point_2v);
-int gcd_euclid (int a, int b);	/* Finds the greatest common divisor  */
-int get_prime_factors (int n, int *f);
-int iterate (struct SURFACE_INFO *C, int mode);
+GMT_LONG gcd_euclid (GMT_LONG a, GMT_LONG b);	/* Finds the greatest common divisor  */
+GMT_LONG get_prime_factors (GMT_LONG n, GMT_LONG *f);
+GMT_LONG iterate (struct SURFACE_INFO *C, GMT_LONG mode);
 void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform);
 
 int main (int argc, char **argv)
 {
 	char	modifier;
 
-	int	i, error = FALSE, n_files = 0;
+	GMT_LONG	i, error = FALSE, n_files = 0, key;
 
 	FILE	*fp_in = NULL;	/* File pointer  */
 	
 	struct SURFACE_INFO C;
 	struct SURFACE_CTRL *Ctrl;
 	
-	void suggest_sizes_for_surface (int factors[], int nx, int ny);
+	void suggest_sizes_for_surface (GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny);
 	void set_grid_parameters (struct SURFACE_INFO *C);
 	void read_data (FILE *fp, struct SURFACE_INFO *C);
 	void throw_away_unusables (struct SURFACE_INFO *C);
 	void remove_planar_trend (struct SURFACE_INFO *C);
-	void rescale_z_values (struct SURFACE_INFO *C);
+	GMT_LONG rescale_z_values (struct SURFACE_INFO *C);
 	void smart_divide (struct SURFACE_INFO *C);
 	void set_offset (struct SURFACE_INFO *C);
 	void set_index (struct SURFACE_INFO *C);
@@ -397,6 +400,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\tRequired arguments to surface:\n");
 		fprintf (stderr, "\t-G sets output grid file name\n");
 		GMT_inc_syntax ('I', 0);
+		fprintf (stderr, "\t\tNote that only gridline registration can be used.\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-A<aspect_ratio>  = 1.0  by default which gives an isotropic solution.\n");
@@ -442,6 +446,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, NULL, &Ctrl->I.active);
+
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
@@ -480,7 +486,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	load_parameters (&C, Ctrl);	/* Pass parameters from parsing control to surface INFO structure */
@@ -488,26 +494,34 @@ int main (int argc, char **argv)
 	GMT_RI_prepare (&C.h);	/* Ensure -R -I consistency and set nx, ny */
 	GMT_err_fail (GMT_grd_RI_verify (&C.h, 1), Ctrl->G.file);
 
+	if (C.h.nx < 4 || C.h.ny < 4) {
+		fprintf (stderr, "%s: GMT ERROR.  Grid must have at least 4 nodes in each direction (you have %d by %d) - abort.\n", GMT_program, C.h.nx, C.h.ny);
+		exit (EXIT_FAILURE);
+	}
+	
 	C.relax_old = 1.0 - C.relax_new;
 
-	C.nx = C.h.nx;
-	C.ny = C.h.ny;
+	C.nx = (GMT_LONG)C.h.nx;
+	C.ny = (GMT_LONG)C.h.ny;
+	C.nxny = GMT_get_nm (C.nx, C.ny);
+	
 	C.mx = C.nx + 4;
 	C.my = C.ny + 4;
+	C.mxmy = GMT_get_nm (C.mx, C.my);
 	C.r_xinc = 1.0 / C.h.x_inc;
 	C.r_yinc = 1.0 / C.h.y_inc;
 	GMT_Surface_Global.x_min = C.h.x_min;
 	GMT_Surface_Global.y_min = C.h.y_min;
 
 	/* New stuff here for v4.3:  Check out the grid dimensions:  */
-	C.grid = gcd_euclid (C.nx-1, C.ny-1);
+	C.grid = (int)gcd_euclid (C.nx-1, C.ny-1);
 
 	if (gmtdefs.verbose || Ctrl->Q.active) {
-		sprintf (C.format, "%s: W: %s E: %s S: %s N: %s nx: %%d ny: %%d\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+		sprintf (C.format, "%s: W: %s E: %s S: %s N: %s nx: %%ld ny: %%ld\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, C.format, C.h.x_min, C.h.x_max, C.h.y_min, C.h.y_max, C.nx-1, C.ny-1);
 	}
 	if (C.grid == 1 && gmtdefs.verbose) fprintf (stderr, "%s:  WARNING:  Your grid dimensions are mutually prime.\n", GMT_program);
-	if ((C.grid == 1 && gmtdefs.verbose) || Ctrl->Q.active) suggest_sizes_for_surface (C.factors, C.nx-1, C.ny-1);
+	if ((C.grid == 1 && gmtdefs.verbose) || Ctrl->Q.active) suggest_sizes_for_surface (C.factors, (int)C.nx-1, (int)C.ny-1);
 	if (Ctrl->Q.active) exit (EXIT_SUCCESS);
 
 	/* New idea: set grid = 1, read data, setting index.  Then throw
@@ -525,13 +539,28 @@ int main (int argc, char **argv)
 	read_data (fp_in, &C);
 	throw_away_unusables (&C);
 	remove_planar_trend (&C);
-	rescale_z_values (&C);
+	key = rescale_z_values (&C);
+	
+	if (key == 1) {	/* Data lies exactly on a plane; just return the plane grid */
+		GMT_free ((void *) C.data);
+		C.u = (float *) GMT_memory (VNULL, (size_t)C.mxmy, sizeof(float), GMT_program);
+		C.ij_sw_corner = 2 * C.my + 2;			/*  Corners of array of actual data  */
+		replace_planar_trend (&C);
+		write_output (&C, Ctrl->G.file);
+		GMT_free ((void *) C.u);
+		Free_Surface_Ctrl (Ctrl);	/* Deallocate control structure */
+
+		GMT_end (argc, argv);
+
+		exit (EXIT_SUCCESS);
+	}
+	
 	load_constraints (&C, TRUE);
 
 	/* Set up factors and reset grid to first value  */
 
-	C.grid = gcd_euclid (C.nx-1, C.ny-1);
-	C.n_fact = get_prime_factors (C.grid, C.factors);
+	C.grid = (int)gcd_euclid (C.nx-1, C.ny-1);
+	C.n_fact = get_prime_factors ((GMT_LONG)C.grid, C.factors);
 	set_grid_parameters (&C);
 	while (C.block_nx < 4 || C.block_ny < 4) {
 		smart_divide (&C);
@@ -544,8 +573,8 @@ int main (int argc, char **argv)
 	/* Allocate more space  */
 
 	C.briggs = (struct SURFACE_BRIGGS *) GMT_memory (VNULL, (size_t)C.npoints, sizeof(struct SURFACE_BRIGGS), GMT_program);
-	C.iu = (char *) GMT_memory (VNULL, (size_t)(C.mx * C.my), sizeof(char), GMT_program);
-	C.u = (float *) GMT_memory (VNULL, (size_t)(C.mx * C.my), sizeof(float), GMT_program);
+	C.iu = (char *) GMT_memory (VNULL, (size_t)C.mxmy, sizeof(char), GMT_program);
+	C.u = (float *) GMT_memory (VNULL, (size_t)C.mxmy, sizeof(float), GMT_program);
 
 	if (C.radius > 0) initialize_grid (&C); /* Fill in nodes with a weighted avg in a search radius  */
 
@@ -628,15 +657,15 @@ void set_coefficients (struct SURFACE_INFO *C)
 
 	C->ij_sw_corner = 2 * C->my + 2;			/*  Corners of array of actual data  */
 	C->ij_se_corner = C->ij_sw_corner + (C->nx - 1) * C->my;
-	C->ij_nw_corner = C->ij_sw_corner + (C->ny - 1);
-	C->ij_ne_corner = C->ij_se_corner + (C->ny - 1);
+	C->ij_nw_corner = C->ij_sw_corner + C->ny - 1;
+	C->ij_ne_corner = C->ij_se_corner + C->ny - 1;
 
 }
 
 void set_offset (struct SURFACE_INFO *C)
 {
-	int	add_w[5], add_e[5], add_s[5], add_n[5], add_w2[5], add_e2[5], add_s2[5], add_n2[5];
-	int	i, j, kase;
+	GMT_LONG	add_w[5], add_e[5], add_s[5], add_n[5], add_w2[5], add_e2[5], add_s2[5], add_n2[5];
+	GMT_LONG	i, j, kase;
 
 	add_w[0] = -C->my; add_w[1] = add_w[2] = add_w[3] = add_w[4] = -C->grid_east;
 	add_w2[0] = -2 * C->my;  add_w2[1] = -C->my - C->grid_east;  add_w2[2] = add_w2[3] = add_w2[4] = -2 * C->grid_east;
@@ -674,8 +703,8 @@ void fill_in_forecast (struct SURFACE_INFO *C) {
 	   after grid is divided.   
 	 */
 
-	int i, j, ii, jj, index_0, index_1, index_2, index_3;
-	int index_new;
+	GMT_LONG i, j, ii, jj, index_0, index_1, index_2, index_3;
+	GMT_LONG index_new;
 	char *iu;
 	double delta_x, delta_y, a0, a1, a2, a3;
 	double old_size;
@@ -754,7 +783,7 @@ int compare_points (const void *point_1v, const void *point_2v)
 		    Sorts on index first, then on radius to node corresponding to index, so that index
 		    goes from low to high, and so does radius.
 		*/
-	int block_i, block_j, index_1, index_2;
+	GMT_LONG block_i, block_j, index_1, index_2;
 	double x0, y0, dist_1, dist_2;
 	struct SURFACE_DATA *point_1, *point_2;
 	
@@ -794,11 +823,11 @@ void set_index (struct SURFACE_INFO *C) {
 		/* recomputes data[k].index for new value of grid,
 		   sorts data on index and radii, and throws away
 		   data which are now outside the usable limits. */
-	int i, j, k, k_skipped = 0;
+	GMT_LONG i, j, k, k_skipped = 0;
 
 	for (k = 0; k < C->npoints; k++) {
-		i = (int)floor(((C->data[k].x-C->h.x_min)*C->r_grid_xinc) + 0.5);
-		j = (int)floor(((C->data[k].y-C->h.y_min)*C->r_grid_yinc) + 0.5);
+		i = (GMT_LONG)floor(((C->data[k].x-C->h.x_min)*C->r_grid_xinc) + 0.5);
+		j = (GMT_LONG)floor(((C->data[k].y-C->h.y_min)*C->r_grid_yinc) + 0.5);
 		if (i < 0 || i >= C->block_nx || j < 0 || j >= C->block_ny) {
 			C->data[k].index = SURFACE_OUTSIDE;
 			k_skipped++;
@@ -814,7 +843,7 @@ void set_index (struct SURFACE_INFO *C) {
 }
 
 void find_nearest_point (struct SURFACE_INFO *C) {
-	int i, j, ij_v2, k, last_index, block_i, block_j, iu_index, briggs_index;
+	GMT_LONG i, j, ij_v2, k, last_index, block_i, block_j, iu_index, briggs_index;
 	double x0, y0, dx, dy, xys, xy1, btemp;
 	double b0, b1, b2, b3, b4, b5;
 	float z_at_node, *u;
@@ -912,15 +941,13 @@ void initialize_grid (struct SURFACE_INFO *C)
 	 * For the initial gridsize, compute weighted averages of data inside the search radius
 	 * and assign the values to u[i,j] where i,j are multiples of gridsize.
 	 */
-	int	irad, jrad, i, j, imin, imax, jmin, jmax, index_1, index_2, k, ki, kj, k_index;
-	char *iu;
+	GMT_LONG	irad, jrad, i, j, imin, imax, jmin, jmax, index_1, index_2, k, ki, kj, k_index;
 	double	r, rfact, sum_w, sum_zw, weight, x0, y0;
 	float *u;
 	u = C->u;
-	iu = C->iu;
 
-	 irad = (int)ceil(C->radius/C->grid_xinc);
-	 jrad = (int)ceil(C->radius/C->grid_yinc);
+	 irad = (GMT_LONG)ceil(C->radius/C->grid_xinc);
+	 jrad = (GMT_LONG)ceil(C->radius/C->grid_yinc);
 	 rfact = -4.5/(C->radius*C->radius);
 	 for (i = 0; i < C->block_nx; i ++ ) {
 	 	x0 = C->h.x_min + i*C->grid_xinc;
@@ -970,10 +997,10 @@ void new_initialize_grid (struct SURFACE_INFO *C)
 	 * For the initial gridsize, load constrained nodes with weighted avg of their data;
 	 * and then do something with the unconstrained ones.
 	 */
-	 int	k, k_index, u_index, block_i, block_j;
-	 char *iu;
-	 double	sum_w, sum_zw, weight, x0, y0, dx, dy, dx_scale, dy_scale;
-	 float *u;
+	GMT_LONG	k, k_index, u_index, block_i, block_j;
+	char *iu;
+	double	sum_w, sum_zw, weight, x0, y0, dx, dy, dx_scale, dy_scale;
+	float *u;
 	u = C->u;
 	iu = C->iu;
 
@@ -1013,9 +1040,10 @@ void new_initialize_grid (struct SURFACE_INFO *C)
 
 void read_data (FILE *fp_in, struct SURFACE_INFO *C)
 {
-	int	i, j, k, kmax = 0, kmin = 0, n_fields, n_expected_fields;
+	GMT_LONG	i, j, k, kmax = 0, kmin = 0;
+	GMT_LONG n_fields, n_expected_fields;
 	double	*in, zmin = DBL_MAX, zmax = -DBL_MAX;
-	char	buffer[BUFSIZ];
+	char	buffer[BUFSIZ], *not_used = NULL;
 
 	C->data = (struct SURFACE_DATA *) GMT_memory (VNULL, (size_t)C->n_alloc, sizeof(struct SURFACE_DATA), GMT_program);
 
@@ -1024,32 +1052,32 @@ void read_data (FILE *fp_in, struct SURFACE_INFO *C)
 	k = 0;
 	C->z_mean = 0;
 	n_expected_fields = (GMT_io.binary[GMT_IN]) ? GMT_io.ncol[GMT_IN] : 3;
-	if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp_in);
+	if (!GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (buffer, BUFSIZ, fp_in);
 
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, k);
+			fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, k);
 			exit (EXIT_FAILURE);
 		}
 
-		if (GMT_is_dnan (in[2])) continue;
+		if (GMT_is_dnan (in[GMT_Z])) continue;
 
-		i = (int)floor(((in[0]-C->h.x_min)*C->r_grid_xinc) + 0.5);
+		i = (GMT_LONG)floor(((in[GMT_X]-C->h.x_min)*C->r_grid_xinc) + 0.5);
 		if (i < 0 || i >= C->block_nx) continue;
-		j = (int)floor(((in[1]-C->h.y_min)*C->r_grid_yinc) + 0.5);
+		j = (GMT_LONG)floor(((in[GMT_Y]-C->h.y_min)*C->r_grid_yinc) + 0.5);
 		if (j < 0 || j >= C->block_ny) continue;
 
 		C->data[k].index = i * C->block_ny + j;
-		C->data[k].x = (float)in[0];
-		C->data[k].y = (float)in[1];
-		C->data[k].z = (float)in[2];
-		if (zmin > in[2]) zmin = in[2], kmin = k;
-		if (zmax < in[2]) zmax = in[2], kmax = k;
+		C->data[k].x = (float)in[GMT_X];
+		C->data[k].y = (float)in[GMT_Y];
+		C->data[k].z = (float)in[GMT_Z];
+		if (zmin > in[GMT_Z]) zmin = in[GMT_Z], kmin = k;
+		if (zmax < in[GMT_Z]) zmax = in[GMT_Z], kmax = k;
 		k++;
-		C->z_mean += in[2];
-		if (k == C->n_alloc) {
-			C->n_alloc += GMT_CHUNK;
+		C->z_mean += in[GMT_Z];
+		if (k == (GMT_LONG)C->n_alloc) {
+			C->n_alloc <<= 1;
 			C->data = (struct SURFACE_DATA *) GMT_memory ((void *)C->data, (size_t)C->n_alloc, sizeof(struct SURFACE_DATA), GMT_program);
 		}
 	}
@@ -1090,7 +1118,7 @@ void read_data (FILE *fp_in, struct SURFACE_INFO *C)
 
 void write_output (struct SURFACE_INFO *C, char *grdfile)
 {	/* Uses v.2.0 netCDF grd format - hence need to transpose original grid to be GMT compatible.  This will be rewritten, eventually */
-	int	index, i, j, k;
+	GMT_LONG	index, i, j, k;
 	float *u, *v2;
 	u = C->u;
 
@@ -1098,7 +1126,7 @@ void write_output (struct SURFACE_INFO *C, char *grdfile)
 		
 	strcpy (C->h.title, GMT_program);
 
-	v2 = (float *) GMT_memory (VNULL, (size_t)(C->nx * C->ny), sizeof (float), GMT_program);
+	v2 = (float *) GMT_memory (VNULL, (size_t)C->nxny, sizeof (float), GMT_program);
 	index = C->ij_sw_corner;
 	for (i = 0; i < C->nx; i++, index += C->my) {
 		for (j = 0; j < C->ny; j++) {
@@ -1115,12 +1143,12 @@ void write_output (struct SURFACE_INFO *C, char *grdfile)
 	if (C->set_high) GMT_free ((void *)C->upper);
 }
 
-int iterate (struct SURFACE_INFO *C, int mode)
+GMT_LONG iterate (struct SURFACE_INFO *C, GMT_LONG mode)
 {
 
-	int	i, j, k, ij, kase, briggs_index, ij_v2;
-	int	x_case, y_case, x_w_case, x_e_case, y_s_case, y_n_case;
-	int	iteration_count = 0;
+	GMT_LONG	i, j, k, ij, kase, briggs_index, ij_v2;
+	GMT_LONG	x_case, y_case, x_w_case, x_e_case, y_s_case, y_n_case;
+	GMT_LONG	iteration_count = 0;
 	char *iu;
 
 	double	current_limit = C->converge_limit / C->grid;
@@ -1134,7 +1162,7 @@ int iterate (struct SURFACE_INFO *C, int mode)
 	double	y_0_const = 4 * C->l_epsilon * (1.0 - C->boundary_tension) / y_denom;
 	double	y_1_const = (C->boundary_tension - 2 * C->l_epsilon * (1.0 - C->boundary_tension) ) / y_denom;
 
-	sprintf(C->format,"%s: %%4d\t%%c\t%%8d\t%s\t%s\t%%10d\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format);
+	sprintf(C->format,"%s: %%4ld\t%%c\t%%8ld\t%s\t%s\t%%10ld\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format);
 
 	u = C->u;
 	iu = C->iu;
@@ -1148,8 +1176,6 @@ int iterate (struct SURFACE_INFO *C, int mode)
 		/* First set d2[]/dn2 = 0 along edges:  */
 		/* New experiment : (1-T)d2[]/dn2 + Td[]/dn = 0  */
 
-
-
 		for (i = 0; i < C->nx; i += C->grid) {
 			/* set d2[]/dy2 = 0 on south side:  */
 			ij = C->ij_sw_corner + i * C->my;
@@ -1358,12 +1384,12 @@ int iterate (struct SURFACE_INFO *C, int mode)
 	if (gmtdefs.verbose && !C->long_verbose) fprintf (stderr, C->format,
 		C->grid, C->mode_type[mode], iteration_count, max_change, current_limit, C->total_iterations);
 
-	return(iteration_count);
+	return ((int)iteration_count);
 }
 
 void check_errors (struct SURFACE_INFO *C) {
 
-	int	i, j, k, ij, n_nodes, move_over[12];
+	GMT_LONG	i, j, k, ij, move_over[12];
 	char *iu;	/* move_over = C->offset[kase][12], but grid = 1 so move_over is easy  */
 
 	double	x0, y0, dx, dy, mean_error, mean_squared_error, z_est, z_err, curvature, c;
@@ -1492,7 +1518,6 @@ void check_errors (struct SURFACE_INFO *C) {
 	 mean_squared_error = sqrt( mean_squared_error / C->npoints);
 	 
 	 curvature = 0.0;
-	 n_nodes = C->nx * C->ny;
 	 
 	 for (i = 0; i < C->nx; i++) {
 	 	for (j = 0; j < C->ny; j++) {
@@ -1504,14 +1529,14 @@ void check_errors (struct SURFACE_INFO *C) {
 	}
 
 	 fprintf (stderr, "%s: Fit info: N data points  N nodes\tmean error\trms error\tcurvature\n", GMT_program);
-	 sprintf (C->format,"%s:\t%%8d\t%%8d\t%s\t%s\t%s\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-	 fprintf (stderr, C->format, C->npoints, n_nodes, mean_error, mean_squared_error, curvature);
+	 sprintf (C->format,"%s:\t%%8ld\t%%8ld\t%s\t%s\t%s\n", GMT_program, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+	 fprintf (stderr, C->format, C->npoints, C->nxny, mean_error, mean_squared_error, curvature);
  }
 
 void remove_planar_trend (struct SURFACE_INFO *C)
 {
 
-	int	i;
+	GMT_LONG	i;
 	double	a, b, c, d, xx, yy, zz;
 	double	sx, sy, sz, sxx, sxy, sxz, syy, syz;
 
@@ -1559,7 +1584,7 @@ void remove_planar_trend (struct SURFACE_INFO *C)
 
 void replace_planar_trend (struct SURFACE_INFO *C)
 {
-	int	i, j, ij;
+	GMT_LONG	i, j, ij;
 	float *u;
 	u = C->u;
 
@@ -1584,7 +1609,7 @@ void throw_away_unusables (struct SURFACE_INFO *C)
 		coefficients, eliminating calls to temp file.
 	*/
 
-	int	last_index, n_outside, k;
+	GMT_LONG	last_index, n_outside, k;
 
 	/* Sort the data  */
 
@@ -1610,14 +1635,14 @@ void throw_away_unusables (struct SURFACE_INFO *C)
 	C->npoints -= n_outside;
 	C->data = (struct SURFACE_DATA *) GMT_memory ((void *)C->data, (size_t)C->npoints, sizeof(struct SURFACE_DATA), GMT_program);
 	if (gmtdefs.verbose && (n_outside)) {
-		fprintf (stderr, "%s: %d unusable points were supplied; these will be ignored.\n", GMT_program, n_outside);
+		fprintf (stderr, "%s: %ld unusable points were supplied; these will be ignored.\n", GMT_program, n_outside);
 		fprintf (stderr, "%s: You should have pre-processed the data with block-mean, -median, or -mode.\n", GMT_program);
 	}
 }
 
-void rescale_z_values (struct SURFACE_INFO *C)
+GMT_LONG rescale_z_values (struct SURFACE_INFO *C)
 {
-	int	i;
+	GMT_LONG	i;
 	double	ssz = 0.0;
 
 	for (i = 0; i < C->npoints; i++) ssz += (C->data[i].z * C->data[i].z);
@@ -1626,9 +1651,10 @@ void rescale_z_values (struct SURFACE_INFO *C)
 
 	C->z_scale = sqrt (ssz / C->npoints);
 
-	if (C->z_scale == 0.0) {
-		fprintf (stderr, "%s: WARNING: Input data lie exactly on a plane - no solution (aborting).\n", GMT_program);
+	if (C->z_scale < GMT_CONV_LIMIT) {
+		fprintf (stderr, "%s: WARNING: Input data lie exactly on a plane.\n", GMT_program);
 		C->r_z_scale = C->z_scale = 1.0;
+		return (1);	/* Flag to tell the main to just write out the plane */
 	}
 	else
 		C->r_z_scale = 1.0 / C->z_scale;
@@ -1636,20 +1662,21 @@ void rescale_z_values (struct SURFACE_INFO *C)
 	for (i = 0; i < C->npoints; i++) C->data[i].z *= (float)C->r_z_scale;
 
 	if (C->converge_limit == 0.0) C->converge_limit = 0.001 * C->z_scale; /* i.e., 1 ppt of L2 scale */
+	return (0);
 }
 
 void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 {
-	int i, j, ij;
+	GMT_LONG i, j, ij;
 	double yy;
 	struct GRD_HEADER hdr;
 
 	/* Load lower/upper limits, verify range, deplane, and rescale */
 
 	if (C->set_low > 0) {
-		C->lower = (float *) GMT_memory (VNULL, (size_t)(C->nx * C->ny), sizeof (float), GMT_program);
+		C->lower = (float *) GMT_memory (VNULL, (size_t)C->nxny, sizeof (float), GMT_program);
 		if (C->set_low < 3)
-			for (i = 0; i < C->nx * C->ny; i++) C->lower[i] = (float)C->low_limit;
+			for (i = 0; i < C->nxny; i++) C->lower[i] = (float)C->low_limit;
 		else {
 			GMT_err_fail (GMT_read_grd_info (C->low_file, &hdr), C->low_file);
 			if (hdr.nx != C->nx || hdr.ny != C->ny) {
@@ -1660,7 +1687,7 @@ void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 		}
 		if (transform) {
 			for (j = ij = 0; j < C->ny; j++) {
-				yy = C->ny - j - 1;
+				yy = (double)(C->ny - j - 1);
 				for (i = 0; i < C->nx; i++, ij++) {
 					if (GMT_is_fnan (C->lower[ij])) continue;
 					C->lower[ij] -= (float)(C->plane_c0 + C->plane_c1 * i + C->plane_c2 * yy);
@@ -1671,9 +1698,9 @@ void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 		C->constrained = TRUE;
 	}
 	if (C->set_high > 0) {
-		C->upper = (float *) GMT_memory (VNULL, (size_t)(C->nx * C->ny), sizeof (float), GMT_program);
+		C->upper = (float *) GMT_memory (VNULL, (size_t)C->nxny, sizeof (float), GMT_program);
 		if (C->set_high < 3)
-			for (i = 0; i < C->nx * C->ny; i++) C->upper[i] = (float)C->high_limit;
+			for (i = 0; i < C->nxny; i++) C->upper[i] = (float)C->high_limit;
 		else {
 			GMT_err_fail (GMT_read_grd_info (C->high_file, &hdr), C->high_file);
 			if (hdr.nx != C->nx || hdr.ny != C->ny) {
@@ -1684,7 +1711,7 @@ void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 		}
 		if (transform) {
 			for (j = ij = 0; j < C->ny; j++) {
-				yy = C->ny - j - 1;
+				yy = (double)(C->ny - j - 1);
 				for (i = 0; i < C->nx; i++, ij++) {
 					if (GMT_is_fnan (C->upper[ij])) continue;
 					C->upper[ij] -= (float)(C->plane_c0 + C->plane_c1 * i + C->plane_c2 * yy);
@@ -1696,7 +1723,7 @@ void load_constraints (struct SURFACE_INFO *C, BOOLEAN transform)
 	}
 }
 
-double guess_surface_time (int factors[], int nx, int ny)
+double guess_surface_time (GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny)
 {
 	/* Routine to guess a number proportional to the operations
 	 * required by surface working on a user-desired grid of
@@ -1723,21 +1750,21 @@ double guess_surface_time (int factors[], int nx, int ny)
 	 *
 	 * W. H. F. Smith, 26 Feb 1992.  */
 
-	int	gcd;		/* Current value of the gcd  */
-	int	nxg, nyg;	/* Current value of the grid dimensions  */
-	int	nfactors = 0;	/* Number of prime factors of current gcd  */
-	int	factor;		/* Currently used factor  */
+	GMT_LONG	gcd;		/* Current value of the gcd  */
+	GMT_LONG	nxg, nyg;	/* Current value of the grid dimensions  */
+	GMT_LONG	nfactors = 0;	/* Number of prime factors of current gcd  */
+	GMT_LONG	factor;		/* Currently used factor  */
 	/* Doubles are used below, even though the values will be integers,
 		because the multiplications might reach sizes of O(n**3)  */
 	double	t_sum;		/* Sum of values of T at each grid cycle  */
 	double	length;		/* Current propagation distance.  */
 
 
-	gcd = gcd_euclid(nx, ny);
+	gcd = gcd_euclid((GMT_LONG)nx, (GMT_LONG)ny);
 	if (gcd > 1) {
 		nfactors = get_prime_factors(gcd, factors);
-		nxg = nx/gcd;
-		nyg = ny/gcd;
+		nxg = (GMT_LONG)(nx/gcd);
+		nyg = (GMT_LONG)(ny/gcd);
 		if (nxg < 3 || nyg < 3) {
 			factor = factors[nfactors - 1];
 			nfactors--;
@@ -1766,7 +1793,7 @@ double guess_surface_time (int factors[], int nx, int ny)
 	return(t_sum);
 }
 
-void suggest_sizes_for_surface (int factors[], int nx, int ny)
+void suggest_sizes_for_surface (GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny)
 {
 	/* Calls guess_surface_time for a variety of trial grid
 	 * sizes, where the trials are highly composite numbers
@@ -1780,14 +1807,14 @@ void suggest_sizes_for_surface (int factors[], int nx, int ny)
 	 *
 	 * W. H. F. Smith, 26 Feb 1992.  */
 
-	double	guess_surface_time(int factors[], int nx, int ny);
+	double	guess_surface_time(GMT_LONG factors[], GMT_LONG nx, GMT_LONG ny);
 	double	users_time;	/* Time for user's nx, ny  */
 	double	current_time;	/* Time for current nxg, nyg  */
-	int	i;
-	int	nxg, nyg;	/* Guessed by this routine  */
-	int	nx2, ny2, nx3, ny3, nx5, ny5;	/* For powers  */
-	int	xstop, ystop;	/* Set to 2*nx, 2*ny  */
-	int	n_sug = 0;	/* N of suggestions found  */
+	GMT_LONG	i;
+	GMT_LONG	nxg, nyg;	/* Guessed by this routine  */
+	GMT_LONG	nx2, ny2, nx3, ny3, nx5, ny5;	/* For powers  */
+	GMT_LONG	xstop, ystop;	/* Set to 2*nx, 2*ny  */
+	GMT_LONG	n_sug = 0;	/* N of suggestions found  */
 	int	compare_sugs(const void *point_1, const void *point_2);	/* Sort suggestions decreasing  */
 	struct SURFACE_SUGGESTION *sug = NULL;
 
@@ -1826,7 +1853,7 @@ void suggest_sizes_for_surface (int factors[], int nx, int ny)
 	if (n_sug) {
 		qsort((void *)sug, (size_t)n_sug, sizeof(struct SURFACE_SUGGESTION), compare_sugs);
 		for (i = 0; i < n_sug && i < 10; i++) {
-			fprintf (stderr, "%s:  HINT:  Choosing nx = %d, ny = %d might cut run time by a factor of %.8g\n",
+			fprintf (stderr, "%s:  HINT:  Choosing nx = %ld, ny = %ld might cut run time by a factor of %.8g\n",
 				GMT_program, sug[i].nx, sug[i].ny, sug[i].factor);
 		}
 		GMT_free ((void *)sug);
@@ -1848,7 +1875,7 @@ int compare_sugs (const void *point_1, const void *point_2)
 		return(0);
 }
 
-int get_prime_factors (int n, int *f)
+GMT_LONG get_prime_factors (GMT_LONG n, GMT_LONG *f)
 {
 	/* Fills the integer array f with the prime factors of n.
 	 * Returns the number of locations filled in f, which is
@@ -1875,25 +1902,29 @@ int get_prime_factors (int n, int *f)
 	 *
 	 * W. H. F. Smith, 26 Feb 1992, after D.E. Knuth, vol. II  */
 
-	int	current_factor;	/* The factor currently being tried  */
-	int	max_factor;	/* Don't try any factors bigger than this  */
-	int	n_factors = 0;	/* Returned; one if n is prime  */
-	int	two_four_toggle = 0;	/* Used to add 2 or 4 to get next trial factor  */
-	int	ten_twenty_toggle = 0;	/* Used to add 10 or 20 to skip_five  */
-	int	skip_five = 25;	/* Used to skip multiples of 5 in the list  */
-	int	m;	/* Used to keep a working copy of n  */
+	GMT_LONG	current_factor;	/* The factor currently being tried  */
+	GMT_LONG	max_factor;	/* Don't try any factors bigger than this  */
+	GMT_LONG	n_factors = 0;	/* Returned; one if n is prime  */
+	GMT_LONG	two_four_toggle = 0;	/* Used to add 2 or 4 to get next trial factor  */
+	GMT_LONG	ten_twenty_toggle = 0;	/* Used to add 10 or 20 to skip_five  */
+	GMT_LONG	skip_five = 25;	/* Used to skip multiples of 5 in the list  */
+	GMT_LONG	m;	/* Used to keep a working copy of n  */
 
 
 	/* Initialize m and max_factor  */
-	m = abs(n);
+#ifdef __LP64__
+	m = labs(n);
+#else
+	m = abs((int)n);
+#endif
 	if (m < 2) return(0);
-	max_factor = (int)floor(sqrt((double)m));
+	max_factor = (GMT_LONG)floor(sqrt((double)m));
 
 	/* First find the 2s  */
 	current_factor = 2;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1902,7 +1933,7 @@ int get_prime_factors (int n, int *f)
 	current_factor = 3;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1911,7 +1942,7 @@ int get_prime_factors (int n, int *f)
 	current_factor = 5;
 	while(!(m%current_factor)) {
 		m /= current_factor;
-		f[n_factors] = current_factor;
+		f[n_factors] = (GMT_LONG)current_factor;
 		n_factors++;
 	}
 	if (m == 1) return(n_factors);
@@ -1950,7 +1981,7 @@ int get_prime_factors (int n, int *f)
 
 		while(!(m%current_factor)) {
 			m /= current_factor;
-			f[n_factors] = current_factor;
+			f[n_factors] = (GMT_LONG)current_factor;
 			n_factors++;
 		}
 	}
@@ -1959,7 +1990,7 @@ int get_prime_factors (int n, int *f)
 
 	if (m > 1) {
 		/* m is an additional prime factor of n  */
-		f[n_factors] = m;
+		f[n_factors] = (GMT_LONG)m;
 		n_factors++;
 	}
 	return (n_factors);
@@ -1968,7 +1999,7 @@ int get_prime_factors (int n, int *f)
 
 #define IABS(i)	(((i) < 0) ? -(i) : (i))
 
-int gcd_euclid (int a, int b)
+GMT_LONG gcd_euclid (GMT_LONG a, GMT_LONG b)
 {
 	/* Returns the greatest common divisor of u and v by Euclid's method.
 	 * I have experimented also with Stein's method, which involves only
@@ -1979,7 +2010,7 @@ int gcd_euclid (int a, int b)
 	 *
 	 * Walter H. F. Smith, 25 Feb 1992, after D. E. Knuth, vol. II  */
 
-	int	u,v,r;
+	GMT_LONG u,v,r;
 
 	u = MAX(IABS(a), IABS(b));
 	v = MIN(IABS(a), IABS(b));
@@ -2019,7 +2050,7 @@ void load_parameters (struct SURFACE_INFO *C, struct SURFACE_CTRL *Ctrl)
 void *New_Surface_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct SURFACE_CTRL *C;
 	
-	C = (struct SURFACE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct SURFACE_CTRL), "New_Surface_Ctrl");
+	C = (struct SURFACE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct SURFACE_CTRL), "New_Surface_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.value = 250;
