@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: triangulate.c,v 1.42 2007/09/30 04:16:49 guru Exp $
+ *	$Id: triangulate.c,v 1.65 2009/07/11 03:16:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -42,7 +42,7 @@
 struct TRIANGULATE_CTRL {
 	struct D {	/* -Dx|y */
 		BOOLEAN active;
-		int dir;
+		GMT_LONG dir;
 	} D;
 	struct E {	/* -E<Ctrl->E.value> */
 		BOOLEAN active;
@@ -59,28 +59,32 @@ struct TRIANGULATE_CTRL {
 		BOOLEAN active;
 		double xinc, yinc;
 	} I;
+	struct Q {	/* -Q */
+		BOOLEAN active;
+	} Q;
 };
 
 struct TRIANGULATE_EDGE {
-	int begin, end;
+	GMT_LONG begin, end;
 };
 
 int main (int argc, char **argv)
 {
-	int i, j, ij, ij1, ij2, ij3, np, k, fno, n = 0, n_alloc = 0, n_files = 0, n_edge;
-	int n_expected_fields, n_args, *link, n_fields, compare_edge(const void *p1, const void *p2);
-	int i_min, i_max, j_min, j_max, p;
+	int *link, compare_edge(const void *p1, const void *p2);
+	GMT_LONG ij, ij1, ij2, ij3, np, n_alloc, n = 0, i, j, k, n_edge;
+	GMT_LONG i_min, i_max, j_min, j_max, p, nm;
+	GMT_LONG fno, n_files = 0, n_expected_fields, n_args, n_fields;
 
 	BOOLEAN error = FALSE, map_them = FALSE;
 	BOOLEAN nofile = TRUE, done = FALSE, do_grid = FALSE;
 
 	double zj, zk, zl, zlj, zkj, *in;
 	double *xx, *yy, *zz = VNULL, vx[4], vy[4], xp, yp, a, b, c, f;
-	double xkj, xlj, ykj, ylj, out[2];
+	double xkj, xlj, ykj, ylj, out[2], *xe, *ye;
 
 	float *grd;
 
-	char line[BUFSIZ];
+	char line[BUFSIZ], *not_used = NULL;
 #ifdef TRIANGLE_D
 	char *algorithm = "Shewchuk";
 #else
@@ -109,11 +113,13 @@ int main (int argc, char **argv)
         
 				case 'H':
 				case 'J':
+				case 'M':
 				case 'R':
 				case 'V':
 				case ':':
 				case 'b':
 				case 'f':
+				case 'm':
 				case '\0':
 					error += GMT_parse_common_options (argv[i], &header.x_min, &header.x_max, &header.y_min, &header.y_max);
 					break;
@@ -150,13 +156,15 @@ int main (int argc, char **argv)
 					}
 					break;
 				case 'L':	/* Obsolete, but backward compatibility prevails [use -f instead] */
-					GMT_io.in_col_type[0] = GMT_io.out_col_type[0] = GMT_IS_LON;
-					GMT_io.in_col_type[1] = GMT_io.out_col_type[1] = GMT_IS_LAT;
+					GMT_io.in_col_type[GMT_X] = GMT_io.out_col_type[GMT_X] = GMT_IS_LON;
+					GMT_io.in_col_type[GMT_Y] = GMT_io.out_col_type[GMT_Y] = GMT_IS_LAT;
 					fprintf (stderr, "%s: Option -L is obsolete (but is processed correctly).  Please use -f instead\n", GMT_program);
 					break;
-				case 'M':
-					GMT_multisegment (&argv[i][2]);
+#ifdef TRIANGLE_D
+				case 'Q':
+					Ctrl->Q.active = TRUE;
 					break;
+#endif
 				default:
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -170,8 +178,12 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
 		fprintf (stderr, "triangulate %s - Optimal (Delaunay) triangulation of Cartesian xyz-data [%s]\n\n", GMT_VERSION, algorithm);
 		fprintf (stderr, "usage: triangulate <infiles> [-Dx|y] [-E<empty>] [-F] [-G<grdfile>]\n");
-		fprintf (stderr, "\t[%s] [%s] [%s] [%s]\n", GMT_H_OPT, GMT_I_OPT, GMT_J_OPT, GMT_M_OPT);
-		fprintf (stderr, "\t[%s] [-V] [%s] [%s]\n\t[%s]\n\n", GMT_Rgeo_OPT, GMT_t_OPT, GMT_b_OPT, GMT_f_OPT);
+#ifdef TRIANGLE_D
+		fprintf (stderr, "\t[%s] [%s] [%s] [-Q]\n", GMT_H_OPT, GMT_I_OPT, GMT_J_OPT);
+#else
+		fprintf (stderr, "\t[%s] [%s] [%s]\n", GMT_H_OPT, GMT_I_OPT, GMT_J_OPT);
+#endif
+		fprintf (stderr, "\t[%s] [-V] [%s] [%s]\n\t[%s] [%s]\n\n", GMT_Rgeo_OPT, GMT_t_OPT, GMT_b_OPT, GMT_f_OPT, GMT_m_OPT);
                 
 		if (GMT_give_synopsis_and_exit) exit (EXIT_FAILURE);
                 
@@ -184,21 +196,26 @@ int main (int argc, char **argv)
 		GMT_explain_option ('H');
 		GMT_inc_syntax ('I', 0);
 		GMT_explain_option ('J');   
-		fprintf (stderr, "\t-M output triangle edges as multiple segments separated by a record\n");
-		fprintf (stderr, "\t   whose first character is <flag> ['>']\n");
-		fprintf (stderr, "\t   [Default is to output the indices of vertices for each Delaunay triangle]\n");
+#ifdef TRIANGLE_D
+		fprintf (stderr, "\t-Q Compute Voronoi polygon edges instead (requires -m and -R) [Delaunay triangulation]\n");
+#endif
 		GMT_explain_option ('R');
 		GMT_explain_option ('V');
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
 		fprintf (stderr, "\t   Default is 2 input columns\n");
-		fprintf (stderr, "\t-bo writes binary integer index table; ignored if -M is selected [Default is ASCII i/o]\n");
+		fprintf (stderr, "\t-bo writes binary integer index table; ignored if -m is selected [Default is ASCII i/o]\n");
 		GMT_explain_option ('f');
+		fprintf (stderr, "\t-m output triangle edges as multiple segments separated by a record\n");
+		fprintf (stderr, "\t   whose first character is <flag> ['>']\n");
+		fprintf (stderr, "\t   [Default is to output the indices of vertices for each Delaunay triangle]\n");
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (GMT_io.binary[GMT_IN] && GMT_io.io_header[GMT_IN]) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
@@ -221,16 +238,28 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option.  Must specify file name\n", GMT_program);
 		error = TRUE;
 	}
-	if ((do_grid = (Ctrl->G.active + Ctrl->I.active + project_info.region_supplied)) > 0 && do_grid != 3) {
+	if (Ctrl->G.active && (do_grid = (Ctrl->I.active + project_info.region_supplied)) != 2) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Must specify -R, -I, -G for gridding\n", GMT_program);
 		error++;
 	}
+	if (Ctrl->G.active && Ctrl->Q.active) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -G option.  Cannot be used with -Q\n", GMT_program);
+		error = TRUE;
+	}
+	if (Ctrl->Q.active && !GMT_io.multi_segments[GMT_OUT]) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -Q option.  Requires -mo\n", GMT_program);
+		error = TRUE;
+	}
+	if (Ctrl->Q.active && !project_info.region_supplied) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -Q option.  Requires -R\n", GMT_program);
+		error = TRUE;
+	}
 
 	if (error) exit (EXIT_FAILURE);
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 #ifdef SET_IO_MODE
@@ -245,8 +274,7 @@ int main (int argc, char **argv)
 		GMT_RI_prepare (&header);	/* Ensure -R -I consistency and set nx, ny */
 		GMT_err_fail (GMT_grd_RI_verify (&header, 1), Ctrl->G.file);
 	}
-
-	if (project_info.region_supplied && project_info.projection > 0) { /* Gave -R -J */
+	if (project_info.region_supplied && project_info.projection != GMT_NO_PROJ) { /* Gave -R -J */
 		map_them = TRUE;
 		GMT_err_fail (GMT_map_setup (header.x_min, header.x_max, header.y_min, header.y_max), "");
 	}
@@ -283,26 +311,33 @@ int main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Reading file %s\n", GMT_program, argv[fno]);
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
+				fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n);
 				exit (EXIT_FAILURE);
 			}
 
-			xx[n] = in[0];
-			yy[n] = in[1];
-			if (do_grid) zz[n] = in[2];
+			xx[n] = in[GMT_X];
+			yy[n] = in[GMT_Y];
+			if (do_grid) zz[n] = in[GMT_Z];
 			n++;
 
 			if (n == n_alloc) {	/* Get more memory */
-				n_alloc += GMT_CHUNK;
+				n_alloc <<= 1;
 				xx = (double *) GMT_memory ((void *)xx, (size_t)n_alloc, sizeof (double), GMT_program);
 				yy = (double *) GMT_memory ((void *)yy, (size_t)n_alloc, sizeof (double), GMT_program);
 				if (do_grid) zz = (double *) GMT_memory ((void *)zz, (size_t)n_alloc, sizeof (double), GMT_program);
 			}
+			if (n == INT_MAX) {
+				fprintf (stderr, "%s: ERROR: Cannot triangulate more than %d points\n", GMT_program, INT_MAX);
+				GMT_free ((void *)xx);
+				GMT_free ((void *)yy);
+				if (do_grid) GMT_free ((void *)zz);
+				exit (EXIT_FAILURE);
+			}
 		}
 
 		if (fp != GMT_stdin) GMT_fclose (fp);
@@ -322,7 +357,13 @@ int main (int argc, char **argv)
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Do Delaunay optimal triangulation on projected coordinates\n", GMT_program);
 
-		np = GMT_delaunay (xxp, yyp, n, &link);
+		if (Ctrl->Q.active) {
+			double we[2];
+			we[0] = project_info.xmin;	we[1] = project_info.xmax;
+			np = GMT_voronoi (xxp, yyp, (int)n, we, &xe, &ye);
+		}
+		else
+			np = GMT_delaunay (xx, yy, (int)n, &link);
 
 		GMT_free ((void *)xxp);
 		GMT_free ((void *)yyp);
@@ -330,19 +371,31 @@ int main (int argc, char **argv)
 	else {
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Do Delaunay optimal triangulation on given coordinates\n", GMT_program);
 
-		np = GMT_delaunay (xx, yy, n, &link);
+		if (Ctrl->Q.active) {
+			double we[2];
+			we[0] = project_info.w;	we[1] = project_info.e;
+			np = GMT_voronoi (xx, yy, (int)n, we, &xe, &ye);
+		}
+		else
+			np = GMT_delaunay (xx, yy, (int)n, &link);
 	}
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: %d Delaunay triangles found\n", GMT_program, np);
-
+	if (gmtdefs.verbose) {
+		if (Ctrl->Q.active)
+			fprintf (stderr, "%s: %ld Voronoi edges found\n", GMT_program, np);
+		else
+			fprintf (stderr, "%s: %ld Delaunay triangles found\n", GMT_program, np);
+	}
+	
 	if (do_grid) {
 
 		header.nx = GMT_get_n (header.x_min, header.x_max, header.x_inc, header.node_offset);
 		header.ny = GMT_get_n (header.y_min, header.y_max, header.y_inc, header.node_offset);
 		header.xy_off = 0.5 * header.node_offset;
-		grd = (float *) GMT_memory (VNULL, (size_t)(header.nx * header.ny), sizeof (float), GMT_program);
+		nm = GMT_get_nm (header.nx, header.ny);
+		grd = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 		if (!Ctrl->E.active) Ctrl->E.value = GMT_d_NaN;
-		for (i = 0; i < header.nx * header.ny; i++) grd[i] = (float)Ctrl->E.value;	/* initialize grid */
+		for (i = 0; i < nm; i++) grd[i] = (float)Ctrl->E.value;	/* initialize grid */
 
 		for (k = ij = 0; k < np; k++) {
 
@@ -387,7 +440,7 @@ int main (int argc, char **argv)
 				for (i = i_min; i <= i_max; i++, p++) {
 					xp = GMT_i_to_x (i, header.x_min, header.x_max, header.x_inc, header.xy_off, header.nx);
 
-					if (!GMT_non_zero_winding (xp, yp, vx, vy, 4)) continue;	/* Outside */
+					if (!GMT_non_zero_winding (xp, yp, vx, vy, (GMT_LONG)4)) continue;	/* Outside */
 
 					if (Ctrl->D.dir == GMT_X)
 						grd[p] = (float)a;
@@ -399,43 +452,69 @@ int main (int argc, char **argv)
 			}
 		}
 		GMT_err_fail (GMT_write_grd (Ctrl->G.file, &header, grd, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
+		GMT_free ((void *)zz);
+		GMT_free ((void *)grd);
 	}
 	if (GMT_io.multi_segments[GMT_OUT]) {	/* Must find unique edges to output only once */
-		n_edge = 3 * np;
-		edge = (struct TRIANGULATE_EDGE *) GMT_memory (VNULL, (size_t)n_edge, sizeof (struct TRIANGULATE_EDGE), GMT_program);
-		for (i = ij1 = 0, ij2 = 1, ij3 = 2; i < np; i++, ij1 += 3, ij2 += 3, ij3 += 3) {
-			edge[ij1].begin = link[ij1];	edge[ij1].end = link[ij2];
-			edge[ij2].begin = link[ij2];	edge[ij2].end = link[ij3];
-			edge[ij3].begin = link[ij1];	edge[ij3].end = link[ij3];
+		if (Ctrl->Q.active) {	/* Voronoi edges */
+			for (i = j = 0; i < np; i++) {
+				fprintf (GMT_stdout, "%c Edge %d\n", GMT_io.EOF_flag[GMT_OUT], (int)i);
+				out[GMT_X] = xe[j];	out[GMT_Y] = ye[j++];
+				GMT_output (GMT_stdout, 2, out);
+				out[GMT_X] = xe[j];	out[GMT_Y] = ye[j++];
+				GMT_output (GMT_stdout, 2, out);
+			}
+			GMT_free ((void *)xe);
+			GMT_free ((void *)ye);
 		}
-		for (i = 0; i < n_edge; i++) if (edge[i].begin > edge[i].end) i_swap (edge[i].begin, edge[i].end);
+		else {	/* Triangle edges */
+			n_edge = 3 * np;
+			edge = (struct TRIANGULATE_EDGE *) GMT_memory (VNULL, (size_t)n_edge, sizeof (struct TRIANGULATE_EDGE), GMT_program);
+			for (i = ij1 = 0, ij2 = 1, ij3 = 2; i < np; i++, ij1 += 3, ij2 += 3, ij3 += 3) {
+				edge[ij1].begin = link[ij1];	edge[ij1].end = link[ij2];
+				edge[ij2].begin = link[ij2];	edge[ij2].end = link[ij3];
+				edge[ij3].begin = link[ij1];	edge[ij3].end = link[ij3];
+			}
+			for (i = 0; i < n_edge; i++) if (edge[i].begin > edge[i].end) i_swap (edge[i].begin, edge[i].end);
 
-		qsort ((void *)edge, (size_t)n_edge, sizeof (struct TRIANGULATE_EDGE), compare_edge);
-		for (i = 1, j = 0; i < n_edge; i++) {
-			if (edge[i].begin != edge[j].begin || edge[i].end != edge[j].end) j++;
-			edge[j] = edge[i];
+			qsort ((void *)edge, (size_t)n_edge, sizeof (struct TRIANGULATE_EDGE), compare_edge);
+			for (i = 1, j = 0; i < n_edge; i++) {
+				if (edge[i].begin != edge[j].begin || edge[i].end != edge[j].end) j++;
+				edge[j] = edge[i];
+			}
+			n_edge = j + 1;
+
+			if (gmtdefs.verbose) fprintf (stderr, "%s: %ld unique triangle edges\n", GMT_program, n_edge);
+
+			for (i = 0; i < n_edge; i++) {
+				fprintf (GMT_stdout, "%c Edge %ld-%ld\n", GMT_io.EOF_flag[GMT_OUT], edge[i].begin, edge[i].end);
+				out[GMT_X] = xx[edge[i].begin];	out[GMT_Y] = yy[edge[i].begin];
+				GMT_output (GMT_stdout, 2, out);
+				out[GMT_X] = xx[edge[i].end];	out[GMT_Y] = yy[edge[i].end];
+				GMT_output (GMT_stdout, 2, out);
+			}
+			GMT_free ((void *)edge);
 		}
-		n_edge = j + 1;
-
-		if (gmtdefs.verbose) fprintf (stderr, "%s: %d unique triangle edges\n", GMT_program, n_edge);
-
-		for (i = 0; i < n_edge; i++) {
-			fprintf (GMT_stdout, "%c Edge %d-%d\n", GMT_io.EOF_flag[GMT_OUT], edge[i].begin, edge[i].end);
-			out[0] = xx[edge[i].begin];	out[1] = yy[edge[i].begin];
-			GMT_output (GMT_stdout, 2, out);
-			out[0] = xx[edge[i].end];	out[1] = yy[edge[i].end];
-			GMT_output (GMT_stdout, 2, out);
-		}
-		GMT_free ((void *)edge);
 	}
 	else if (GMT_io.binary[GMT_OUT])
-		GMT_fwrite ((void *)link, sizeof (int), (size_t)(3*np), GMT_stdout);
+		i = GMT_fwrite ((void *)link, sizeof (int), (size_t)(3*np), GMT_stdout);
 	else
 		for (i = ij = 0; i < np; i++, ij += 3) fprintf (GMT_stdout, "%d\t%d\t%d\n", link[ij], link[ij+1], link[ij+2]);
 
 	GMT_free ((void *) xx);
 	GMT_free ((void *) yy);
-	GMT_free ((void *) link);
+#ifdef TRIANGLE_D
+#ifdef DEBUG
+	/* Shewchuk's function allocated the memory separately */
+	GMT_memtrack_off (GMT_mem_keeper);
+#endif
+#endif
+	if (!Ctrl->Q.active) GMT_free ((void *) link);
+#ifdef TRIANGLE_D
+#ifdef DEBUG
+	GMT_memtrack_on (GMT_mem_keeper);
+#endif
+#endif
 
 	if (gmtdefs.verbose) fprintf (stderr, "%s: Done!\n", GMT_program);
 
@@ -469,7 +548,7 @@ int compare_edge (const void *p1, const void *p2)
 void *New_Triangulate_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct TRIANGULATE_CTRL *C;
 	
-	C = (struct TRIANGULATE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct TRIANGULATE_CTRL), "New_Triangulate_Ctrl");
+	C = (struct TRIANGULATE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct TRIANGULATE_CTRL), "New_Triangulate_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->D.dir = -1;	/* No derivatives */
