@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: xyz2grd.c,v 1.78 2008/03/24 08:58:32 guru Exp $
+ *	$Id: xyz2grd.c,v 1.88 2009/07/11 03:16:31 guru Exp $
  *
- *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2009 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -78,10 +78,9 @@ int main (int argc, char **argv)
 	BOOLEAN error = FALSE, do_grid = FALSE, count = FALSE, average = TRUE;
 	BOOLEAN b_only = FALSE, done = FALSE, nofile = TRUE;
 	
-	int i, ii, jj , z = 2, fno, n_args, *flag = VNULL;
-	int n_expected_fields, n_fields, n_files = 0, high_low = 0;
-	
-	GMT_LONG gmt_ij, nm = 0, n_read = 0, n_filled = 0, n_used = 0;
+	GMT_LONG z = 2, fno, n_args, *flag = VNULL;
+	GMT_LONG n_expected_fields, n_fields, n_files = 0, high_low = 0;
+	GMT_LONG i, ii, jj , gmt_ij, nm = 0, n_read = 0, n_filled = 0, n_used = 0;
 	GMT_LONG ij = -1;	/* Will be incremented to 0 or set first time around */
 	GMT_LONG n_empty = 0, n_stuffed = 0, n_bad = 0, n_confused = 0;
 
@@ -89,7 +88,7 @@ int main (int argc, char **argv)
 
 	float no_data_f, *a = VNULL;
 
-	char line[BUFSIZ];
+	char line[BUFSIZ], *not_used = NULL;
 
 	FILE *fp = NULL;
 
@@ -177,7 +176,7 @@ int main (int argc, char **argv)
 					break;
 				case 'Z':
 					Ctrl->Z.active = TRUE;
-					error += GMT_parse_z_io (&argv[i][2], &Ctrl->Z);
+					error += (BOOLEAN)GMT_parse_z_io (&argv[i][2], &Ctrl->Z);
 					break;
 				default:
 					error = TRUE;
@@ -229,7 +228,8 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t   Append x if gridline-registered, periodic data in x without repeating column at xmax\n");
 		fprintf (stderr, "\t   Append y if gridline-registered, periodic data in y without repeating row at ymax\n");
 		fprintf (stderr, "\t   Specify one of the following data types (all binary except a):\n");
-		fprintf (stderr, "\t     a  Ascii\n");
+		fprintf (stderr, "\t     A  Ascii (multiple floating point values per record)\n");
+		fprintf (stderr, "\t     a  Ascii (one value per record)\n");
 		fprintf (stderr, "\t     c  signed 1-byte character\n");
 		fprintf (stderr, "\t     u  unsigned 1-byte character\n");
 		fprintf (stderr, "\t     h  signed short 2-byte integer\n");
@@ -250,6 +250,8 @@ int main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 
+	GMT_check_lattice (&Ctrl->I.xinc, &Ctrl->I.yinc, &Ctrl->F.active, &Ctrl->I.active);
+
 	if (Ctrl->S.active && !Ctrl->Z.active) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR option -S:  Must also specify -Z\n", GMT_program);
 		error++;
@@ -279,7 +281,7 @@ int main (int argc, char **argv)
 			error++;
 		}
 	}
-	if (!Ctrl->G.active && !Ctrl->G.file) {
+	if (!Ctrl->S.active && !(Ctrl->G.active || Ctrl->G.file)) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR option -G:  Must specify output file\n", GMT_program);
 		error++;
 	}
@@ -331,7 +333,7 @@ int main (int argc, char **argv)
 
 	if (GMT_io.binary[GMT_IN] && gmtdefs.verbose) {
 		char *type[2] = {"double", "single"};
-		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
+		fprintf (stderr, "%s: Expects %ld-column %s-precision binary data\n", GMT_program, GMT_io.ncol[GMT_IN], type[GMT_io.single_precision[GMT_IN]]);
 	}
 
 	if (GMT_io.binary[GMT_IN] && GMT_io.in_col_type[2] & GMT_IS_RATIME && GMT_io.single_precision[GMT_IN]) {
@@ -350,10 +352,10 @@ int main (int argc, char **argv)
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: nx = %d  ny = %d\n", GMT_program, grd.nx, grd.ny);
 
-		nm = grd.nx * grd.ny;
+		nm = GMT_get_nm (grd.nx, grd.ny);
 
 		a = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
-		flag = (int *) GMT_memory (VNULL, (size_t)nm, sizeof (int), GMT_program);
+		flag = (GMT_LONG *) GMT_memory (VNULL, (size_t)nm, sizeof (GMT_LONG), GMT_program);
 
 		n_expected_fields = (Ctrl->Z.active) ? 1 : 3;
 		z = (Ctrl->Z.active) ? 0 : 2;
@@ -380,6 +382,12 @@ int main (int argc, char **argv)
 			GMT_setmode (GMT_IN);
 #endif
 		}
+#ifdef WIN32
+		else if (Ctrl->E.active && (fp = fopen (argv[fno], GMT_io.r_mode)) == NULL) {
+			fprintf (stderr, "%s: Cannot open file %s\n", GMT_program, argv[fno]);
+			continue;
+		}
+#endif
 		else if ((fp = GMT_fopen (argv[fno], GMT_io.r_mode)) == NULL) {
 			fprintf (stderr, "%s: Cannot open file %s\n", GMT_program, argv[fno]);
 			continue;
@@ -387,7 +395,7 @@ int main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 
-		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
+		if (GMT_io.io_header[GMT_IN]) for (i = 0; i < GMT_io.n_header_recs; i++) not_used = GMT_fgets (line, BUFSIZ, fp);
 
 		if (Ctrl->S.active) {	/* Just swap data and exit */
 			FILE *fps = NULL;
@@ -442,35 +450,35 @@ int main (int argc, char **argv)
 			float value;
 			
 			grd.node_offset = 0;
-			GMT_fgets (line, BUFSIZ, fp);
+			not_used = fgets (line, BUFSIZ, fp);
 			if (sscanf (line, "%*s %d", &grd.nx) != 1) {
 				fprintf (stderr, "%s: Error decoding ncols record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
-			GMT_fgets (line, BUFSIZ, fp);
+			not_used = fgets (line, BUFSIZ, fp);
 			if (sscanf (line, "%*s %d", &grd.ny) != 1) {
 				fprintf (stderr, "%s: Error decoding ncols record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
-			GMT_fgets (line, BUFSIZ, fp);
+			not_used = fgets (line, BUFSIZ, fp);
 			if (sscanf (line, "%*s %lf", &grd.x_min) != 1) {
 				fprintf (stderr, "%s: Error decoding xll record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
 			if (!strncmp (line, "xllcorner", (size_t)9)) grd.node_offset = 1;	/* Pixel grid */
-			GMT_fgets (line, BUFSIZ, fp);
+			not_used = fgets (line, BUFSIZ, fp);
 			if (sscanf (line, "%*s %lf", &grd.y_min) != 1) {
 				fprintf (stderr, "%s: Error decoding yll record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
 			if (!strncmp (line, "yllcorner", (size_t)9)) grd.node_offset = 1;	/* Pixel grid */
-			GMT_fgets (line, BUFSIZ, fp);
+			not_used = fgets (line, BUFSIZ, fp);
 			if (sscanf (line, "%*s %lf", &grd.x_inc) != 1) {
 				fprintf (stderr, "%s: Error decoding cellsize record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
 			if (!Ctrl->E.set) {	/* Only read if not specified in -E */
-				GMT_fgets (line, BUFSIZ, fp);
+				not_used = fgets (line, BUFSIZ, fp);
 				if (sscanf (line, "%*s %lf", &Ctrl->E.nodata) != 1) {
 					fprintf (stderr, "%s: Error decoding nodata_value record\n", GMT_program);
 					exit (EXIT_FAILURE);
@@ -484,7 +492,7 @@ int main (int argc, char **argv)
 			GMT_err_fail (GMT_grd_RI_verify (&grd, 1), Ctrl->G.file);
 
 			if (gmtdefs.verbose) fprintf (stderr, "%s: nx = %d  ny = %d\n", GMT_program, grd.nx, grd.ny);
-			nm = n_left = grd.nx * grd.ny;
+			nm = n_left = GMT_get_nm (grd.nx, grd.ny);
 
 			a = (float *) GMT_memory (VNULL, (size_t)nm, sizeof (float), GMT_program);
 			/* ESRI grids are scanline oriented (top to bottom) as are GMT grids */
@@ -506,20 +514,20 @@ int main (int argc, char **argv)
 			while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_read);
+					fprintf (stderr, "%s: Mismatch between actual (%ld) and expected (%ld) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_read);
 					exit (EXIT_FAILURE);
 				}
 				n_read++;
 
-				if (GMT_y_is_outside (in[1],  wesn[2], wesn[3])) continue;	/* Outside y-range */
-				if (GMT_x_is_outside (&in[0], wesn[0], wesn[1])) continue;	/* Outside x-range */
+				if (GMT_y_is_outside (in[GMT_Y],  wesn[2], wesn[3])) continue;	/* Outside y-range */
+				if (GMT_x_is_outside (&in[GMT_X], wesn[0], wesn[1])) continue;	/* Outside x-range */
 
 				/* Ok, we are inside the region - process data */
 
-				ii = GMT_x_to_i (in[0], grd.x_min, grd.x_inc, grd.xy_off, grd.nx);
+				ii = GMT_x_to_i (in[GMT_X], grd.x_min, grd.x_inc, grd.xy_off, grd.nx);
 				if (ii == -1) ii++, n_confused++;
 				if (ii == grd.nx) ii--, n_confused++;
-				jj = GMT_y_to_j (in[1], grd.y_min, grd.y_inc, grd.xy_off, grd.ny);
+				jj = GMT_y_to_j (in[GMT_Y], grd.y_min, grd.y_inc, grd.xy_off, grd.ny);
 				if (jj == -1) jj++, n_confused++;
 				if (jj == grd.ny) jj--, n_confused++;
 				ij = GMT_IJ (jj, ii, grd.nx);
@@ -539,7 +547,13 @@ int main (int argc, char **argv)
 				n_used++;
 			}
 		}
+#ifdef WIN32
+		if (fp != GMT_stdin) {
+			(Ctrl->E.active) ? fclose (fp) : GMT_fclose (fp);
+		}
+#else
 		if (fp != GMT_stdin) GMT_fclose (fp);
+#endif
 	}
 
 	if (!(Ctrl->Z.active || Ctrl->E.active)) {	/* xyz data could have resulted in duplicates */
